@@ -7,12 +7,22 @@
 /* define if GNOME WM hints are enabled */
 #undef GNOME
 
+/* define if extended WM hints are enabled */
+#undef HAVE_EWMH
+
 /* Where to search for images.  */
 #undef FVWM_IMAGEPATH
 
 /* Define if Xpm library is used.  */
 #undef XPM
 
+/* Define if ppm library is used.  */
+#undef HAVE_PNG
+
+/* Define if your processor stores words with the most significant
+   byte first (like Motorola and SPARC, unlike Intel and VAX).  */
+#undef WORDS_BIGENDIAN
+
 /* Define if rplay library is used.  */
 #undef HAVE_RPLAY
 
@@ -22,6 +32,15 @@
 /* Define if Xinerama library is used. */
 #undef HAVE_XINERAMA
 
+/* Define if Xrender library is used. */
+#undef HAVE_XRENDER
+
+/* Define if Xft library is used. */
+#undef HAVE_XFT
+
+/* Define if Xft library can handle utf8 encoding  */
+#undef HAVE_XFT_UTF8
+
 /* Define if stroke library is used. */
 #undef HAVE_STROKE
 
@@ -36,6 +55,25 @@
 /* Define if readline is available.  */
 #undef HAVE_READLINE
 
+/* Define if iconv (in the libc) or libiconv is available */
+#undef HAVE_ICONV
+
+/* define if we use libiconv (not needed in general: for example iconv is
+* native with recent glibc) */
+#undef USE_LIBICONV
+
+/* define if second arg of iconv use const */
+#undef ICONV_ARG_USE_CONST
+
+#ifdef USE_LIBICONV
+/* define to use locale_charset in the place of nl_langinfog if libiconv
+ * is used */ 
+#undef HAVE_LIBCHARSET
+#endif
+
+/* Define if nl_langinfo is available */
+#undef HAVE_CODESET
+
 /* Define if you want the Shaped window extensions.
  * Shaped window extensions seem to increase the window managers RSS
  * by about 60 Kbytes. They provide for leaving a title-bar on the window
@@ -53,20 +91,20 @@
  * the currently running X server supports it.  */
 #undef SHAPE
 
+/* Define if fribidi library is used.  */
+#undef HAVE_BIDI
+
 /* Enables the "MiniIcon" Style option to specify a small pixmap which
  * can be used as one of the title-bar buttons, shown in window list,
  * utilized by modules, etc.  Requires PIXMAP_BUTTONS to be defined
  * (see below).  */
 /* #undef MINI_ICONS */
 /* NOTE: hard coded to 1 */
-#define MINI_ICONS 1
-
-/* Enables stacked button styles.  There is a slight memory
- * penalty for each additional style. See the man page entries for
- * AddButtonStyle and AddTitleStyle for usage information.  */
-/* #undef MULTISTYLE */
-/* NOTE: hard coded to 1 */
-#define MULTISTYLE 1
+#if 1
+#define FMiniIconsSupported 1
+#else
+#define FMiniIconsSupported 0
+#endif
 
 /* Enables tagged general decoration styles which can be assigned to
  * windows using the UseDecor Style option, or dynamically updated
@@ -77,18 +115,24 @@
 /* NOTE: hard coded to 1 */
 #define USEDECOR 1
 
+/* Enables multi-pixmap themeable titlebars */
+#undef FANCY_TITLEBARS
+
 /* Enables session management functionality. */
 #undef SESSION
 
-/* Enables X11 compound text conversion for windows and icons names */
+/* Enables X11 multibyte character support (was I18N_MB) */
+#undef MULTIBYTE
+
+/* Enables compund text */
 #undef COMPOUND_TEXT
 
-/* Enables X11 multibyte character support */
-#undef I18N_MB
-
-/* Enables to use setlocale() provided by X */;
+/* Enables to use setlocale() provided by X */
 #undef X_LOCALE
 
+/* Enable X output method */
+#undef HAVE_XOUTPUT_METHOD
+
 /* Specify a type for sig_atomic_t if it's not available.  */
 #undef sig_atomic_t
 
