@@ -18,10 +18,8 @@
 
 /* This file contains all command prototypes. */
 void CMD_Plus(F_CMD_ARGS);
-#ifdef MULTISTYLE
 void CMD_AddButtonStyle(F_CMD_ARGS);
 void CMD_AddTitleStyle(F_CMD_ARGS);
-#endif /* MULTISTYLE */
 #ifdef USEDECOR
 void CMD_AddToDecor(F_CMD_ARGS);
 #endif /* USEDECOR */
@@ -29,7 +27,9 @@ void CMD_AddToFunc(F_CMD_ARGS);
 void CMD_AddToMenu(F_CMD_ARGS);
 void CMD_All(F_CMD_ARGS);
 void CMD_AnimatedMove(F_CMD_ARGS);
+void CMD_Any(F_CMD_ARGS);
 void CMD_Beep(F_CMD_ARGS);
+void CMD_Break(F_CMD_ARGS);
 void CMD_BorderStyle(F_CMD_ARGS);
 void CMD_BugOpts(F_CMD_ARGS);
 void CMD_BusyCursor(F_CMD_ARGS);
@@ -39,11 +39,14 @@ void CMD_ButtonStyle(F_CMD_ARGS);
 void CMD_ChangeDecor(F_CMD_ARGS);
 #endif /* USEDECOR */
 void CMD_ChangeMenuStyle(F_CMD_ARGS);
+void CMD_CleanupColorsets(F_CMD_ARGS);
 void CMD_ClickTime(F_CMD_ARGS);
 void CMD_Close(F_CMD_ARGS);
 void CMD_ColorLimit(F_CMD_ARGS);
 void CMD_ColormapFocus(F_CMD_ARGS);
 void CMD_Colorset(F_CMD_ARGS);
+void CMD_Cond(F_CMD_ARGS);
+void CMD_CondCase(F_CMD_ARGS);
 void CMD_CopyMenuStyle(F_CMD_ARGS);
 void CMD_Current(F_CMD_ARGS);
 void CMD_CursorMove(F_CMD_ARGS);
@@ -54,7 +57,9 @@ void CMD_DefaultFont(F_CMD_ARGS);
 void CMD_DefaultIcon(F_CMD_ARGS);
 void CMD_DefaultLayers(F_CMD_ARGS);
 void CMD_Delete(F_CMD_ARGS);
+void CMD_Deschedule(F_CMD_ARGS);
 void CMD_Desk(F_CMD_ARGS);
+void CMD_DesktopName(F_CMD_ARGS);
 void CMD_DesktopSize(F_CMD_ARGS);
 void CMD_Destroy(F_CMD_ARGS);
 #ifdef USEDECOR
@@ -67,6 +72,7 @@ void CMD_DestroyModuleConfig(F_CMD_ARGS);
 void CMD_DestroyStyle(F_CMD_ARGS);
 void CMD_Direction(F_CMD_ARGS);
 void CMD_Echo(F_CMD_ARGS);
+void CMD_EdgeCommand(F_CMD_ARGS); 
 void CMD_EdgeResistance(F_CMD_ARGS);
 void CMD_EdgeScroll(F_CMD_ARGS);
 void CMD_EdgeThickness(F_CMD_ARGS);
@@ -120,24 +126,30 @@ void CMD_PipeRead(F_CMD_ARGS);
 void CMD_PixmapPath(F_CMD_ARGS);
 void CMD_PlaceAgain(F_CMD_ARGS);
 void CMD_PointerKey(F_CMD_ARGS);
+void CMD_PointerWindow(F_CMD_ARGS);
 void CMD_Popup(F_CMD_ARGS);
 void CMD_Prev(F_CMD_ARGS);
+void CMD_PropertyChange(F_CMD_ARGS);
 void CMD_Quit(F_CMD_ARGS);
 void CMD_QuitScreen(F_CMD_ARGS);
 void CMD_QuitSession(F_CMD_ARGS);
 void CMD_Raise(F_CMD_ARGS);
 void CMD_RaiseLower(F_CMD_ARGS);
 void CMD_Read(F_CMD_ARGS);
+void CMD_ReadWriteColors(F_CMD_ARGS);
 void CMD_Recapture(F_CMD_ARGS);
 void CMD_RecaptureWindow(F_CMD_ARGS);
 void CMD_Refresh(F_CMD_ARGS);
 void CMD_RefreshWindow(F_CMD_ARGS);
 void CMD_Repeat(F_CMD_ARGS);
 void CMD_Resize(F_CMD_ARGS);
+void CMD_ResizeMaximize(F_CMD_ARGS);
 void CMD_ResizeMove(F_CMD_ARGS);
+void CMD_ResizeMoveMaximize(F_CMD_ARGS);
 void CMD_Restart(F_CMD_ARGS);
 void CMD_SaveQuitSession(F_CMD_ARGS);
 void CMD_SaveSession(F_CMD_ARGS);
+void CMD_Schedule(F_CMD_ARGS);
 void CMD_Scroll(F_CMD_ARGS);
 void CMD_Send_ConfigInfo(F_CMD_ARGS);
 void CMD_Send_WindowList(F_CMD_ARGS);
@@ -150,12 +162,15 @@ void CMD_SetEnv(F_CMD_ARGS);
 void CMD_Silent(F_CMD_ARGS);
 void CMD_SnapAttraction(F_CMD_ARGS);
 void CMD_SnapGrid(F_CMD_ARGS);
+void CMD_State(F_CMD_ARGS);
 void CMD_Stick(F_CMD_ARGS);
 #ifdef HAVE_STROKE
 void CMD_Stroke(F_CMD_ARGS);
 void CMD_StrokeFunc(F_CMD_ARGS);
 #endif /* HAVE_STROKE */
 void CMD_Style(F_CMD_ARGS);
+void CMD_TearMenuOff(F_CMD_ARGS);
+void CMD_ThisWindow(F_CMD_ARGS);
 void CMD_Title(F_CMD_ARGS);
 void CMD_TitleStyle(F_CMD_ARGS);
 void CMD_UnsetEnv(F_CMD_ARGS);
@@ -174,5 +189,7 @@ void CMD_XineramaSls(F_CMD_ARGS);
 void CMD_XineramaSlsSize(F_CMD_ARGS);
 void CMD_XorPixmap(F_CMD_ARGS);
 void CMD_XorValue(F_CMD_ARGS);
+void CMD_XSync(F_CMD_ARGS);
+void CMD_XSynchronize(F_CMD_ARGS);
 
 #endif /* COMMANDS_H */
