@@ -54,6 +54,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -66,6 +67,7 @@
 #include "lang-strings.h"
 #include "update.h"
 #include "style.h"
+#include "geometry.h"
 
 
 extern Atom _XA_MwmAtom;
@@ -113,12 +115,11 @@ typedef PropMotifWmHints        PropMwmHints;
 #define OL_DECOR_RESIZEH              (1L << 1)
 #define OL_DECOR_HEADER               (1L << 2)
 #define OL_DECOR_ICON_NAME            (1L << 3)
-#define OL_DECOR_ALL                  (OL_DECOR_CLOSE | OL_DECOR_RESIZEH | OL_DECOR_HEADER | OL_DECOR_ICON_NAME)
+#define OL_DECOR_ALL                  \
+  (OL_DECOR_CLOSE | OL_DECOR_RESIZEH | OL_DECOR_HEADER | OL_DECOR_ICON_NAME)
 /* indicates if there are any OL hints */
 #define OL_ANY_HINTS                  (1L << 7)
 
-extern FvwmWindow *Tmp_win;
-
 /****************************************************************************
  *
  * Reads the property MOTIF_WM_HINTS
@@ -135,7 +136,7 @@ void GetMwmHints(FvwmWindow *t)
     XFree((char *)t->mwm_hints);
     t->mwm_hints = NULL;
   }
-  if(XGetWindowProperty (dpy, t->w, _XA_MwmAtom, 0L, 20L, False,
+  if(XGetWindowProperty (dpy, FW_W(t), _XA_MwmAtom, 0L, 20L, False,
 			 _XA_MwmAtom, &actual_type, &actual_format, &nitems,
 			 &bytesafter,(unsigned char **)&t->mwm_hints)==Success)
   {
@@ -186,7 +187,7 @@ void GetOlHints(FvwmWindow *t)
 
   t->ol_hints = OL_DECOR_ALL;
 
-  if (XGetWindowProperty (dpy, t->w, _XA_OL_WIN_ATTR, 0L, 20L, False,
+  if (XGetWindowProperty (dpy, FW_W(t), _XA_OL_WIN_ATTR, 0L, 20L, False,
                           _XA_OL_WIN_ATTR, &actual_type, &actual_format,
 			  &nitems, &bytesafter,
 			  (unsigned char **)&hints)==Success)
@@ -224,7 +225,7 @@ void GetOlHints(FvwmWindow *t)
       XFree (hints);
   }
 
-  if(XGetWindowProperty (dpy, t->w, _XA_OL_DECOR_ADD, 0L, 20L, False,
+  if(XGetWindowProperty (dpy, FW_W(t), _XA_OL_DECOR_ADD, 0L, 20L, False,
 			 XA_ATOM, &actual_type, &actual_format, &nitems,
 			 &bytesafter,(unsigned char **)&hints)==Success)
   {
@@ -244,7 +245,7 @@ void GetOlHints(FvwmWindow *t)
       XFree (hints);
   }
 
-  if(XGetWindowProperty (dpy, t->w, _XA_OL_DECOR_DEL, 0L, 20L, False,
+  if(XGetWindowProperty (dpy, FW_W(t), _XA_OL_DECOR_DEL, 0L, 20L, False,
 			 XA_ATOM, &actual_type, &actual_format, &nitems,
 			 &bytesafter,(unsigned char **)&hints)==Success)
   {
@@ -278,6 +279,7 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
   int i;
   short border_width;
   short handle_width;
+  short used_width;
   PropMwmHints *prop;
   style_flags *sflags = &(pstyle->flags);
 
@@ -419,7 +421,7 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
       (!SDO_DECORATE_TRANSIENT(sflags) && IS_TRANSIENT(t)))
     decor &= ~MWM_DECOR_TITLE;
 
-  if (SHAS_NO_BORDER(sflags) ||
+  if (SHAS_NO_HANDLES(sflags) ||
       (!SDO_DECORATE_TRANSIENT(sflags) && IS_TRANSIENT(t)))
     decor &= ~MWM_DECOR_RESIZEH;
 
@@ -436,14 +438,14 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
   /* Assume no decorations, and build up */
   SET_HAS_TITLE(t, 0);
   SET_HAS_BORDER(t, 0);
+  SET_HAS_HANDLES(t, 0);
 
-  t->boundary_width = 0;
-
+  used_width = 0;
   if(decor & MWM_DECOR_BORDER)
   {
     /* A narrow border is displayed (5 pixels - 2 relief, 1 top,
      * (2 shadow) */
-    t->boundary_width = border_width;
+    used_width = border_width;
   }
   if(decor & MWM_DECOR_TITLE)
   {
@@ -455,9 +457,12 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
   {
     /* A wide border, with corner tiles is desplayed
      * (10 pixels - 2 relief, 2 shadow) */
-    SET_HAS_BORDER(t, 1);
-    t->boundary_width = handle_width;
+    SET_HAS_HANDLES(t, 1);
+    used_width = handle_width;
   }
+  SET_HAS_BORDER(t, (used_width > 0));
+  SET_HAS_HANDLES(t, (HAS_BORDER(t) && HAS_HANDLES(t)));
+  set_window_border_size(t, used_width);
   if(!(decor & MWM_DECOR_MENU))
   {
     /*  title-bar menu button omitted
@@ -520,11 +525,7 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
       t->nr_right_buttons--;
   }
 
-  if(t->boundary_width <= 0)
-  {
-    t->boundary_width = 0;
-    SET_HAS_BORDER(t, 0);
-  }
+  return;
 }
 
 /*
@@ -532,91 +533,175 @@ void SelectDecor(FvwmWindow *t, window_style *pstyle, short *buttons)
 ** check_allowed_function2 partially overlapping in their checks, so I
 ** combined them here and made them wrapper functions instead.
 */
-int check_if_function_allowed(int function, FvwmWindow *t,
-			      Bool override_allowed, char *menu_string)
+Bool is_function_allowed(
+        int function, char *action_string, FvwmWindow *t, Bool is_user_request,
+        Bool do_allow_override_mwm_hints)
 {
-  if (!t)
-    return 1;
+        unsigned int functions;
+        char *functionlist[] = {
+                MOVE_STRING,
+                RESIZE_STRING1,
+                RESIZE_STRING2,
+                MINIMIZE_STRING,
+                MINIMIZE_STRING2,
+                MAXIMIZE_STRING,
+                CLOSE_STRING1,
+                CLOSE_STRING2,
+                CLOSE_STRING3,
+                CLOSE_STRING4,
+                NULL
+        };
 
-  if (override_allowed && HAS_MWM_OVERRIDE_HINTS(t))
-    return 1;
+        if (t == NULL)
+        {
+		return True;  /* this logic come from animated menu */
+        }
+        if (do_allow_override_mwm_hints && HAS_MWM_OVERRIDE_HINTS(t))
+        {
+                /* allow everything */
+                functions = ~0;
+        }
+        else
+        {
+                /* restrict by mwm hints */
+                functions = t->functions;
+        }
 
-  switch(function)
-  {
-    case F_DELETE:
-      if (!WM_DELETES_WINDOW(t))
-	return 0;
-      /* fall through to close clause */
-    case F_CLOSE:
-      if (!(t->functions & MWM_FUNC_CLOSE))
-	return 0;
-      break;
-    case F_DESTROY: /* shouldn't destroy always be allowed??? */
-      if (!(t->functions & MWM_FUNC_CLOSE))
-	return 0;
-      break;
-    case F_RESIZE:
-      if (!(t->functions & MWM_FUNC_RESIZE))
-	return 0;
-      break;
-    case F_ICONIFY:
-      if ((!IS_ICONIFIED(t))&&
-	  (!(t->functions & MWM_FUNC_MINIMIZE)))
-	return 0;
-      break;
-    case F_MAXIMIZE:
-      if (!(t->functions & MWM_FUNC_MAXIMIZE))
-	return 0;
-      break;
-    case F_MOVE:
-      /* Move is a funny hint. Keeps it out of the menu, but you're
-       * still allowed to move. */
-      if((!(t->functions & MWM_FUNC_MOVE))&&menu_string)
-	return 0;
-      break;
-    case F_FUNCTION:
-      /* Hard part! What to do now? */
-      /* Hate to do it, but for lack of a better idea,
-       * check based on the menu entry name */
-      /* Complex functions are a little tricky, ignore them if no menu item*/
-      if (menu_string)
-      {
-	if((!(t->functions & MWM_FUNC_MOVE))&&
-	   (StrEquals(menu_string,MOVE_STRING)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_RESIZE))&&
-	   (StrEquals(menu_string,RESIZE_STRING1)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_RESIZE))&&
-	   (StrEquals(menu_string,RESIZE_STRING2)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_MINIMIZE))&&
-	   (!IS_ICONIFIED(t))&&
-	   (StrEquals(menu_string,MINIMIZE_STRING)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_MINIMIZE))&&
-	   (StrEquals(menu_string,MINIMIZE_STRING2)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_MAXIMIZE))&&
-	   (StrEquals(menu_string,MAXIMIZE_STRING)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_CLOSE))&&
-	   (StrEquals(menu_string,CLOSE_STRING1)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_CLOSE))&&
-	   (StrEquals(menu_string,CLOSE_STRING2)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_CLOSE))&&
-	   (StrEquals(menu_string,CLOSE_STRING3)))
-	  return 0;
-	if((!(t->functions & MWM_FUNC_CLOSE))&&
-	   (StrEquals(menu_string,CLOSE_STRING4)))
-	  return 0;
-      }
-      break;
-    default:
-      break;
-  } /* end of switch */
-  /* if we fell through, just return a 1 */
-  return 1;
+        /* Hate to do it, but for lack of a better idea, check based on the
+         * menu entry name */
+        /* Complex functions are a little tricky, ignore them if no menu item*/
+        if (function == F_FUNCTION && action_string != NULL)
+        {
+                int i;
+
+                /* remap to regular actions */
+
+                i = GetTokenIndex(action_string, functionlist, -1, NULL);
+                switch (i)
+                {
+                case 0:
+                        function = F_MOVE;
+                        break;
+                case 1:
+                        function = F_RESIZE;
+                        break;
+                case 2:
+                        function = F_RESIZE;
+                        break;
+                case 3:
+                        function = F_ICONIFY;
+                        break;
+                case 4:
+                        function = F_ICONIFY;
+                        break;
+                case 5:
+                        function = F_MAXIMIZE;
+                        break;
+                case 6:
+                        function = F_CLOSE;
+                        break;
+                case 7:
+                        function = F_DELETE;
+                        break;
+                case 8:
+                        function = F_DESTROY;
+                        break;
+                case 9:
+                        function = F_QUIT;
+                        break;
+                default:
+                        break;
+                }
+        }
+        /* now do the real checks */
+        switch(function)
+        {
+        case F_DELETE:
+		if (IS_TEAR_OFF_MENU(t))
+		{
+		  /* always allow this on tear off menus */
+		  break;
+		}
+                if (!WM_DELETES_WINDOW(t))
+		{
+                        return False;
+		}
+                /* fall through to close clause */
+        case F_CLOSE:
+		if (IS_TEAR_OFF_MENU(t))
+		{
+		  /* always allow this on tear off menus */
+		  break;
+		}
+                if (!(functions & MWM_FUNC_CLOSE))
+		{
+                        return False;
+		}
+                break;
+        case F_DESTROY: /* shouldn't destroy always be allowed??? */
+		if (IS_TEAR_OFF_MENU(t))
+		{
+		  /* always allow this on tear off menus */
+		  break;
+		}
+                if (!(functions & MWM_FUNC_CLOSE))
+		{
+                        return False;
+		}
+                break;
+        case F_RESIZE:
+                if (!HAS_OVERRIDE_SIZE_HINTS(t) &&
+                    t->hints.min_width == t->hints.min_width &&
+                    t->hints.min_height == t->hints.max_height)
+                {
+                        return False;
+                }
+                if (is_user_request && IS_SIZE_FIXED(t))
+                {
+                        return False;
+                }
+                else if (!is_user_request && IS_PSIZE_FIXED(t))
+                {
+                        return False;
+                }
+                if (is_user_request && !(functions & MWM_FUNC_RESIZE))
+                {
+                        return False;
+                }
+                break;
+        case F_ICONIFY:
+                if (!IS_ICONIFIED(t) && !(functions & MWM_FUNC_MINIMIZE))
+		{
+                        return False;
+		}
+                break;
+        case F_MAXIMIZE:
+                if (is_user_request && !(functions & MWM_FUNC_MAXIMIZE))
+                {
+                        return False;
+                }
+                break;
+        case F_MOVE:
+                /* Move is a funny hint. Keeps it out of the menu, but you're
+                 * still allowed to move. */
+                if (is_user_request && IS_FIXED(t))
+                {
+                        return False;
+                }
+                else if (!is_user_request && IS_FIXED_PPOS(t))
+                {
+                        return False;
+                }
+                if (is_user_request && !(functions & MWM_FUNC_MOVE))
+                {
+                        return False;
+                }
+                break;
+        case F_FUNCTION:
+        default:
+                break;
+        } /* end of switch */
+
+        /* if we fell through, just return True */
+        return True;
 }
