@@ -36,6 +36,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/Parse.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "commands.h"
 #include "externs.h"
@@ -46,13 +47,16 @@
 #include "module_interface.h"
 #include "misc.h"
 #include "screen.h"
+#include "geometry.h"
 #include "repeat.h"
 #include "read.h"
 #include "virtual.h"
-#include "libs/Colorset.h"
+#include "colorset.h"
+#include "ewmh.h"
+#include "schedule.h"
 
 extern XEvent Event;
-extern FvwmWindow *Tmp_win;
+extern FvwmWindow *Fw;
 extern char const * const Fvwm_VersionInfo;
 
 /* forward declarations */
@@ -71,6 +75,10 @@ void CMD_Title(F_CMD_ARGS)
 {
   return;
 }
+void CMD_TearMenuOff(F_CMD_ARGS)
+{
+  return;
+}
 
 /*
  * be sure to keep this list properly ordered for bsearch routine!
@@ -90,11 +98,9 @@ void CMD_Title(F_CMD_ARGS)
 static const func_type func_config[] =
 {
   CMD_ENTRY("+", CMD_Plus, F_ADDMENU2, 0),
-#ifdef MULTISTYLE
   CMD_ENTRY("addbuttonstyle", CMD_AddButtonStyle, F_ADD_BUTTON_STYLE,
 	    FUNC_DECOR),
   CMD_ENTRY("addtitlestyle", CMD_AddTitleStyle, F_ADD_TITLE_STYLE, FUNC_DECOR),
-#endif /* MULTISTYLE */
 #ifdef USEDECOR
   CMD_ENTRY("addtodecor", CMD_AddToDecor, F_ADD_DECOR, 0),
 #endif /* USEDECOR */
@@ -103,8 +109,10 @@ static const func_type func_config[] =
   CMD_ENTRY("all", CMD_All, F_ALL, 0),
   CMD_ENTRY("animatedmove", CMD_AnimatedMove, F_ANIMATED_MOVE,
 	    FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("any", CMD_Any, F_ANY, 0),
   CMD_ENTRY("beep", CMD_Beep, F_BEEP, 0),
   CMD_ENTRY("borderstyle", CMD_BorderStyle, F_BORDERSTYLE, FUNC_DECOR),
+  CMD_ENTRY("break", CMD_Break, F_BREAK, 0),
   CMD_ENTRY("bugopts", CMD_BugOpts, F_BUG_OPTS, 0),
   CMD_ENTRY("busycursor", CMD_BusyCursor, F_BUSY_CURSOR, 0),
   CMD_ENTRY("buttonstate", CMD_ButtonState, F_BUTTON_STATE, 0),
@@ -113,11 +121,14 @@ static const func_type func_config[] =
   CMD_ENTRY("changedecor", CMD_ChangeDecor, F_CHANGE_DECOR, FUNC_NEEDS_WINDOW),
 #endif /* USEDECOR */
   CMD_ENTRY("changemenustyle", CMD_ChangeMenuStyle, F_CHANGE_MENUSTYLE, 0),
+  CMD_ENTRY("cleanupcolorsets", CMD_CleanupColorsets, F_NOP, 0),
   CMD_ENTRY("clicktime", CMD_ClickTime, F_CLICK, 0),
   CMD_ENTRY("close", CMD_Close, F_CLOSE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("colorlimit", CMD_ColorLimit, F_COLOR_LIMIT, 0),
   CMD_ENTRY("colormapfocus", CMD_ColormapFocus, F_COLORMAP_FOCUS, 0),
   CMD_ENTRY("colorset", CMD_Colorset, F_NOP, 0),
+  CMD_ENTRY("cond", CMD_Cond, F_COND, 0),
+  CMD_ENTRY("condcase", CMD_CondCase, F_CONDCASE, 0),
   CMD_ENTRY("copymenustyle", CMD_CopyMenuStyle, F_COPY_MENU_STYLE, 0),
   CMD_ENTRY("current", CMD_Current, F_CURRENT, 0),
   CMD_ENTRY("cursormove", CMD_CursorMove, F_MOVECURSOR, 0),
@@ -128,7 +139,9 @@ static const func_type func_config[] =
   CMD_ENTRY("defaulticon", CMD_DefaultIcon, F_DFLT_ICON, 0),
   CMD_ENTRY("defaultlayers", CMD_DefaultLayers, F_DFLT_LAYERS, 0),
   CMD_ENTRY("delete", CMD_Delete, F_DELETE, FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("deschedule", CMD_Deschedule, F_DESCHEDULE, 0),
   CMD_ENTRY("desk", CMD_Desk, F_GOTO_DESK, 0),
+  CMD_ENTRY("desktopname", CMD_DesktopName, F_DESKTOP_NAME, 0),
   CMD_ENTRY("desktopsize", CMD_DesktopSize, F_SETDESK, 0),
   CMD_ENTRY("destroy", CMD_Destroy, F_DESTROY, FUNC_NEEDS_WINDOW),
 #ifdef USEDECOR
@@ -141,11 +154,15 @@ static const func_type func_config[] =
   CMD_ENTRY("destroystyle", CMD_DestroyStyle, F_DESTROY_STYLE, 0),
   CMD_ENTRY("direction", CMD_Direction, F_DIRECTION, 0),
   CMD_ENTRY("echo", CMD_Echo, F_ECHO, 0),
+  CMD_ENTRY("edgecommand", CMD_EdgeCommand, F_EDGE_COMMAND, 0),
   CMD_ENTRY("edgeresistance", CMD_EdgeResistance, F_EDGE_RES, 0),
   CMD_ENTRY("edgescroll", CMD_EdgeScroll, F_EDGE_SCROLL, 0),
   CMD_ENTRY("edgethickness", CMD_EdgeThickness, F_NOP, 0),
   CMD_ENTRY("emulate", CMD_Emulate, F_EMULATE, 0),
   CMD_ENTRY("escapefunc", CMD_EscapeFunc, F_ESCAPE_FUNC, 0),
+  CMD_ENTRY("ewmhbasestrut", CMD_EwmhBaseStrut, F_EWMH_BASE_STRUT, 0),
+  CMD_ENTRY("ewmhnumberofdesktops", CMD_EwmhNumberOfDesktops,
+    F_EWMH_NUMBER_OF_DESKTOPS, 0),
   CMD_ENTRY("exec", CMD_Exec, F_EXEC, 0),
   CMD_ENTRY("execuseshell", CMD_ExecUseShell, F_EXEC_SETUP, 0),
   CMD_ENTRY("fakeclick", CMD_FakeClick, F_FAKE_CLICK, 0),
@@ -195,24 +212,32 @@ static const func_type func_config[] =
   CMD_ENTRY("pixmappath", CMD_PixmapPath, F_PIXMAP_PATH, 0),
   CMD_ENTRY("placeagain", CMD_PlaceAgain, F_PLACEAGAIN, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("pointerkey", CMD_PointerKey, F_POINTERKEY, 0),
+  CMD_ENTRY("pointerwindow", CMD_PointerWindow, F_POINTERWINDOW, 0),
   CMD_ENTRY("popup", CMD_Popup, F_POPUP, 0),
   CMD_ENTRY("prev", CMD_Prev, F_PREV, 0),
+  CMD_ENTRY("propertychange", CMD_PropertyChange, F_NOP, 0),
   CMD_ENTRY("quit", CMD_Quit, F_QUIT, 0),
   CMD_ENTRY("quitscreen", CMD_QuitScreen, F_QUIT_SCREEN, 0),
   CMD_ENTRY("quitsession", CMD_QuitSession, F_QUIT_SESSION, 0),
   CMD_ENTRY("raise", CMD_Raise, F_RAISE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("raiselower", CMD_RaiseLower, F_RAISELOWER, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("read", CMD_Read, F_READ, 0),
+  CMD_ENTRY("readwritecolors", CMD_ReadWriteColors, F_NOP, 0),
   CMD_ENTRY("recapture", CMD_Recapture, F_RECAPTURE, 0),
   CMD_ENTRY("recapturewindow", CMD_RecaptureWindow, F_RECAPTURE_WINDOW, 0),
   CMD_ENTRY("refresh", CMD_Refresh, F_REFRESH, 0),
   CMD_ENTRY("refreshwindow", CMD_RefreshWindow, F_REFRESH, FUNC_NEEDS_WINDOW),
   CMD_ENTRY(PRE_REPEAT, CMD_Repeat, F_REPEAT, FUNC_DONT_REPEAT),
   CMD_ENTRY("resize", CMD_Resize, F_RESIZE, FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("resizemaximize", CMD_ResizeMaximize, F_RESIZE_MAXIMIZE,
+	    FUNC_NEEDS_WINDOW),
   CMD_ENTRY("resizemove", CMD_ResizeMove, F_RESIZEMOVE, FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("resizemovemaximize", CMD_ResizeMoveMaximize,
+	    F_RESIZEMOVE_MAXIMIZE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("restart", CMD_Restart, F_RESTART, 0),
   CMD_ENTRY("savequitsession", CMD_SaveQuitSession, F_SAVE_QUIT_SESSION, 0),
   CMD_ENTRY("savesession", CMD_SaveSession, F_SAVE_SESSION, 0),
+  CMD_ENTRY("schedule", CMD_Schedule, F_SCHEDULE, 0),
   CMD_ENTRY("scroll", CMD_Scroll, F_SCROLL, 0),
   CMD_ENTRY("send_configinfo", CMD_Send_ConfigInfo, F_CONFIG_LIST,
 	    FUNC_DONT_REPEAT),
@@ -229,12 +254,14 @@ static const func_type func_config[] =
   CMD_ENTRY(PRE_SILENT, CMD_Silent, F_SILENT, 0),
   CMD_ENTRY("snapattraction", CMD_SnapAttraction, F_SNAP_ATT, 0),
   CMD_ENTRY("snapgrid", CMD_SnapGrid, F_SNAP_GRID, 0),
+  CMD_ENTRY("state", CMD_State, F_STATE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("stick", CMD_Stick, F_STICK, FUNC_NEEDS_WINDOW),
 #ifdef HAVE_STROKE
   CMD_ENTRY("stroke", CMD_Stroke, F_STROKE, 0),
   CMD_ENTRY("strokefunc", CMD_StrokeFunc, F_STROKE_FUNC, 0),
 #endif /* HAVE_STROKE */
   CMD_ENTRY("style", CMD_Style, F_STYLE, 0),
+  CMD_ENTRY("tearmenuoff", CMD_TearMenuOff, F_TEARMENUOFF, 0),
   CMD_ENTRY("thiswindow", CMD_ThisWindow, F_THISWINDOW, 0),
   CMD_ENTRY("title", CMD_Title, F_TITLE, 0),
   CMD_ENTRY("titlestyle", CMD_TitleStyle, F_TITLESTYLE, FUNC_DECOR),
@@ -255,6 +282,8 @@ static const func_type func_config[] =
   CMD_ENTRY("xineramaslssize", CMD_XineramaSlsSize, F_XINERAMASLSSIZE, 0),
   CMD_ENTRY("xorpixmap", CMD_XorPixmap, F_XOR, 0),
   CMD_ENTRY("xorvalue", CMD_XorValue, F_XOR, 0),
+  CMD_ENTRY("xsync", CMD_XSync, F_XSYNC, 0),
+  CMD_ENTRY("xsynchronize", CMD_XSynchronize, F_XSYNCHRONIZE, 0),
   {"",0,0,0}
 };
 
@@ -312,12 +341,39 @@ static char *function_vars[] =
   "w.y",
   "w.width",
   "w.height",
+  "cw.x",
+  "cw.y",
+  "cw.width",
+  "cw.height",
+  "it.x",
+  "it.y",
+  "it.width",
+  "it.height",
+  "ip.x",
+  "ip.y",
+  "ip.width",
+  "ip.height",
+  "i.x",
+  "i.y",
+  "i.width",
+  "i.height",
   "screen",
+  "desk.name",
+  "schedule.last",
+  "schedule.next",
+  "cond.rc",
+  "pointer.x",
+  "pointer.y",
+  "pointer.wx",
+  "pointer.wy",
+  "pointer.cx",
+  "pointer.cy",
   NULL
 };
 
 static int expand_extended_var(
-  char *var_name, char *output, FvwmWindow *tmp_win)
+  char *var_name, char *output, FvwmWindow *fw,
+  fvwm_cond_func_rc *cond_rc)
 {
   char *s;
   char *rest;
@@ -326,9 +382,14 @@ static int expand_extended_var(
   int cs = -1;
   int n;
   int i;
+  int l;
+  int x;
+  int y;
   Pixel pixel = 0;
   int val = -12345678;
   Bool is_numeric = False;
+  Bool is_x;
+  Window context_w = Scr.Root;
 
   /* allow partial matches for *.cs variables */
   switch ((i = GetTokenIndex(var_name, function_vars, -1, &rest)))
@@ -347,7 +408,7 @@ static int expand_extended_var(
       return 0;
     if (cs < 0)
       return 0;
-    AllocColorset(cs);
+    alloc_colorset(cs);
     switch (i)
     {
     case 0:
@@ -368,6 +429,35 @@ static int expand_extended_var(
       break;
     }
     return pixel_to_color_string(dpy, Pcmap, pixel, target, False);
+    break;
+  case 33:
+    if (sscanf(rest, "%d%n", &cs, &n) < 1)
+      return 0;
+    if (*(rest + n) != 0)
+      /* trailing characters */
+      return 0;
+    s = GetDesktopName(cs);
+    if (s == NULL)
+    {
+      s = (char *)safemalloc(23*sizeof(char));
+      sprintf(s,"Desk %i", cs);
+      l = strlen(s);
+      if (output)
+      {
+	strcpy(output,s);
+      }
+      free(s);
+    }
+    else
+    {
+      l = strlen(s);
+      if (output)
+      {
+	strcpy(output,s);
+      }
+    }
+    return l;
+    break;
   default:
     break;
   }
@@ -415,33 +505,35 @@ static int expand_extended_var(
     is_numeric = True;
     val = (int)(Scr.Vy / Scr.MyDisplayHeight);
     break;
-
   case 12:
   case 13:
   case 14:
   case 15:
-    if (!tmp_win || IS_ICONIFIED(tmp_win))
+    if (!fw || IS_ICONIFIED(fw) || IS_EWMH_DESKTOP(FW_W(fw)))
       return 0;
     else
     {
+      rectangle g;
+
       is_numeric = True;
+      get_unshaded_geometry(fw, &g);
       switch (i)
       {
       case 12:
 	/* w.x */
-	val = tmp_win->frame_g.x;
+	val = g.x;
 	break;
       case 13:
 	/* w.y */
-	val = tmp_win->frame_g.y;
+	val = g.y;
 	break;
       case 14:
 	/* w.width */
-	val = tmp_win->frame_g.width;
+	val = g.width;
 	break;
       case 15:
 	/* w.height */
-	val = tmp_win->frame_g.height;
+	val = g.height;
 	break;
       default:
 	return 0;
@@ -449,10 +541,231 @@ static int expand_extended_var(
     }
     break;
   case 16:
+  case 17:
+  case 18:
+  case 19:
+    if (!fw || IS_ICONIFIED(fw) || IS_EWMH_DESKTOP(FW_W(fw)))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      is_numeric = True;
+      get_client_geometry(fw, &g);
+      switch (i)
+      {
+      case 16:
+	/* cw.x */
+	val = g.x;
+	break;
+      case 17:
+	/* cw.y */
+	val = g.y;
+	break;
+      case 18:
+	/* cw.width */
+	val = g.width;
+	break;
+      case 19:
+	/* cw.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 20:
+  case 21:
+  case 22:
+  case 23:
+    if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_title_geometry(fw, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 20:
+	/* it.x */
+	val = g.x;
+	break;
+      case 21:
+	/* it.y */
+	val = g.y;
+	break;
+      case 22:
+	/* it.width */
+	val = g.width;
+	break;
+      case 23:
+	/* it.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 24:
+  case 25:
+  case 26:
+  case 27:
+    if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_picture_geometry(fw, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 24:
+	/* ip.x */
+	val = g.x;
+	break;
+      case 25:
+	/* ip.y */
+	val = g.y;
+	break;
+      case 26:
+	/* ip.width */
+	val = g.width;
+	break;
+      case 27:
+	/* ip.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 28:
+  case 29:
+  case 30:
+  case 31:
+    if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_geometry(fw, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 28:
+	/* i.x */
+	val = g.x;
+	break;
+      case 29:
+	/* i.y */
+	val = g.y;
+	break;
+      case 30:
+	/* i.width */
+	val = g.width;
+	break;
+      case 31:
+	/* i.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 32:
     /* screen */
     is_numeric = True;
     val = Scr.screen;
     break;
+  case 34:
+    /* schedule.last */
+    is_numeric = True;
+    val = squeue_get_last_id();
+    break;
+  case 35:
+    /* schedule.next */
+    is_numeric = True;
+    val = squeue_get_next_id();
+    break;
+  case 36:
+    /* cond.rc */
+    if (cond_rc == NULL)
+    {
+      return 0;
+    }
+    switch (*cond_rc)
+    {
+    case COND_RC_OK:
+    case COND_RC_NO_MATCH:
+    case COND_RC_ERROR:
+      val = (int)(*cond_rc);
+      break;
+    default:
+      return 0;
+    }
+    is_numeric = True;
+    break;
+  case 37:
+  case 38:
+    /* pointer.x and pointer.y */
+    if (is_numeric == False)
+    {
+      is_numeric = True;
+      context_w = Scr.Root;
+    }
+    /* fall through */
+  case 39:
+  case 40:
+    /* pointer.wx and pointer.wy */
+    if (is_numeric == False)
+    {
+      if (!fw || IS_ICONIFIED(fw) || IS_EWMH_DESKTOP(FW_W(fw)))
+      {
+        return 0;
+      }
+      is_numeric = True;
+      context_w = FW_W_FRAME(fw);
+    }
+    /* fall through */
+  case 41:
+  case 42:
+    /* pointer.cx and pointer.cy */
+    if (is_numeric == False)
+    {
+      if (!fw || IS_ICONIFIED(fw) || IS_SHADED(fw) ||
+          IS_EWMH_DESKTOP(FW_W(fw)))
+      {
+        return 0;
+      }
+      is_numeric = True;
+      context_w = FW_W(fw);
+    }
+    is_x = (i & 1) ? True : False;
+    if (XQueryPointer(dpy, context_w, &JunkRoot, &JunkChild,
+                      &JunkX, &JunkY, &x, &y, &JunkMask) == False)
+    {
+      /* pointer is on a different screen, don't expand */
+      return 0;
+    }
+    val = (is_x) ? x : y;
+    break;
   default:
     /* unknown variable - try to find it in the environment */
     s = getenv(var_name);
@@ -479,7 +792,8 @@ static int expand_extended_var(
 }
 
 static char *expand(
-  char *input, char *arguments[], FvwmWindow *tmp_win, Bool addto, Bool ismod)
+  char *input, char *arguments[], FvwmWindow *fw, Bool addto, Bool ismod,
+  fvwm_cond_func_rc *cond_rc)
 {
   int l,i,l2,n,k,j,m;
   int xlen;
@@ -521,7 +835,7 @@ static char *expand(
 	  k = strlen(var);
 	  if (!addto)
 	  {
-	    xlen = expand_extended_var(var, NULL, tmp_win);
+	    xlen = expand_extended_var(var, NULL, fw, cond_rc);
 	    if (xlen > 0)
 	      l2 += xlen - (k + 2);
 	  }
@@ -561,23 +875,31 @@ static char *expand(
 	i++;
 	break;
       case 'c':
-	if (tmp_win && tmp_win->class.res_class &&
-	    tmp_win->class.res_class[0])
-	{
-	  string = tmp_win->class.res_class;
-	}
-	break;
       case 'r':
-	if (tmp_win && tmp_win->class.res_name &&
-	    tmp_win->class.res_name[0])
-	{
-	  string = tmp_win->class.res_name;
-	}
-	break;
       case 'n':
-	if (tmp_win && tmp_win->name && tmp_win->name[0])
+	if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 	{
-	  string = tmp_win->name;
+	  switch(input[i+1])
+	  {
+	  case 'c':
+	    if (fw->class.res_class && fw->class.res_class[0])
+	    {
+	      string = fw->class.res_class;
+	    }
+	    break;
+	  case 'r':
+	    if (fw->class.res_name && fw->class.res_name[0])
+	    {
+	      string = fw->class.res_name;
+	    }
+	    break;
+	  case 'n':
+	    if (fw->name.name && fw->name.name[0])
+	    {
+	      string = fw->name.name;
+	    }
+	    break;
+	  }
 	}
 	break;
       case 'v':
@@ -628,7 +950,7 @@ static char *expand(
 	  input[m] = 0;
 	  /* handle variable name */
 	  k = strlen(var);
-	  xlen = expand_extended_var(var, &out[j], tmp_win);
+	  xlen = expand_extended_var(var, &out[j], fw, cond_rc);
 	  input[m] = ']';
 	  if (xlen > 0)
 	  {
@@ -690,8 +1012,8 @@ static char *expand(
 	is_string = True;
 	break;
       case 'w':
-	if(tmp_win)
-	  sprintf(&out[j],"0x%x",(unsigned int)tmp_win->w);
+	if(fw && !IS_EWMH_DESKTOP(FW_W(fw)))
+	  sprintf(&out[j],"0x%x",(unsigned int)FW_W(fw));
 	else
 	  sprintf(&out[j],"$w");
 	j += strlen(&out[j]);
@@ -714,25 +1036,31 @@ static char *expand(
 	break;
 
       case 'c':
-	if (tmp_win && tmp_win->class.res_class &&
-	    tmp_win->class.res_class[0])
-	{
-	  string = tmp_win->class.res_class;
-	}
-	is_string = True;
-	break;
       case 'r':
-	if (tmp_win && tmp_win->class.res_name &&
-	    tmp_win->class.res_name[0])
-	{
-	  string = tmp_win->class.res_name;
-	}
-	is_string = True;
-	break;
       case 'n':
-	if (tmp_win && tmp_win->name && tmp_win->name[0])
+	if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 	{
-	  string = tmp_win->name;
+	  switch(input[i+1])
+	  {
+	  case 'c':
+	    if (fw->class.res_class && fw->class.res_class[0])
+	    {
+	      string = fw->class.res_class;
+	    }
+	    break;
+	  case 'r':
+	    if (fw->class.res_name && fw->class.res_name[0])
+	    {
+	      string = fw->class.res_name;
+	    }
+	    break;
+	  case 'n':
+	    if (fw->name.name && fw->name.name[0])
+	    {
+	      string = fw->name.name;
+	    }
+	    break;
+	  }
 	}
 	is_string = True;
 	break;
@@ -787,7 +1115,6 @@ static char *expand(
  * clicking, but is moving the cursor
  *
  ****************************************************************************/
-#define DEBUG_CAT 0
 static cfunc_action_type CheckActionType(
   int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed)
 {
@@ -797,12 +1124,6 @@ static cfunc_action_type CheckActionType(
   XEvent old_event;
   extern Time lastTimestamp;
   Bool do_sleep = False;
-#if DEBUG_CAT
-int cev = 0;
-int clp = 0;
-int csleep = 0;
-int cc[5] = {0,0,0,0,0};
-#endif
 
   xcurrent = x;
   ycurrent = y;
@@ -812,30 +1133,18 @@ int cc[5] = {0,0,0,0,0};
   while ((total < Scr.ClickTime && lastTimestamp - t0 < Scr.ClickTime) ||
 	 !may_time_out)
   {
-#if DEBUG_CAT
-clp++;
-#endif
     if (!(x - xcurrent <= dist && xcurrent - x <= dist &&
 	  y - ycurrent <= dist && ycurrent - y <= dist))
     {
-#if DEBUG_CAT
-fprintf(stderr, "cat stats (1): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n", clp, cc[0], cc[1], cc[2], cc[3], cc[4], cev, csleep);
-#endif
       return (is_button_pressed) ? CF_MOTION : CF_TIMEOUT;
     }
 
     if (do_sleep)
     {
-#if DEBUG_CAT
-csleep += 20000;
-#endif
       usleep(20000);
     }
     else
     {
-#if DEBUG_CAT
-csleep += 1;
-#endif
       usleep(1);
       do_sleep = 1;
     }
@@ -843,23 +1152,13 @@ csleep += 1;
     if (XCheckMaskEvent(dpy, ButtonReleaseMask|ButtonMotionMask|
 			PointerMotionMask|ButtonPressMask|ExposureMask, d))
     {
-#if DEBUG_CAT
-cev++;
-#endif
       do_sleep = 0;
       StashEventTime(d);
       switch (d->xany.type)
       {
       case ButtonRelease:
-#if DEBUG_CAT
-cc[0]++;
-fprintf(stderr, "cat stats (2): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n", clp, cc[0], cc[1], cc[2], cc[3], cc[4], cev, csleep);
-#endif
 	return CF_CLICK;
       case MotionNotify:
-#if DEBUG_CAT
-cc[1]++;
-#endif
 	if ((d->xmotion.state & DEFAULT_ALL_BUTTONS_MASK) ||
             !is_button_pressed)
 	{
@@ -868,44 +1167,28 @@ cc[1]++;
 	}
         else
 	{
-#if DEBUG_CAT
-fprintf(stderr, "cat stats (3): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n", clp, cc[0], cc[1], cc[2], cc[3], cc[4], cev, csleep);
-#endif
 	  return CF_CLICK;
 	}
 	break;
       case ButtonPress:
-#if DEBUG_CAT
-cc[2]++;
-#endif
 	if (may_time_out)
 	  is_button_pressed = True;
 	break;
       case Expose:
-#if DEBUG_CAT
-cc[3]++;
-#endif
 	/* must handle expose here so that raising a window with "I" works */
 	memcpy(&old_event, &Event, sizeof(XEvent));
 	memcpy(&Event, d, sizeof(XEvent));
-	/* note: handling Expose events will never modify the global Tmp_win */
+	/* note: handling Expose events will never modify the global Fw */
 	DispatchEvent(True);
 	memcpy(&Event, &old_event, sizeof(XEvent));
 	break;
       default:
-#if DEBUG_CAT
-cc[4]++;
-fprintf(stderr,"%d ", d->xany.type);
-#endif
 	/* can't happen */
 	break;
       }
     }
   }
 
-#if DEBUG_CAT
-fprintf(stderr, "cat stats (4): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n", clp, cc[0], cc[1], cc[2], cc[3], cc[4], cev, csleep);
-#endif
   return (is_button_pressed) ? CF_HOLD : CF_TIMEOUT;
 }
 
@@ -917,7 +1200,7 @@ fprintf(stderr, "cat stats (4): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n",
  *
  *  Inputs:
  *	Action	- the action to execute
- *	tmp_win	- the fvwm window structure
+ *	fw	- the fvwm window structure
  *	eventp	- pointer to the event that caused the function
  *	context - the context in which the button was pressed
  *
@@ -925,7 +1208,7 @@ fprintf(stderr, "cat stats (4): clp: %d (%d/%d/%d/%d/%d)  cev: %d csleep: %d\n",
 void execute_function(exec_func_args_type *efa)
 {
   static unsigned int func_depth = 0;
-  FvwmWindow *s_Tmp_win = Tmp_win;
+  FvwmWindow *s_Fw = Fw;
   Window w;
   int j;
   char *function;
@@ -969,7 +1252,7 @@ void execute_function(exec_func_args_type *efa)
       arguments[j] = NULL;
   }
 
-  if (efa->tmp_win == NULL)
+  if (efa->fw == NULL || IS_EWMH_DESKTOP(FW_W(efa->fw)))
   {
     if (efa->flags.is_window_unmanaged)
       w = efa->win;
@@ -988,7 +1271,7 @@ void execute_function(exec_func_args_type *efa)
     }
     else
     {
-      w = efa->tmp_win->w;
+      w = FW_W(efa->fw);
     }
   }
 
@@ -1027,7 +1310,8 @@ void execute_function(exec_func_args_type *efa)
 
   function = PeekToken(taction, NULL);
   if (function)
-    function = expand(function, arguments, efa->tmp_win, False, False);
+    function = expand(
+     function, arguments, efa->fw, False, False, efa->cond_rc);
   if (function && function[0] != '*')
   {
 #if 1
@@ -1067,8 +1351,9 @@ void execute_function(exec_func_args_type *efa)
 
   if (!(efa->flags.exec & FUNC_DONT_EXPAND_COMMAND))
   {
-    expaction = expand(taction, arguments, efa->tmp_win,
-      (bif) ? !!(bif->flags & FUNC_ADD_TO) : False, taction[0] == '*');
+    expaction = expand(taction, arguments, efa->fw,
+      (bif) ? !!(bif->flags & FUNC_ADD_TO) : False, (taction[0] == '*'),
+      efa->cond_rc);
     if (func_depth <= 1)
       must_free_string = set_repeat_data(expaction, REPEAT_COMMAND, bif);
     else
@@ -1106,7 +1391,9 @@ void execute_function(exec_func_args_type *efa)
     char *runaction;
 
     runaction = SkipNTokens(expaction, 1);
-    bif->action(efa->eventp,w,efa->tmp_win,efa->context,runaction,&efa->module);
+    bif->action(
+      efa->cond_rc, efa->eventp, w, efa->fw, efa->context, runaction,
+      &efa->module);
   }
   else
   {
@@ -1124,13 +1411,13 @@ void execute_function(exec_func_args_type *efa)
     }
 
     execute_complex_function(
-      efa->eventp,w,efa->tmp_win,efa->context,runaction, &efa->module,
-      &desperate);
+      efa->cond_rc, efa->eventp, w, efa->fw, efa->context, runaction,
+      &efa->module, &desperate);
     if (!bif && desperate)
     {
       if (executeModuleDesperate(
-	efa->eventp, w, efa->tmp_win, efa->context, runaction, &efa->module) ==
-	  -1 && *function != 0)
+	efa->cond_rc, efa->eventp, w, efa->fw, efa->context, runaction,
+	&efa->module) == -1 && *function != 0 && !set_silent)
       {
 	fvwm_msg(
 	  ERR, "execute_function", "No such command '%s'", function);
@@ -1151,10 +1438,10 @@ void execute_function(exec_func_args_type *efa)
   }
   if (efa->flags.do_save_tmpwin)
   {
-    if (check_if_fvwm_window_exists(s_Tmp_win))
-      Tmp_win = s_Tmp_win;
+    if (check_if_fvwm_window_exists(s_Fw))
+      Fw = s_Fw;
     else
-      Tmp_win = NULL;
+      Fw = NULL;
   }
   func_depth--;
 
@@ -1162,22 +1449,24 @@ void execute_function(exec_func_args_type *efa)
 }
 
 void old_execute_function(
-  char *action, FvwmWindow *tmp_win, XEvent *eventp, unsigned long context,
-  int Module, FUNC_FLAGS_TYPE exec_flags, char *args[])
+	fvwm_cond_func_rc *cond_rc, char *action, FvwmWindow *fw,
+	XEvent *eventp, unsigned long context, int Module,
+	FUNC_FLAGS_TYPE exec_flags, char *args[])
 {
-  exec_func_args_type efa;
+	exec_func_args_type efa;
 
-  memset(&efa, 0, sizeof(efa));
-  efa.eventp = eventp;
-  efa.tmp_win = tmp_win;
-  efa.action = action;
-  efa.args = args;
-  efa.context = context;
-  efa.module = Module;
-  efa.flags.exec = exec_flags;
-  execute_function(&efa);
+	memset(&efa, 0, sizeof(efa));
+	efa.cond_rc = cond_rc;
+	efa.eventp = eventp;
+	efa.fw = fw;
+	efa.action = action;
+	efa.args = args;
+	efa.context = context;
+	efa.module = Module;
+	efa.flags.exec = exec_flags;
+	execute_function(&efa);
 
-  return;
+	return;
 }
 
 
@@ -1190,7 +1479,7 @@ void old_execute_function(
  *  Inputs:
  *      eventp  - pointer to XEvent to patch up
  *      w       - pointer to Window to patch up
- *      tmp_win - pointer to FvwmWindow Structure to patch up
+ *      fw - pointer to FvwmWindow Structure to patch up
  *	context	- the context in which the mouse button was pressed
  *	func	- the function to defer
  *	cursor	- the cursor to display while waiting
@@ -1199,7 +1488,7 @@ void old_execute_function(
  *
  ***********************************************************************/
 int DeferExecution(
-  XEvent *eventp, Window *w,FvwmWindow **tmp_win, unsigned long *context,
+  XEvent *eventp, Window *w,FvwmWindow **fw, unsigned long *context,
   cursor_type cursor, int FinishEvent)
 {
   int done;
@@ -1209,7 +1498,8 @@ int DeferExecution(
 
   original_w = *w;
 
-  if((*context != C_ROOT)&&(*context != C_NO_CONTEXT)&&(*tmp_win != NULL))
+  if((*context != C_ROOT)&&(*context != C_NO_CONTEXT)&&(*fw != NULL)
+     && *context != C_EWMH_DESKTOP)
   {
     if((FinishEvent == ButtonPress)||((FinishEvent == ButtonRelease) &&
                                       (eventp->type != ButtonPress)))
@@ -1288,42 +1578,51 @@ int DeferExecution(
     *w = eventp->xbutton.subwindow;
     eventp->xany.window = *w;
   }
-  if (*w == Scr.Root)
+  if (*w == Scr.Root || IS_EWMH_DESKTOP(*w))
   {
     *context = C_ROOT;
     XBell(dpy, 0);
     return TRUE;
   }
-  if (XFindContext (dpy, *w, FvwmContext, (caddr_t *)tmp_win) == XCNOENT)
+  if (XFindContext (dpy, *w, FvwmContext, (caddr_t *)fw) == XCNOENT)
   {
-    *tmp_win = NULL;
+    *fw = NULL;
     XBell(dpy, 0);
     return (TRUE);
   }
 
-  if(*w == (*tmp_win)->Parent)
+  if(*w == FW_W_PARENT(*fw))
   {
-    *w = (*tmp_win)->w;
+    *w = FW_W(*fw);
   }
 
-  if(original_w == (*tmp_win)->Parent)
+  if(original_w == FW_W_PARENT(*fw))
   {
-    original_w = (*tmp_win)->w;
+    original_w = FW_W(*fw);
   }
 
   /* this ugly mess attempts to ensure that the release and press
    * are in the same window. */
   if(*w != original_w && original_w != Scr.Root &&
-     original_w != None && original_w != Scr.NoFocusWin)
+     original_w != None && original_w != Scr.NoFocusWin &&
+     !IS_EWMH_DESKTOP(original_w))
   {
-    if (*w != (*tmp_win)->frame || original_w != (*tmp_win)->w)
+    if (*w != FW_W_FRAME(*fw) || original_w != FW_W(*fw))
     {
       *context = C_ROOT;
       XBell(dpy, 0);
       return TRUE;
     }
   }
-  *context = GetContext(*tmp_win,eventp,&dummy);
+
+  if (IS_EWMH_DESKTOP(FW_W(*fw)))
+  {
+    *context = C_ROOT;
+    XBell(dpy, 0);
+    return TRUE;
+  }
+
+  *context = GetContext(*fw,eventp,&dummy);
 
   return FALSE;
 }
@@ -1563,6 +1862,7 @@ static void cf_cleanup(unsigned int *depth, char **arguments)
 
 static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 {
+  fvwm_cond_func_rc cond_func_rc = COND_RC_OK;
   cfunc_action_type type = CF_MOTION;
   char c;
   FunctionItem *fi;
@@ -1639,7 +1939,7 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 
   if(ImmediateNeedsTarget)
   {
-    if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_SELECT,ButtonPress))
+    if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT,ButtonPress))
     {
       func->use_depth--;
       cf_cleanup(&depth, arguments);
@@ -1658,19 +1958,20 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
     cf_cleanup(&depth, arguments);
     return;
   }
-  for (fi = func->first_item; fi != NULL; fi = fi->next_item)
+  for (fi = func->first_item; fi != NULL && cond_func_rc != COND_RC_BREAK;
+       fi = fi->next_item)
   {
     /* c is already lowercase here */
     c = fi->condition;
     switch (c)
     {
     case CF_IMMEDIATE:
-      if (tmp_win)
-	w = tmp_win->frame;
+      if (fw)
+	w = FW_W_FRAME(fw);
       else
 	w = None;
       old_execute_function(
-	fi->action, tmp_win, eventp, context, -1, 0, arguments);
+	&cond_func_rc, fi->action, fw, eventp, context, -1, 0, arguments);
       break;
     case CF_DOUBLE_CLICK:
       HaveDoubleClick = True;
@@ -1686,7 +1987,7 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
     }
   }
 
-  if(!Persist)
+  if(!Persist || cond_func_rc == COND_RC_BREAK)
   {
     func->use_depth--;
     cf_cleanup(&depth, arguments);
@@ -1698,7 +1999,7 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
    * a window to operate on */
   if(NeedsTarget)
   {
-    if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_SELECT,ButtonPress))
+    if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT,ButtonPress))
     {
       func->use_depth--;
       cf_cleanup(&depth, arguments);
@@ -1718,8 +2019,13 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
     XCheckMaskEvent(dpy, ButtonPressMask, &d);
     break;
   default:
-    XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
-		   &x,&y,&JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      x = 0;
+      y = 0;
+    }
     break;
   }
 
@@ -1730,7 +2036,7 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
   {
     ev = &d;
     /* If it was a click, wait to see if its a double click */
-    if(HaveDoubleClick)
+    if (HaveDoubleClick)
     {
       type = CheckActionType(x, y, &d, True, False);
       switch (type)
@@ -1769,20 +2075,20 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
    * select text. */
   UngrabEm(GRAB_NORMAL);
 #endif
-  while(fi != NULL)
+  while (fi != NULL)
   {
     /* make lower case */
     c = fi->condition;
-    if(isupper(c))
+    if (isupper(c))
       c=tolower(c);
-    if(c == type)
+    if (c == type)
     {
-      if(tmp_win)
-	w = tmp_win->frame;
+      if (fw)
+	w = FW_W_FRAME(fw);
       else
 	w = None;
       old_execute_function(
-	fi->action, tmp_win, ev, context, -1, 0, arguments);
+	&cond_func_rc, fi->action, fw, ev, context, -1, 0, arguments);
     }
     fi = fi->next_item;
   }
