@@ -41,6 +41,8 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/Flocale.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -52,9 +54,10 @@
 #include "builtins.h"
 #include "menus.h"
 #include "module_interface.h"
-#include "libs/Colorset.h"
+#include "colorset.h"
 #include "icccm2.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "icons.h"
 #include "add_window.h"
 #include "fvwmsignal.h"
@@ -69,16 +72,14 @@
 #include "update.h"
 #include "window_flags.h"
 #include "move_resize.h"
+#include "borders.h"
+#include "frame.h"
 
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 /* need to get prototype for XrmUniqueQuark for XUniqueContext call */
 #include <X11/Xresource.h>
 
-#if defined(I18N_MB) || defined(COMPOUND_TEXT)
-#include <X11/Xlocale.h>
-#endif
-
 #if HAVE_SYS_SYSTEMINFO_H
 /* Solaris has sysinfo instead of gethostname.  */
 #include <sys/systeminfo.h>
@@ -94,7 +95,6 @@ static char sccsid[] __attribute__((__unused__))
 int master_pid;			/* process number of 1st fvwm process */
 
 ScreenInfo Scr;		        /* structures for the screen */
-MenuInfo Menus;                 /* structures for menus */
 Display *dpy = NULL;		/* which display are we talking to */
 
 Bool fFvwmInStartup = True;     /* Set to False when startup has finished */
@@ -202,26 +202,7 @@ int main(int argc, char **argv)
 
   setVersionInfo();
 
-#if defined(I18N_MB) || defined(COMPOUND_TEXT)
-  {
-    char *s;
-
-    if ((s = setlocale(LC_CTYPE, "")) == NULL)
-    {
-      fvwm_msg(ERR, "main","Can't set locale. Check your $LC_CTYPE or $LANG.\n");
-    }
-    else if (!XSupportsLocale())
-    {
-      fvwm_msg(WARN, "main", "X does not support current locale: %s\n", s);
-    }
-#ifdef I18N_MB
-    else if (XSetLocaleModifiers("") == NULL)
-    {
-      fvwm_msg(WARN, "main", "Cannot set locale modifiers\n");
-    }
-#endif
-  }
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FVWM");
 
   /* Put the default module directory into the environment so it can be used
      later by the config file, etc.  */
@@ -327,8 +308,7 @@ int main(int argc, char **argv)
     {
       /* obsolete option */
       fvwm_msg(WARN, "main", "The -blackout option is obsolete, it may be "
-	       "removed in the future.",
-              VERSION,__DATE__,__TIME__);
+	       "removed in the future.");
     }
     else if (strncasecmp(argv[i], "-replace", 8) == 0)
     {
@@ -489,7 +469,7 @@ int main(int argc, char **argv)
   Scr.Root = RootWindow(dpy, Scr.screen);
   if(Scr.Root == None)
   {
-    fvwm_msg(ERR,"main","Screen %d is not a valid screen",(char *)Scr.screen);
+    fvwm_msg(ERR,"main","Screen %d is not a valid screen", (int)Scr.screen);
     exit(1);
   }
 
@@ -551,7 +531,7 @@ int main(int argc, char **argv)
   }
 
   /* make a copy of the visual stuff so that XorPixmap can swap with root */
-  SaveFvwmVisual();
+  PictureSaveFvwmVisual();
 
   FShapeInit(dpy);
   Scr.pscreen = XScreenOfDisplay(dpy, Scr.screen);
@@ -564,6 +544,7 @@ int main(int argc, char **argv)
   XChangeProperty(dpy, Scr.Root, _XA_MIT_PRIORITY_COLORS,
 		  XA_CARDINAL, 32, PropModeReplace, NULL, 0);
 
+  Scr.FvwmCursors = CreateCursors(dpy);
   /* create a window which will accept the keyboard focus when no other
      windows have it */
   /* do this before any RC parsing as some GC's are created from this window
@@ -571,26 +552,29 @@ int main(int argc, char **argv)
   attributes.event_mask = XEVMASK_NOFOCUSW;
   attributes.override_redirect = True;
   attributes.colormap = Pcmap;
+  attributes.cursor = Scr.FvwmCursors[CRS_DEFAULT];
   attributes.background_pixmap = None;
   attributes.border_pixel = 0;
-  Scr.NoFocusWin=XCreateWindow(dpy, Scr.Root, -10, -10, 10, 10, 0, Pdepth,
-                               InputOutput, Pvisual,
-                               CWEventMask | CWOverrideRedirect | CWColormap
-                               | CWBackPixmap | CWBorderPixel, &attributes);
+  Scr.NoFocusWin=XCreateWindow(
+	  dpy, Scr.Root, -10, -10, 10, 10, 0, Pdepth, InputOutput, Pvisual,
+	  CWEventMask | CWOverrideRedirect | CWColormap | CWBackPixmap |
+	  CWBorderPixel | CWCursor, &attributes);
   XMapWindow(dpy, Scr.NoFocusWin);
-
   SetMWM_INFO(Scr.NoFocusWin);
   FOCUS_SET(Scr.NoFocusWin);
 
-  XSync(dpy, 0);
+  frame_init();
+  XFlush(dpy);
   if(debugging)
-    XSynchronize(dpy,1);
+  {
+	  sync_server(1);
+  }
 
-  SetupICCCM2 (replace_wm);
+  SetupICCCM2(replace_wm);
   XSetErrorHandler(CatchRedirectError);
   XSetIOErrorHandler(CatchFatal);
   XSelectInput(dpy, Scr.Root, XEVMASK_ROOTW);
-  XSync(dpy, 0);
+  XFlush(dpy);
 
   XSetErrorHandler(FvwmErrorHandler);
 
@@ -633,11 +617,10 @@ int main(int argc, char **argv)
    */
   LoadWindowStates(state_filename);
 
-  Scr.FvwmCursors = CreateCursors(dpy);
   InitVariables();
   if (visualClass != -1 || visualId != -1) {
     /* this is so that menus use the (non-default) fvwm colormap */
-    Scr.FvwmRoot.w = Scr.NoFocusWin;
+    FW_W(&Scr.FvwmRoot) = Scr.NoFocusWin;
     Scr.FvwmRoot.number_cmap_windows = 1;
     Scr.FvwmRoot.cmap_windows = &Scr.NoFocusWin;
   }
@@ -661,7 +644,7 @@ int main(int argc, char **argv)
     {
       DoingCommandLine = True;
       old_execute_function(
-	config_commands[i], NULL, &Event, C_ROOT, 1, 0, NULL);
+	NULL, config_commands[i], NULL, &Event, C_ROOT, 1, 0, NULL);
       free(config_commands[i]);
     }
     DoingCommandLine = False;
@@ -727,6 +710,7 @@ int main(int argc, char **argv)
 
   SessionInit();
 
+  EWMH_Init();
   GNOME_Init();
 
   DBUG("main","Entering HandleEvents loop...");
@@ -774,6 +758,10 @@ void StartupStuff(void)
 #define startFuncName "StartFunction"
   const char *initFuncName;
 
+  /* make sure colorset 0 exists in case modules get started that use colorsets
+   * and the startup files/commands have not created any */
+  alloc_colorset(0);
+
   CaptureAllWindows(False);
   /* Turn off the SM stuff after the initial capture so that new windows will
    * not be matched by accident. */
@@ -785,6 +773,9 @@ void StartupStuff(void)
 
   fFvwmInStartup = False;
 
+  /* Make sure the geometry window uses the current font */
+  resize_geometry_window();
+
   /* Make sure we have the correct click time now. */
   if (Scr.ClickTime < 0)
     Scr.ClickTime = -Scr.ClickTime;
@@ -797,14 +788,14 @@ void StartupStuff(void)
   /* migo (04-Sep-1999): execute StartFunction */
   if (FindFunction(startFuncName)) {
     char *action = "Function " startFuncName;
-    old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
+    old_execute_function(NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
   }
 
   /* migo (03-Jul-1999): execute [Session]{Init|Restart}Function */
   initFuncName = getInitFunctionName(Restarting == True);
   if (FindFunction(initFuncName)) {
     char *action = safestrdup(CatString2("Function ", initFuncName));
-    old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
+    old_execute_function(NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
 
@@ -835,7 +826,7 @@ void SetRCDefaults(void)
   /* set up default colors, fonts, etc */
   char *defaults[] = {
     "XORValue 0",
-    "DefaultFont fixed",
+    "DefaultFont",
     "DefaultColors black grey",
     DEFAULT_MENU_STYLE,
     "TitleStyle Centered -- Raised",
@@ -856,7 +847,7 @@ void SetRCDefaults(void)
 
   while (defaults[i])
   {
-    old_execute_function(defaults[i], NULL, &Event, C_ROOT, 1, 0, NULL);
+    old_execute_function(NULL, defaults[i], NULL, &Event, C_ROOT, 1, 0, NULL);
     i++;
   }
 } /* SetRCDefaults */
@@ -898,6 +889,9 @@ Atom _XA_WM_WINDOW_ROLE;
 Atom _XA_WM_CLIENT_LEADER;
 Atom _XA_SM_CLIENT_ID;
 
+Atom _XA_XROOTPMAP_ID;
+Atom _XA_XSETROOT_ID;
+
 static void InternUsefulAtoms (void)
 {
   /*
@@ -930,6 +924,9 @@ static void InternUsefulAtoms (void)
   _XA_WM_WINDOW_ROLE=XInternAtom(dpy, "WM_WINDOW_ROLE",False);
   _XA_WM_CLIENT_LEADER=XInternAtom(dpy, "WM_CLIENT_LEADER",False);
   _XA_SM_CLIENT_ID=XInternAtom(dpy, "SM_CLIENT_ID",False);
+
+  _XA_XROOTPMAP_ID=XInternAtom(dpy, "_XROOTPMAP_ID",False);
+  _XA_XSETROOT_ID=XInternAtom(dpy, "_XSETROOT_ID",False);
   return;
 }
 
@@ -1110,78 +1107,78 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
   case 0:
   case 4:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 22;
     v->y[0] = 39;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 78;
     v->y[1] = 39;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 78;
     v->y[2] = 61;
     v->x[3] = 22;
     v->y[3] = 61;
     v->x[4] = 22;
     v->y[4] = 39;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   case 1:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 32;
     v->y[0] = 45;
     v->x[1] = 68;
     v->y[1] = 45;
     v->x[2] = 68;
     v->y[2] = 55;
-    v->line_style |= (1 << 2);
+    v->c[2] = 1;
     v->x[3] = 32;
     v->y[3] = 55;
-    v->line_style |= (1 << 3);
+    v->c[3] = 1;
     v->x[4] = 32;
     v->y[4] = 45;
     break;
   case 2:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 49;
     v->y[0] = 49;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 51;
     v->y[1] = 49;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 51;
     v->y[2] = 51;
     v->x[3] = 49;
     v->y[3] = 51;
     v->x[4] = 49;
     v->y[4] = 49;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   case 3:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 32;
     v->y[0] = 45;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 68;
     v->y[1] = 45;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 68;
     v->y[2] = 55;
     v->x[3] = 32;
     v->y[3] = 55;
     v->x[4] = 32;
     v->y[4] = 45;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   }
 }
@@ -1203,79 +1200,79 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
   case 0:
   case 3:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 25;
     v->y[0] = 25;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 75;
     v->y[1] = 25;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 75;
     v->y[2] = 75;
     v->x[3] = 25;
     v->y[3] = 75;
     v->x[4] = 25;
     v->y[4] = 25;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   case 1:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 39;
     v->y[0] = 39;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 61;
     v->y[1] = 39;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 61;
     v->y[2] = 61;
     v->x[3] = 39;
     v->y[3] = 61;
     v->x[4] = 39;
     v->y[4] = 39;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   case 2:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 49;
     v->y[0] = 49;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 51;
     v->y[1] = 49;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 51;
     v->y[2] = 51;
     v->x[3] = 49;
     v->y[3] = 51;
     v->x[4] = 49;
     v->y[4] = 49;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   case 4:
     v->num = 5;
-    v->line_style = 0;
-    v->x = (int *) safemalloc (sizeof(int) * v->num);
-    v->y = (int *) safemalloc (sizeof(int) * v->num);
+    v->x = safemalloc(sizeof(char) * v->num);
+    v->y = safemalloc(sizeof(char) * v->num);
+    v->c = safecalloc(v->num, sizeof(char));
     v->x[0] = 36;
     v->y[0] = 36;
-    v->line_style |= (1 << 0);
+    v->c[0] = 1;
     v->x[1] = 64;
     v->y[1] = 36;
-    v->line_style |= (1 << 1);
+    v->c[1] = 1;
     v->x[2] = 64;
     v->y[2] = 64;
     v->x[3] = 36;
     v->y[3] = 64;
     v->x[4] = 36;
     v->y[4] = 36;
-    v->line_style |= (1 << 4);
+    v->c[4] = 1;
     break;
   }
 }
@@ -1367,6 +1364,7 @@ static void CreateGCs(void)
   Scr.ScratchGC3 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.ScratchGC4 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.TitleGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
+  Scr.TileGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.TransMaskGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   c.pixel = GetColor(DEFAULT_FORE_COLOR);
   Scr.ScratchMonoPixmap = XCreatePixmap(dpy, Scr.Root, 1, 1, 1);
@@ -1389,21 +1387,14 @@ static void InitVariables(void)
 
   Scr.functions = NULL;
 
-  Menus.all = NULL;
-  Menus.DefaultStyle = NULL;
-  Menus.LastStyle = NULL;
+  menus_init();
+
   Scr.last_added_item.type = ADDED_NONE;
 
   Scr.DefaultIcon = NULL;
 
   Scr.DefaultColorset = -1;
-  AllocColorset(0);
-  /* set up colorset 0 so that if FvwmTheme fails to start any modules
-   * using colorsets don't appear black on black */
-  Colorset[0].fg = Scr.StdFore = GetColor(DEFAULT_FORE_COLOR);
-  Colorset[0].bg = Scr.StdBack = GetColor(DEFAULT_BACK_COLOR);
-  Colorset[0].hilite = Scr.StdHilite = GetHilite(Scr.StdBack);
-  Colorset[0].shadow = Scr.StdShadow = GetShadow(Scr.StdBack);
+
   Scr.StdGC = 0;
   Scr.StdReliefGC = 0;
   Scr.StdShadowGC = 0;
@@ -1419,16 +1410,11 @@ static void InitVariables(void)
     /* a number > than the builtin table! */
     Scr.ColorLimit = 255;
   }
-  Scr.FvwmRoot.w = Scr.Root;
+  FW_W(&Scr.FvwmRoot) = Scr.Root;
   Scr.FvwmRoot.next = 0;
 
   init_stack_and_layers();
 
-  if (!XGetWindowAttributes(dpy,Scr.Root,&(Scr.FvwmRoot.attr)))
-  {
-    fvwm_msg(ERR, "InitVariables", "Could not get root window attributes");
-    exit(0);
-  }
   Scr.root_pushes = 0;
   Scr.fvwm_pushes = 0;
   Scr.pushed_window = &Scr.FvwmRoot;
@@ -1443,7 +1429,8 @@ static void InitVariables(void)
    * fvwm. */
   Scr.Ungrabbed = NULL;
 
-  Scr.DefaultFont.font = NULL;
+
+  Scr.DefaultFont = NULL;
 
   Scr.VxMax = 2*Scr.MyDisplayWidth;
   Scr.VyMax = 2*Scr.MyDisplayHeight;
@@ -1457,8 +1444,7 @@ static void InitVariables(void)
    * compilations */
   Scr.CurrentDesk = 0;
 
-  Scr.EdgeScrollX = DEFAULT_EDGE_SCROLL * Scr.MyDisplayWidth / 100;
-  Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL * Scr.MyDisplayHeight / 100;
+  Scr.EdgeScrollX = Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL;
   Scr.ScrollResistance = DEFAULT_SCROLL_RESISTANCE;
   Scr.MoveResistance = DEFAULT_MOVE_RESISTANCE;
   Scr.XiMoveResistance = DEFAULT_XIMOVE_RESISTANCE;
@@ -1478,6 +1464,21 @@ static void InitVariables(void)
   Scr.randomx = Scr.randomy = 0;
   Scr.buttons2grab = 0;
 
+  /* initialisation of the head of the desktops info */
+  Scr.Desktops = (DesktopsInfo *)safemalloc(sizeof(DesktopsInfo));
+  Scr.Desktops->name = NULL;
+  Scr.Desktops->desk = 0; /* not desk 0 */
+  Scr.Desktops->ewmh_dyn_working_area.x = Scr.Desktops->ewmh_working_area.x = 0;
+  Scr.Desktops->ewmh_dyn_working_area.y = Scr.Desktops->ewmh_working_area.y = 0;
+  Scr.Desktops->ewmh_dyn_working_area.width =
+    Scr.Desktops->ewmh_working_area.width = Scr.MyDisplayWidth;
+  Scr.Desktops->ewmh_dyn_working_area.height =
+    Scr.Desktops->ewmh_working_area.height = Scr.MyDisplayHeight;
+  Scr.Desktops->next = NULL;
+
+  /* ewmh desktop */
+  Scr.EwmhDesktop = NULL;
+
   InitFvwmDecor(&Scr.DefaultDecor);
 #ifdef USEDECOR
   Scr.DefaultDecor.tag = "Default";
@@ -1506,15 +1507,19 @@ static void InitVariables(void)
   Scr.bo.DisableConfigureNotify = 0;
   Scr.bo.InstallRootCmap = 0;
   Scr.bo.FlickeringQtDialogsWorkaround = 1;
+  Scr.bo.EWMHIconicStateWorkaround = 0;
 
   Scr.gs.EmulateMWM = DEFAULT_EMULATE_MWM;
   Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
   Scr.gs.use_active_down_buttons = DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
   Scr.gs.use_inactive_buttons = DEFAULT_USE_INACTIVE_BUTTONS;
   /* Not the right place for this, should only be called once somewhere .. */
-  Scr.flags.is_pointer_on_this_screen = !!XQueryPointer(
-	  dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX, &JunkY,
-	  &JunkMask);
+
+  /* EdgeCommands - no edge commands by default */
+  Scr.PanFrameTop.command    = NULL ; 
+  Scr.PanFrameBottom.command = NULL ; 
+  Scr.PanFrameRight.command  = NULL ; 
+  Scr.PanFrameLeft.command   = NULL ; 
 
   return;
 }
@@ -1559,7 +1564,7 @@ void Done(int restart, char *command)
   if (FindFunction(exitFuncName))
   {
     char *action = safestrdup(CatString2("Function ", exitFuncName));
-    old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
+    old_execute_function(NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
 
@@ -1568,6 +1573,7 @@ void Done(int restart, char *command)
     Reborder();
   }
 
+  EWMH_ExitStuff();
   if (restart)
   {
     Bool doPreserveState = True;
@@ -1656,6 +1662,14 @@ void Done(int restart, char *command)
             "Use Restart without parameters if you mean to restart the same WM."
           );
         }
+        /* If we are at it, warn against a 'Restart fvwm' usage as well */
+        if (n == 1 && strcmp(my_argv[0], "fvwm") == 0)
+        {
+          fvwm_msg(WARN, "Done",
+            "`Restart fvwm' might not do what you want, see the man page.\n\t"
+            "Use Restart without parameters if you mean to restart the same WM."
+          );
+        }
         execvp(my_argv[0], my_argv);
         fvwm_msg(ERR, "Done", "Call of '%s' failed! (restarting '%s' instead)",
           my_argv[0], g_argv[0]);
@@ -1768,7 +1782,7 @@ void SaveDesktopState()
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
     data[0] = (unsigned long) t->Desk;
-    XChangeProperty (dpy, t->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
+    XChangeProperty (dpy, FW_W(t), _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
                      PropModeReplace, (unsigned char *) data, 1);
   }
 
@@ -1776,7 +1790,7 @@ void SaveDesktopState()
   XChangeProperty (dpy, Scr.Root, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
                    PropModeReplace, (unsigned char *) data, 1);
 
-  XSync(dpy, 0);
+  XFlush(dpy);
 }
 
 
@@ -1963,20 +1977,32 @@ static void setVersionInfo(void)
 #ifdef XPM
   strcat(support_str, " XPM,");
 #endif
+#ifdef HAVE_PNG
+  strcat(support_str, " PNG,");
+#endif
 #ifdef GNOME
   strcat(support_str, " GNOME WM hints,");
+#endif
+#ifdef HAVE_EWMH
+  strcat(support_str, " EWMH hints,");
 #endif
   if (FHaveShapeExtension)
     strcat(support_str, " Shape,");
 #ifdef SESSION
   strcat(support_str, " SM,");
 #endif
-#ifdef I18N_MB
+#ifdef MULTIBYTE
   strcat(support_str, " Multibyte,");
 #endif
+#ifdef HAVE_BIDI
+  strcat(support_str, " bidi text,");
+#endif
 #ifdef HAVE_XINERAMA
   strcat(support_str, " Xinerama,");
 #endif
+#ifdef HAVE_XFT
+  strcat(support_str, " XFT,");
+#endif
 
   support_len = strlen(support_str);
   if (support_len > 0)
