@@ -26,16 +26,17 @@
 #include "externs.h"
 #include "cursor.h"
 #include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "gnome.h"
+#include "move_resize.h"
 #include "stack.h"
 #include "update.h"
 #include "style.h"
 #include "virtual.h"
 #include "window_flags.h"
 #include "borders.h"
+#include "decorations.h"
 
 
 /*********************************************************/
@@ -303,7 +304,7 @@ GNOME_GetHintIcons(FvwmWindow *fwin)
   Pixmap	       mask;
 
   atom_get = XInternAtom(dpy, XA_WIN_ICONS, False);
-  retval = AtomGet(fwin->w, atom_get, XA_PIXMAP, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_PIXMAP, &size);
   if (retval)
   {
     for (i = 0; i < (size / (sizeof(CARD32))); i += 2)
@@ -323,7 +324,7 @@ GNOME_GetHintLayer(FvwmWindow *fwin)
   int size;
 
   atom_get = XInternAtom(dpy, XA_WIN_LAYER, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     set_layer(fwin, *retval);
@@ -339,14 +340,18 @@ GNOME_GetHintState(FvwmWindow *fwin)
   int size;
 
   atom_get = XInternAtom(dpy, XA_WIN_STATE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     if (*retval & WIN_STATE_STICKY)
       SET_STICKY(fwin, 1);
 
     if (*retval & WIN_STATE_SHADED)
+    {
+      /* Fixme: how do we find out the correct shade direction here? */
       SET_SHADED(fwin, 1);
+      SET_SHADED_DIR(fwin, DIR_N);
+    }
 
     if (*retval & WIN_STATE_FIXED_POSITION)
       SET_FIXED(fwin, 1);
@@ -364,7 +369,7 @@ GNOME_GetHintAppState(FvwmWindow *fwin)
 
   /* have nothing interesting to do with an app state (lamp) right now */
   atom_get = XInternAtom(dpy, XA_WIN_APP_STATE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     free(retval);
@@ -380,7 +385,7 @@ GNOME_GetHintDesktop(FvwmWindow *fwin)
   int *desk;
 
   atom_get = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     desk = (int *)retval;
@@ -400,7 +405,7 @@ GNOME_GetHint(FvwmWindow *fwin)
   int size;
 
   atom_get = XInternAtom(dpy, XA_WIN_HINTS, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     if (*retval & WIN_HINTS_SKIP_WINLIST)
@@ -430,7 +435,7 @@ GNOME_GetExpandedSize(FvwmWindow *fwin)
   int expanded_x, expanded_y, expanded_width, expanded_height;
 
   atom_get = XInternAtom(dpy, XA_WIN_EXPANDED_SIZE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     expanded_x = retval[0];
@@ -469,10 +474,10 @@ GNOME_SetHints(FvwmWindow *fwin)
     val |= WIN_STATE_STICKY;
   if (IS_SHADED(fwin))
     val |= WIN_STATE_SHADED;
-  if (IS_FIXED(fwin))
+  if (!is_function_allowed(F_MOVE, NULL, fwin, True, False))
     val |= WIN_STATE_FIXED_POSITION;
 
-  XChangeProperty(dpy, fwin->w, atom_set, XA_CARDINAL, 32,
+  XChangeProperty(dpy, FW_W(fwin), atom_set, XA_CARDINAL, 32,
 		  PropModeReplace, (unsigned char *)&val, 1);
 }
 
@@ -487,11 +492,11 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
   unsigned char *retval;
   int size;
 
-  if (DO_IGNORE_GNOME_HINTS(fwin))
+  if (SDO_IGNORE_GNOME_HINTS(style->flags))
     return;
   /* Desktop */
   atom_get = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     SSET_START_DESK(*style, *(int*)retval);
@@ -507,7 +512,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
 
   /* Layer */
   atom_get = XInternAtom(dpy, XA_WIN_LAYER, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     SSET_LAYER(*style, *(int*)retval);
@@ -518,7 +523,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
 
   /* State */
   atom_get = XInternAtom(dpy, XA_WIN_STATE, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     if (*(int*)retval & WIN_STATE_STICKY)
@@ -536,7 +541,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
       */
     }
 
-    if (*(int*)retval & WIN_STATE_FIXED_POSITION)
+    if ((*(int*)retval & WIN_STATE_FIXED_POSITION))
     {
       SFSET_IS_FIXED(*style, 1);
       SMSET_IS_FIXED(*style, 1);
@@ -550,7 +555,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
 
   /* Hints */
   atom_get = XInternAtom(dpy, XA_WIN_HINTS, False);
-  retval = AtomGet(fwin->w, atom_get, XA_CARDINAL, &size);
+  retval = AtomGet(FW_W(fwin), atom_get, XA_CARDINAL, &size);
   if (retval)
   {
     if (*retval & WIN_HINTS_SKIP_WINLIST)
@@ -580,7 +585,7 @@ GNOME_SetDesk(FvwmWindow *fwin)
   {
     GNOME_SetDeskCount();
   }
-  XChangeProperty(dpy, fwin->w, atom_set, XA_CARDINAL, 32,
+  XChangeProperty(dpy, FW_W(fwin), atom_set, XA_CARDINAL, 32,
 		  PropModeReplace, (unsigned char *)&val, 1);
 }
 
@@ -593,12 +598,12 @@ GNOME_SetLayer(FvwmWindow *fwin)
 
   atom_set = XInternAtom(dpy, XA_WIN_LAYER, False);
   val = get_layer(fwin);
-  XChangeProperty(dpy, fwin->w, atom_set, XA_CARDINAL, 32,
+  XChangeProperty(dpy, FW_W(fwin), atom_set, XA_CARDINAL, 32,
 		  PropModeReplace, (unsigned char *)&val, 1);
 }
 
 
-/*** INITALIZE GNOME WM SUPPORT ***/
+/*** INITIALIZE GNOME WM SUPPORT ***/
 
 /* sets the virtual desktop grid properties */
 void
@@ -721,7 +726,7 @@ GNOME_SetClientList(void)
   {
     if (!DO_SKIP_WINDOW_LIST(t))
     {
-      wl[i++] = t->w;
+      wl[i++] = FW_W(t);
     }
   }
 
@@ -761,7 +766,7 @@ GNOME_SetWinArea(FvwmWindow *w)
       if (val[1] < 0 && w->frame_g.y + Scr.Vy + w->frame_g.height > 0)
         val[1] = 0;
     }
-    XChangeProperty(dpy, w->w, atom_set, XA_CARDINAL, 32,
+    XChangeProperty(dpy, FW_W(w), atom_set, XA_CARDINAL, 32,
 		    PropModeReplace, (unsigned char *)val, 2);
   }
 }
@@ -824,7 +829,7 @@ GNOME_Init(void)
 		  PropModeReplace, (unsigned char *)&val, 1);
 
 
-  /* some inital settings */
+  /* some initial settings */
   GNOME_SetDeskCount();
   GNOME_SetCurrentDesk();
   GNOME_SetDeskNames();
@@ -926,7 +931,7 @@ void GNOME_HandlePropRequest(unsigned int propm,
 
   for (fwin = Scr.FvwmRoot.next; fwin; fwin = fwin->next)
   {
-    if (fwin->w == win)
+    if (FW_W(fwin) == win)
       break;
   }
 /*-----------------------------------------------------------------------
@@ -934,7 +939,7 @@ void GNOME_HandlePropRequest(unsigned int propm,
     somthing.
  -----------------------------------------------------------------------*/
 #ifdef FVWM_DEBUG_MSGS
-  fvwm_msg(DBG, "HandleGnomePropRequest", "window is %d", fwin->w);
+  fvwm_msg(DBG, "HandleGnomePropRequest", "window is %d", FW_W(fwin));
 #endif
   if (fwin == NULL)
     return;
@@ -945,14 +950,14 @@ void GNOME_HandlePropRequest(unsigned int propm,
   fvwm_msg(DBG, "HandleGnomePropRequest", "prop req is %d", prop);
   fvwm_msg(DBG, "HandleGnomePropRequest", "propm req is %d", propm);
 #endif
-  XGetWindowProperty(dpy, fwin->w, a, 0L, 3L, False, a,
+  XGetWindowProperty(dpy, FW_W(fwin), a, 0L, 3L, False, a,
 		     &atype, &aformat, &nitems, &bytes_remain, &indi);
   oldprop &= ~(propm);
   oldprop |= (propm & prop);
 #ifdef FVWM_DEBUG_MSGS
   fvwm_msg(DBG, "HandleGnomePropRequest", "oldprop req is %d", oldprop);
 #endif
-  XChangeProperty(dpy, fwin->w, a, XA_CARDINAL, 32,
+  XChangeProperty(dpy, FW_W(fwin), a, XA_CARDINAL, 32,
 		  PropModeReplace, (unsigned char *)&oldprop, 1);
 
 /*-----------------------------------------------------------------------
@@ -972,7 +977,8 @@ void GNOME_HandlePropRequest(unsigned int propm,
     {
       SET_STICKY(fwin, 0);
     }
-    DrawDecorations(fwin, DRAW_TITLE, (Scr.Hilite==fwin), True, None);
+    border_draw_decorations(
+      fwin, PART_TITLE, (Scr.Hilite==fwin), True, CLEAR_ALL, NULL, NULL);
   }
 /*-----------------------------------------------------------------------
     WindowShade
@@ -984,13 +990,13 @@ void GNOME_HandlePropRequest(unsigned int propm,
     {
       /* shade up */
       old_execute_function(
-	"WindowShade True", fwin, ev, C_WINDOW, -1, 0, NULL);
+	NULL, "WindowShade True", fwin, ev, C_WINDOW, -1, 0, NULL);
     }
     else
     {
       /* shade down */
       old_execute_function(
-	"WindowShade False", fwin, ev, C_WINDOW, -1, 0, NULL);
+        NULL, "WindowShade False", fwin, ev, C_WINDOW, -1, 0, NULL);
 
     }
   }
