@@ -28,6 +28,7 @@
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
+#include "events.h"
 
 Time managing_since;
 
@@ -80,20 +81,10 @@ SetupICCCM2 (Bool replace_wm)
     XChangeWindowAttributes (dpy, running_wm_win, CWEventMask, &attr);
   }
 
-  /* add PropChange to NoFocusWin events */
-  attr.event_mask = PropertyChangeMask;
-  XChangeWindowAttributes (dpy, Scr.NoFocusWin, CWEventMask, &attr);
-
   /* We are not yet in the event loop, thus lastTimestamp will not
      be ready. Have to get a timestamp manually by provoking a
      PropertyNotify. */
-  XChangeProperty (dpy, Scr.NoFocusWin, XA_WM_CLASS, XA_STRING, 8,
-		   PropModeAppend, NULL, 0);
-  XWindowEvent (dpy, Scr.NoFocusWin, PropertyChangeMask, &xev);
-  attr.event_mask = 0L;
-  XChangeWindowAttributes (dpy, Scr.NoFocusWin, CWEventMask, &attr);
-
-  managing_since = xev.xproperty.time;
+  managing_since = get_server_time();
 
   XSetSelectionOwner (dpy, _XA_WM_SX, Scr.NoFocusWin, managing_since);
   if (XGetSelectionOwner (dpy, _XA_WM_SX) != Scr.NoFocusWin) {
@@ -120,7 +111,7 @@ SetupICCCM2 (Bool replace_wm)
   }
 
   /* restore NoFocusWin event mask */
-  attr.event_mask = NO_FOCUS_WIN_EVMASK;
+  attr.event_mask = XEVMASK_NOFOCUSW;
   XChangeWindowAttributes (dpy, Scr.NoFocusWin, CWEventMask, &attr);
 }
 
@@ -131,8 +122,8 @@ void
 CloseICCCM2 (void)
 {
   DBUG("CloseICCCM2", "good luck, new wm");
-  XSelectInput(dpy, Scr.Root, 0 );
-  XSync(dpy, 0);
+  XSelectInput(dpy, Scr.Root, NoEventMask);
+  XFlush(dpy);
 }
 
 /* FIXME: property change actually succeeded */
@@ -158,7 +149,8 @@ convertProperty (Window w, Atom target, Atom property)
      and send the SelectionNotify once we are sure the property
      has arrived. Problem: this needs a list of pending
      SelectionNotifys. */
-  XSync (dpy, 0);
+  XFlush(dpy);
+
   return True;
 }
 
@@ -202,8 +194,10 @@ HandleSelectionRequest (void)
       reply.property = ev.property;
   }
 
-  XSync (dpy, 0);
   XSendEvent (dpy, ev.requestor, False, 0L, (XEvent*)&reply);
+  XFlush(dpy);
+
+  return;
 }
 
 /* If another wm is requesting ownership of the selection,
@@ -216,8 +210,3 @@ HandleSelectionClear (void)
   DBUG("HandleSelectionClear", "I lost my selection!");
   Done(0, NULL);
 }
-
-
-
-
-
