@@ -19,16 +19,18 @@
 #ifdef NO_ICONS
 #define ICON_HEIGHT(t) 1
 #else
-#define ICON_HEIGHT(t) ((t)->icon_font.height + 6)
+#define ICON_HEIGHT(t) ((t)->icon_font->height + 6)
 #endif
 
-void AutoPlaceIcon(FvwmWindow *);
+int get_visible_icon_window_count(FvwmWindow *fw);
+void clear_icon(FvwmWindow *fw);
+void GetIconPicture(FvwmWindow *fw, Bool no_icon_window);
+void AutoPlaceIcon(FvwmWindow *t, initial_window_options_type *win_opts);
+void ChangeIconPixmap(FvwmWindow *fw);
 void RedoIconName(FvwmWindow *);
 void DrawIconWindow(FvwmWindow *);
-void CreateIconWindow(FvwmWindow *tmp_win, int def_x, int def_y);
-void GetIconWindow(FvwmWindow *tmp_win);
-void GetIconBitmap(FvwmWindow *tmp_win);
-void Iconify(FvwmWindow *, int, int);
+void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y);
+void Iconify(FvwmWindow *fw, initial_window_options_type *win_opts);
 void DeIconify(FvwmWindow *);
 void SetMapStateProp(FvwmWindow *, int);
 
