@@ -38,8 +38,10 @@
 #endif
 
 #include "libs/fvwmlib.h"
-#include "libs/Colorset.h"
 #include "libs/FScreen.h"
+#include "libs/Module.h"
+#include "libs/queue.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -47,9 +49,11 @@
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
+#include "colorset.h"
 #include "module_interface.h"
 #include "read.h"
 #include "events.h"
+#include "geometry.h"
 #include "fvwmsignal.h"
 
 /*
@@ -62,23 +66,46 @@
 int npipes;
 int *readPipes;
 int *writePipes;
-int *pipeOn;
+static int *pipeOn;
 char **pipeName;
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
 char **pipeAlias;  /* as given in: Module FvwmPager MyAlias */
-#endif
 
-unsigned long *PipeMask;
-unsigned long *SyncMask;
-unsigned long *NoGrabMask;
-struct queue_buff_struct **pipeQueue;
+typedef struct
+{
+  unsigned long *data;
+  int size;
+  int done;
+} mqueue_object_type;
+
+msg_masks_type *PipeMask;
+static msg_masks_type *SyncMask;
+static msg_masks_type *NoGrabMask;
+fqueue *pipeQueue;
 
 extern fd_set init_fdset;
 
 static void DeleteMessageQueueBuff(int module);
-static void AddToMessageQueue(int module, unsigned long *ptr, int size, int done);
 static void AddToCommandQueue(Window w, int module, char * command);
 
+/*
+ * Sets the mask to the specific value.  If M_EXTENDED_MSG is set in mask, the
+ * function operates only on the extended messages, otherwise it operates only
+ * on normal messages.
+ */
+static void set_message_mask(msg_masks_type *mask, unsigned long msg)
+{
+	if (msg & M_EXTENDED_MSG)
+	{
+		mask->m2 = (msg & ~M_EXTENDED_MSG);
+	}
+	else
+	{
+		mask->m1 = msg;
+	}
+
+	return;
+}
+
 void initModules(void)
 {
   int i;
@@ -88,30 +115,28 @@ void initModules(void)
   writePipes = (int *)safemalloc(sizeof(int)*npipes);
   readPipes = (int *)safemalloc(sizeof(int)*npipes);
   pipeOn = (int *)safemalloc(sizeof(int)*npipes);
-  PipeMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
-  SyncMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
-  NoGrabMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
+  PipeMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
+  SyncMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
+  NoGrabMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
   pipeName = (char **)safemalloc(sizeof(char *)*npipes);
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-  pipeAlias= (char **)safemalloc(sizeof(char *)*npipes);
-#endif
-  pipeQueue=(struct queue_buff_struct **)
-    safemalloc(sizeof(struct queue_buff_struct *)*npipes);
+  pipeAlias = (char **)safemalloc(sizeof(char *)*npipes);
+  pipeQueue = (fqueue *)safemalloc(sizeof(fqueue)*npipes);
 
-  for(i=0;i<npipes;i++)
-    {
-      writePipes[i]= -1;
-      readPipes[i]= -1;
-      pipeOn[i] = -1;
-      PipeMask[i] = MAX_MASK;
-      SyncMask[i] = 0;
-      NoGrabMask[i] = 0;
-      pipeQueue[i] = (struct queue_buff_struct *)NULL;
-      pipeName[i] = NULL;
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-      pipeAlias[i] = NULL;
-#endif
-    }
+  for (i=0; i < npipes; i++)
+  {
+    writePipes[i]= -1;
+    readPipes[i]= -1;
+    pipeOn[i] = -1;
+    PipeMask[i].m1 = DEFAULT_MASK;
+    PipeMask[i].m2 = DEFAULT_XMASK;
+    SyncMask[i].m1 = 0;
+    SyncMask[i].m2 = 0;
+    NoGrabMask[i].m1 = 0;
+    NoGrabMask[i].m2 = 0;
+    fqueue_init(&pipeQueue[i]);
+    pipeName[i] = NULL;
+    pipeAlias[i] = NULL;
+  }
   DBUG("initModules", "Zeroing init module array\n");
   FD_ZERO(&init_fdset);
 }
@@ -131,14 +156,12 @@ void ClosePipes(void)
         free(pipeName[i]);
         pipeName[i] = 0;
       }
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
       if(pipeAlias[i] != NULL)
       {
         free(pipeAlias[i]);
         pipeAlias[i] = NULL;
       }
-#endif
-      while(pipeQueue[i] != NULL)
+      while (!FQUEUE_IS_EMPTY(&pipeQueue[i]))
       {
         DeleteMessageQueueBuff(i);
       }
@@ -173,8 +196,8 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
     return -1;
   }
 
-  if(tmp_win)
-    win = tmp_win->w;
+  if(fw)
+    win = FW_W(fw);
   else
     win = None;
 
@@ -269,14 +292,12 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
   {
     args = (char **)saferealloc((void *)args, (nargs + 2) * sizeof(char *));
     args[nargs] = token;
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
     if (pipeAlias[i] == NULL)
     {
       const char *ptr = skipModuleAliasToken(args[nargs]);
       if (ptr && *ptr == '\0')
         pipeAlias[i] = stripcpy(args[nargs]);
     }
-#endif
   }
   args[nargs] = NULL;
 
@@ -296,11 +317,14 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
     writePipes[i] = fvwm_to_app[1];
     readPipes[i] = app_to_fvwm[0];
     pipeOn[i] = -1;
-    PipeMask[i] = MAX_MASK;
-    SyncMask[i] = 0;
-    NoGrabMask[i] = 0;
+    PipeMask[i].m1 = DEFAULT_MASK;
+    PipeMask[i].m2 = DEFAULT_XMASK;
+    SyncMask[i].m1 = 0;
+    SyncMask[i].m2 = 0;
+    NoGrabMask[i].m1 = 0;
+    NoGrabMask[i].m2 = 0;
     free(arg1);
-    pipeQueue[i] = NULL;
+    fqueue_init(&pipeQueue[i]);
     if (DoingCommandLine) {
       /* add to the list of command line modules */
       DBUG("executeModule", "starting commandline module\n");
@@ -396,7 +420,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
   Bool need_ungrab = False;
   char *escape = NULL;
   XEvent tmpevent;
-  extern FvwmWindow *Tmp_win;
+  extern FvwmWindow *Fw;
 
   if (!action)
     return;
@@ -465,7 +489,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
       FD_ZERO(&out_fdset);
       if (readPipes[pipe_slot] >= 0)
         FD_SET(readPipes[pipe_slot], &in_fdset);
-      if (pipeQueue[pipe_slot] != NULL)
+      if (!FQUEUE_IS_EMPTY(&pipeQueue[pipe_slot]))
         FD_SET(writePipes[pipe_slot], &out_fdset);
 
       timeout.tv_sec = 0;
@@ -519,7 +543,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
                             STROKE_ARG(0)
                             tmpevent.xkey.keycode,
                             tmpevent.xkey.state, GetUnusedModifiers(),
-                            GetContext(Tmp_win, &tmpevent, &targetWindow),
+                            GetContext(Fw, &tmpevent, &targetWindow),
                             KEY_BINDING);
       if (escape != NULL)
       {
@@ -615,11 +639,11 @@ void ExecuteModuleCommand(Window w, int module, char *text)
 {
   extern FvwmWindow *ButtonWindow;
   extern int Context;
-  FvwmWindow *tmp_win;
+  FvwmWindow *fw;
 
-  if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &tmp_win) == XCNOENT)
+  if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &fw) == XCNOENT)
   {
-    tmp_win = NULL;
+    fw = NULL;
     w = None;
   }
 
@@ -635,7 +659,7 @@ void ExecuteModuleCommand(Window w, int module, char *text)
     Event.xbutton.window = Scr.Root;
     Event.xbutton.button = 1;
     Event.xbutton.subwindow = None;
-    tmp_win = NULL;
+    fw = NULL;
     ButtonWindow = NULL;
   }
   else
@@ -643,8 +667,8 @@ void ExecuteModuleCommand(Window w, int module, char *text)
     Event.xbutton.window = w;
     Event.xbutton.button = 1;
     Event.xbutton.subwindow = None;
-    Context = GetContext(tmp_win,&Event,&w);
-    ButtonWindow = tmp_win;
+    Context = GetContext(fw,&Event,&w);
+    ButtonWindow = fw;
   }
   /* If a module does XUngrabPointer(), it can now get proper Popups */
   if(StrEquals(text, "popup"))
@@ -653,8 +677,8 @@ void ExecuteModuleCommand(Window w, int module, char *text)
     Event.xbutton.type = ButtonRelease;
   Event.xbutton.x = 0;
   Event.xbutton.y = 0;
-  Context = GetContext(tmp_win,&Event,&w);
-  old_execute_function(text, tmp_win, &Event, Context, module, 0, NULL);
+  Context = GetContext(fw,&Event,&w);
+  old_execute_function(NULL, text, fw, &Event, Context, module, 0, NULL);
   ButtonWindow = NULL;
 }
 
@@ -671,8 +695,8 @@ void KillModule(int channel)
   readPipes[channel] = -1;
   writePipes[channel] = -1;
   pipeOn[channel] = -1;
-  while(pipeQueue[channel] != NULL)
-    {
+  while (!FQUEUE_IS_EMPTY(&pipeQueue[channel]))
+  {
       DeleteMessageQueueBuff(channel);
     }
   if(pipeName[channel] != NULL)
@@ -680,13 +704,11 @@ void KillModule(int channel)
       free(pipeName[channel]);
       pipeName[channel] = NULL;
     }
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
   if(pipeAlias[channel] != NULL)
     {
       free(pipeAlias[channel]);
       pipeAlias[channel] = NULL;
     }
-#endif
   if (fFvwmInStartup) {
     /* remove from list of command line modules */
     DBUG("killModule", "ending command line module\n");
@@ -696,11 +718,7 @@ void KillModule(int channel)
   return;
 }
 
-static void KillModuleByName(char *name
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-  , char *alias
-#endif
-)
+static void KillModuleByName(char *name, char *alias)
 {
   int i = 0;
 
@@ -710,9 +728,7 @@ static void KillModuleByName(char *name
   while(i<npipes)
     {
       if((pipeName[i] != NULL)&&(matchWildcards(name,pipeName[i]))
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
           && (!alias || (pipeAlias[i] && matchWildcards(alias, pipeAlias[i])))
-#endif
         )
 	{
 	  KillModule(i);
@@ -725,20 +741,14 @@ static void KillModuleByName(char *name
 void CMD_KillModule(F_CMD_ARGS)
 {
   char *module;
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
   char *alias = NULL;
-#endif
 
   action = GetNextToken(action,&module);
   if (!module)
     return;
 
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
   GetNextToken(action, &alias);
   KillModuleByName(module, alias);
-#else
-  KillModuleByName(module);
-#endif
   free(module);
 }
 
@@ -895,9 +905,9 @@ static void BroadcastNewPacket(unsigned long event_type,
 
 
 /* this is broken, the flags may not fit in a word */
-#define CONFIGARGS(_t) 24,\
-            (_t)->w,\
-            (_t)->frame,\
+#define CONFIGARGS(_t) 27,\
+            FW_W(_t),\
+            FW_W_FRAME(_t),\
             (unsigned long)(_t),\
             (_t)->frame_g.x,\
             (_t)->frame_g.y,\
@@ -905,7 +915,7 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->frame_g.height,\
             (_t)->Desk,\
             (_t)->flags,\
-            (_t)->title_g.height,\
+            (_t)->title_thickness,\
             (_t)->boundary_width,\
 	    (_t)->hints.base_width,\
 	    (_t)->hints.base_height,\
@@ -915,16 +925,19 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->hints.min_height,\
             (_t)->hints.max_width,\
             (_t)->hints.max_height,\
-            (_t)->icon_w,\
-            (_t)->icon_pixmap_w,\
+            FW_W_ICON_TITLE(_t),\
+            FW_W_ICON_PIXMAP(_t),\
             (_t)->hints.win_gravity,\
             (_t)->colors.fore,\
-            (_t)->colors.back
+            (_t)->colors.back,\
+            (_t)->ewmh_hint_layer,\
+            (_t)->ewmh_hint_desktop,\
+            (_t)->ewmh_window_type
 
 #ifndef DISABLE_MBC
-#define OLDCONFIGARGS(_t) 24,\
-            (_t)->w,\
-            (_t)->frame,\
+#define OLDCONFIGARGS(_t) 27,\
+            FW_W(_t),\
+            FW_W_FRAME(_t),\
             (unsigned long)(_t),\
             (_t)->frame_g.x,\
             (_t)->frame_g.y,\
@@ -932,7 +945,7 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->frame_g.height,\
             (_t)->Desk,\
             old_flags,\
-            (_t)->title_g.height,\
+            (_t)->title_thickness,\
             (_t)->boundary_width,\
 	    (_t)->hints.base_width,\
 	    (_t)->hints.base_height,\
@@ -942,15 +955,18 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->hints.min_height,\
             (_t)->hints.max_width,\
             (_t)->hints.max_height,\
-            (_t)->icon_w,\
-            (_t)->icon_pixmap_w,\
+            FW_W_ICON_TITLE(_t),\
+            FW_W_ICON_PIXMAP(_t),\
             (_t)->hints.win_gravity,\
             (_t)->colors.fore,\
-            (_t)->colors.back
+            (_t)->colors.back,\
+            (_t)->ewmh_hint_layer,\
+            (_t)->ewmh_hint_desktop,\
+            (_t)->ewmh_window_type
 
 #define SETOLDFLAGS \
 { int i = 1; \
-  old_flags |= DO_START_ICONIC(t)		? i : 0; i<<=1; \
+  old_flags |= 0 /* was start_iconic */		? i : 0; i<<=1; \
   old_flags |= False /* OnTop */		? i : 0; i<<=1; \
   old_flags |= IS_STICKY(t)			? i : 0; i<<=1; \
   old_flags |= DO_SKIP_WINDOW_LIST(t)		? i : 0; i<<=1; \
@@ -996,11 +1012,11 @@ static void BroadcastNewPacket(unsigned long event_type,
         as a dummy to preserve alignment of the other fields in the
         old packet: we should drop this before the next release.
 *****************************************************************/
-#define CONFIGARGSNEW(_t) 25,\
+#define CONFIGARGSNEW(_t) 28,\
 	    (unsigned long)(sizeof(unsigned long)),\
-            &(*(_t))->w,\
+            &FW_W(*(_t)),\
 	    (unsigned long)(sizeof(unsigned long)),\
-            &(*(_t))->frame,\
+            &FW_W_FRAME(*(_t)),\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(_t),\
 	    (unsigned long)(0),\
@@ -1016,7 +1032,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 	    (unsigned long)(0),\
             &(*(_t))->layer,\
 	    (unsigned long)(0),\
-            &(*(_t))->title_g.height,\
+            &(*(_t))->title_thickness,\
 	    (unsigned long)(0),\
             &(*(_t))->boundary_width,\
 	    (unsigned long)(0),\
@@ -1036,15 +1052,21 @@ static void BroadcastNewPacket(unsigned long event_type,
 	    (unsigned long)(0),\
             &(*(_t))->hints.max_height,\
 	    (unsigned long)(0),\
-            &(*(_t))->icon_w,\
+            &FW_W_ICON_TITLE(*(_t)),\
 	    (unsigned long)(sizeof(unsigned long)),\
-            &(*(_t))->icon_pixmap_w,\
+            &FW_W_ICON_PIXMAP(*(_t)),\
 	    (unsigned long)(0),\
             &(*(_t))->hints.win_gravity,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->colors.fore,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->colors.back,\
+            (unsigned long)(0),\
+            &(*(_t))->ewmh_hint_layer,\
+	    (unsigned long)(sizeof(unsigned long)),\
+            &(*(_t))->ewmh_hint_desktop,\
+	    (unsigned long)(0),\
+            &(*(_t))->ewmh_window_type,\
 	    (unsigned long)(sizeof((*(_t))->flags)),\
             &(*(_t))->flags
 
@@ -1091,7 +1113,7 @@ const FvwmWindow **t1 = &t;
 
 static unsigned long *
 make_named_packet(int *len, unsigned long event_type, const char *name,
-                  int num, ...)
+		  int num, ...)
 {
   unsigned long *body;
   va_list ap;
@@ -1158,47 +1180,97 @@ BroadcastName(unsigned long event_type,
   free(body);
 }
 
-#ifdef MINI_ICONS
-static void
-SendMiniIcon(int module, unsigned long event_type,
-             unsigned long data1, unsigned long data2,
-             unsigned long data3, unsigned long data4,
-             unsigned long data5, unsigned long data6,
-             unsigned long data7, unsigned long data8,
-             const char *name)
+void
+BroadcastWindowIconNames(FvwmWindow *t, Bool window, Bool icon)
 {
-  unsigned long *body;
-  int l;
+	if (window)
+	{
+		BroadcastName(
+			M_WINDOW_NAME,FW_W(t), FW_W_FRAME(t), (unsigned long)t,
+			t->name.name);
+		BroadcastName(
+			M_VISIBLE_NAME,FW_W(t),FW_W_FRAME(t),(unsigned long)t,
+			t->visible_name);
+	}
+	if (icon)
+	{
+		BroadcastName(
+			M_ICON_NAME, FW_W(t), FW_W_FRAME(t), (unsigned long)t,
+			t->icon_name.name);
+		BroadcastName(
+			MX_VISIBLE_ICON_NAME,FW_W(t),FW_W_FRAME(t),
+			(unsigned long)t, t->visible_icon_name);
+	}
 
-  if ((name == NULL) || (event_type != M_MINI_ICON))
-    return;
-
-  body = make_named_packet(&l, event_type, name, 8, data1, data2, data3,
-                           data4, data5, data6, data7, data8);
-  PositiveWrite(module, body, l*sizeof(unsigned long));
-  free(body);
+	return;
 }
 
 void
-BroadcastMiniIcon(unsigned long event_type,
-                  unsigned long data1, unsigned long data2,
-                  unsigned long data3, unsigned long data4,
-                  unsigned long data5, unsigned long data6,
-                  unsigned long data7, unsigned long data8,
-                  const char *name)
+SendFvwmPicture(int module, unsigned long event_type,
+		unsigned long data1, unsigned long data2,
+		unsigned long data3, FvwmPicture *picture, char *name)
 {
-  unsigned long *body;
-  int i, l;
+        unsigned long *body;
+	unsigned int data4 = 0, data5 = 0, data6 = 0,
+		data7 = 0, data8 = 0, data9 = 0;
+        int l;
 
-  body = make_named_packet(&l, event_type, name, 8, data1, data2, data3,
-                           data4, data5, data6, data7, data8);
+        if (!FMiniIconsSupported)
+        {
+                return;
+        }
+	if ((name == NULL) || (event_type != M_MINI_ICON))
+                return;
 
-  for (i=0; i < npipes; i++)
-    PositiveWrite(i, body, l*sizeof(unsigned long));
+	if (picture != NULL)
+	{
+		data4 = picture->width;
+		data5 = picture->height;
+		data6 = picture->depth;
+		data7 = picture->picture;
+		data8 = picture->mask;
+		data9 = picture->alpha;
+	}
+        body = make_named_packet(&l, event_type, name, 9, data1, data2, data3,
+				 data4, data5, data6, data7, data8, data9);
 
-  free(body);
+	PositiveWrite(module, body, l*sizeof(unsigned long));
+        free(body);
+
+	return;
+}
+
+void
+BroadcastFvwmPicture(unsigned long event_type,
+		     unsigned long data1, unsigned long data2,
+		     unsigned long data3, FvwmPicture *picture, char *name)
+{
+        unsigned long *body;
+	unsigned int data4 = 0, data5 = 0, data6 = 0,
+		data7 = 0, data8 = 0, data9 = 0;
+        int i, l;
+
+        if (!FMiniIconsSupported)
+        {
+                return;
+        }
+	if (picture != NULL)
+	{
+		data4 = picture->width;
+		data5 = picture->height;
+		data6 = picture->depth;
+		data7 = picture->picture;
+		data8 = picture->mask;
+		data9 = picture->alpha;
+	}
+        body = make_named_packet(&l, event_type, name, 9, data1, data2, data3,
+				 data4, data5, data6, data7, data8, data9);
+
+        for (i=0; i < npipes; i++)
+                PositiveWrite(i, body, l*sizeof(unsigned long));
+
+        free(body);
 }
-#endif /* MINI_ICONS */
 
 /**********************************************************************
  * Reads a colorset command from a module and broadcasts it back out
@@ -1217,6 +1289,21 @@ void BroadcastColorset(int n)
   }
 }
 
+/**********************************************************************
+ * Broadcasts a string to all modules as M_CONFIG_INFO.
+ **********************************************************************/
+void BroadcastPropertyChange(unsigned long argument, unsigned long data1,
+			     unsigned long data2, char *string)
+{
+  int i;
+
+  for (i = 0; i < npipes; i++)
+  {
+    /* just a quick check to save us lots of overhead */
+    if (pipeOn[i] >= 0)
+      SendName(i, MX_PROPERTY_CHANGE, argument, data1, data2, string);
+  }
+}
 
 /**********************************************************************
  * Broadcasts a string to all modules as M_CONFIG_INFO.
@@ -1244,6 +1331,20 @@ void broadcast_xinerama_state(void)
 }
 
 
+/**********************************************************************
+ * Broadcasts the ignored modifiers to all modules as M_CONFIG_INFO.
+ **********************************************************************/
+void broadcast_ignore_modifiers(void)
+{
+  char msg[32];
+
+  sprintf(msg, "IgnoreModifiers %d", GetUnusedModifiers());
+  BroadcastConfigInfoString(msg);
+
+  return;
+}
+
+
 /*
 ** send an arbitrary string to all instances of a module
 */
@@ -1263,11 +1364,11 @@ void CMD_SendToModule(F_CMD_ARGS)
     return;
   str = safestrdup(action + strlen(module) + 1);
 
-  if (tmp_win) {
+  if (fw) {
     /* Modules may need to know which window this applies to */
-    data0 = tmp_win->w;
-    data1 = tmp_win->frame;
-    data2 = (unsigned long)tmp_win;
+    data0 = FW_W(fw);
+    data1 = FW_W_FRAME(fw);
+    data2 = (unsigned long)fw;
   } else {
     data0 = 0;
     data1 = 0;
@@ -1276,10 +1377,7 @@ void CMD_SendToModule(F_CMD_ARGS)
   for (i=0;i<npipes;i++)
   {
     if ((pipeName[i] != NULL && matchWildcards(module,pipeName[i]))
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-      || (pipeAlias[i] && matchWildcards(module, pipeAlias[i]))
-#endif
-    )
+      || (pipeAlias[i] && matchWildcards(module, pipeAlias[i])))
     {
       SendName(i,M_STRING,data0,data1,data2,str);
       FlushMessageQueue(i);
@@ -1290,7 +1388,6 @@ void CMD_SendToModule(F_CMD_ARGS)
   free(str);
 }
 
-
 /* This used to be marked "fvwm_inline".  I removed this
    when I added the lockonsend logic.  The routine seems too big to
    want to inline.  dje 9/4/98 */
@@ -1298,186 +1395,220 @@ extern int myxgrabcount;                /* defined in libs/Grab.c */
 extern char *ModuleUnlock;		/* defined in libs/Module.c */
 void PositiveWrite(int module, unsigned long *ptr, int size)
 {
-  extern int moduleTimeout;
+	extern int moduleTimeout;
+	msg_masks_type mask;
 
-  if(ptr == NULL)
-  {
-   return;
-  }
-  if((pipeOn[module]<0)||(!((PipeMask[module]) & ptr[1])))
-  {
-    return;
-  }
+	if(ptr == NULL)
+	{
+		return;
+	}
+	if (pipeOn[module] < 0)
+	{
+		return;
+	}
+	if (!IS_MESSAGE_IN_MASK(&PipeMask[module], ptr[1]))
+	{
+		return;
+	}
 
-  /* a dirty hack to prevent FvwmAnimate triggering during Recapture */
-  /* would be better to send RecaptureStart and RecaptureEnd messages. */
-  /* If module is lock on send for iconify message and it's an
-   * iconify event and server grabbed, then return */
-  if ((NoGrabMask[module] & SyncMask[module] & ptr[1]) && (myxgrabcount != 0))
-  {
-    return;
-  }
+	/* a dirty hack to prevent FvwmAnimate triggering during Recapture */
+	/* would be better to send RecaptureStart and RecaptureEnd messages. */
+	/* If module is lock on send for iconify message and it's an
+	 * iconify event and server grabbed, then return */
+	mask.m1 = (NoGrabMask[module].m1 & SyncMask[module].m1);
+	mask.m2 = (NoGrabMask[module].m2 & SyncMask[module].m2);
+	if (IS_MESSAGE_IN_MASK(&mask, ptr[1]) && myxgrabcount != 0)
+	{
+		return;
+	}
 
-  AddToMessageQueue(module,ptr,size,0);
+	/* DV: This was once the AddToMessageQueue function.  Since it was only
+	 * called once, put it in here for better performance. */
+	{
+		mqueue_object_type *c;
 
-  /* dje, from afterstep, for FvwmAnimate, allows modules to sync with fvwm.
-   * this is disabled when the server is grabbed, otherwise deadlocks happen.
-   * M_LOCKONSEND has been replaced by a separated mask which defines on
-   * which messages the fvwm-to-module communication need to be lock
-   * on send. olicha Nov 13, 1999 */
-  /* migo (19-Aug-2000): removed !myxgrabcount to sync M_DESTROY_WINDOW */
-/*if ((SyncMask[module] & ptr[1]) && !myxgrabcount) */
+		c = (mqueue_object_type *)malloc(
+			sizeof(mqueue_object_type) + size);
+		if (c == NULL)
+		{
+			fvwm_msg(ERR, "PositiveWrite", "malloc failed\n");
+			exit(1);
+		}
+		c->size = size;
+		c->done = 0;
+		c->data = (unsigned long *)(c + 1);
+		memcpy((void*)c->data, (const void*)ptr, size);
 
-  if ((SyncMask[module] & ptr[1]))
-  {
-    Window targetWindow;
-    fd_set readSet;
-    struct timeval timeout;
-    int channel = readPipes[module];
+		fqueue_add_at_end(&pipeQueue[module], c);
+	}
 
-    FlushMessageQueue(module);
+	/* dje, from afterstep, for FvwmAnimate, allows modules to sync with
+	 * fvwm. this is disabled when the server is grabbed, otherwise
+	 * deadlocks happen. M_LOCKONSEND has been replaced by a separated
+	 * mask which defines on which messages the fvwm-to-module
+	 * communication need to be lock on send. olicha Nov 13, 1999 */
+	/* migo (19-Aug-2000): removed !myxgrabcount to sync M_DESTROY_WINDOW */
+	/*if ((SyncMask[module] & ptr[1]) && !myxgrabcount) */
+	if (IS_MESSAGE_IN_MASK(&SyncMask[module], ptr[1]))
+	{
+		Window targetWindow;
+		fd_set readSet;
+		struct timeval timeout;
+		int channel = readPipes[module];
 
-    do
-    {
-      int rc = 0;
-      /*
-       * We give the read a long timeout; if the module fails to
-       * respond within this time then it deserves to be KILLED!
-       *
-       * NOTE: rather than impose an arbitrary timeout on the user,
-       *       we will make this a configuration parameter.
-       */
+		FlushMessageQueue(module);
 
-      do
-      {
-        timeout.tv_sec = moduleTimeout;
-        timeout.tv_usec = 0;
-        FD_ZERO(&readSet);
-        FD_SET(channel, &readSet);
+		do
+		{
+			int rc = 0;
+			/*
+			 * We give the read a long timeout; if the module
+			 * fails to respond within this time then it deserves
+			 * to be KILLED!
+			 *
+			 * NOTE: rather than impose an arbitrary timeout on the
+			 * user, we will make this a configuration parameter. */
+			do
+			{
+				timeout.tv_sec = moduleTimeout;
+				timeout.tv_usec = 0;
+				FD_ZERO(&readSet);
+				FD_SET(channel, &readSet);
 
-        /* Wait for input to arrive on just one descriptor, with a timeout */
-        /* (fvwmSelect <= 0) or read() returning wrong size is bad news */
-        rc = fvwmSelect(channel + 1, &readSet, NULL, NULL, &timeout);
-        /* retry if select() failed with EINTR */
-      } while ((rc < 0) && !isTerminated && (errno == EINTR));
+				/* Wait for input to arrive on just one
+				 * descriptor, with a timeout (fvwmSelect <= 0)
+				 * or read() returning wrong size is bad news */
+				rc = fvwmSelect(
+					channel + 1, &readSet, NULL, NULL,
+					&timeout);
+				/* retry if select() failed with EINTR */
+			} while ((rc < 0) && !isTerminated && (errno == EINTR));
 
-      if ( isTerminated ) break;
+			if ( isTerminated ) break;
 
-      if (rc <= 0 || read(channel, &targetWindow, sizeof(targetWindow))
-           != sizeof(targetWindow))
-      {
-        /* Doh! Something has gone wrong - get rid of the offender!! */
-        fvwm_msg(ERR, "PositiveWrite",
-                 "Failed to read descriptor from '%s':\n"
-                 "- data available=%c\n"
-                 "- terminate signal=%c\n",
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-                 pipeAlias[module]
-                   ? CatString3(pipeName[module], " ", pipeAlias[module])
-                   : pipeName[module],
-#else
-                 pipeName[module],
-#endif
-                 (FD_ISSET(channel, &readSet) ? 'Y' : 'N'),
-                 isTerminated ? 'Y' : 'N');
-        KillModule(module);
-        break;
-      }
+			if (rc <= 0 || read(channel, &targetWindow,
+					    sizeof(targetWindow))
+			    != sizeof(targetWindow))
+			{
+				char *name;
 
-      /* Execute all messages from the module until UNLOCK is received
-       * N.B. This may cause recursion if a command results in a sync message
-       * to another module, which in turn may send a command that results in
-       * another sync message to this module.
+				if (pipeName[module] != NULL)
+				{
+					if (pipeAlias[module] != NULL)
+					{
+						name = CatString3(
+							pipeName[module], " ",
+							pipeAlias[module]);
+					}
+					else
+					{
+						name = pipeName[module];
+					}
+				}
+				else
+				{
+					name = "(null)";
+				}
 
-       * Hippo: I don't think this will cause deadlocks, but the third time we
-       * get here the first times UNLOCK will be read and then on returning up
-       * the third level UNLOCK will be read at the first level. This could be
-       * difficult to fix without turning queueing on.  Turning queueing on
-       * may be bad because it can be useful for modules to be able to inject
-       * commands from modules in a synchronous manner. e.g. FvwmIconMan
-       * can tell FvwmAnimate to do an animation when a window is de-iconified
-       * from the IconMan, queueing make s this happen too late.
-       */
-    }
-    while (!HandleModuleInput(targetWindow, module, ModuleUnlock, False));
-  }
+				/* Doh! Something has gone wrong - get rid of
+				 * the offender!! */
+				fvwm_msg(ERR, "PositiveWrite",
+					 "Failed to read descriptor from"
+					 " '%s':\n"
+					 "- data available=%c\n"
+					 "- terminate signal=%c\n",
+					 name,
+					 (FD_ISSET(channel, &readSet) ?
+					  'Y' : 'N'),
+					 isTerminated ? 'Y' : 'N');
+				KillModule(module);
+				break;
+			}
+
+			/* Execute all messages from the module until UNLOCK is
+			 * received N.B. This may cause recursion if a command
+			 * results in a sync message to another module, which
+			 * in turn may send a command that results in another
+			 * sync message to this module.
+			 * Hippo: I don't think this will cause deadlocks, but
+			 * the third time we get here the first times UNLOCK
+			 * will be read and then on returning up the third
+			 * level UNLOCK will be read at the first level. This
+			 * could be difficult to fix without turning queueing
+			 * on.  Turning queueing on may be bad because it can
+			 * be useful for modules to be able to inject commands
+			 * from modules in a synchronous manner. e.g.
+			 * FvwmIconMan can tell FvwmAnimate to do an animation
+			 * when a window is de-iconified from the IconMan,
+			 * queueing make s this happen too late. */
+		}
+		while (!HandleModuleInput(
+			       targetWindow, module, ModuleUnlock, False));
+	}
+
+	return;
 }
 
 
-static void AddToMessageQueue(int module, unsigned long *ptr, int size, int done)
-{
-  struct queue_buff_struct *c,*e;
-  unsigned long *d;
-
-  c = (struct queue_buff_struct *)safemalloc(sizeof(struct queue_buff_struct));
-  c->next = NULL;
-  c->size = size;
-  c->done = done;
-  d = (unsigned long *)safemalloc(size);
-  c->data = d;
-  memcpy((void*)d,(const void*)ptr,size);
-
-  e = pipeQueue[module];
-  if(e == NULL)
-    {
-      pipeQueue[module] = c;
-      return;
-    }
-  while(e->next != NULL)
-    e = e->next;
-  e->next = c;
-}
-
 static void DeleteMessageQueueBuff(int module)
 {
-  struct queue_buff_struct *a;
+	mqueue_object_type *obj;
 
-  if(pipeQueue[module] == NULL)
-     return;
-  a = pipeQueue[module];
-  pipeQueue[module] = a->next;
-  if (a->data != NULL)
-    free(a->data);
-  free(a);
-  return;
+	if (fqueue_get_first(&pipeQueue[module], (void **)&obj) == 1)
+	{
+		/* remove from queue */
+		fqueue_remove_or_operate_from_front(
+			&pipeQueue[module], NULL, NULL);
+		/* we don't need to free the ocj->data here because it's in the
+		 * same malloced block as the obj itself. */
+		free(obj);
+	}
+
+	return;
 }
 
 void FlushMessageQueue(int module)
 {
-  char *dptr;
-  struct queue_buff_struct *d;
-  int a;
-  extern int errno;
+	mqueue_object_type *obj;
+	char *dptr;
+	int a;
+	extern int errno;
 
-  if((pipeOn[module] <= 0)||(pipeQueue[module] == NULL))
-    return;
-
-  while(pipeQueue[module] != NULL)
-    {
-      d = pipeQueue[module];
-      dptr = (char *)d->data;
-      while(d->done < d->size)
+	if (pipeOn[module] <= 0)
 	{
-	  a = write(writePipes[module],&dptr[d->done], d->size - d->done);
-	  if(a >=0)
-	    d->done += a;
-	  /* the write returns EWOULDBLOCK or EAGAIN if the pipe is full.
-	   * (This is non-blocking I/O). SunOS returns EWOULDBLOCK, OSF/1
-	   * returns EAGAIN under these conditions. Hopefully other OSes
-	   * return one of these values too. Solaris 2 doesn't seem to have
-	   * a man page for write(2) (!) */
-	  else if ((errno == EWOULDBLOCK)||(errno == EAGAIN)||(errno==EINTR))
-	    {
-	      return;
-	    }
-	  else
-	    {
-	      KillModule(module);
-	      return;
-	    }
+		return;
+	}
+
+	while (fqueue_get_first(&pipeQueue[module], (void **)&obj) == 1)
+	{
+		dptr = (char *)obj->data;
+		while (obj->done < obj->size)
+		{
+			a = write(writePipes[module], &dptr[obj->done],
+				  obj->size - obj->done);
+			if (a >=0)
+			{
+				obj->done += a;
+			}
+			/* the write returns EWOULDBLOCK or EAGAIN if the pipe
+			 * is full. (This is non-blocking I/O). SunOS returns
+			 * EWOULDBLOCK, OSF/1 returns EAGAIN under these
+			 * conditions. Hopefully other OSes return one of these
+			 * values too. Solaris 2 doesn't seem to have a man
+			 * page for write(2) (!) */
+			else if ((errno == EWOULDBLOCK) || (errno == EAGAIN) ||
+				 (errno==EINTR))
+			{
+				return;
+			}
+			else
+			{
+				KillModule(module);
+				return;
+			}
+		}
+		DeleteMessageQueueBuff(module);
 	}
-      DeleteMessageQueueBuff(module);
-    }
 }
 
 void FlushAllMessageQueues(void)
@@ -1492,16 +1623,14 @@ void FlushAllMessageQueues(void)
 }
 
 /* A queue of commands from the modules */
-typedef struct CommandQueue
+typedef struct
 {
-  struct CommandQueue *next;
   Window window;
   int module;
   char *command;
-} CommandQueue;
+} cqueue_object_type;
 
-static CommandQueue *CQstart = NULL;
-static CommandQueue *CQlast = NULL;
+static fqueue cqueue = FQUEUE_INIT;
 
 /***********************************************************************
  *
@@ -1512,57 +1641,49 @@ static CommandQueue *CQlast = NULL;
 
 static void AddToCommandQueue(Window window, int module, char *command)
 {
-  CommandQueue *new;
+	cqueue_object_type *new;
 
-  if (!command)
-    return;
+	if (!command)
+		return;
 
-  new = (CommandQueue *)safemalloc(sizeof(CommandQueue));
+	new = (cqueue_object_type *)safemalloc(sizeof(cqueue_object_type));
+	new->window = window;
+	new->module = module;
+	new->command = safestrdup(command);
+	DBUG("AddToCommandQueue", command);
+	fqueue_add_at_end(&cqueue, new);
 
-  new->window = window;
-  new->module = module;
-  new->command = safestrdup(command);
-  new->next = NULL;
-
-  DBUG("AddToCommandQueue", command);
-
-  if (CQlast)
-    CQlast->next = new;
-  CQlast = new;
-  if (!CQstart)
-    CQstart = CQlast;
+	return;
 }
 
 /***********************************************************************
  *
  *  Procedure:
- *	EmptyCommandQueue - runs command from the module command queue
+ *	ExecuteCommandQueue - runs command from the module command queue
  *	This may be called recursively if a module command runs a function
  *	that does a Wait, so it must be re-entrant
  *
  ************************************************************************/
 void ExecuteCommandQueue(void)
 {
-  CommandQueue *temp;
+	cqueue_object_type *obj;
 
-  while (CQstart)
-  {
-    /* remember the first command */
-    temp = CQstart;
-    /* remove it from the queue */
-    CQstart = CQstart->next;
-    /* fix the end pointer if the queue is now empty */
-    if (CQstart == NULL)
-      CQlast = NULL;
-    /* run the first command (this may add stuff to the queue) */
-    if (temp->command)
-    {
-      DBUG("EmptyCommandQueue", temp->command);
-      ExecuteModuleCommand(temp->window, temp->module, temp->command);
-      free(temp->command);
-    }
-    free(temp);
-  }
+	while (fqueue_get_first(&cqueue, (void **)&obj) == 1)
+	{
+		/* remove from queue */
+		fqueue_remove_or_operate_from_front(&cqueue, NULL, NULL);
+		/* execute and destroy */
+		if (obj->command)
+		{
+			DBUG("EmptyCommandQueue", temp->command);
+			ExecuteModuleCommand(
+				obj->window, obj->module, obj->command);
+			free(obj->command);
+		}
+		free(obj);
+	}
+
+	return;
 }
 
 void CMD_Send_WindowList(F_CMD_ARGS)
@@ -1578,8 +1699,8 @@ void CMD_Send_WindowList(F_CMD_ARGS)
       if(Scr.Hilite != NULL)
       {
 	SendPacket(*Module, M_FOCUS_CHANGE, 5,
-                   Scr.Hilite->w,
-                   Scr.Hilite->frame,
+                   FW_W(Scr.Hilite),
+                   FW_W_FRAME(Scr.Hilite),
 		   (unsigned long)True,
 		   Scr.Hilite->hicolors.fore,
 		   Scr.Hilite->hicolors.back);
@@ -1597,42 +1718,45 @@ void CMD_Send_WindowList(F_CMD_ARGS)
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
 	{
 	  SendConfig(*Module,M_CONFIGURE_WINDOW,t);
-	  SendName(*Module,M_WINDOW_NAME,t->w,t->frame,
-		   (unsigned long)t,t->name);
-	  SendName(*Module,M_ICON_NAME,t->w,t->frame,
-		   (unsigned long)t,t->icon_name);
-
+	  SendName(*Module,M_WINDOW_NAME,FW_W(t),FW_W_FRAME(t),
+		   (unsigned long)t,t->name.name);
+	  SendName(*Module,M_ICON_NAME,FW_W(t),FW_W_FRAME(t),
+		   (unsigned long)t,t->icon_name.name);
+	  SendName(*Module,M_VISIBLE_NAME,FW_W(t),FW_W_FRAME(t),
+		   (unsigned long)t,t->visible_name);
+	  SendName(*Module,MX_VISIBLE_ICON_NAME,FW_W(t),FW_W_FRAME(t),
+		   (unsigned long)t,t->visible_icon_name);
           if (t->icon_bitmap_file != NULL
               && t->icon_bitmap_file != Scr.DefaultIcon)
-            SendName(*Module,M_ICON_FILE,t->w,t->frame,
+            SendName(*Module,M_ICON_FILE,FW_W(t),FW_W_FRAME(t),
                      (unsigned long)t,t->icon_bitmap_file);
 
-	  SendName(*Module,M_RES_CLASS,t->w,t->frame,
+	  SendName(*Module,M_RES_CLASS,FW_W(t),FW_W_FRAME(t),
 		   (unsigned long)t,t->class.res_class);
-	  SendName(*Module,M_RES_NAME,t->w,t->frame,
+	  SendName(*Module,M_RES_NAME,FW_W(t),FW_W_FRAME(t),
 		   (unsigned long)t,t->class.res_name);
 
 	  if((IS_ICONIFIED(t))&&(!IS_ICON_UNMAPPED(t)))
-	    SendPacket(*Module, M_ICONIFY, 7, t->w, t->frame,
-		       (unsigned long)t,
-		       t->icon_g.x, t->icon_g.y,
-		       t->icon_g.width, t->icon_g.height+t->icon_p_height);
+	  {
+	    rectangle r;
+	    Bool rc;
+
+	    rc = get_visible_icon_geometry(t, &r);
+	    if (rc == True)
+	    {
+	      SendPacket(*Module, M_ICONIFY, 7, FW_W(t), FW_W_FRAME(t),
+			 (unsigned long)t, r.x, r.y, r.width, r.height);
+	    }
+	  }
 
 	  if((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
-	    SendPacket(*Module, M_ICONIFY, 7, t->w, t->frame,
+	    SendPacket(*Module, M_ICONIFY, 7, FW_W(t), FW_W_FRAME(t),
 		       (unsigned long)t,
                        0, 0, 0, 0);
-#ifdef MINI_ICONS
-	  if (t->mini_icon != NULL)
-            SendMiniIcon(*Module, M_MINI_ICON,
-                         t->w, t->frame, (unsigned long)t,
-                         t->mini_icon->width,
-                         t->mini_icon->height,
-                         t->mini_icon->depth,
-                         t->mini_icon->picture,
-                         t->mini_icon->mask,
-                         t->mini_pixmap_file);
-#endif
+	  if (FMiniIconsSupported && t->mini_icon != NULL)
+            SendFvwmPicture(*Module, M_MINI_ICON,
+			    FW_W(t), FW_W_FRAME(t), (unsigned long)t,
+			    t->mini_icon, t->mini_pixmap_file);
 	}
 
       if(Scr.Hilite == NULL)
@@ -1645,8 +1769,8 @@ void CMD_Send_WindowList(F_CMD_ARGS)
       else
       {
 	  BroadcastPacket(M_FOCUS_CHANGE, 5,
-                          Scr.Hilite->w,
-                          Scr.Hilite->frame,
+                          FW_W(Scr.Hilite),
+                          FW_W(Scr.Hilite),
                           (unsigned long)True,
                           Scr.Hilite->hicolors.fore,
                           Scr.Hilite->hicolors.back);
@@ -1658,26 +1782,30 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 
 void CMD_set_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  PipeMask[*Module] = (unsigned long)val;
+  /*GetIntegerArguments(action, NULL, &val, 1);*/
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&PipeMask[*Module], (unsigned long)val);
 }
 
 void CMD_set_sync_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  SyncMask[*Module] = (unsigned long)val;
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&SyncMask[*Module], (unsigned long)val);
 }
 
 void CMD_set_nograb_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  NoGrabMask[*Module] = (unsigned long)val;
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&NoGrabMask[*Module], (unsigned long)val);
 }
 
 /*
