@@ -17,22 +17,12 @@
 #define FVWM_MODULE_INTERFACE_H
 
 #include "Module.h"
-
-
-struct queue_buff_struct
-{
-  struct queue_buff_struct *next;
-  unsigned long *data;
-  int size;
-  int done;
-};
+#include "libs/queue.h"
 
 extern int npipes;
 extern int *readPipes;
 extern int *writePipes;
-extern struct queue_buff_struct **pipeQueue;
-
-
+extern fqueue *pipeQueue;
 
 /*
  * MAX_MASK is used to initialize the pipeMask array.  In a few places
@@ -52,26 +42,40 @@ extern struct queue_buff_struct **pipeQueue;
  * module.c, modconf.c and event.c.  dje 10/2/98
  */
 /* this is a bit long winded to allow MAX_MESSAGE to be 32 and not get an
- * integer overflow with (1 << MAX_MESSAGES) */
-#define MAX_MASK             ((((1<<(MAX_MESSAGES-1))-1)+(1<<(MAX_MESSAGES-1)))\
-                              &~(M_NOTUSED + M_SENDCONFIG))
+ * integer overflow with (1 << MAX_MESSAGES) and even with
+ * (1<<(MAX_MESSAGES-1)) - 1 */
+#define DEFAULT_MASK   (MAX_MSG_MASK & ~(M_SENDCONFIG))
+#define DEFAULT_XMASK  (DEFAULT_XMSG_MASK)
 
+/*
+ * Returns zero if the msg is not selected by the mask. Takes care of normal
+ * and extended messages.
+ */
+#define IS_MESSAGE_IN_MASK(mask, msg) \
+  (((msg) & M_EXTENDED_MSG) ? ((mask)->m2 & (msg)) : ((mask)->m1 & (msg)))
+
+/*
+ * Returns non zero if one of the specified messages is selected for the module
+ */
+/* please don't use msg_masks_type and PipeMask outside of module_interface.c.
+ * They are only global to allow to access the IS_MESSAGE_SELECTED macro without
+ * having to call a function. */
+typedef struct
+{
+	unsigned long m1;
+	unsigned long m2;
+} msg_masks_type;
+extern msg_masks_type *PipeMask;
+#define IS_MESSAGE_SELECTED(module, msg_mask) \
+  IS_MESSAGE_IN_MASK(&PipeMask[(module)], (msg_mask))
 
 /*
- * M_NOTUSED is not used. It was M_LOCKONSEND which is not more needed: it
- * has been replaced by a separated mask which defines on which messages
- * the fvwm-to-module communication need to be lock. olicha Nov 13 1999.
- *
  * M_SENDCONFIG for   modules to tell  fvwm that  they  want to  see each
  * module configuration command as   it is entered.  Causes  modconf.c to
  * look at each active module, find  the ones that sent M_SENDCONFIG, and
  * send a copy of the command in an M_CONFIG_INFO command.
  */
 
-void module_zapper(F_CMD_ARGS);
-void executeModule(F_CMD_ARGS);
-int executeModuleDesperate(F_CMD_ARGS);
-void executeModuleSync(F_CMD_ARGS);
 void initModules(void);
 void ExecuteModuleCommand(Window w, int channel, char *command);
 Bool HandleModuleInput(Window w, int channel, char *expect, Bool queue);
@@ -81,27 +85,29 @@ void BroadcastPacket(unsigned long event_type, unsigned long num_datum, ...);
 void BroadcastConfig(unsigned long event_type, const FvwmWindow *t);
 void BroadcastName(unsigned long event_type, unsigned long data1,
 		   unsigned long data2, unsigned long data3, const char *name);
-void BroadcastMiniIcon(unsigned long event_type,
-		       unsigned long data1, unsigned long data2,
-		       unsigned long data3, unsigned long data4,
-		       unsigned long data5, unsigned long data6,
-		       unsigned long data7, unsigned long data8,
-		       const char *name);
+void BroadcastWindowIconNames(FvwmWindow *t, Bool window, Bool icon);
+void BroadcastFvwmPicture(unsigned long event_type,
+			  unsigned long data1, unsigned long data2,
+			  unsigned long data3, FvwmPicture *picture, char *name);
+void BroadcastPropertyChange(unsigned long argument, unsigned long data1,
+			     unsigned long data2, char *string);
 void BroadcastColorset(int n);
+void BroadcastConfigInfoString(char *string);
+void broadcast_xinerama_state(void);
+void broadcast_ignore_modifiers(void);
+
 void SendPacket(int channel, unsigned long event_type,
                 unsigned long num_datum, ...);
 void SendConfig(int Module, unsigned long event_type, const FvwmWindow *t);
 void SendName(int channel, unsigned long event_type, unsigned long data1,
 	      unsigned long data2, unsigned long data3, const char *name);
-void SendStrToModule(F_CMD_ARGS);
-void send_list_func(F_CMD_ARGS);
 void FlushAllMessageQueues(void);
 void FlushMessageQueue(int Module);
 void ExecuteCommandQueue(void);
 void PositiveWrite(int module, unsigned long *ptr, int size);
 RETSIGTYPE DeadPipe(int nonsense);
-void set_mask_function(F_CMD_ARGS);
-void setSyncMaskFunc(F_CMD_ARGS);
-void setNoGrabMaskFunc(F_CMD_ARGS);
+char *skipModuleAliasToken(const char *string);
+int executeModuleDesperate(F_CMD_ARGS);
+int is_message_selected(int module, unsigned long msg_mask);
 
 #endif /* MODULE_H */
