@@ -16,18 +16,18 @@
 #ifndef _MOVE_RESIZE_
 #define _MOVE_RESIZE_
 
-void MoveOutline(int x, int  y, int  width, int height);
+#include "menus.h"
+
+void switch_move_resize_grid(Bool state);
 void AnimatedMoveOfWindow(
   Window w,int startX,int startY,int endX, int endY,Bool fWarpPointerToo,
-  int cusDelay, float *ppctMovement);
+  int cusDelay, float *ppctMovement, MenuRoot *menu_root);
 void AnimatedMoveFvwmWindow(
-  FvwmWindow *tmp_win, Window w, int startX, int startY, int endX, int endY,
+  FvwmWindow *fw, Window w, int startX, int startY, int endX, int endY,
   Bool fWarpPointerToo, int cmsDelay, float *ppctMovement);
-Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
+Bool moveLoop(FvwmWindow *fw, int XOffset, int YOffset, int Width,
 	      int Height, int *FinalX, int *FinalY,Bool do_move_opaque);
-void move_window_doit(
-  XEvent *eventp,Window w,FvwmWindow *tmp_win, unsigned long context,
-  char *action,int* Module, Bool do_animate, Bool do_move_to_page);
 void handle_stick(F_CMD_ARGS, int toggle);
+void resize_geometry_window(void);
 
 #endif /* _MOVE_RESIZE_ */
