@@ -30,6 +30,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -46,6 +47,7 @@
 #include "virtual.h"
 #include "stack.h"
 #include "add_window.h"
+#include "ewmh.h"
 
 #ifndef MIN
 #define MIN(A,B) ((A)<(B)? (A):(B))
@@ -60,10 +62,10 @@ static int get_next_x(
 static int get_next_y(
   FvwmWindow *t, rectangle *screen_g, int y, int pdeltay, int use_percent);
 static float test_fit(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int x11, int y11, float aoimin, int pdeltax, int pdeltay, int use_percent);
 static void CleverPlacement(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int *x, int *y, int pdeltax, int pdeltay, int use_percent);
 
 static int SmartPlacement(
@@ -83,10 +85,11 @@ static int SmartPlacement(
   int tx = 0;
   int ty = 0;
   int th = 0;
-  FvwmWindow *test_window;
+  FvwmWindow *test_fw;
   int stickyx;
   int stickyy;
-  Bool do_test;
+  rectangle g;
+  Bool rc;
 
   test_x = PageLeft;
   test_y = PageTop;
@@ -99,15 +102,16 @@ static int SmartPlacement(
     while (test_x + temp_w < PageRight && !loc_ok)
     {
       loc_ok = True;
-      test_window = Scr.FvwmRoot.next;
-      for ( ; test_window != NULL && loc_ok; test_window = test_window->next)
+      test_fw = Scr.FvwmRoot.next;
+      for ( ; test_fw != NULL && loc_ok; test_fw = test_fw->next)
       {
-	if (t == test_window)
+	if (t == test_fw || IS_EWMH_DESKTOP(FW_W(test_fw)))
 	  continue;
+
         /*  RBW - account for sticky windows...  */
-        if (test_window->Desk == t->Desk || IS_STICKY(test_window))
+        if (test_fw->Desk == t->Desk || IS_STICKY(test_fw))
         {
-          if (IS_STICKY(test_window))
+          if (IS_STICKY(test_fw))
 	  {
 	    stickyx = pdeltax;
 	    stickyy = pdeltay;
@@ -117,27 +121,14 @@ static int SmartPlacement(
 	    stickyx = 0;
 	    stickyy = 0;
 	  }
-	  do_test = False;
-          if(PLACEMENT_AVOID_ICON != 0 && IS_ICONIFIED(test_window) &&
-	     !IS_ICON_UNMAPPED(test_window) && test_window->icon_w)
-          {
-            tw=test_window->icon_p_width;
-            th=test_window->icon_p_height+
-              test_window->icon_g.height;
-            tx = test_window->icon_g.x - stickyx;
-            ty = test_window->icon_g.y - stickyy;
-	    do_test = True;
-          }
-          else if(!IS_ICONIFIED(test_window))
-          {
-            tw = test_window->frame_g.width;
-            th = test_window->frame_g.height;
-            tx = test_window->frame_g.x - stickyx;
-            ty = test_window->frame_g.y - stickyy;
-	    do_test = True;
-          }
-	  if (do_test)
+	  rc = get_visible_window_or_icon_geometry(test_fw, &g);
+	  if (rc == True &&
+	      (PLACEMENT_AVOID_ICON == 0 || !IS_ICONIFIED(test_fw)))
 	  {
+	    tx = g.x - stickyx;
+	    ty = g.y - stickyy;
+	    tw = g.width;
+	    th = g.height;
             if (tx < test_x + width  && test_x < tx + tw &&
 		ty < test_y + height && test_y < ty + th)
             {
@@ -146,7 +137,7 @@ static int SmartPlacement(
               test_x = tx + tw - 1;
             }
 	  }
-        } /* if (test_window->Desk == t->Desk || IS_STICKY(test_window)) */
+        } /* if (test_fw->Desk == t->Desk || IS_STICKY(test_fw)) */
       } /* for */
       if (!loc_ok)
 	test_x +=1;
@@ -171,7 +162,7 @@ static int SmartPlacement(
  * interference, fine.  Otherwise, it places it so that the area of of
  * interference between the new window and the other windows is minimized */
 static void CleverPlacement(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int *x, int *y, int pdeltax, int pdeltay, int use_percent)
 {
   int test_x;
@@ -187,7 +178,7 @@ static void CleverPlacement(
   test_x = PageLeft;
   test_y = PageTop;
   aoi = test_fit(
-    t, screen_g, test_x, test_y, -1, pdeltax, pdeltay, use_percent);
+    t, sflags, screen_g, test_x, test_y, -1, pdeltax, pdeltay, use_percent);
   aoimin = aoi;
   xbest = test_x;
   ybest = test_y;
@@ -207,10 +198,10 @@ static void CleverPlacement(
       test_y = get_next_y(t, screen_g, test_y, pdeltay, use_percent);
     }
     aoi = test_fit(
-      t, screen_g, test_x, test_y, aoimin, pdeltax, pdeltay, use_percent);
-    /* I've added +0.00001 because whith my machine the < test fail with
+      t, sflags, screen_g, test_x,test_y, aoimin, pdeltax, pdeltay, use_percent);
+    /* I've added +0.0001 because whith my machine the < test fail with
      * certain *equal* float numbers! */
-    if (aoi >= 0 && aoi + 0.00001 < aoimin)
+    if (aoi >= 0 && aoi + 0.0001 < aoimin)
     {
       xbest = test_x;
       ybest = test_y;
@@ -223,6 +214,7 @@ static void CleverPlacement(
   return;
 }
 
+#define GET_NEXT_STEP 5
 static int get_next_x(
   FvwmWindow *t, rectangle *screen_g, int x, int y, int pdeltax, int pdeltay,
   int use_percent)
@@ -232,19 +224,37 @@ static int get_next_x(
   int xtest;
   int PageLeft = screen_g->x - pdeltax;
   int PageRight = PageLeft + screen_g->width;
-  int minx_inc = screen_g->width / 20;
   int stickyx;
   int stickyy;
+  int start,i;
+  int win_left;
+  rectangle g;
+  Bool rc;
+
+  if (use_percent)
+    start = 0;
+  else
+    start = GET_NEXT_STEP;
 
   /* Test window at far right of screen */
   xnew = PageRight;
   xtest = PageRight - t->frame_g.width;
+  if (xtest > x)
+    xnew = MIN(xnew, xtest);
+  /* test the borders of the working area */
+  xtest = PageLeft + Scr.Desktops->ewmh_working_area.x;
+  if (xtest > x)
+    xnew = MIN(xnew, xtest);
+  xtest = PageLeft +
+    (Scr.Desktops->ewmh_working_area.x + Scr.Desktops->ewmh_working_area.width)
+    - t->frame_g.width;
   if (xtest > x)
     xnew = MIN(xnew, xtest);
   /* Test the values of the right edges of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)) ||
+	IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
     if (IS_STICKY(testw))
@@ -260,34 +270,50 @@ static int get_next_x(
 
     if (IS_ICONIFIED(testw))
     {
-      if (y < testw->icon_p_height + testw->icon_g.height + testw->icon_g.y
-	  - stickyy &&
-	  testw->icon_g.y - stickyy < t->frame_g.height + y)
+      rc = get_visible_icon_geometry(testw, &g);
+      if (rc == True &&
+	  y < g.y + g.height - stickyy && g.y - stickyy < t->frame_g.height + y)
       {
-        xtest = PageLeft + testw->icon_p_width + testw->icon_g.x - stickyx;
-        if (xtest > x)
-          xnew = MIN(xnew, xtest);
-        xtest = PageLeft + testw->icon_g.x - stickyx - t->frame_g.width;
-        if (xtest > x)
-          xnew = MIN(xnew, xtest);
+	win_left = PageLeft + g.x - stickyx - t->frame_g.width;
+	for (i = start; i <= GET_NEXT_STEP; i++)
+	{
+	  xtest = win_left + g.width * (GET_NEXT_STEP - i) / GET_NEXT_STEP;
+	  if (xtest > x)
+	    xnew = MIN(xnew, xtest);
+	}
+	win_left = PageLeft + g.x - stickyx;
+	for(i=start; i <= GET_NEXT_STEP; i++)
+	{
+	  xtest = (win_left) + g.width * i / GET_NEXT_STEP;
+	  if (xtest > x)
+	    xnew = MIN(xnew, xtest);
+	}
       }
     }
     else if (y < testw->frame_g.height + testw->frame_g.y - stickyy &&
 	     testw->frame_g.y - stickyy < t->frame_g.height + y)
     {
-      xtest = PageLeft + testw->frame_g.width + testw->frame_g.x - stickyx;
-      if (xtest > x)
-        xnew = MIN(xnew, xtest);
-      xtest = PageLeft + testw->frame_g.x - stickyx - t->frame_g.width;
-      if (xtest > x)
-        xnew = MIN(xnew, xtest);
+      win_left = PageLeft + testw->frame_g.x - stickyx - t->frame_g.width;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	xtest = (win_left) + (testw->frame_g.width) *
+	  (GET_NEXT_STEP - i)/GET_NEXT_STEP;
+	if (xtest > x)
+	  xnew = MIN(xnew, xtest);
+      }
+      win_left = PageLeft + testw->frame_g.x - stickyx;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	xtest = (win_left) + (testw->frame_g.width) * i/GET_NEXT_STEP;
+	if (xtest > x)
+	  xnew = MIN(xnew, xtest);
+      }
     }
   }
-  if (use_percent)
-    xnew = MIN(xnew, x + minx_inc);
 
   return xnew;
 }
+
 static int get_next_y(
   FvwmWindow *t, rectangle *screen_g, int y, int pdeltay, int use_percent)
 {
@@ -295,18 +321,36 @@ static int get_next_y(
   int ynew;
   int ytest;
   int PageBottom = screen_g->y + screen_g->height - pdeltay;
-  int miny_inc = (screen_g->height / 20);
   int stickyy;
+  int win_top;
+  int start;
+  int i;
+  rectangle g;
+
+  if (use_percent)
+    start = 0;
+  else
+    start = GET_NEXT_STEP;
 
   /* Test window at far bottom of screen */
   ynew = PageBottom;
   ytest = PageBottom - t->frame_g.height;
+  if (ytest > y)
+    ynew = MIN(ynew, ytest);
+  /* test the borders of the working area */
+  ytest = screen_g->y + Scr.Desktops->ewmh_working_area.y - pdeltay;
+  if (ytest > y)
+    ynew = MIN(ynew, ytest);
+  ytest = screen_g->y +
+    (Scr.Desktops->ewmh_working_area.y + Scr.Desktops->ewmh_working_area.height)
+    - t->frame_g.height;
   if (ytest > y)
     ynew = MIN(ynew, ytest);
   /* Test the values of the bottom edge of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+	|| IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
     if (IS_STICKY(testw))
@@ -320,32 +364,47 @@ static int get_next_y(
 
     if (IS_ICONIFIED(testw))
     {
-      ytest =
-	testw->icon_p_height + testw->icon_g.height + testw->icon_g.y - stickyy;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
-      ytest = testw->icon_g.y - stickyy - t->frame_g.height;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
+      get_visible_icon_geometry(testw, &g);
+      win_top = g.y - stickyy;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + g.height * i / GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
+      win_top = g.y - stickyy - t->frame_g.height;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + g.height * (GET_NEXT_STEP - i) / GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
     }
     else
     {
-      ytest = testw->frame_g.height + testw->frame_g.y - stickyy;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
-      ytest = testw->frame_g.y - stickyy - t->frame_g.height;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
+      win_top = testw->frame_g.y - stickyy;;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = (win_top) + (testw->frame_g.height) * i/GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
+      win_top = testw->frame_g.y - stickyy - t->frame_g.height;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + (testw->frame_g.height) *
+	  (GET_NEXT_STEP - i)/GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
     }
   }
-  if (use_percent)
-    ynew = MIN(ynew,y+miny_inc);
 
   return ynew;
 }
 
 static float test_fit(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int x11, int y11, float aoimin, int pdeltax, int pdeltay, int use_percent)
 {
   FvwmWindow *testw;
@@ -363,11 +422,13 @@ static float test_fit(
   /* area of interference */
   float aoi = 0;
   float anew;
-  float cover_factor = 1;
+  float cover_factor = 0;
   float avoidance_factor;
   int PageRight  = screen_g->x + screen_g->width - pdeltax;
   int PageBottom = screen_g->y + screen_g->height - pdeltay;
   int stickyx, stickyy;
+  rectangle g;
+  Bool rc;
 
   x12 = x11 + t->frame_g.width;
   y12 = y11 + t->frame_g.height;
@@ -378,7 +439,8 @@ static float test_fit(
     return -2;
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+	|| IS_EWMH_DESKTOP(FW_W(testw)))
        continue;
 
     if (IS_STICKY(testw))
@@ -392,22 +454,11 @@ static float test_fit(
       stickyy = 0;
     }
 
-    if (IS_ICONIFIED(testw))
-    {
-       if (testw->icon_w == None || IS_ICON_UNMAPPED(testw))
-	  continue;
-       x21 = testw->icon_g.x - stickyx;
-       y21 = testw->icon_g.y - stickyy;
-       x22 = x21 + testw->icon_p_width;
-       y22 = y21 + testw->icon_p_height + testw->icon_g.height;
-    }
-    else
-    {
-       x21 = testw->frame_g.x - stickyx;
-       y21 = testw->frame_g.y - stickyy;
-       x22 = x21 + testw->frame_g.width;
-       y22 = y21 + testw->frame_g.height;
-    }
+    rc = get_visible_window_or_icon_geometry(testw, &g);
+    x21 = g.x - stickyx;
+    y21 = g.y - stickyy;
+    x22 = x21 + g.width;
+    y22 = y21 + g.height;
     if (x11 < x22 && x12 > x21 &&
 	y11 < y22 && y12 > y21)
     {
@@ -418,44 +469,59 @@ static float test_fit(
       yb = MIN(y12, y22);
       anew = (xr - xl) * (yb - yt);
       if (IS_ICONIFIED(testw))
-        avoidance_factor = PLACEMENT_AVOID_ICON;
+        avoidance_factor = ICON_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) > 0)
-        avoidance_factor = PLACEMENT_AVOID_ONTOP;
+        avoidance_factor = ONTOP_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) < 0)
-        avoidance_factor = PLACEMENT_AVOID_BELOW;
+        avoidance_factor = BELOW_PLACEMENT_PENALTY(testw);
       else if(IS_STICKY(testw))
-        avoidance_factor = PLACEMENT_AVOID_STICKY;
+        avoidance_factor = STICKY_PLACEMENT_PENALTY(testw);
       else
-        avoidance_factor = 1.0;
+        avoidance_factor = NORMAL_PLACEMENT_PENALTY(testw);
 
       if (use_percent)
       {
-	/* normalisation */
+	cover_factor = 0;
 	if ((x22 - x21) * (y22 - y21) != 0 && (x12 - x11) * (y12 - y11) != 0)
 	{
 	  anew = 100 * MAX(anew / ((x22 - x21) * (y22 - y21)),
-			   anew / ((x12 - x11) * (y12 - y11)));
+			      anew / ((x12 - x11) * (y12 - y11)));
 	  if (anew >= 99)
-	    cover_factor = PLACEMENT_AVOID_COVER_99;
-	  else if (anew >= 95)
-	    cover_factor = PLACEMENT_AVOID_COVER_95;
-	  else if (anew >= 85)
-	    cover_factor = PLACEMENT_AVOID_COVER_85;
-	  else if (anew >= 75)
-	    cover_factor = PLACEMENT_AVOID_COVER_75;
+	    cover_factor = PERCENTAGE_99_PENALTY(testw);
+	  else if (anew > 94)
+	    cover_factor = PERCENTAGE_95_PENALTY(testw);
+	  else if (anew > 84)
+	    cover_factor = PERCENTAGE_85_PENALTY(testw);
+	  else if (anew > 74)
+	    cover_factor = PERCENTAGE_75_PENALTY(testw);
 	}
-	if (avoidance_factor>1)
-	  avoidance_factor =
-	    avoidance_factor + (cover_factor > 1) ? cover_factor : 0;
-	else
-	  avoidance_factor = cover_factor;
+	avoidance_factor += (avoidance_factor >= 1)? cover_factor : 0;
       }
+
+      if (SEWMH_PLACEMENT_MODE(sflags) == EWMH_USE_DYNAMIC_WORKING_AREA &&
+	  !DO_EWMH_IGNORE_STRUT_HINTS(testw) &&
+	  (testw->dyn_strut.left > 0 || testw->dyn_strut.right > 0 ||
+	   testw->dyn_strut.top > 0 || testw->dyn_strut.bottom > 0))
+      {
+	/* if we intersect a window which reserves space */
+	avoidance_factor += (avoidance_factor >= 1)?
+	  EWMH_STRUT_PLACEMENT_PENALTY(t) : 0;
+      }
+
       anew *= avoidance_factor;
       aoi += anew;
       if (aoi > aoimin && aoimin != -1)
+      {
         return aoi;
+      }
     }
   }
+  /* now handle the working area */
+  if (SEWMH_PLACEMENT_MODE(sflags) == EWMH_USE_WORKING_AREA)
+  {
+    aoi += EWMH_STRUT_PLACEMENT_PENALTY(t) *
+      EWMH_GetStrutIntersection(x11, y11, x12, y12, use_percent);
+  }
   return aoi;
 }
 
@@ -472,8 +538,9 @@ static float test_fit(
  *
  **************************************************************************/
 Bool PlaceWindow(
-  FvwmWindow *tmp_win, style_flags *sflags,
-  int Desk, int PageX, int PageY, int XineramaScreen, int mode)
+	FvwmWindow *fw, style_flags *sflags, rectangle *attr_g,
+	int Desk, int PageX, int PageY, int XineramaScreen, int mode,
+	initial_window_options_type *win_opts)
 {
   FvwmWindow *t;
   int xl = -1;
@@ -489,6 +556,7 @@ Bool PlaceWindow(
   int PageRight;
   int PageBottom;
   rectangle screen_g;
+  size_borders b;
   Bool rc = False;
   struct
   {
@@ -499,7 +567,6 @@ Bool PlaceWindow(
     unsigned do_not_use_wm_placement : 1;
   } flags;
   extern Bool Restarting;
-  extern Bool PPosOverride;
 
   memset(&flags, 0, sizeof(flags));
 
@@ -523,7 +590,8 @@ Bool PlaceWindow(
      * Honor the flag unless...
      * it's a restart or recapture, and that option's disallowed...
      */
-    if (PPosOverride && (Restarting || (Scr.flags.windows_captured)) &&
+    if (win_opts->flags.do_override_ppos &&
+	(Restarting || (Scr.flags.windows_captured)) &&
 	!SRECAPTURE_HONORS_STARTS_ON_PAGE(sflags))
     {
       flags.do_honor_starts_on_page = False;
@@ -532,7 +600,8 @@ Bool PlaceWindow(
     /*
      * it's a cold start window capture, and that's disallowed...
      */
-    if (PPosOverride && (!Restarting && !(Scr.flags.windows_captured)) &&
+    if (win_opts->flags.do_override_ppos &&
+	(!Restarting && !(Scr.flags.windows_captured)) &&
 	!SCAPTURE_HONORS_STARTS_ON_PAGE(sflags))
     {
       flags.do_honor_starts_on_page = False;
@@ -541,8 +610,8 @@ Bool PlaceWindow(
     /*
      * it's ActivePlacement and SkipMapping, and that's disallowed.
      */
-    if (!PPosOverride &&
-	(DO_NOT_SHOW_ON_MAP(tmp_win) &&
+    if (!win_opts->flags.do_override_ppos &&
+	(DO_NOT_SHOW_ON_MAP(fw) &&
 	 (SPLACEMENT_MODE(sflags) == PLACE_MANUAL ||
 	  SPLACEMENT_MODE(sflags) == PLACE_MANUAL_B ||
 	  SPLACEMENT_MODE(sflags) == PLACE_TILEMANUAL) &&
@@ -582,6 +651,15 @@ Bool PlaceWindow(
       NULL, FSCREEN_CURRENT,
       &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height);
   }
+
+  if (SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAPPERCENT &&
+      SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAP)
+  {
+    EWMH_GetWorkAreaIntersection(fw,
+      &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height,
+      SEWMH_PLACEMENT_MODE(sflags));
+  }
+
   PageLeft   = screen_g.x - pdeltax;
   PageTop    = screen_g.y - pdeltay;
   PageRight  = PageLeft + screen_g.width;
@@ -589,47 +667,50 @@ Bool PlaceWindow(
   yt = PageTop;
 
   /* Don't alter the existing desk location during Capture/Recapture.  */
-  if (!PPosOverride)
+  if (!win_opts->flags.do_override_ppos)
   {
-    tmp_win->Desk = Scr.CurrentDesk;
+    fw->Desk = Scr.CurrentDesk;
   }
   if (SIS_STICKY(*sflags))
-    tmp_win->Desk = Scr.CurrentDesk;
+    fw->Desk = Scr.CurrentDesk;
   else if (SUSE_START_ON_DESK(sflags) && Desk && flags.do_honor_starts_on_page)
-    tmp_win->Desk = (Desk > -1) ? Desk - 1 : Desk;
+    fw->Desk = (Desk > -1) ? Desk - 1 : Desk;
   else
   {
-    if ((DO_USE_WINDOW_GROUP_HINT(tmp_win)) &&
-	(tmp_win->wmhints) && (tmp_win->wmhints->flags & WindowGroupHint)&&
-	(tmp_win->wmhints->window_group != None) &&
-	(tmp_win->wmhints->window_group != Scr.Root))
+    if ((DO_USE_WINDOW_GROUP_HINT(fw)) &&
+	(fw->wmhints) && (fw->wmhints->flags & WindowGroupHint)&&
+	(fw->wmhints->window_group != None) &&
+	(fw->wmhints->window_group != Scr.Root))
     {
       /* Try to find the group leader or another window in the group */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if (t->w == tmp_win->wmhints->window_group)
+	if (FW_W(t) == fw->wmhints->window_group)
 	{
 	  /* found the group leader, break out */
-	  tmp_win->Desk = t->Desk;
+	  fw->Desk = t->Desk;
 	  break;
 	}
 	else if (t->wmhints && (t->wmhints->flags & WindowGroupHint) &&
-		 (t->wmhints->window_group == tmp_win->wmhints->window_group))
+		 (t->wmhints->window_group == fw->wmhints->window_group))
 	{
 	  /* found a window from the same group, but keep looking for the group
 	   * leader */
-	  tmp_win->Desk = t->Desk;
+	  fw->Desk = t->Desk;
 	}
       }
     }
-    if ((IS_TRANSIENT(tmp_win))&&(tmp_win->transientfor!=None)&&
-	(tmp_win->transientfor != Scr.Root))
+    if ((IS_TRANSIENT(fw))&&(FW_W_TRANSIENTFOR(fw)!=None)&&
+	(FW_W_TRANSIENTFOR(fw) != Scr.Root))
     {
       /* Try to find the parent's desktop */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if(t->w == tmp_win->transientfor)
-	  tmp_win->Desk = t->Desk;
+	if (FW_W(t) == FW_W_TRANSIENTFOR(fw))
+        {
+	  fw->Desk = t->Desk;
+          break;
+        }
       }
     }
 
@@ -643,12 +724,12 @@ Bool PlaceWindow(
 
       if (
 	XGetWindowProperty(
-	  dpy, tmp_win->w, _XA_WM_DESKTOP, 0L, 1L, True, _XA_WM_DESKTOP,
+	  dpy, FW_W(fw), _XA_WM_DESKTOP, 0L, 1L, True, _XA_WM_DESKTOP,
 	  &atype, &aformat, &nitems, &bytes_remain, &prop) == Success)
       {
 	if (prop != NULL)
 	{
-	  tmp_win->Desk = *(unsigned long *)prop;
+	  fw->Desk = *(unsigned long *)prop;
 	  XFree(prop);
 	}
       }
@@ -658,9 +739,9 @@ Bool PlaceWindow(
   /* I think it would be good to switch to the selected desk
    * whenever a new window pops up, except during initialization */
   /*  RBW - 11/02/1998  --  I dont. */
-  if ((!PPosOverride)&&(!DO_NOT_SHOW_ON_MAP(tmp_win)))
+  if ((!win_opts->flags.do_override_ppos)&&(!DO_NOT_SHOW_ON_MAP(fw)))
   {
-    goto_desk(tmp_win->Desk);
+    goto_desk(fw->Desk);
   }
 
   /* Don't move viewport if SkipMapping, or if recapturing the window,
@@ -675,7 +756,7 @@ Bool PlaceWindow(
       py = PageY - 1;
       px *= Scr.MyDisplayWidth;
       py *= Scr.MyDisplayHeight;
-      if ( (!PPosOverride) && (!DO_NOT_SHOW_ON_MAP(tmp_win)))
+      if ( (!win_opts->flags.do_override_ppos) && (!DO_NOT_SHOW_ON_MAP(fw)))
       {
 	MoveViewport(px,py,True);
       }
@@ -715,7 +796,7 @@ Bool PlaceWindow(
     Bool has_ppos = False;
     Bool has_uspos = False;
 
-    if (IS_TRANSIENT(tmp_win))
+    if (IS_TRANSIENT(fw))
     {
       override_ppos = SUSE_NO_TRANSIENT_PPOSITION(sflags);
       override_uspos = SUSE_NO_TRANSIENT_USPOSITION(sflags);
@@ -725,11 +806,11 @@ Bool PlaceWindow(
       override_ppos = SUSE_NO_PPOSITION(sflags);
       override_uspos = SUSE_NO_USPOSITION(sflags);
     }
-    if ((tmp_win->hints.flags & PPosition) && !override_ppos)
+    if ((fw->hints.flags & PPosition) && !override_ppos)
     {
       has_ppos = True;
     }
-    if ((tmp_win->hints.flags & USPosition) && !override_uspos)
+    if ((fw->hints.flags & USPosition) && !override_uspos)
     {
       has_uspos = True;
     }
@@ -742,13 +823,13 @@ Bool PlaceWindow(
     {
       flags.do_not_use_wm_placement = True;
     }
-    else if (PPosOverride)
+    else if (win_opts->flags.do_override_ppos)
     {
       flags.do_not_use_wm_placement = True;
     }
     else if (!flags.do_honor_starts_on_page &&
-	     tmp_win->wmhints && (tmp_win->wmhints->flags & StateHint) &&
-	     tmp_win->wmhints->initial_state == IconicState)
+	     fw->wmhints && (fw->wmhints->flags & StateHint) &&
+	     fw->wmhints->initial_state == IconicState)
     {
       flags.do_forbid_manual_placement = True;
     }
@@ -759,7 +840,7 @@ Bool PlaceWindow(
     unsigned int placement_mode = SPLACEMENT_MODE(sflags);
 
     /* override if Manual placement happen */
-    SET_PLACED_BY_FVWM(tmp_win,True);
+    SET_PLACED_BY_FVWM(fw,True);
 
     if (flags.do_forbid_manual_placement)
     {
@@ -783,7 +864,7 @@ Bool PlaceWindow(
     case PLACE_TILEMANUAL:
       flags.is_smartly_placed =
 	SmartPlacement(
-	  tmp_win, &screen_g, tmp_win->frame_g.width, tmp_win->frame_g.height,
+	  fw, &screen_g, fw->frame_g.width, fw->frame_g.height,
 	  &xl, &yt, pdeltax, pdeltay);
       if (flags.is_smartly_placed)
 	break;
@@ -804,7 +885,7 @@ Bool PlaceWindow(
 
 	/* Grabbed the pointer - continue */
 	MyXGrabServer(dpy);
-	if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
+	if (XGetGeometry(dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY,
 			 (unsigned int *)&DragWidth,
 			 (unsigned int *)&DragHeight,
 			 &JunkBW,  &JunkDepth) == 0)
@@ -813,14 +894,17 @@ Bool PlaceWindow(
           UngrabEm(GRAB_NORMAL);
 	  return False;
 	}
-	SET_PLACED_BY_FVWM(tmp_win,False);
+	SET_PLACED_BY_FVWM(fw,False);
 	MyXGrabKeyboard(dpy);
-	DragWidth = tmp_win->frame_g.width;
-	DragHeight = tmp_win->frame_g.height;
+	DragWidth = fw->frame_g.width;
+	DragHeight = fw->frame_g.height;
 
-	XMapRaised(dpy, Scr.SizeWindow);
+	if (Scr.SizeWindow != None)
+	{
+	  XMapRaised(dpy, Scr.SizeWindow);
+	}
 	FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
-	if (moveLoop(tmp_win, mx, my, DragWidth, DragHeight,
+	if (moveLoop(fw, mx, my, DragWidth, DragHeight,
 		     &xl, &yt, False))
 	{
 	  /* resize too */
@@ -831,7 +915,10 @@ Bool PlaceWindow(
 	  /* ok */
 	  rc = False;
 	}
-	XUnmapWindow(dpy, Scr.SizeWindow);
+	if (Scr.SizeWindow != None)
+	{
+	  XUnmapWindow(dpy, Scr.SizeWindow);
+	}
 	MyXUngrabKeyboard(dpy);
 	MyXUngrabServer(dpy);
 	UngrabEm(GRAB_NORMAL);
@@ -848,17 +935,18 @@ Bool PlaceWindow(
         xl -= pdeltax;
         yt -= pdeltay;
       }
-      tmp_win->attr.y = yt;
-      tmp_win->attr.x = xl;
+      attr_g->y = yt;
+      attr_g->x = xl;
       break;
     case PLACE_MINOVERLAPPERCENT:
-      CleverPlacement(tmp_win, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
+      CleverPlacement(
+	      fw, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
       flags.is_smartly_placed = True;
       break;
     case PLACE_TILECASCADE:
       flags.is_smartly_placed =
 	SmartPlacement(
-	  tmp_win, &screen_g, tmp_win->frame_g.width, tmp_win->frame_g.height,
+	  fw, &screen_g, fw->frame_g.width, fw->frame_g.height,
 	  &xl, &yt, pdeltax, pdeltay);
       if (flags.is_smartly_placed)
 	break;
@@ -871,46 +959,37 @@ Bool PlaceWindow(
 	  the old Style Dumb/Smart, Random/Active, Smart/SmartOff (i.e.:
 	  Dumb+Random+Smart or Dumb+Active+Smart)
       */
-      if ((Scr.randomx += tmp_win->title_g.height) > screen_g.width / 2)
+      if ((Scr.randomx += fw->title_thickness) > screen_g.width / 2)
       {
-	Scr.randomx = tmp_win->title_g.height;
+	Scr.randomx = fw->title_thickness;
       }
-      if ((Scr.randomy += 2 * tmp_win->title_g.height) > screen_g.height / 2)
+      if ((Scr.randomy += 2 * fw->title_thickness) > screen_g.height / 2)
       {
-	Scr.randomy = 2 * tmp_win->title_g.height;
+	Scr.randomy = 2 * fw->title_thickness;
       }
-      tmp_win->attr.x = Scr.randomx + PageLeft - pdeltax;
-      tmp_win->attr.y = Scr.randomy + PageTop - pdeltay;
+      attr_g->x = Scr.randomx + PageLeft - pdeltax;
+      attr_g->y = Scr.randomy + PageTop - pdeltay;
       /* try to keep the window on the screen */
-      tmp_win->frame_g.x = PageLeft + tmp_win->attr.x + tmp_win->old_bw + 10;
-      tmp_win->frame_g.y = PageTop + tmp_win->attr.y + tmp_win->old_bw + 10;
+      fw->frame_g.x = PageLeft + attr_g->x + fw->attr_backup.border_width + 10;
+      fw->frame_g.y = PageTop + attr_g->y + fw->attr_backup.border_width + 10;
 
-      if (tmp_win->attr.x + tmp_win->frame_g.width >= PageRight)
+      get_window_borders(fw, &b);
+      if (attr_g->x + fw->frame_g.width >= PageRight)
       {
-	tmp_win->attr.x = PageRight - tmp_win->attr.width
-	  - tmp_win->old_bw - 2*tmp_win->boundary_width;
+	attr_g->x = PageRight - attr_g->width - fw->attr_backup.border_width -
+		b.total_size.width;
 	Scr.randomx = 0;
       }
-      if (tmp_win->attr.y + tmp_win->frame_g.height >= PageBottom)
+      if (attr_g->y + fw->frame_g.height >= PageBottom)
       {
-	tmp_win->attr.y = PageBottom - tmp_win->attr.height
-	  - tmp_win->old_bw - tmp_win->title_g.height -
-	  2*tmp_win->boundary_width;
+	attr_g->y = PageBottom - attr_g->height - fw->attr_backup.border_width -
+		b.total_size.height;
 	Scr.randomy = 0;
       }
-
-      /* the left and top sides are more important in huge windows */
-      if (tmp_win->attr.x < 0)
-      {
-	tmp_win->attr.x = 0;
-      }
-      if (tmp_win->attr.y < 0)
-      {
-	tmp_win->attr.y = 0;
-      }
       break;
     case PLACE_MINOVERLAP:
-      CleverPlacement(tmp_win, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
+      CleverPlacement(
+	      fw, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
       flags.is_smartly_placed = True;
       break;
     default:
@@ -921,14 +1000,14 @@ Bool PlaceWindow(
     if (flags.is_smartly_placed)
     {
       /* "smart" placement succed, we have done ... */
-      tmp_win->attr.x = xl;
-      tmp_win->attr.y = yt;
+      attr_g->x = xl;
+      attr_g->y = yt;
     }
   } /* !flags.do_not_use_wm_placement */
   else
   {
-    if (!PPosOverride)
-      SET_PLACED_BY_FVWM(tmp_win,False);
+    if (!win_opts->flags.do_override_ppos)
+      SET_PLACED_BY_FVWM(fw,False);
     /* the USPosition was specified, or the window is a transient,
      * or it starts iconic so place it automatically */
 
@@ -949,26 +1028,25 @@ Bool PlaceWindow(
        */
 
       FScreenTranslateCoordinates(
-	NULL, XineramaScreen, NULL, FSCREEN_GLOBAL,
-	&tmp_win->attr.x, &tmp_win->attr.y);
+	NULL, XineramaScreen, NULL, FSCREEN_GLOBAL, &attr_g->x, &attr_g->y);
     }
 
     /*
      *  If SkipMapping, and other legalities are observed, adjust for
      * StartsOnPage.
      */
-    if ( ( DO_NOT_SHOW_ON_MAP(tmp_win) && flags.do_honor_starts_on_page )  &&
+    if ( ( DO_NOT_SHOW_ON_MAP(fw) && flags.do_honor_starts_on_page )  &&
 
-	 ( (!(IS_TRANSIENT(tmp_win)) ||
+	 ( (!(IS_TRANSIENT(fw)) ||
 	    SUSE_START_ON_PAGE_FOR_TRANSIENT(sflags)) &&
 
 	   ((SUSE_NO_PPOSITION(sflags)) ||
-	    !(tmp_win->hints.flags & PPosition)) &&
+	    !(fw->hints.flags & PPosition)) &&
 
            /*  RBW - allow StartsOnPage to go through, even if iconic.  */
-           ( ((!((tmp_win->wmhints)&&
-		 (tmp_win->wmhints->flags & StateHint)&&
-		 (tmp_win->wmhints->initial_state == IconicState)))
+           ( ((!((fw->wmhints)&&
+		 (fw->wmhints->flags & StateHint)&&
+		 (fw->wmhints->initial_state == IconicState)))
               || (flags.do_honor_starts_on_page)) )
 
 	   ) )
@@ -982,32 +1060,30 @@ Bool PlaceWindow(
        * were only one page, no virtual desktop), then 2) readjust
        * relative to the current page.
        */
-      if (tmp_win->attr.x < 0)
+      if (attr_g->x < 0)
       {
-	tmp_win->attr.x =
-	  ((Scr.MyDisplayWidth + tmp_win->attr.x) % Scr.MyDisplayWidth);
+	attr_g->x = ((Scr.MyDisplayWidth + attr_g->x) % Scr.MyDisplayWidth);
       }
       else
       {
-	tmp_win->attr.x = tmp_win->attr.x % Scr.MyDisplayWidth;
+	attr_g->x = attr_g->x % Scr.MyDisplayWidth;
       }
       /*
        * Noticed a quirk here. With some apps (e.g., xman), we find the
        * placement has moved 1 pixel away from where we originally put it when
        * we come through here. Why is this happening?
-       * Probably old_bw, try xclock -borderwidth 100
+       * Probably attr_backup.border_width, try xclock -borderwidth 100
        */
-      if (tmp_win->attr.y < 0)
+      if (attr_g->y < 0)
       {
-	tmp_win->attr.y =
-	  ((Scr.MyDisplayHeight + tmp_win->attr.y) % Scr.MyDisplayHeight);
+	attr_g->y = ((Scr.MyDisplayHeight + attr_g->y) % Scr.MyDisplayHeight);
       }
       else
       {
-	tmp_win->attr.y = tmp_win->attr.y % Scr.MyDisplayHeight;
+	attr_g->y = attr_g->y % Scr.MyDisplayHeight;
       }
-      tmp_win->attr.x -= pdeltax;
-      tmp_win->attr.y -= pdeltay;
+      attr_g->x -= pdeltax;
+      attr_g->y -= pdeltay;
     }
 
     /* put it where asked, mod title bar */
@@ -1017,9 +1093,9 @@ Bool PlaceWindow(
       int gravx;
       int gravy;
 
-      gravity_get_offsets(tmp_win->hints.win_gravity, &gravx, &gravy);
-      final_g.x = tmp_win->attr.x + gravx * tmp_win->old_bw;
-      final_g.y = tmp_win->attr.y + gravy * tmp_win->old_bw;
+      gravity_get_offsets(fw->hints.win_gravity, &gravx, &gravy);
+      final_g.x = attr_g->x + gravx * fw->attr_backup.border_width;
+      final_g.y = attr_g->y + gravy * fw->attr_backup.border_width;
       /* Virtually all applications seem to share a common bug: they request
        * the top left pixel of their *border* as their origin instead of the
        * top left pixel of their client window, e.g. 'xterm -g +0+0' creates an
@@ -1028,19 +1104,19 @@ Bool PlaceWindow(
        * at (1 1) instead.  This clearly violates the ICCCM, but trying to
        * change this is like tilting at windmills.  So we have to add the
        * border width here. */
-      final_g.x += tmp_win->old_bw;
-      final_g.y += tmp_win->old_bw;
+      final_g.x += fw->attr_backup.border_width;
+      final_g.y += fw->attr_backup.border_width;
       final_g.width = 0;
       final_g.height = 0;
       if (mode == PLACE_INITIAL)
       {
+	get_window_borders(fw, &b);
         gravity_resize(
-          tmp_win->hints.win_gravity, &final_g,
-          2 * tmp_win->boundary_width,
-          2 * tmp_win->boundary_width + tmp_win->title_g.height);
+          fw->hints.win_gravity, &final_g,
+          b.total_size.width, b.total_size.height);
       }
-      tmp_win->attr.x = final_g.x;
-      tmp_win->attr.y = final_g.y;
+      attr_g->x = final_g.x;
+      attr_g->y = final_g.y;
     }
   }
 
@@ -1050,32 +1126,44 @@ Bool PlaceWindow(
 
 void CMD_PlaceAgain(F_CMD_ARGS)
 {
-  int x;
-  int y;
-  char *token;
-  float noMovement[1] = {1.0};
-  float *ppctMovement = noMovement;
-  window_style style;
+	char *token;
+	float noMovement[1] = {1.0};
+	float *ppctMovement = noMovement;
+	window_style style;
+	rectangle attr_g;
+	XWindowAttributes attr;
+	initial_window_options_type win_opts;
 
-  if (DeferExecution(eventp, &w, &tmp_win, &context, CRS_SELECT,
-		     ButtonRelease))
-    return;
+	if (DeferExecution(
+		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
+	{
+		return;
+	}
+	if (!XGetWindowAttributes(dpy, FW_W(fw), &attr))
+	{
+		return;
+	}
+	memset(&win_opts, 0, sizeof(win_opts));
+	lookup_style(fw, &style);
+	attr_g.x = attr.x;
+	attr_g.y = attr.y;
+	attr_g.width = attr.width;
+	attr_g.height = attr.height;
+	PlaceWindow(
+		fw, &style.flags, &attr_g, SGET_START_DESK(style),
+		SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
+		SGET_START_SCREEN(style), PLACE_AGAIN, &win_opts);
 
-  lookup_style(tmp_win, &style);
-  PlaceWindow(
-    tmp_win, &style.flags,
-    SGET_START_DESK(style), SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
-    SGET_START_SCREEN(style), PLACE_AGAIN);
-  x = tmp_win->attr.x;
-  y = tmp_win->attr.y;
+	/* Possibly animate the movement */
+	token = PeekToken(action, NULL);
+	if (token && StrEquals("Anim", token))
+	{
+		ppctMovement = NULL;
+	}
 
-  /* Possibly animate the movement */
-  token = PeekToken(action, NULL);
-  if(token && StrEquals("ANIM", token))
-    ppctMovement = NULL;
+	AnimatedMoveFvwmWindow(
+		fw, FW_W_FRAME(fw), -1, -1, attr_g.x, attr_g.y, False, -1,
+		ppctMovement);
 
-  AnimatedMoveFvwmWindow(tmp_win, tmp_win->frame, -1, -1, x, y, False, -1,
-			 ppctMovement);
-
-  return;
+	return;
 }
