@@ -32,9 +32,6 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 
 #include "libs/fvwmlib.h"
 #include "fvwm.h"
@@ -110,7 +107,7 @@ const char *get_current_read_dir(void)
 /**
  * Read and execute each line from stream.
  **/
-void run_command_stream( FILE* f, XEvent *eventp, FvwmWindow *tmp_win,
+void run_command_stream( FILE* f, XEvent *eventp, FvwmWindow *fw,
 			 unsigned long context, int Module )
 {
   char *tline;
@@ -135,12 +132,17 @@ void run_command_stream( FILE* f, XEvent *eventp, FvwmWindow *tmp_win,
     tline=line;
     while(isspace((unsigned char)*tline))
       tline++;
-    if (debugging)
-      fvwm_msg(DBG,"ReadSubFunc","Module switch %d, about to exec: '%.*s'",
-	       Module,strlen(tline)-1,tline);
+    l = strlen(tline);
+    if (l > 0 && tline[l - 1] == '\n')
+      tline[l - 1] = '\0';
 
-    old_execute_function(tline, tmp_win, eventp, context, Module, 0, NULL);
-    tline = fgets(line,(sizeof line)-1,f);
+    if (debugging)
+      fvwm_msg(DBG, "ReadSubFunc", "Module switch %d, about to exec: '%s'",
+	       Module, tline);
+
+    old_execute_function(
+      NULL, tline, fw, eventp, context, Module, 0, NULL);
+    tline = fgets(line, (sizeof line) - 1, f);
   }
 }
 
@@ -186,7 +188,7 @@ static int parse_filename(
 /**
  * Returns FALSE if file not found
  **/
-int run_command_file( char* filename, XEvent *eventp, FvwmWindow *tmp_win,
+int run_command_file( char* filename, XEvent *eventp, FvwmWindow *fw,
 		      unsigned long context, int Module )
 {
   char *full_filename;
@@ -213,7 +215,7 @@ int run_command_file( char* filename, XEvent *eventp, FvwmWindow *tmp_win,
   if (push_read_file(full_filename) == FALSE)
     return FALSE;
 
-  run_command_stream( f, eventp, tmp_win, context, Module );
+  run_command_stream( f, eventp, fw, context, Module );
   fclose( f );
 
   pop_read_file();
@@ -265,12 +267,18 @@ void CMD_Read(F_CMD_ARGS)
   if ( !parse_filename( "Read", action, &filename, &read_quietly ) )
     return;
   cursor_control(True);
-  if ( !run_command_file( filename, eventp, tmp_win, context, *Module ) &&
+  if ( !run_command_file( filename, eventp, fw, context, *Module ) &&
        !read_quietly )
   {
-    fvwm_msg( ERR, "Read",
-	      "file '%s' not found in %s or "FVWM_DATADIR,
-	      filename, fvwm_userdir );
+    if (filename[0] == '/')
+    {
+      fvwm_msg( ERR, "Read",
+                "file '%s' not found",filename);
+    } else {
+      fvwm_msg( ERR, "Read",
+                "file '%s' not found in %s or "FVWM_DATADIR,
+                filename, fvwm_userdir );
+    }
   }
   free( filename );
   cursor_control(False);
@@ -304,7 +312,7 @@ void CMD_PipeRead(F_CMD_ARGS)
   }
   free(command);
 
-  run_command_stream(f, eventp, tmp_win, context, *Module);
+  run_command_stream(f, eventp, fw, context, *Module);
   pclose(f);
   cursor_control(False);
 }
