@@ -54,11 +54,19 @@
 #define COLORMAP_FOLLOWS_MOUSE 1 /* default */
 #define COLORMAP_FOLLOWS_FOCUS 2
 
+#ifdef FANCY_TITLEBARS
+#define TITLE_PADDING 5
+enum tb_pixmap_enum {TBP_MAIN, TBP_LEFT_MAIN, TBP_RIGHT_MAIN, TBP_UNDER_TEXT,
+                     TBP_LEFT_OF_TEXT, TBP_RIGHT_OF_TEXT, TBP_LEFT_END,
+                     TBP_RIGHT_END, TBP_BUTTONS, TBP_LEFT_BUTTONS,
+                     TBP_RIGHT_BUTTONS, NUM_TB_PIXMAPS};
+#endif
 
 typedef struct
 {
   Window win;
   int isMapped;
+  char * command ; /* command which is executed when the pan frame is entered */ 
 } PanFrame;
 
 typedef enum
@@ -70,9 +78,10 @@ typedef enum
     GradientButton            ,
     PixmapButton              ,
     TiledPixmapButton         ,
-#ifdef MINI_ICONS
-    MiniIconButton            ,
+#ifdef FANCY_TITLEBARS
+    MultiPixmap               ,
 #endif
+    MiniIconButton            ,
     SolidButton
 } DecorFaceType;
 
@@ -88,7 +97,7 @@ typedef enum
 
 typedef struct
 {
-  unsigned face_type : 3; /* was DecorFaceType : 3 */
+  unsigned face_type : 4;
   struct
   {
     unsigned h_justification : 2; /* was JustificationType : 2 */
@@ -119,81 +128,89 @@ typedef struct
 
 typedef struct DecorFace
 {
-  DecorFaceStyle style;
-  struct
-  {
-    Picture *p;
-    Pixel back;
-    struct
-    {
-      int npixels;
-      Pixel *pixels;
-      char gradient_type;
-    } grad;
-    struct vector_coords
-    {
-      int num;
-      int *x;
-      int *y;
-      unsigned long line_style;
-      unsigned long use_fgbg;
-    } vector;
-  } u;
-
-#ifdef MULTISTYLE
-  struct DecorFace *next;
+	DecorFaceStyle style;
+	struct
+	{
+		FvwmPicture *p;
+#ifdef FANCY_TITLEBARS
+		FvwmPicture **multi_pixmaps;
+		short multi_stretch_flags;
 #endif
+		Pixel back;
+		struct
+		{
+			int npixels;
+			Pixel *pixels;
+			char gradient_type;
+		} grad;
+		struct vector_coords
+		{
+			int num;
+			signed char *x;
+			signed char *y;
+			signed char *c;
+			unsigned use_fgbg : 1;
+		} vector;
+	} u;
 
-  struct
-  {
-    unsigned has_changed : 1;
-  } flags;
+	struct DecorFace *next;
+
+	struct
+	{
+		unsigned has_changed : 1;
+	} flags;
 } DecorFace;
 
-enum ButtonState
-{
-  BS_All = -1,
-  BS_ActiveUp,
-  BS_ActiveDown,
-  BS_Inactive,
-  BS_ToggledActiveUp,
-  BS_ToggledActiveDown,
-  BS_ToggledInactive,
-  BS_MaxButtonState
-};
-
 typedef enum
 {
-  /* The first five are used in title buttons.  These can't be renumbered
-   * without extending the mwm_decor_flags member below and adapting the style
-   * structure. */
-  MWM_DECOR_MENU     = 0x1,
-  MWM_DECOR_MINIMIZE = 0x2,
-  MWM_DECOR_MAXIMIZE = 0x4,
-  MWM_DECOR_SHADE    = 0x8,
-  MWM_DECOR_STICK    = 0x10,
-  /* --- */
-  MWM_DECOR_BORDER   = 0x20,
-  MWM_DECOR_RESIZEH  = 0x40,
-  MWM_DECOR_TITLE    = 0x80,
-  MWM_DECOR_ALL      = 0x100,
-  MWM_DECOR_EVERYTHING = 0xff
+	BS_All = -1,
+	BS_ActiveUp,
+	BS_ActiveDown,
+	BS_Inactive,
+	BS_ToggledActiveUp,
+	BS_ToggledActiveDown,
+	BS_ToggledInactive,
+	BS_MaxButtonState
+} ButtonState;
+
+typedef enum
+{
+	/* The first five are used in title buttons.  These can't be
+	 * renumbered without extending the mwm_decor_flags member below and
+	 * adapting the style structure. */
+	MWM_DECOR_MENU     = 0x1,
+	MWM_DECOR_MINIMIZE = 0x2,
+	MWM_DECOR_MAXIMIZE = 0x4,
+	MWM_DECOR_SHADE    = 0x8,
+	MWM_DECOR_STICK    = 0x10,
+	/* --- */
+	MWM_DECOR_BORDER   = 0x20,
+	MWM_DECOR_RESIZEH  = 0x40,
+	MWM_DECOR_TITLE    = 0x80,
+	MWM_DECOR_ALL      = 0x100,
+	MWM_DECOR_EVERYTHING = 0xff
 } mwm_flags;
 
 typedef struct
 {
-  unsigned just : 2; /* was JustificationType : 2 */
-  struct
-  {
-    unsigned has_changed : 1;
-    mwm_flags mwm_decor_flags : 9;
-  } flags;
-  DecorFace state[BS_MaxButtonState];
+	unsigned just : 2; /* was JustificationType : 2 */
+	int layer;
+	struct
+	{
+		unsigned has_changed : 1;
+		mwm_flags mwm_decor_flags : 9;
+		/* Support {ButtonStyle <number> - Layer 4} construction, so
+		 * button can be rendered 'pressed in' when the window is
+		 * assigned to a particular layer. */
+		unsigned has_layer : 1;
+	} flags;
+	DecorFace state[BS_MaxButtonState];
 } TitleButton;
 
 #define TB_FLAGS(tb)              ((tb).flags)
 #define TB_STATE(tb)              ((tb).state)
 #define TB_JUSTIFICATION(tb)      ((tb).just)
+#define TB_LAYER(tb)              ((tb).layer)
 #define TB_MWM_DECOR_FLAGS(tb)    ((tb).flags.mwm_decor_flags)
 #define TB_HAS_CHANGED(tb)     \
   (!!((tb).flags.has_changed))
@@ -231,6 +248,28 @@ typedef struct FvwmDecor
   } flags;
 } FvwmDecor;
 
+typedef struct DesktopsInfo
+{
+  int desk;
+  char *name;
+
+  struct
+  {
+    int x;
+    int y;
+    int width;
+    int height;
+  } ewmh_working_area;
+  struct
+  {
+    int x;
+    int y;
+    int width;
+    int height;
+  } ewmh_dyn_working_area;
+
+  struct DesktopsInfo *next;
+} DesktopsInfo;
 
 typedef struct ScreenInfo
 {
@@ -273,7 +312,7 @@ typedef struct ScreenInfo
 
   FvwmFunction *functions;
 
-  FvwmFont DefaultFont;     	/* font structure */
+  FlocaleFont *DefaultFont;     	/* font structure */
 
   GC TransMaskGC;               /* GC for transparency masks */
   Pixel StdFore, StdBack, StdHilite, StdShadow; /* don't change the order */
@@ -290,6 +329,7 @@ typedef struct ScreenInfo
   GC ScratchGC3;
   GC ScratchGC4;
   GC TitleGC;
+  GC TileGC;                    /* Used for tiling (clipmask and tile) */
   int SizeStringWidth;	        /* minimum width of size window */
 
   FvwmDecor DefaultDecor;	/* decoration style(s) */
@@ -304,6 +344,8 @@ typedef struct ScreenInfo
   Window UnknownWinFocused;      /* None, if the focus is nowhere or on an fvwm
 				 * managed window. Set to id of otherwindow
 				 * with focus otherwise */
+  Window StolenFocusWin;        /* The window that the UnknownWinFocused window
+				 * stole the focus from. */
   FvwmWindow *Ungrabbed;
   int EdgeScrollX;              /* #pixels to scroll on screen edge */
   int EdgeScrollY;              /* #pixels to scroll on screen edge */
@@ -353,6 +395,7 @@ typedef struct ScreenInfo
     unsigned InstallRootCmap : 1;
     unsigned RaiseOverUnmanaged : 1;
     unsigned FlickeringQtDialogsWorkaround : 1;
+    unsigned EWMHIconicStateWorkaround : 1;
   } bo; /* bug workaround control options */
   struct
   {
@@ -385,6 +428,10 @@ typedef struct ScreenInfo
     unsigned edge_wrap_y : 1;
   } flags;
 
+  DesktopsInfo *Desktops; /* info for some desktops; the first entries should be
+			  * generic info correct for any desktop not in the
+			  * list */
+  FvwmWindow *EwmhDesktop;   /* the window of desktop type if any */
   struct
   {
     last_added_item_type type;
@@ -392,6 +439,10 @@ typedef struct ScreenInfo
   } last_added_item;
 } ScreenInfo;
 
+/* A macro to to simplify he "ewmh desktop code" */
+#define IS_EWMH_DESKTOP(win) \
+  (Scr.EwmhDesktop && win == Scr.EwmhDesktop->wins.client)
+
 /*
    Macro which gets specific decor or default decor.
    This saves an indirection in case you don't want
