@@ -24,13 +24,11 @@
 #include "config.h"
 
 #include <stdio.h>
-#ifdef HAVE_GETPWUID
 #include <pwd.h>
-#endif
 #include <signal.h>
-#include <fcntl.h>
 
 #include "libs/fvwmlib.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -70,6 +68,7 @@ typedef struct _match
   int                 layer;
   int                 used;
   int                 gravity;
+  unsigned long       ewmh_hint_desktop;
   window_flags        flags;
 }
 Match;
@@ -150,30 +149,13 @@ static void setRealStateFilename(char *filename)
 static char *getUniqueStateFilename(void)
 {
   const char *path = getenv("SM_SAVE_DIR");
-  char *filename;
-  int fd;
+  struct passwd *pwd;
 
-  if ( ! path)
-    path = getenv ("HOME");
-
-#ifdef HAVE_GETPWUID
   if (!path) {
-    struct passwd *pwd = getpwuid(getuid());
-    if (pwd)
-      path = pwd->pw_dir;
+    pwd = getpwuid(getuid());
+    path = pwd->pw_dir;
   }
-#endif
-  if ( ! path)
-    return NULL;
-
-  filename = tempnam(path, ".fs-");
-  if ((fd = open (filename, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1) {
-    free (filename);
-    filename = NULL;
-  } else {
-    close (fd);
-  }
-  return filename;
+  return tempnam(path, ".fs-");
 }
 
 #else
@@ -332,9 +314,8 @@ char *get_version_string()
 {
   /* migo (14-Mar-2001): it is better to manually update a version string
    * in the stable branch, othervise saving sessions becomes useless */
-  /*return CatString3(VERSION, ", ",__DATE__);*/
-
-  return "2.4-4";
+  return CatString3(VERSION, ", ", __DATE__);
+  /* return "2.6-0"; */
 }
 
 /*
@@ -366,8 +347,8 @@ static Bool VerifyVersionInfo(char *filename)
       {
 	fvwm_msg(
 	  ERR, "VerifyVersionInfo",
-	  "State file version (%s) does not match the current version (%s)\n"
-	  "State file will be ignored", s1, current_v);
+	  "State file version (%s) does not match the current version (%s), "
+	  "state file is ignored.", s1, current_v);
 	break;
       }
     }
@@ -412,7 +393,9 @@ SaveWindowStates(FILE *f)
   int wm_command_count;
   FvwmWindow *ewin;
   rectangle save_g;
+  rectangle ig;
   int i;
+  int layer;
 
   for (ewin = get_next_window_in_stack_ring(&Scr.FvwmRoot);
        ewin != &Scr.FvwmRoot;
@@ -420,7 +403,7 @@ SaveWindowStates(FILE *f)
   {
     Bool is_icon_sticky;
 
-    if (!XGetGeometry(dpy, ewin->w, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
+    if (!XGetGeometry(dpy, FW_W(ewin), &JunkRoot, &JunkX, &JunkY, &JunkWidth,
 		      &JunkHeight, &JunkBW, &JunkDepth))
     {
       /* Don't save the state of windows that already died (i.e. modules)!
@@ -429,16 +412,16 @@ SaveWindowStates(FILE *f)
     }
     is_icon_sticky = (IS_STICKY(ewin) ||
 		     (IS_ICONIFIED(ewin) && IS_ICON_STICKY(ewin)));
-    fprintf(f, "[CLIENT] %lx\n", ewin->w);
+    fprintf(f, "[CLIENT] %lx\n", FW_W(ewin));
 
-    client_id = GetClientID(ewin->w);
+    client_id = GetClientID(FW_W(ewin));
     if (client_id)
     {
       fprintf(f, "  [CLIENT_ID] %s\n", client_id);
       XFree(client_id);
     }
 
-    window_role = GetWindowRole(ewin->w);
+    window_role = GetWindowRole(FW_W(ewin));
     if (window_role)
     {
       fprintf(f, "  [WINDOW_ROLE] %s\n", window_role);
@@ -450,12 +433,12 @@ SaveWindowStates(FILE *f)
         fprintf(f, "  [RES_NAME] %s\n", ewin->class.res_name);
       if (ewin->class.res_name)
         fprintf(f, "  [RES_CLASS] %s\n", ewin->class.res_class);
-      if (ewin->name)
-        fprintf(f, "  [WM_NAME] %s\n", ewin->name);
+      if (ewin->name.name)
+        fprintf(f, "  [WM_NAME] %s\n", ewin->name.name);
 
       wm_command = NULL;
       wm_command_count = 0;
-      if (XGetCommand(dpy, ewin->w, &wm_command, &wm_command_count) &&
+      if (XGetCommand(dpy, FW_W(ewin), &wm_command, &wm_command_count) &&
 	  wm_command && wm_command_count > 0)
       {
         fprintf(f, "  [WM_COMMAND] %i", wm_command_count);
@@ -477,17 +460,24 @@ SaveWindowStates(FILE *f)
       save_g.x -= Scr.Vx;
       save_g.y -= Scr.Vy;
     }
+    get_visible_icon_geometry(ewin, &ig);
     fprintf(
       f, "  [GEOMETRY] %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i\n",
       save_g.x, save_g.y, save_g.width, save_g.height,
       ewin->max_g.x, ewin->max_g.y, ewin->max_g.width, ewin->max_g.height,
       ewin->max_g_defect.width, ewin->max_g_defect.height,
-      ewin->icon_g.x + ((!is_icon_sticky) ? Scr.Vx : 0),
-      ewin->icon_g.y + ((!is_icon_sticky) ? Scr.Vy : 0),
+      ig.x + ((!is_icon_sticky) ? Scr.Vx : 0),
+      ig.y + ((!is_icon_sticky) ? Scr.Vy : 0),
       ewin->hints.win_gravity,
       ewin->max_offset.x, ewin->max_offset.y);
     fprintf(f, "  [DESK] %i\n", ewin->Desk);
-    fprintf(f, "  [LAYER] %i\n", get_layer(ewin));
+    /* set the layer to the default layer if the layer has been set by
+     * an ewmh hint */
+    layer = get_layer(ewin);
+    if (layer == ewin->ewmh_hint_layer && layer > 0)
+      layer = Scr.DefaultLayer;
+    fprintf(f, "  [LAYER] %i\n", layer);
+    fprintf(f, "  [EWMH_DESKTOP] %lu\n", ewin->ewmh_hint_desktop);
     fprintf(f, "  [FLAGS] ");
     for (i = 0; i < sizeof(window_flags); i++)
       fprintf(f, "%02x ", (int)(((unsigned char *)&(ewin->flags))[i]));
@@ -602,6 +592,11 @@ LoadWindowStates(char *filename)
       sscanf(s, "%*s %i",
 	     &(matches[num_match - 1].layer));
     }
+    else if (!strcmp(s1, "[EWMH_DESKTOP]"))
+    {
+      sscanf(s, "%*s %lu",
+	     &(matches[num_match - 1].ewmh_hint_desktop));
+    }
     else if (!strcmp(s1, "[FLAGS]"))
     {
       char *ts = s;
@@ -676,14 +671,14 @@ static Bool matchWin(FvwmWindow *w, Match *m)
   int found;
 
   found = 0;
-  client_id = GetClientID(w->w);
+  client_id = GetClientID(FW_W(w));
 
   if (xstreq(client_id, m->client_id))
   {
 
     /* client_id's match */
 
-    window_role = GetWindowRole(w->w);
+    window_role = GetWindowRole(FW_W(w));
 
     if (window_role || m->window_role)
     {
@@ -699,7 +694,7 @@ static Bool matchWin(FvwmWindow *w, Match *m)
       if (xstreq(w->class.res_name, m->res_name) &&
           xstreq(w->class.res_class, m->res_class) &&
           (IS_NAME_CHANGED(w) || IS_NAME_CHANGED(m) ||
-           xstreq(w->name, m->wm_name)))
+           xstreq(w->name.name, m->wm_name)))
       {
         if (client_id)
         {
@@ -711,7 +706,7 @@ static Bool matchWin(FvwmWindow *w, Match *m)
         {
           /* for non-SM-aware clients we also compare WM_COMMAND */
 
-	  if (!XGetCommand (dpy, w->w, &wm_command, &wm_command_count))
+	  if (!XGetCommand (dpy, FW_W(w), &wm_command, &wm_command_count))
 	  {
 	    wm_command = NULL;
 	    wm_command_count = 0;
@@ -727,7 +722,7 @@ static Bool matchWin(FvwmWindow *w, Match *m)
             if (i == wm_command_count)
             {
               /* migo (21/Oct/1999): on restarts compare window ids too */
-              if (!Restarting || w->w == m->win)
+              if (!Restarting || FW_W(w) == m->win)
                 found = 1;
             }
           } /* if (wm_command_count ==... */
@@ -763,7 +758,9 @@ static Bool matchWin(FvwmWindow *w, Match *m)
   to be set up correctly beforehand!
 */
 Bool
-MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
+MatchWinToSM(
+	FvwmWindow *ewin, int *do_shade, int *shade_dir, int *do_max,
+	initial_window_options_type *win_opts)
 {
   int i;
 
@@ -793,17 +790,17 @@ MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
 	SET_FOCUS_MODE(ewin, GET_FOCUS_MODE(&(matches[i])));
 	if (matches[i].wm_name)
 	{
-	  if (ewin->name)
-	  {
-	    free(ewin->name);
-	  }
-	  ewin->name = matches[i].wm_name;
+	  free_window_names(ewin, True, False);
+	  ewin->name.name = matches[i].wm_name;
+	  setup_visible_name(ewin, False);
 	}
       }
       SET_PLACED_WB3(ewin,IS_PLACED_WB3(&(matches[i])));
       SET_PLACED_BY_FVWM(ewin,IS_PLACED_BY_FVWM(&(matches[i])));
       *do_shade = IS_SHADED(&(matches[i]));
+      *shade_dir = SHADED_DIR(&(matches[i]));
       *do_max = IS_MAXIMIZED(&(matches[i]));
+      SET_USER_STATES(ewin, GET_USER_STATES(&(matches[i])));
       SET_ICON_MOVED(ewin, IS_ICON_MOVED(&(matches[i])));
       if (IS_ICONIFIED(&(matches[i])))
       {
@@ -811,19 +808,17 @@ MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
 	  ICON_MOVED is necessary to make fvwm use icon_[xy]_loc
 	  for icon placement
 	*/
-	SET_DO_START_ICONIC(ewin, 1);
-	if (!IS_ICON_MOVED(ewin))
+	win_opts->initial_state = IconicState;
+	win_opts->flags.use_initial_icon_xy = 1;
+	win_opts->initial_icon_x = matches[i].icon_x;
+	win_opts->initial_icon_y = matches[i].icon_y;
+	if (!IS_STICKY(&(matches[i])) &&
+	  !(IS_ICONIFIED(&(matches[i])) && IS_ICON_STICKY(&(matches[i]))))
 	{
-	  /* only temporary to initially place the icon */
-	  SET_ICON_MOVED(ewin, 1);
-	  SET_DELETE_ICON_MOVED(ewin, 1);
+		win_opts->initial_icon_x -= Scr.Vx;
+		win_opts->initial_icon_y -= Scr.Vy;
 	}
       }
-      else
-      {
-	SET_DO_START_ICONIC(ewin, 0);
-      }
-
       ewin->normal_g.x = matches[i].x;
       ewin->normal_g.y = matches[i].y;
       ewin->normal_g.width = matches[i].w;
@@ -839,18 +834,23 @@ MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
       SET_STICKY(ewin, IS_STICKY(&(matches[i])));
       ewin->Desk = (IS_STICKY(ewin)) ? Scr.CurrentDesk : matches[i].desktop;
       set_layer(ewin, matches[i].layer);
-
-      /* this is not enough to fight fvwms attempts to
-	 put icons on the current page */
-      ewin->icon_g.x = matches[i].icon_x;
-      ewin->icon_g.y = matches[i].icon_y;
-      if (!IS_STICKY(&(matches[i])) &&
-	  !(IS_ICONIFIED(&(matches[i])) && IS_ICON_STICKY(&(matches[i]))))
-      {
-	ewin->icon_g.x -= Scr.Vx;
-	ewin->icon_g.y -= Scr.Vy;
-      }
-
+      /* Note: the Modal, skip pager, skip taskbar and "stacking order" state
+       * are not restored here: there are restored in EWMH_ExitStuff */
+      ewin->ewmh_hint_desktop = matches[i].ewmh_hint_desktop;
+      SET_HAS_EWMH_INIT_WM_DESKTOP(
+	  ewin, HAS_EWMH_INIT_WM_DESKTOP(&(matches[i])));
+      SET_HAS_EWMH_INIT_FULLSCREEN_STATE(
+	  ewin, HAS_EWMH_INIT_FULLSCREEN_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_HIDDEN_STATE(
+	  ewin, HAS_EWMH_INIT_HIDDEN_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_MAXHORIZ_STATE(
+	  ewin, HAS_EWMH_INIT_MAXHORIZ_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_MAXVERT_STATE(
+	  ewin, HAS_EWMH_INIT_MAXVERT_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_SHADED_STATE(
+	  ewin, HAS_EWMH_INIT_SHADED_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_STICKY_STATE(
+	  ewin, HAS_EWMH_INIT_STICKY_STATE(&(matches[i])));
       return True;
     }
   }
