@@ -60,7 +60,9 @@ void RestartInSession (char *filename, Bool isNative, Bool doPreserveState);
 **  The dimensions returned in x, y, w, h should be used when the
 **  window is to be maximized.
  */
-Bool MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max);
+Bool MatchWinToSM(
+	FvwmWindow *ewin, int *do_shade, int *shade_dir, int *do_max,
+	initial_window_options_type *win_opts);
 
 #ifdef SESSION
 void SetClientID(char *client_id);
@@ -98,8 +100,5 @@ void ProcessICEMsgs(void);
 Bool quitSession(void);
 Bool saveSession(void);
 Bool saveQuitSession(void);
-void quitSession_func(F_CMD_ARGS);
-void saveSession_func(F_CMD_ARGS);
-void saveQuitSession_func(F_CMD_ARGS);
 
 #endif
