@@ -29,16 +29,15 @@
 #define SHAS_ICON_BOXES(sf)           ((sf)->has_icon_boxes)
 #define SHAS_MAX_WINDOW_SIZE(sf)      ((sf)->has_max_window_size)
 #define SHAS_WINDOW_SHADE_STEPS(sf)   ((sf)->has_window_shade_steps)
-#ifdef MINI_ICONS
 #define SHAS_MINI_ICON(sf)            ((sf)->has_mini_icon)
-#endif
 #define SHAS_MWM_DECOR(sf)            ((sf)->has_mwm_decor)
 #define SHAS_MWM_FUNCTIONS(sf)        ((sf)->has_mwm_functions)
-#define SHAS_NO_BORDER(sf)            ((sf)->has_no_border)
+#define SHAS_NO_HANDLES(sf)           ((sf)->has_no_handles)
 #define SHAS_NO_TITLE(sf)             ((sf)->has_no_title)
 #define SHAS_OL_DECOR(sf)             ((sf)->has_ol_decor)
 #define SIS_BUTTON_DISABLED(sf)       ((sf)->is_button_disabled)
 #define SPLACEMENT_MODE(sf)           ((sf)->placement_mode)
+#define SEWMH_PLACEMENT_MODE(sf)      ((sf)->ewmh_placement_mode)
 #define SUSE_BACKING_STORE(sf)        ((sf)->use_backing_store)
 #define SUSE_PARENT_RELATIVE(sf)      ((sf)->use_parent_relative)
 #define SUSE_COLORSET(sf)             ((sf)->use_colorset)
@@ -47,20 +46,22 @@
 #define SUSE_BORDER_COLORSET_HI(sf)   ((sf)->use_border_colorset_hi)
 #define SUSE_LAYER(sf)                ((sf)->use_layer)
 #define SUSE_NO_PPOSITION(sf)         ((sf)->use_no_pposition)
+#define SUSE_NO_USPOSITION(sf)         ((sf)->use_no_usposition)
+#define SUSE_NO_TRANSIENT_PPOSITION(sf)  ((sf)->use_no_transient_pposition)
+#define SUSE_NO_TRANSIENT_USPOSITION(sf) ((sf)->use_no_transient_usposition)
 #define SUSE_START_ON_DESK(sf)        ((sf)->use_start_on_desk)
 #define SUSE_START_ON_PAGE_FOR_TRANSIENT(sf) \
   ((sf)->use_start_on_page_for_transient)
+#define SUSE_START_ON_SCREEN(sf)      ((sf)->use_start_on_screen)
 #define SMANUAL_PLACEMENT_HONORS_STARTS_ON_PAGE(sf) \
   ((sf)->manual_placement_honors_starts_on_page)
 #define SCAPTURE_HONORS_STARTS_ON_PAGE(sf) \
   ((sf)->capture_honors_starts_on_page)
 #define SRECAPTURE_HONORS_STARTS_ON_PAGE(sf) \
   ((sf)->recapture_honors_starts_on_page)
-
-
-
-
-
+#define SHAS_PLACEMENT_PENALTY(sf) ((sf)->has_placement_penalty)
+#define SHAS_PLACEMENT_PERCENTAGE_PENALTY(sf) \
+  ((sf)->has_placement_percentage_penalty)
 
 
 /* access to common flags */
@@ -70,16 +71,16 @@
 #define SMGET_COMMON_STATIC_FLAGS(st) ((st).flag_mask.common.s)
 #define SCGET_COMMON_FLAGS(st) ((st).change_mask.common)
 #define SCGET_COMMON_STATIC_FLAGS(st) ((st).change_mask.common.s)
-#define SHAS_BOTTOM_TITLE(sf) ((sf).common.s.has_bottom_title)
-#define SFHAS_BOTTOM_TITLE(st) ((st).flags.common.s.has_bottom_title)
-#define SMHAS_BOTTOM_TITLE(st) ((st).flag_mask.common.s.has_bottom_title)
-#define SCHAS_BOTTOM_TITLE(st) ((st).change_mask.common.s.has_bottom_title)
-#define SFSET_HAS_BOTTOM_TITLE(st,x) \
-          ((st).flags.common.s.has_bottom_title = !!(x))
-#define SMSET_HAS_BOTTOM_TITLE(st,x) \
-          ((st).flag_mask.common.s.has_bottom_title = !!(x))
-#define SCSET_HAS_BOTTOM_TITLE(st,x) \
-          ((st).change_mask.common.s.has_bottom_title = !!(x))
+#define STITLE_DIR(sf)  ((sf).common.title_dir)
+#define SFTITLE_DIR(st) ((st).flags.common.title_dir)
+#define SMTITLE_DIR(st) ((st).flag_mask.common.title_dir)
+#define SCTITLE_DIR(st) ((st).change_mask.common.title_dir)
+#define SFSET_TITLE_DIR(st,x) \
+          ((st).flags.common.title_dir = x)
+#define SMSET_TITLE_DIR(st,x) \
+          ((st).flag_mask.common.title_dir = DIR_MAJOR_MASK * !!(x))
+#define SCSET_TITLE_DIR(st,x) \
+          ((st).change_mask.common.title_dir = DIR_MAJOR_MASK * !!(x))
 #define SIS_STICKY(sf) ((sf).common.is_sticky)
 #define SFIS_STICKY(st) ((st).flags.common.is_sticky)
 #define SMIS_STICKY(st) ((st).flag_mask.common.is_sticky)
@@ -185,6 +186,13 @@
 #define SFSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(st,x) ((st).flags.common.s.do_not_raise_click_focus_click = !!(x))
 #define SMSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(st,x) ((st).flag_mask.common.s.do_not_raise_click_focus_click = !!(x))
 #define SCSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(st,x) ((st).change_mask.common.s.do_not_raise_click_focus_click = !!(x))
+#define SDO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(sf)  ((sf).common.s.do_ignore_mouse_focus_click_motion)
+#define SFDO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st) ((st).flags.common.s.do_ignore_mouse_focus_click_motion)
+#define SMDO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st) ((st).flag_mask.common.s.do_ignore_mouse_focus_click_motion)
+#define SCDO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st) ((st).change_mask.common.s.do_ignore_mouse_focus_click_motion)
+#define SFSET_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st,x) ((st).flags.common.s.do_ignore_mouse_focus_click_motion = !!(x))
+#define SMSET_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st,x) ((st).flag_mask.common.s.do_ignore_mouse_focus_click_motion = !!(x))
+#define SCSET_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(st,x) ((st).change_mask.common.s.do_ignore_mouse_focus_click_motion = !!(x))
 #define SDO_RAISE_MOUSE_FOCUS_CLICK(sf)  ((sf).common.s.do_raise_mouse_focus_click)
 #define SFDO_RAISE_MOUSE_FOCUS_CLICK(st) ((st).flags.common.s.do_raise_mouse_focus_click)
 #define SMDO_RAISE_MOUSE_FOCUS_CLICK(st) ((st).flag_mask.common.s.do_raise_mouse_focus_click)
@@ -220,13 +228,6 @@
 #define SFSET_DO_STACK_TRANSIENT_PARENT(st,x) ((st).flags.common.s.do_stack_transient_parent = !!(x))
 #define SMSET_DO_STACK_TRANSIENT_PARENT(st,x) ((st).flag_mask.common.s.do_stack_transient_parent = !!(x))
 #define SCSET_DO_STACK_TRANSIENT_PARENT(st,x) ((st).change_mask.common.s.do_stack_transient_parent = !!(x))
-#define SDO_START_ICONIC(sf) ((sf).common.s.do_start_iconic)
-#define SFDO_START_ICONIC(st) ((st).flags.common.s.do_start_iconic)
-#define SMDO_START_ICONIC(st) ((st).flag_mask.common.s.do_start_iconic)
-#define SCDO_START_ICONIC(st) ((st).change_mask.common.s.do_start_iconic)
-#define SFSET_DO_START_ICONIC(st,x) ((st).flags.common.s.do_start_iconic = !!(x))
-#define SMSET_DO_START_ICONIC(st,x) ((st).flag_mask.common.s.do_start_iconic = !!(x))
-#define SCSET_DO_START_ICONIC(st,x) ((st).change_mask.common.s.do_start_iconic = !!(x))
 #define SDO_WINDOW_LIST_SKIP(sf) ((sf).common.s.do_window_list_skip)
 #define SFDO_WINDOW_LIST_SKIP(st) ((st).flags.common.s.do_window_list_skip)
 #define SMDO_WINDOW_LIST_SKIP(st) ((st).flag_mask.common.s.do_window_list_skip)
@@ -311,6 +312,13 @@
 #define SFSET_ICON_OVERRIDE(st,x) ((st).flags.common.s.icon_override = (x))
 #define SMSET_ICON_OVERRIDE(st,x) ((st).flag_mask.common.s.icon_override = (x))
 #define SCSET_ICON_OVERRIDE(st,x) ((st).change_mask.common.s.icon_override = (x))
+#define SIS_BOTTOM_TITLE_ROTATED(sf) ((sf).common.s.is_bottom_title_rotated)
+#define SFIS_BOTTOM_TITLE_ROTATED(st) ((st).flags.common.s.is_bottom_title_rotated)
+#define SMIS_BOTTOM_TITLE_ROTATED(st) ((st).flag_mask.common.s.is_bottom_title_rotated)
+#define SCIS_BOTTOM_TITLE_ROTATED(st) ((st).change_mask.common.s.is_bottom_title_rotated)
+#define SFSET_IS_BOTTOM_TITLE_ROTATED(st,x) ((st).flags.common.s.is_bottom_title_rotated = !!(x))
+#define SMSET_IS_BOTTOM_TITLE_ROTATED(st,x) ((st).flag_mask.common.s.is_bottom_title_rotated = !!(x))
+#define SCSET_IS_BOTTOM_TITLE_ROTATED(st,x) ((st).change_mask.common.s.is_bottom_title_rotated = !!(x))
 #define SIS_FIXED(sf) ((sf).common.s.is_fixed)
 #define SFIS_FIXED(st) ((st).flags.common.s.is_fixed)
 #define SMIS_FIXED(st) ((st).flag_mask.common.s.is_fixed)
@@ -318,6 +326,13 @@
 #define SFSET_IS_FIXED(st,x) ((st).flags.common.s.is_fixed = !!(x))
 #define SMSET_IS_FIXED(st,x) ((st).flag_mask.common.s.is_fixed = !!(x))
 #define SCSET_IS_FIXED(st,x) ((st).change_mask.common.s.is_fixed = !!(x))
+#define SIS_FIXED_PPOS(sf) ((sf).common.s.is_fixed_ppos)
+#define SFIS_FIXED_PPOS(st) ((st).flags.common.s.is_fixed_ppos)
+#define SMIS_FIXED_PPOS(st) ((st).flag_mask.common.s.is_fixed_ppos)
+#define SCIS_FIXED_PPOS(st) ((st).change_mask.common.s.is_fixed_ppos)
+#define SFSET_IS_FIXED_PPOS(st,x) ((st).flags.common.s.is_fixed_ppos = !!(x))
+#define SMSET_IS_FIXED_PPOS(st,x) ((st).flag_mask.common.s.is_fixed_ppos = !!(x))
+#define SCSET_IS_FIXED_PPOS(st,x) ((st).change_mask.common.s.is_fixed_ppos = !!(x))
 #define SIS_ICON_STICKY(sf) ((sf).common.s.is_icon_sticky)
 #define SFIS_ICON_STICKY(st) ((st).flags.common.s.is_icon_sticky)
 #define SMIS_ICON_STICKY(st) ((st).flag_mask.common.s.is_icon_sticky)
@@ -332,6 +347,13 @@
 #define SFSET_IS_ICON_SUPPRESSED(st,x) ((st).flags.common.s.is_icon_suppressed = !!(x))
 #define SMSET_IS_ICON_SUPPRESSED(st,x) ((st).flag_mask.common.s.is_icon_suppressed = !!(x))
 #define SCSET_IS_ICON_SUPPRESSED(st,x) ((st).change_mask.common.s.is_icon_suppressed = !!(x))
+#define SIS_LEFT_TITLE_ROTATED_CW(sf) ((sf).common.s.is_left_title_rotated_cw)
+#define SFIS_LEFT_TITLE_ROTATED_CW(st) ((st).flags.common.s.is_left_title_rotated_cw)
+#define SMIS_LEFT_TITLE_ROTATED_CW(st) ((st).flag_mask.common.s.is_left_title_rotated_cw)
+#define SCIS_LEFT_TITLE_ROTATED_CW(st) ((st).change_mask.common.s.is_left_title_rotated_cw)
+#define SFSET_IS_LEFT_TITLE_ROTATED_CW(st,x) ((st).flags.common.s.is_left_title_rotated_cw = !!(x))
+#define SMSET_IS_LEFT_TITLE_ROTATED_CW(st,x) ((st).flag_mask.common.s.is_left_title_rotated_cw = !!(x))
+#define SCSET_IS_LEFT_TITLE_ROTATED_CW(st,x) ((st).change_mask.common.s.is_left_title_rotated_cw = !!(x))
 #define SIS_LENIENT(sf) ((sf).common.s.is_lenient)
 #define SFIS_LENIENT(st) ((st).flags.common.s.is_lenient)
 #define SMIS_LENIENT(st) ((st).flag_mask.common.s.is_lenient)
@@ -339,6 +361,34 @@
 #define SFSET_IS_LENIENT(st,x) ((st).flags.common.s.is_lenient = !!(x))
 #define SMSET_IS_LENIENT(st,x) ((st).flag_mask.common.s.is_lenient = !!(x))
 #define SCSET_IS_LENIENT(st,x) ((st).change_mask.common.s.is_lenient = !!(x))
+#define SIS_SIZE_FIXED(sf) ((sf).common.s.is_size_fixed)
+#define SFIS_SIZE_FIXED(st) ((st).flags.common.s.is_size_fixed)
+#define SMIS_SIZE_FIXED(st) ((st).flag_mask.common.s.is_size_fixed)
+#define SCIS_SIZE_FIXED(st) ((st).change_mask.common.s.is_size_fixed)
+#define SFSET_IS_SIZE_FIXED(st,x) ((st).flags.common.s.is_size_fixed = !!(x))
+#define SMSET_IS_SIZE_FIXED(st,x) ((st).flag_mask.common.s.is_size_fixed = !!(x))
+#define SCSET_IS_SIZE_FIXED(st,x) ((st).change_mask.common.s.is_size_fixed = !!(x))
+#define SIS_PSIZE_FIXED(sf) ((sf).common.s.is_psize_fixed)
+#define SFIS_PSIZE_FIXED(st) ((st).flags.common.s.is_psize_fixed)
+#define SMIS_PSIZE_FIXED(st) ((st).flag_mask.common.s.is_psize_fixed)
+#define SCIS_PSIZE_FIXED(st) ((st).change_mask.common.s.is_psize_fixed)
+#define SFSET_IS_PSIZE_FIXED(st,x) ((st).flags.common.s.is_psize_fixed = !!(x))
+#define SMSET_IS_PSIZE_FIXED(st,x) ((st).flag_mask.common.s.is_psize_fixed = !!(x))
+#define SCSET_IS_PSIZE_FIXED(st,x) ((st).change_mask.common.s.is_psize_fixed = !!(x))
+#define SIS_RIGHT_TITLE_ROTATED_CW(sf) ((sf).common.s.is_right_title_rotated_cw)
+#define SFIS_RIGHT_TITLE_ROTATED_CW(st) ((st).flags.common.s.is_right_title_rotated_cw)
+#define SMIS_RIGHT_TITLE_ROTATED_CW(st) ((st).flag_mask.common.s.is_right_title_rotated_cw)
+#define SCIS_RIGHT_TITLE_ROTATED_CW(st) ((st).change_mask.common.s.is_right_title_rotated_cw)
+#define SFSET_IS_RIGHT_TITLE_ROTATED_CW(st,x) ((st).flags.common.s.is_right_title_rotated_cw = !!(x))
+#define SMSET_IS_RIGHT_TITLE_ROTATED_CW(st,x) ((st).flag_mask.common.s.is_right_title_rotated_cw = !!(x))
+#define SCSET_IS_RIGHT_TITLE_ROTATED_CW(st,x) ((st).change_mask.common.s.is_right_title_rotated_cw = !!(x))
+#define SIS_TOP_TITLE_ROTATED(sf) ((sf).common.s.is_top_title_rotated)
+#define SFIS_TOP_TITLE_ROTATED(st) ((st).flags.common.s.is_top_title_rotated)
+#define SMIS_TOP_TITLE_ROTATED(st) ((st).flag_mask.common.s.is_top_title_rotated)
+#define SCIS_TOP_TITLE_ROTATED(st) ((st).change_mask.common.s.is_top_title_rotated)
+#define SFSET_IS_TOP_TITLE_ROTATED(st,x) ((st).flags.common.s.is_top_title_rotated = !!(x))
+#define SMSET_IS_TOP_TITLE_ROTATED(st,x) ((st).flag_mask.common.s.is_top_title_rotated = !!(x))
+#define SCSET_IS_TOP_TITLE_ROTATED(st,x) ((st).change_mask.common.s.is_top_title_rotated = !!(x))
 #define SUSE_ICON_POSITION_HINT(sf) ((sf).common.s.use_icon_position_hint)
 #define SFUSE_ICON_POSITION_HINT(st) ((st).flags.common.s.use_icon_position_hint)
 #define SMUSE_ICON_POSITION_HINT(st) ((st).flag_mask.common.s.use_icon_position_hint)
@@ -346,14 +396,76 @@
 #define SFSET_USE_ICON_POSITION_HINT(st,x) ((st).flags.common.s.use_icon_position_hint = !!(x))
 #define SMSET_USE_ICON_POSITION_HINT(st,x) ((st).flag_mask.common.s.use_icon_position_hint = !!(x))
 #define SCSET_USE_ICON_POSITION_HINT(st,x) ((st).change_mask.common.s.use_icon_position_hint = !!(x))
-
-
-
-
-
-
-
-
+#define SUSE_INDEXED_WINDOW_NAME(sf) ((sf).common.s.use_indexed_window_name)
+#define SFUSE_INDEXED_WINDOW_NAME(st) ((st).flags.common.s.use_indexed_window_name)
+#define SMUSE_INDEXED_WINDOW_NAME(st) ((st).flag_mask.common.s.use_indexed_window_name)
+#define SCUSE_INDEXED_WINDOW_NAME(st) ((st).change_mask.common.s.use_indexed_window_name)
+#define SFSET_USE_INDEXED_WINDOW_NAME(st,x) ((st).flags.common.s.use_indexed_window_name = !!(x))
+#define SMSET_USE_INDEXED_WINDOW_NAME(st,x) ((st).flag_mask.common.s.use_indexed_window_name = !!(x))
+#define SCSET_USE_INDEXED_WINDOW_NAME(st,x) ((st).change_mask.common.s.use_indexed_window_name = !!(x))
+#define SUSE_INDEXED_ICON_NAME(sf) ((sf).common.s.use_indexed_icon_name)
+#define SFUSE_INDEXED_ICON_NAME(st) ((st).flags.common.s.use_indexed_icon_name)
+#define SMUSE_INDEXED_ICON_NAME(st) ((st).flag_mask.common.s.use_indexed_icon_name)
+#define SCUSE_INDEXED_ICON_NAME(st) ((st).change_mask.common.s.use_indexed_icon_name)
+#define SFSET_USE_INDEXED_ICON_NAME(st,x) ((st).flags.common.s.use_indexed_icon_name = !!(x))
+#define SMSET_USE_INDEXED_ICON_NAME(st,x) ((st).flag_mask.common.s.use_indexed_icon_name = !!(x))
+#define SCSET_USE_INDEXED_ICON_NAME(st,x) ((st).change_mask.common.s.use_indexed_icon_name = !!(x))
+#define SWINDOWSHADE_LAZINESS(sf) ((sf).common.s.windowshade_laziness)
+#define SFWINDOWSHADE_LAZINESS(st) ((st).flags.common.s.windowshade_laziness)
+#define SMWINDOWSHADE_LAZINESS(st) ((st).flag_mask.common.s.windowshade_laziness)
+#define SCWINDOWSHADE_LAZINESS(st) ((st).change_mask.common.s.windowshade_laziness)
+#define SFSET_WINDOWSHADE_LAZINESS(st,x) ((st).flags.common.s.windowshade_laziness = (x))
+#define SMSET_WINDOWSHADE_LAZINESS(st,x) ((st).flag_mask.common.s.windowshade_laziness = (x))
+#define SCSET_WINDOWSHADE_LAZINESS(st,x) ((st).change_mask.common.s.windowshade_laziness = (x))
+#define SDO_EWMH_MINI_ICON_OVERRIDE(sf) ((sf).common.s.do_ewmh_mini_icon_override)
+#define SFDO_EWMH_MINI_ICON_OVERRIDE(st) ((st).flags.common.s.do_ewmh_mini_icon_override)
+#define SMDO_EWMH_MINI_ICON_OVERRIDE(st) ((st).flag_mask.common.s.do_ewmh_mini_icon_override)
+#define SCDO_EWMH_MINI_ICON_OVERRIDE(st) ((st).change_mask.common.s.do_ewmh_mini_icon_override)
+#define SFSET_DO_EWMH_MINI_ICON_OVERRIDE(st,x) ((st).flags.common.s.do_ewmh_mini_icon_override = !!(x))
+#define SMSET_DO_EWMH_MINI_ICON_OVERRIDE(st,x) ((st).flag_mask.common.s.do_ewmh_mini_icon_override = !!(x))
+#define SCSET_DO_EWMH_MINI_ICON_OVERRIDE(st,x) ((st).change_mask.common.s.do_ewmh_mini_icon_override = !!(x))
+#define SDO_EWMH_DONATE_ICON(sf) ((sf).common.s.do_ewmh_donate_icon)
+#define SFDO_EWMH_DONATE_ICON(st) ((st).flags.common.s.do_ewmh_donate_icon)
+#define SMDO_EWMH_DONATE_ICON(st) ((st).flag_mask.common.s.do_ewmh_donate_icon)
+#define SCDO_EWMH_DONATE_ICON(st) ((st).change_mask.common.s.do_ewmh_donate_icon)
+#define SFSET_DO_EWMH_DONATE_ICON(st,x) ((st).flags.common.s.do_ewmh_donate_icon = !!(x))
+#define SMSET_DO_EWMH_DONATE_ICON(st,x) ((st).flag_mask.common.s.do_ewmh_donate_icon = !!(x))
+#define SCSET_DO_EWMH_DONATE_ICON(st,x) ((st).change_mask.common.s.do_ewmh_donate_icon = !!(x))
+#define SDO_EWMH_DONATE_MINI_ICON(sf) ((sf).common.s.do_ewmh_donate_mini_icon)
+#define SFDO_EWMH_DONATE_MINI_ICON(st) ((st).flags.common.s.do_ewmh_donate_mini_icon)
+#define SMDO_EWMH_DONATE_MINI_ICON(st) ((st).flag_mask.common.s.do_ewmh_donate_mini_icon)
+#define SCDO_EWMH_DONATE_MINI_ICON(st) ((st).change_mask.common.s.do_ewmh_donate_mini_icon)
+#define SFSET_DO_EWMH_DONATE_MINI_ICON(st,x) ((st).flags.common.s.do_ewmh_donate_mini_icon = !!(x))
+#define SMSET_DO_EWMH_DONATE_MINI_ICON(st,x) ((st).flag_mask.common.s.do_ewmh_donate_mini_icon = !!(x))
+#define SCSET_DO_EWMH_DONATE_MINI_ICON(st,x) ((st).change_mask.common.s.do_ewmh_donate_mini_icon = !!(x))
+#define SDO_EWMH_USE_STACKING_HINTS(sf) ((sf).common.s.do_ewmh_use_stacking_hints)
+#define SFDO_EWMH_USE_STACKING_HINTS(st) ((st).flags.common.s.do_ewmh_use_stacking_hints)
+#define SMDO_EWMH_USE_STACKING_HINTS(st) ((st).flag_mask.common.s.do_ewmh_use_stacking_hints)
+#define SCDO_EWMH_USE_STACKING_HINTS(st) ((st).change_mask.common.s.do_ewmh_use_stacking_hints)
+#define SFSET_DO_EWMH_USE_STACKING_HINTS(st,x) ((st).flags.common.s.do_ewmh_use_stacking_hints = !!(x))
+#define SMSET_DO_EWMH_USE_STACKING_HINTS(st,x) ((st).flag_mask.common.s.do_ewmh_use_stacking_hints = !!(x))
+#define SCSET_DO_EWMH_USE_STACKING_HINTS(st,x) ((st).change_mask.common.s.do_ewmh_use_stacking_hints = !!(x))
+#define SDO_EWMH_IGNORE_STRUT_HINTS(sf) ((sf).common.s.do_ewmh_ignore_strut_hints)
+#define SFDO_EWMH_IGNORE_STRUT_HINTS(st) ((st).flags.common.s.do_ewmh_ignore_strut_hints)
+#define SMDO_EWMH_IGNORE_STRUT_HINTS(st) ((st).flag_mask.common.s.do_ewmh_ignore_strut_hints)
+#define SCDO_EWMH_IGNORE_STRUT_HINTS(st) ((st).change_mask.common.s.do_ewmh_ignore_strut_hints)
+#define SFSET_DO_EWMH_IGNORE_STRUT_HINTS(st,x) ((st).flags.common.s.do_ewmh_ignore_strut_hints = !!(x))
+#define SMSET_DO_EWMH_IGNORE_STRUT_HINTS(st,x) ((st).flag_mask.common.s.do_ewmh_ignore_strut_hints = !!(x))
+#define SCSET_DO_EWMH_IGNORE_STRUT_HINTS(st,x) ((st).change_mask.common.s.do_ewmh_ignore_strut_hints = !!(x))
+#define SDO_EWMH_IGNORE_STATE_HINTS(sf) ((sf).common.s.do_ewmh_ignore_state_hints)
+#define SFDO_EWMH_IGNORE_STATE_HINTS(st) ((st).flags.common.s.do_ewmh_ignore_state_hints)
+#define SMDO_EWMH_IGNORE_STATE_HINTS(st) ((st).flag_mask.common.s.do_ewmh_ignore_state_hints)
+#define SCDO_EWMH_IGNORE_STATE_HINTS(st) ((st).change_mask.common.s.do_ewmh_ignore_state_hints)
+#define SFSET_DO_EWMH_IGNORE_STATE_HINTS(st,x) ((st).flags.common.s.do_ewmh_ignore_state_hints = !!(x))
+#define SMSET_DO_EWMH_IGNORE_STATE_HINTS(st,x) ((st).flag_mask.common.s.do_ewmh_ignore_state_hints = !!(x))
+#define SCSET_DO_EWMH_IGNORE_STATE_HINTS(st,x) ((st).change_mask.common.s.do_ewmh_ignore_state_hints = !!(x))
+#define SEWMH_MAXIMIZE_MODE(sf) ((sf).common.s.ewmh_maximize_mode)
+#define SFEWMH_MAXIMIZE_MODE(st) ((st).flags.common.s.ewmh_maximize_mode)
+#define SMEWMH_MAXIMIZE_MODE(st) ((st).flag_mask.common.s.ewmh_maximize_mode)
+#define SCEWMH_MAXIMIZE_MODE(st) ((st).change_mask.common.s.ewmh_maximize_mode)
+#define SFSET_EWMH_MAXIMIZE_MODE(st,x) ((st).flags.common.s.ewmh_maximize_mode = (x))
+#define SMSET_EWMH_MAXIMIZE_MODE(st,x) ((st).flag_mask.common.s.ewmh_maximize_mode = (x))
+#define SCSET_EWMH_MAXIMIZE_MODE(st,x) ((st).change_mask.common.s.ewmh_maximize_mode = (x))
 
 
 
@@ -368,10 +480,8 @@
 #define SSET_NAME(s,x)                ((s).name = (x))
 #define SGET_ICON_NAME(s)             ((s).icon_name)
 #define SSET_ICON_NAME(s,x)           ((s).icon_name = (x))
-#ifdef MINI_ICONS
 #define SGET_MINI_ICON_NAME(s)        ((s).mini_icon_name)
 #define SSET_MINI_ICON_NAME(s,x)      ((s).mini_icon_name = (x))
-#endif
 #ifdef USEDECOR
 #define SGET_DECOR_NAME(s)            ((s).decor_name)
 #define SSET_DECOR_NAME(s,x)          ((s).decor_name = (x))
@@ -409,6 +519,8 @@
 #define SSET_START_PAGE_X(s,x)        ((s).start_page_x = (x))
 #define SGET_START_PAGE_Y(s)          ((s).start_page_y)
 #define SSET_START_PAGE_Y(s,x)        ((s).start_page_y = (x))
+#define SGET_START_SCREEN(s)          ((s).start_screen)
+#define SSET_START_SCREEN(s,x)        ((s).start_screen = (x))
 #define SGET_MAX_WINDOW_WIDTH(s)      ((s).max_window_width)
 #define SSET_MAX_WINDOW_WIDTH(s,x)    ((s).max_window_width = (x))
 #define SGET_MAX_WINDOW_HEIGHT(s)     ((s).max_window_height)
@@ -417,17 +529,44 @@
 #define SSET_WINDOW_SHADE_STEPS(s,x)  ((s).shade_anim_steps = (x))
 #define SGET_ICON_BOXES(s)            ((s).icon_boxes)
 #define SSET_ICON_BOXES(s,x)          ((s).icon_boxes = (x))
-
+#define SGET_NORMAL_PLACEMENT_PENALTY(s)       ((s).placement_penalty[0])
+#define SGET_ONTOP_PLACEMENT_PENALTY(s)        ((s).placement_penalty[1])
+#define SGET_ICON_PLACEMENT_PENALTY(s)         ((s).placement_penalty[2])
+#define SGET_STICKY_PLACEMENT_PENALTY(s)       ((s).placement_penalty[3])
+#define SGET_BELOW_PLACEMENT_PENALTY(s)        ((s).placement_penalty[4])
+#define SGET_EWMH_STRUT_PLACEMENT_PENALTY(s)   ((s).placement_penalty[5])
+#define SSET_NORMAL_PLACEMENT_PENALTY(s,x)     ((s).placement_penalty[0] = (x))
+#define SSET_ONTOP_PLACEMENT_PENALTY(s,x)      ((s).placement_penalty[1] = (x))
+#define SSET_ICON_PLACEMENT_PENALTY(s,x)       ((s).placement_penalty[2] = (x))
+#define SSET_STICKY_PLACEMENT_PENALTY(s,x)     ((s).placement_penalty[3] = (x))
+#define SSET_BELOW_PLACEMENT_PENALTY(s,x)      ((s).placement_penalty[4] = (x))
+#define SSET_EWMH_STRUT_PLACEMENT_PENALTY(s,x) ((s).placement_penalty[5] = (x))
+#define SGET_99_PLACEMENT_PERCENTAGE_PENALTY(s) \
+                                    ((s).placement_percentage_penalty[0])
+#define SGET_95_PLACEMENT_PERCENTAGE_PENALTY(s) \
+                                    ((s).placement_percentage_penalty[1])
+#define SGET_85_PLACEMENT_PERCENTAGE_PENALTY(s) \
+                                    ((s).placement_percentage_penalty[2])
+#define SGET_75_PLACEMENT_PERCENTAGE_PENALTY(s) \
+                                    ((s).placement_percentage_penalty[3])
+#define SSET_99_PLACEMENT_PERCENTAGE_PENALTY(s,x) \
+                                    ((s).placement_percentage_penalty[0] = (x))
+#define SSET_95_PLACEMENT_PERCENTAGE_PENALTY(s,x) \
+                                    ((s).placement_percentage_penalty[1] = (x))
+#define SSET_85_PLACEMENT_PERCENTAGE_PENALTY(s,x) \
+                                    ((s).placement_percentage_penalty[2] = (x))
+#define SSET_75_PLACEMENT_PERCENTAGE_PENALTY(s,x) \
+                                    ((s).placement_percentage_penalty[3] = (x))
 
 /* function prototypes */
-void lookup_style(FvwmWindow *tmp_win, window_style *styles);
+void lookup_style(FvwmWindow *fw, window_style *styles);
 Bool blockcmpmask(char *blk1, char *blk2, char *mask, int length);
 void check_window_style_change(
   FvwmWindow *t, update_win *flags, window_style *ret_style);
 void reset_style_changes(void);
 void update_style_colorset(int colorset);
-void update_window_color_style(FvwmWindow *tmp_win, window_style *style);
-void update_window_color_hi_style(FvwmWindow *tmp_win, window_style *style);
+void update_window_color_style(FvwmWindow *fw, window_style *style);
+void update_window_color_hi_style(FvwmWindow *fw, window_style *style);
 void free_icon_boxes(icon_boxes *ib);
 
 #endif /* _STYLE_ */
