@@ -40,6 +40,12 @@
           (fw)->flags.common.s.do_not_raise_click_focus_click = !!(x)
 #define SETM_DO_NOT_RAISE_CLICK_FOCUS_CLICK(fw,x) \
           (fw)->flag_mask.common.s.do_not_raise_click_focus_click = !!(x)
+#define DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw)  \
+          ((fw)->flags.common.s.do_ignore_mouse_focus_click_motion)
+#define SET_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw,x) \
+          (fw)->flags.common.s.do_ignore_mouse_focus_click_motion = !!(x)
+#define SETM_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw,x) \
+          (fw)->flag_mask.common.s.do_ignore_mouse_focus_click_motion = !!(x)
 #define DO_RAISE_MOUSE_FOCUS_CLICK(fw)  \
           ((fw)->flags.common.s.do_raise_mouse_focus_click)
 #define SET_DO_RAISE_MOUSE_FOCUS_CLICK(fw,x) \
@@ -80,17 +86,13 @@
           (fw)->flag_mask.common.s.do_window_list_skip = !!(x)
 #define DO_STACK_TRANSIENT_PARENT(fw) \
           ((fw)->flags.common.s.do_stack_transient_parent)
-#define DO_START_ICONIC(fw)    ((fw)->flags.common.s.do_start_iconic)
-#define SET_DO_START_ICONIC(fw,x) \
-          (fw)->flags.common.s.do_start_iconic = !!(x)
-#define SETM_DO_START_ICONIC(fw,x) \
-          (fw)->flag_mask.common.s.do_start_iconic = !!(x)
-
-#define HAS_BOTTOM_TITLE(fw)   ((fw)->flags.common.s.has_bottom_title)
-#define SET_HAS_BOTTOM_TITLE(fw,x) \
-          (fw)->flags.common.s.has_bottom_title = !!(x)
-#define SETM_HAS_BOTTOM_TITLE(fw,x) \
-          (fw)->flag_mask.common.s.has_bottom_title = !!(x)
+#define GET_TITLE_DIR(fw)    ((fw)->flags.common.title_dir)
+#define HAS_TITLE_DIR(fw,x)  ((fw)->flags.common.title_dir == x)
+#define SET_TITLE_DIR(fw,x)  ((fw)->flags.common.title_dir = x)
+#define SETM_TITLE_DIR(fw,x) \
+	  (fw)->flag_mask.common.title_dir = DIR_MAJOR_MASK * !!(x)
+#define HAS_VERTICAL_TITLE(fw) \
+          (HAS_TITLE_DIR(fw,DIR_W) || HAS_TITLE_DIR(fw,DIR_E))
 #define HAS_STIPPLED_TITLE(fw)  \
           ((fw)->flags.common.s.has_stippled_title)
 #define SET_HAS_STIPPLED_TITLE(fw,x) \
@@ -100,9 +102,10 @@
 #define ICON_OVERRIDE_MODE(fw)  \
           ((fw)->flags.common.s.icon_override)
 #define SET_ICON_OVERRIDE_MODE(fw,x)  \
-          (fw)->flags.common.s.icon_override = !!(x)
+          (fw)->flags.common.s.icon_override = ((x) & ICON_OVERRIDE_MASK)
 #define SETM_ICON_OVERRIDE_MODE(fw,x) \
-          (fw)->flag_mask.common.s.icon_override = !!(x)
+          (fw)->flag_mask.common.s.icon_override = \
+                  ((x) ? ICON_OVERRIDE_MASK : 0)
 #define IS_ICON_STICKY(fw)     ((fw)->flags.common.s.is_icon_sticky)
 #define SET_ICON_STICKY(fw,x)  (fw)->flags.common.s.is_icon_sticky = !!(x)
 #define SETM_ICON_STICKY(fw,x) \
@@ -113,6 +116,68 @@
           (fw)->flags.common.s.use_icon_position_hint = !!(x)
 #define SETM_USE_ICON_POSITION_HINT(fw,x) \
           (fw)->flag_mask.common.s.use_icon_position_hint = !!(x)
+#define USE_INDEXED_WINDOW_NAME(fw) \
+          ((fw)->flags.common.s.use_indexed_window_name)
+#define SET_USE_INDEXED_WINDOW_NAME(fw,x) \
+          (fw)->flags.common.s.use_indexed_window_name = !!(x)
+#define SETM_USE_INDEXED_WINDOW_NAME(fw,x) \
+          (fw)->flag_mask.common.s.use_indexed_window_name = !!(x)
+#define USE_INDEXED_ICON_NAME(fw) \
+          ((fw)->flags.common.s.use_indexed_icon_name)
+#define SET_USE_INDEXED_ICON_NAME(fw,x) \
+          (fw)->flags.common.s.use_indexed_icon_name = !!(x)
+#define SETM_USE_INDEXED_ICON_NAME(fw,x) \
+          (fw)->flag_mask.common.s.use_indexed_icon_name = !!(x)
+#define WINDOWSHADE_LAZINESS(fw)  \
+          ((fw)->flags.common.s.windowshade_laziness)
+#define SET_WINDOWSHADE_LAZINESS(fw,x)  \
+          (fw)->flags.common.s.windowshade_laziness = \
+		((x) & WINDOWSHADE_LAZY_MASK)
+#define SETM_WINDOWSHADE_LAZINESS(fw,x) \
+          (fw)->flag_mask.common.s.windowshade_laziness = \
+		((x) ? WINDOWSHADE_LAZY_MASK : 0)
+#define DO_EWMH_MINI_ICON_OVERRIDE(fw) \
+          ((fw)->flags.common.s.do_ewmh_mini_icon_override)
+#define SET_DO_EWMH_MINI_ICON_OVERRIDE(fw,x) \
+          (fw)->flags.common.s.do_ewmh_mini_icon_override = !!(x)
+#define SETM_DO_EWMH_MINI_ICON_OVERRIDE(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_mini_icon_override = !!(x)
+#define DO_EWMH_DONATE_ICON(fw) \
+          ((fw)->flags.common.s.do_ewmh_donate_icon)
+#define SET_DO_EWMH_DONATE_ICON(fw,x) \
+          (fw)->flags.common.s.do_ewmh_donate_icon = !!(x)
+#define SETM_DO_EWMH_DONATE_ICON(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_donate_icon = !!(x)
+#define DO_EWMH_DONATE_MINI_ICON(fw) \
+          ((fw)->flags.common.s.do_ewmh_donate_mini_icon)
+#define SET_DO_EWMH_DONATE_MINI_ICON(fw,x) \
+          (fw)->flags.common.s.do_ewmh_donate_mini_icon = !!(x)
+#define SETM_DO_EWMH_DONATE_MINI_ICON(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_donate_mini_icon = !!(x)
+#define DO_EWMH_USE_STACKING_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_use_stacking_hints)
+#define SET_DO_EWMH_USE_STACKING_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_use_stacking_hints = !!(x)
+#define SETM_DO_EWMH_USE_STACKING_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_use_stacking_hints = !!(x)
+#define DO_EWMH_IGNORE_STRUT_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_ignore_strut_hints)
+#define SET_DO_EWMH_IGNORE_STRUT_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_ignore_strut_hints = !!(x)
+#define SETM_DO_EWMH_IGNORE_STRUT_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_ignore_strut_hints = !!(x)
+#define DO_EWMH_IGNORE_STATE_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_ignore_state_hints)
+#define SET_DO_EWMH_IGNORE_STATE_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_ignore_state_hints = !!(x)
+#define SETM_DO_EWMH_IGNORE_STATE_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_ignore_state_hints = !!(x)
+#define EWMH_MAXIMIZE_MODE(fw) \
+          ((fw)->flags.common.s.ewmh_maximize_mode)
+#define SET_EWMH_MAXIMIZE_MODE(fw,x) \
+          (fw)->flags.common.s.ewmh_maximize_mode = (x)
+#define SETM_EWMH_MAXIMIZE_MODE(fw,x) \
+          (fw)->flag_mask.common.s.ewmh_maximize_mode = (x)
 #define IS_ICON_SUPPRESSED(fw) \
           ((fw)->flags.common.s.is_icon_suppressed)
 #define SET_ICON_SUPPRESSED(fw,x)  \
@@ -173,16 +238,45 @@
 #define IS_FIXED(fw)           ((fw)->flags.common.s.is_fixed)
 #define SET_FIXED(fw,x)        (fw)->flags.common.s.is_fixed = !!(x)
 #define SETM_FIXED(fw,x)       (fw)->flag_mask.common.s.is_fixed = !!(x)
+#define IS_FIXED_PPOS(fw)      ((fw)->flags.common.s.is_fixed_ppos)
+#define SET_FIXED_PPOS(fw,x)   (fw)->flags.common.s.is_fixed_ppos = !!(x)
+#define SETM_FIXED_PPOS(fw,x)  (fw)->flag_mask.common.s.is_fixed_ppos = !!(x)
+#define IS_SIZE_FIXED(fw)      ((fw)->flags.common.s.is_size_fixed)
+#define SET_SIZE_FIXED(fw,x)   (fw)->flags.common.s.is_size_fixed = !!(x)
+#define SETM_SIZE_FIXED(fw,x)  (fw)->flag_mask.common.s.is_size_fixed = !!(x)
+#define IS_PSIZE_FIXED(fw) ((fw)->flags.common.s.is_psize_fixed)
+#define SET_PSIZE_FIXED(fw,x) \
+                               (fw)->flags.common.s.is_psize_fixed = !!(x)
+#define SETM_PSIZE_FIXED(fw,x) \
+                            (fw)->flag_mask.common.s.is_psize_fixed = !!(x)
 #define HAS_DEPRESSABLE_BORDER(fw) ((fw)->flags.common.s.has_depressable_border)
 
+#define IS_LEFT_TITLE_ROTATED_CW(fw) \
+          ((fw)->flags.common.s.is_left_title_rotated_cw)
+#define SET_IS_LEFT_TITLE_ROTATED_CW(fw,x) \
+          (fw)->flags.common.s.is_left_title_rotated_cw = !!(x)
+#define SETM_IS_LEFT_TITLE_ROTATED_CW(fw,x) \
+          (fw)->flag_mask.common.s.is_left_title_rotated_cw = !!(x)
+#define IS_RIGHT_TITLE_ROTATED_CW(fw) \
+          ((fw)->flags.common.s.is_right_title_rotated_cw)
+#define SET_IS_RIGHT_TITLE_ROTATED_CW(fw,x) \
+          (fw)->flags.common.s.is_right_title_rotated_cw = !!(x)
+#define SETM_IS_RIGHT_TITLE_ROTATED_CW(fw,x) \
+          (fw)->flag_mask.common.s.is_right_title_rotated_cw = !!(x)
+#define IS_BOTTOM_TITLE_ROTATED(fw) \
+          ((fw)->flags.common.s.is_bottom_title_rotated)
+#define SET_IS_BOTTOM_TITLE_ROTATED(fw,x) \
+          (fw)->flags.common.s.is_bottom_title_rotated = !!(x)
+#define SETM_IS_BOTTOM_TITLE_ROTATED(fw,x) \
+          (fw)->flag_mask.common.s.is_bottom_title_rotated = !!(x)
+#define IS_BOTTOM_TITLE_ROTATED(fw) \
+          ((fw)->flags.common.s.is_bottom_title_rotated)
+#define SET_IS_BOTTOM_TITLE_ROTATED(fw,x) \
+          (fw)->flags.common.s.is_bottom_title_rotated = !!(x)
+#define SETM_IS_BOTTOM_TITLE_ROTATED(fw,x) \
+          (fw)->flag_mask.common.s.is_bottom_title_rotated = !!(x)
 
 /* access to the special flags of a window */
-#define DO_DELETE_ICON_MOVED(fw) \
-                               ((fw)->flags.do_delete_icon_moved)
-#define SET_DELETE_ICON_MOVED(fw,x) \
-                               (fw)->flags.do_delete_icon_moved = !!(x)
-#define SETM_DELETE_ICON_MOVED(fw,x) \
-                               (fw)->flag_mask.do_delete_icon_moved = !!(x)
 #define DO_REUSE_DESTROYED(fw) ((fw)->flags.do_reuse_destroyed)
 #define SET_DO_REUSE_DESTROYED(fw,x) \
           (fw)->flags.do_reuse_destroyed = !!(x)
@@ -191,6 +285,12 @@
 #define HAS_BORDER(fw)         ((fw)->flags.has_border)
 #define SET_HAS_BORDER(fw,x)   (fw)->flags.has_border = !!(x)
 #define SETM_HAS_BORDER(fw,x)  (fw)->flag_mask.has_border = !!(x)
+#define HAS_HANDLES(fw)        ((fw)->flags.has_handles)
+#define SET_HAS_HANDLES(fw,x)  (fw)->flags.has_handles = !!(x)
+#define SETM_HAS_HANDLES(fw,x) (fw)->flag_mask.has_handles = !!(x)
+#define HAS_ICON_CHANGED(fw)         ((fw)->flags.has_icon_changed)
+#define SET_HAS_ICON_CHANGED(fw,x)   (fw)->flags.has_icon_changed = !!(x)
+#define SETM_HAS_ICON_CHANGED(fw,x)  (fw)->flag_mask.has_icon_changed = !!(x)
 #define HAS_TITLE(fw)          ((fw)->flags.has_title)
 #define SET_HAS_TITLE(fw,x)    (fw)->flags.has_title = !!(x)
 #define SETM_HAS_TITLE(fw,x)   (fw)->flag_mask.has_title = !!(x)
@@ -247,14 +347,22 @@
 #define IS_PLACED_WB3(fw)      ((fw)->flags.is_placed_wb3)
 #define SET_PLACED_WB3(fw,x)   (fw)->flags.is_placed_wb3 = !!(x)
 #define SETM_PLACED_WB3(fw,x)  (fw)->flag_mask.is_placed_wb3 = !!(x)
+#define IS_PLACED_BY_FVWM(fw)      ((fw)->flags.is_placed_by_fvwm)
+#define SET_PLACED_BY_FVWM(fw,x)   (fw)->flags.is_placed_by_fvwm = (x)
+#define SETM_PLACED_BY_FVWM(fw,x)  (fw)->flag_mask.is_placed_by_fvwm = (x)
 #define IS_SCHEDULED_FOR_DESTROY(fw)    ((fw)->flags.is_scheduled_for_destroy)
 #define SET_SCHEDULED_FOR_DESTROY(fw,x) \
           (fw)->flags.is_scheduled_for_destroy = !!(x)
 #define IS_SCHEDULED_FOR_RAISE(fw)    ((fw)->flags.is_scheduled_for_raise)
 #define SET_SCHEDULED_FOR_RAISE(fw,x) (fw)->flags.is_scheduled_for_raise = !!(x)
 #define IS_SHADED(fw)          ((fw)->flags.is_window_shaded)
+#define SHADED_DIR(fw)         ((fw)->flags.shaded_dir)
 #define SET_SHADED(fw,x)       (fw)->flags.is_window_shaded = !!(x)
+#define SET_SHADED_DIR(fw,x)   (fw)->flags.shaded_dir = (x)
 #define SETM_SHADED(fw,x)      (fw)->flag_mask.is_window_shaded = !!(x)
+#define IS_TEAR_OFF_MENU(fw)   ((fw)->flags.is_tear_off_menu)
+#define SET_TEAR_OFF_MENU(fw,x) (fw)->flags.is_tear_off_menu = !!(x)
+#define SETM_TEAR_OFF_MENU(fw,x) (fw)->flag_mask.is_tear_off_menu = !!(x)
 #define IS_TRANSIENT(fw)       ((fw)->flags.is_transient)
 #define SET_TRANSIENT(fw,x)    (fw)->flags.is_transient = !!(x)
 #define SETM_TRANSIENT(fw,x)   (fw)->flag_mask.is_transient = !!(x)
@@ -269,6 +377,9 @@
 #define IS_SIZE_INC_SET(fw)     ((fw)->flags.is_size_inc_set)
 #define SET_SIZE_INC_SET(fw,x)  (fw)->flags.is_size_inc_set = !!(x)
 #define SETM_SIZE_INC_SET(fw,x) (fw)->flag_mask.is_size_inc_set = !!(x)
+#define IS_STYLE_DELETED(fw)     ((fw)->flags.is_style_deleted)
+#define SET_STYLE_DELETED(fw,x)  (fw)->flags.is_style_deleted = !!(x)
+#define SETM_STYLE_DELETED(fw,x) (fw)->flag_mask.is_style_deleted = !!(x)
 #define IS_VIEWPORT_MOVED(fw)  ((fw)->flags.is_viewport_moved)
 #define SET_VIEWPORT_MOVED(fw,x) \
           (fw)->flags.is_viewport_moved = !!(x)
@@ -323,6 +434,12 @@
           (fw)->flags.does_wm_take_focus = !!(x)
 #define SETM_WM_TAKES_FOCUS(fw,x) \
           (fw)->flag_mask.does_wm_take_focus = !!(x)
+#define DO_FORCE_NEXT_CR(fw)        ((fw)->flags.do_force_next_cr)
+#define SET_FORCE_NEXT_CR(fw,x)  (fw)->flags.do_force_next_cr = !!(x)
+#define SETM_FORCE_NEXT_CR(fw,x) (fw)->flag_mask.do_force_next_cr = !!(x)
+#define DO_FORCE_NEXT_PN(fw)     ((fw)->flags.do_force_next_pn)
+#define SET_FORCE_NEXT_PN(fw,x)  (fw)->flags.do_force_next_pn = !!(x)
+#define SETM_FORCE_NEXT_PN(fw,x) (fw)->flag_mask.do_force_next_pn = !!(x)
 #define USING_DEFAULT_WINDOW_FONT(fw) \
           ((fw)->flags.using_default_window_font)
 #define SET_USING_DEFAULT_WINDOW_FONT(fw,x) \
@@ -347,6 +464,97 @@
           (fw)->flags.was_icon_name_provided = (x)
 #define SETM_WAS_ICON_NAME_PROVIDED(fw,x) \
           (fw)->flag_mask.was_icon_name_provided = (x)
-
+#define HAS_EWMH_WM_NAME(fw)        ((fw)->flags.has_ewmh_wm_name)
+#define SET_HAS_EWMH_WM_NAME(fw,x)  (fw)->flags.has_ewmh_wm_name = !!(x)
+#define SETM_HAS_EWMH_WM_NAME(fw,x) (fw)->flag_mask.has_ewmh_wm_name = !!(x)
+#define HAS_EWMH_WM_ICON_NAME(fw) \
+          ((fw)->flags.has_ewmh_wm_icon_name)
+#define SET_HAS_EWMH_WM_ICON_NAME(fw,x) \
+          (fw)->flags.has_ewmh_wm_icon_name = !!(x)
+#define SETM_HAS_EWMH_WM_ICON_NAME(fw,x) \
+          (fw)->flag_mask.has_ewmh_wm_icon_name = !!(x)
+#define HAS_EWMH_WM_ICON_HINT(fw) ((fw)->flags.has_ewmh_wm_icon_hint)
+#define SET_HAS_EWMH_WM_ICON_HINT(fw,x)   (fw)->flags.has_ewmh_wm_icon_hint = (x)
+#define SETM_HAS_EWMH_WM_ICON_HINT(fw,x)  (fw)->flag_mask.has_ewmh_wm_icon_hint = (x)
+#define USE_EWMH_ICON(fw) ((fw)->flags.use_ewmh_icon)
+#define SET_USE_EWMH_ICON(fw,x) (fw)->flags.use_ewmh_icon = !!(x)
+#define SETM_USE_EWMH_ICON(fw,x) (fw)->flag_mask.use_ewmh_icon = !!(x)
+#define HAS_EWMH_MINI_ICON(fw) ((fw)->flags.has_ewmh_mini_icon)
+#define SET_HAS_EWMH_MINI_ICON(fw,x) (fw)->flags.has_ewmh_mini_icon = !!(x)
+#define SETM_HAS_EWMH_MINI_ICON(fw,x) (fw)->flag_mask.has_ewmh_mini_icon = !!(x)
+#define IS_EWMH_MODAL(fw)     ((fw)->flags.is_ewmh_modal)
+#define SET_EWMH_MODAL(fw,x)  (fw)->flags.is_ewmh_modal = !!(x)
+#define SETM_EWMH_MODAL(fw,x) \
+          (fw)->flag_mask.is_ewmh_modal = !!(x)
+#define SET_HAS_EWMH_INIT_FULLSCREEN_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_fullscreen_state = (x)
+#define SETM_HAS_EWMH_INIT_FULLSCREEN_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_fullscreen_state = (x)
+#define HAS_EWMH_INIT_FULLSCREEN_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_fullscreen_state)
+#define SET_HAS_EWMH_INIT_HIDDEN_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_hidden_state = (x)
+#define SETM_HAS_EWMH_INIT_HIDDEN_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_hidden_state = (x)
+#define HAS_EWMH_INIT_HIDDEN_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_hidden_state)
+#define SET_HAS_EWMH_INIT_MAXHORIZ_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_maxhoriz_state = (x)
+#define SETM_HAS_EWMH_INIT_MAXHORIZ_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_maxhoriz_state = (x)
+#define HAS_EWMH_INIT_MAXHORIZ_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_maxhoriz_state)
+#define SET_HAS_EWMH_INIT_MAXVERT_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_maxvert_state = (x)
+#define SETM_HAS_EWMH_INIT_MAXVERT_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_maxvert_state = (x)
+#define HAS_EWMH_INIT_MAXVERT_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_fullscreen_state)
+#define SET_HAS_EWMH_INIT_MODAL_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_modal_state = (x)
+#define SETM_HAS_EWMH_INIT_MODAL_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_modal_state = (x)
+#define HAS_EWMH_INIT_MODAL_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_modal_state)
+#define SET_HAS_EWMH_INIT_SHADED_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_shaded_state = (x)
+#define SETM_HAS_EWMH_INIT_SHADED_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_shaded_state = (x)
+#define HAS_EWMH_INIT_SHADED_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_shaded_state)
+#define SET_HAS_EWMH_INIT_SKIP_PAGER_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_skip_pager_state = (x)
+#define SETM_HAS_EWMH_INIT_SKIP_PAGER_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_skip_pager_state = (x)
+#define HAS_EWMH_INIT_SKIP_PAGER_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_skip_pager_state)
+#define SET_HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_skip_taskbar_state = (x)
+#define SETM_HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_skip_taskbar_state = (x)
+#define HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_skip_taskbar_state)
+#define SET_HAS_EWMH_INIT_STICKY_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_sticky_state = (x)
+#define SETM_HAS_EWMH_INIT_STICKY_STATE(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_sticky_state = (x)
+#define HAS_EWMH_INIT_STICKY_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_sticky_state)
+#define SET_HAS_EWMH_INIT_WM_DESKTOP(fw,x) \
+          (fw)->flags.has_ewmh_init_wm_desktop = (x)
+#define SETM_HAS_EWMH_INIT_WM_DESKTOP(fw,x) \
+          (fw)->flag_mask.has_ewmh_init_wm_desktop = (x)
+#define HAS_EWMH_INIT_WM_DESKTOP(fw) \
+          ((fw)->flags.has_ewmh_init_wm_desktop)
+#define SET_USER_STATES(fw, mask) \
+          (fw)->flags.user_states |= (mask)
+#define CLEAR_USER_STATES(fw, mask) \
+          (fw)->flags.user_states &= ~(mask)
+#define TOGGLE_USER_STATES(fw, mask) \
+          (fw)->flags.user_states ^= (mask)
+#define SETM_USER_STATES(fw, mask) \
+          (fw)->flag_mask.user_states |= (mask)
+#define GET_USER_STATES(fw) \
+          ((fw)->flags.user_states)
 
 #endif /* _WINDOW_FLAGS_ */
