@@ -38,24 +38,33 @@ static Bool is_grabbing_everything = False;
 /* The keys must be in lower case! */
 static struct charstring win_contexts[]=
 {
-  {'w',C_WINDOW},
-  {'t',C_TITLE},
-  {'i',C_ICON},
-  {'r',C_ROOT},
-  {'f',C_FRAME},
-  {'s',C_SIDEBAR},
-  {'1',C_L1},
-  {'2',C_R1},
-  {'3',C_L2},
-  {'4',C_R2},
-  {'5',C_L3},
-  {'6',C_R3},
-  {'7',C_L4},
-  {'8',C_R4},
-  {'9',C_L5},
-  {'0',C_R5},
-  {'a',C_ALL},
-  {0,0}
+  {'d', C_EWMH_DESKTOP},
+  {'f', C_FRAME},
+  {'<', C_F_TOPLEFT},
+  {'^', C_F_TOPRIGHT},
+  {'>', C_F_BOTTOMRIGHT},
+  {'v', C_F_BOTTOMLEFT},
+  {'i', C_ICON},
+  {'r', C_ROOT},
+  {'s', C_SIDEBAR},
+  {'[', C_SB_LEFT},
+  {']', C_SB_RIGHT},
+  {'-', C_SB_TOP},
+  {'_', C_SB_BOTTOM},
+  {'t', C_TITLE},
+  {'w', C_WINDOW},
+  {'1', C_L1},
+  {'2', C_R1},
+  {'3', C_L2},
+  {'4', C_R2},
+  {'5', C_L3},
+  {'6', C_R3},
+  {'7', C_L4},
+  {'8', C_R4},
+  {'9', C_L5},
+  {'0', C_R5},
+  {'a', C_ALL},
+  {0, 0}
 };
 
 /* The keys musat be in lower case! */
