@@ -35,13 +35,17 @@ typedef struct {
   unsigned int shape_width : 12;
   unsigned int shape_height : 12;
   unsigned int shape_type : 2;
-#ifdef FVWMTHEME_PRIVATE
-  /* FvwmTheme use only. */
+#ifdef FVWM_COLORSET_PRIVATE
+  /* fvwm/colorset.c use only */
   Pixmap mask;
-  unsigned int color_flags : 6;
-  Picture *picture;
+  Pixmap alpha_pixmap;
+  unsigned int color_flags : 7;
+  FvwmPicture *picture;
   Pixel *pixels;
   int nalloc_pixels;
+  Pixel tint;
+  int tint_percent;
+  Bool do_tint_use_mask;
 #endif
 } colorset_struct;
 
@@ -54,21 +58,22 @@ typedef struct {
 #define SHAPE_TILED 0
 #define SHAPE_STRETCH 1
 #define SHAPE_STRETCH_ASPECT 2
-
-#ifdef FVWMTHEME_PRIVATE
+#ifdef FVWM_COLORSET_PRIVATE
 #define FG_SUPPLIED 0x1
 #define BG_SUPPLIED 0x2
 #define HI_SUPPLIED 0x4
 #define SH_SUPPLIED 0x8
 #define FG_CONTRAST 0x10
 #define BG_AVERAGE  0x20
+#define TINT_SUPPLIED  0x40
 #endif
 
 /* colorsets are stored as an array of structs to permit fast dereferencing */
 extern colorset_struct *Colorset;
 
-#ifndef FVWMTHEME_PRIVATE
-/* Create n new colorsets, FvwmTheme does it's own thing (different size) */
+#ifndef FVWM_COLORSET_PRIVATE
+/* Create n new colorsets, fvwm/colorset.c does its own thing (different size)
+ */
 void AllocColorset(int n);
 #endif
 
@@ -80,12 +85,6 @@ int LoadColorset(char *line);
 Pixmap CreateBackgroundPixmap(Display *dpy, Window win, int width, int height,
 			      colorset_struct *colorset, unsigned int depth,
 			      GC gc, Bool is_mask);
-Pixmap ScrollPixmap(
-	Display *dpy, Pixmap p, GC gc, int x_off, int y_off, int width,
-	int height, unsigned int depth);
-void SetWindowBackgroundWithOffset(
-	Display *dpy, Window win, int x_off, int y_off, int width, int height,
-	colorset_struct *colorset, unsigned int depth, GC gc, Bool clear_area);
 void SetWindowBackground(Display *dpy, Window win, int width, int height,
 			 colorset_struct *colorset, unsigned int depth, GC gc,
 			 Bool clear_area);
