@@ -137,7 +137,7 @@ static int                 randr_error_base  = -1;
 #endif
 
 #ifdef USE_XINERAMA_EMULATION
-static Window blank_w, vert_w;
+static Window blank_w, vert_w, blank2_w, blank3_w;
 #endif
 
 static int FScreenParseScreenBit(char *arg, char default_screen);
@@ -187,6 +187,8 @@ static void FScreenUpdateEmulationMapState(void)
     if (!is_mapped)
     {
       XMapRaised(disp, blank_w);
+      XMapRaised(disp, blank2_w);
+      XMapRaised(disp, blank3_w);
       XMapRaised(disp, vert_w);
       is_mapped = True;
     }
@@ -196,6 +198,8 @@ static void FScreenUpdateEmulationMapState(void)
     if (is_mapped)
     {
       XUnmapWindow(disp, blank_w);
+      XUnmapWindow(disp, blank2_w);
+      XUnmapWindow(disp, blank3_w);
       XUnmapWindow(disp, vert_w);
       is_mapped = False;
     }
@@ -278,23 +282,33 @@ void FScreenInit(Display *dpy)
     h = DisplayHeight(disp, scr);
     screens_xi[1].screen_number = 0;
     screens_xi[1].x_org         = 0;
-    screens_xi[1].y_org         = 0;
+    screens_xi[1].y_org         = h / 16;
     screens_xi[1].width         = ws;
     screens_xi[1].height        = 7 * h / 8;
     screens_xi[2].screen_number = 1;
     screens_xi[2].x_org         = ws;
     screens_xi[2].y_org         = 0;
     screens_xi[2].width         = w - ws;
-    screens_xi[2].height        = h;
+    screens_xi[2].height        = 7 * h / 8;
     /* add delimiter */
     attributes.background_pixel = WhitePixel(disp, scr);
     attributes.override_redirect = True;
     blank_w = XCreateWindow(
-      disp, root, 0, 7 * h / 8 - 1, ws, 2, 0, CopyFromParent, CopyFromParent,
+      disp, root, 0, screens_xi[1].y_org - 1, screens_xi[1].width, 2, 0,
+      CopyFromParent, CopyFromParent, CopyFromParent,
+      CWBackPixel|CWOverrideRedirect, &attributes);
+    blank2_w = XCreateWindow(
+      disp, root, 0, screens_xi[1].y_org + screens_xi[1].height - 1,
+      screens_xi[1].width, 2, 0, CopyFromParent, CopyFromParent,
+      CopyFromParent, CWBackPixel|CWOverrideRedirect, &attributes);
+    blank3_w = XCreateWindow(
+      disp, root, screens_xi[2].x_org, screens_xi[2].height - 1,
+      w - screens_xi[2].x_org, 2, 0, CopyFromParent, CopyFromParent,
       CopyFromParent, CWBackPixel|CWOverrideRedirect, &attributes);
     vert_w = XCreateWindow(
-      disp, root, ws - 1, 0, 2, h, 0, CopyFromParent, CopyFromParent,
-      CopyFromParent, CWBackPixel|CWOverrideRedirect, &attributes);
+      disp, root, screens_xi[2].x_org - 1, 0, 2, h, 0, CopyFromParent,
+      CopyFromParent, CopyFromParent, CWBackPixel|CWOverrideRedirect,
+      &attributes);
   }
   else
 #endif
@@ -346,6 +360,11 @@ void FScreenSLSOnOff(Bool do_enable)
   if (do_enable)
   {
     total_screens = total_screens_sls;
+    if (!screens_sls)
+    {
+      /* Sls not configured yet, use whole screen by default */
+      FScreenConfigureSLS(1, 1);
+    }
     screens = screens_sls;
   }
   else
@@ -614,6 +633,34 @@ Bool FScreenGetScrRect(
   return !(screen == 0 && num_screens > 1);
 }
 
+/* Translates the coodinates *x *y from the screen specified by arg_src and
+ * screen_src to coordinates on the screen specified by arg_dest and
+ * screen_dest. (see FScreenGetScrRect for more details). */
+void FScreenTranslateCoordinates(
+  fscreen_scr_arg *arg_src, int screen_src,
+  fscreen_scr_arg *arg_dest, int screen_dest,
+  int *x, int *y)
+{
+  int x_src;
+  int y_src;
+  int x_dest;
+  int y_dest;
+
+  FScreenGetScrRect(arg_src, screen_src, &x_src, &y_src, NULL, NULL);
+  FScreenGetScrRect(arg_dest, screen_dest, &x_dest, &y_dest, NULL, NULL);
+
+  if (x)
+  {
+    *x = *x + x_src - x_dest;
+  }
+  if (y)
+  {
+    *y = *y + y_src - y_dest;
+  }
+
+  return;
+}
+
 /* Arguments work exactly like for FScreenGetScrRect() */
 int FScreenClipToScreen(
   fscreen_scr_arg *arg, int screen, int *x, int *y, int w, int h)
@@ -743,6 +790,7 @@ Bool FScreenIsRectangleOnScreen(
   int sh;
 
   FScreenGetScrRect(arg, screen, &sx, &sy, &sw, &sh);
+
   return (rec->x + rec->width > sx && rec->x < sx + sw &&
 	  rec->y + rec->height > sy && rec->y < sy + sh) ? True : False;
 }
@@ -760,6 +808,8 @@ static int FScreenParseScreenBit(char *scr_spec, char default_screen)
     scr = FSCREEN_CURRENT;
   else if (c == FSCREEN_SPEC_PRIMARY)
     scr = FSCREEN_PRIMARY;
+  else if (c == FSCREEN_SPEC_WINDOW)
+    scr = FSCREEN_XYPOS;
   else if (isdigit(c))
     scr = atoi(scr_spec);
   else
@@ -771,6 +821,8 @@ static int FScreenParseScreenBit(char *scr_spec, char default_screen)
       scr = FSCREEN_CURRENT;
     else if (c == FSCREEN_SPEC_PRIMARY)
       scr = FSCREEN_PRIMARY;
+    else if (c == FSCREEN_SPEC_WINDOW)
+      scr = FSCREEN_XYPOS;
     else if (isdigit(c))
       scr = atoi(scr_spec);
   }
@@ -786,7 +838,6 @@ int FScreenGetScreenArgument(char *scr_spec, char default_screen)
   return FScreenParseScreenBit(scr_spec, default_screen);
 }
 
-#if 1
 /*
  * FScreenParseGeometry
  *     Does the same as XParseGeometry, but handles additional "@scr".
@@ -1094,7 +1145,6 @@ int FScreenGetGeometry(
 
   return ret;
 }
-#endif
 
 
 /* no rand_r for now */
