@@ -20,12 +20,14 @@
 */
 
 #include <X11/X.h>
+#include "libs/fvwmlib.h"
 
 
 /**
  * FVWM sends packets of this type to modules.
  **/
 
+
 typedef struct {
     unsigned long start_pattern;       /* always holds START_FLAG value */
     unsigned long type;                /* one of the M_xxx values, below */
@@ -91,15 +93,36 @@ typedef struct {
 #define M_MINI_ICON          (1<<23)
 #define M_WINDOWSHADE        (1<<24)
 #define M_DEWINDOWSHADE      (1<<25)
-/* The next one is not used, so we have a place for a new type (it was
- * M_LOCKONSEND which is no more needed). olicha Nov 13, 1999 */
-#define M_NOTUSED            (1<<26)
+#define M_VISIBLE_NAME       (1<<26)
 #define M_SENDCONFIG         (1<<27)
 #define M_RESTACK            (1<<28)
 #define M_ADD_WINDOW         (1<<29)
 #define M_CONFIGURE_WINDOW   (1<<30)
+#define M_EXTENDED_MSG       (1<<31)
 #define MAX_MESSAGES         31
+#define MAX_MSG_MASK         0x7fffffff
 
+/* to get more than the old maximum of 32 messages, the 32nd bit is reserved to
+ * mark another 31 messages that have this bit and another one set.
+ * When handling received messages, the message type can be compared to the
+ * MX_... macro.  When using one of the calls that accepts a message mask, a
+ * separate call has to be made that ors the MX_... macros.  The normal
+ * M_... and MX_... macros must *never* be or'ed in one of these operations'
+ */
+#define MX_VISIBLE_ICON_NAME      ((1<<0) | M_EXTENDED_MSG)
+#define MX_ENTER_WINDOW           ((1<<1) | M_EXTENDED_MSG)
+#define MX_LEAVE_WINDOW           ((1<<2) | M_EXTENDED_MSG)
+#define MX_PROPERTY_CHANGE        ((1<<3) | M_EXTENDED_MSG)
+#define MAX_EXTENDED_MESSAGES     4
+#define DEFAULT_XMSG_MASK         0x00000000
+#define MAX_XMSG_MASK             0x0000000f
+
+#define MAX_TOTAL_MESSAGES   (MAX_MESSAGES + MAX_EXTENDED_MESSAGES)
+
+/* for MX_PROPERTY_CHANGE */
+#define MX_PROPERTY_CHANGE_NONE        0
+#define MX_PROPERTY_CHANGE_BACKGROUND  1
+#define MX_PROPERTY_CHANGE_SWALLOW     2
 
 /**
  * Reads a single packet of info from FVWM.
@@ -199,6 +222,11 @@ void InitGetConfigLine(int *fd, char *match);
 void GetConfigLine(int *fd, char **line);
 
 
+/* expands certain variables in a command to be sent by a module */
+char *module_expand_action(
+	Display *dpy, int screen , char *in_action, rectangle *r,
+	char *forecolor, char *backcolor);
+
 /**
  * Parse the command line arguments given to the module by FVWM.
  * Input is the argc & argv from main(), and a flag to indicate
