@@ -98,31 +98,31 @@ char *SkipQuote(char *s, const char *qlong, const char *qstart,
   if (*s == '\\' && s[1] != 0)
     return s+2;
   else if (*qlong && (t = strchr(qlong, *s)))
-    {
-      char c = *t;
+  {
+    char c = *t;
 
-      s++;
-      while(*s && *s != c)
-	{
-	  /* Skip over escaped text, ie \quote */
-	  if(*s == '\\' && *(s+1) != 0)
-	    s++;
-	  s++;
-	}
-      if(*s == c)
+    s++;
+    while(*s && *s != c)
+    {
+      /* Skip over escaped text, ie \quote */
+      if(*s == '\\' && *(s+1) != 0)
 	s++;
-      return s;
+      s++;
     }
+    if(*s == c)
+      s++;
+    return s;
+  }
   else if (*qstart && (t = strchr(qstart, *s)))
-    {
-      char c = *((t - qstart) + qend);
+  {
+    char c = *((t - qstart) + qend);
 
-      while (*s && *s != c)
-	s = SkipQuote(s, qlong, "", "");
-      return (*s == *t) ? ++s : s;
-    }
-  else
-    return ++s;
+    while (*s && *s != c)
+      s = SkipQuote(s, qlong, "", "");
+    return (*s == c) ? ++s : s;
+  }
+
+  return ++s;
 }
 
 /* Returns a string up to the first character from the string delims in a
@@ -297,19 +297,20 @@ char *PeekToken(char *indata, char **outdata)
 /* Tries to seek up to n tokens in indata. Returns the number of tokens
  * actually found (up to a maximum of n). */
 
-/* int CheckNTokens(char *indata, unsigned int n) */
-/* { */
-/*   unsigned int i; */
-/*   char *token; */
+#if 0
+int CheckNTokens(char *indata, unsigned int n)
+{
+  unsigned int i;
+  char *token;
 
-/*   for (i = 0; i < n; i++) */
-/*     { */
-/*       token = PeekToken(indata, NULL); */
-/*       if (token == NULL) */
-/* 	break; */
-/*     } */
-/*   return i; */
-/* } */
+  for (i = 0, token = (char *)1; i < n && token != NULL; i++)
+  {
+    token = PeekToken(indata, NULL);
+  }
+
+  return i;
+}
+#endif
 
 /*
 ** MatchToken: does case-insensitive compare on next token in string, leaving
@@ -326,6 +327,7 @@ int MatchToken(char *pstr,char *tok)
   return rc;
 }
 
+/* unused at the moment */
 /*
    function:		XCmpToken
    description:	        compare 1st word of s to 1st word of t
@@ -337,19 +339,21 @@ int MatchToken(char *pstr,char *tok)
 
 int XCmpToken(const char *s, const char **t)
 {
-    register const char *w=*t;
+	register const char *w=*t;
 
-    if (w==NULL) return 1;		/* non existant word */
-    if (s==NULL) return -1;		/* non existant string */
+	if (w==NULL)
+		return 1;		/* non existant word */
+	if (s==NULL)
+		return -1;		/* non existant string */
 
-    while ( *w && (*s==*w || toupper(*s)==toupper(*w)) )
-	s++,w++;
+	while ( *w && (*s==*w || toupper(*s)==toupper(*w)) )
+		s++,w++;
 
-    if ((*s=='\0' && (ispunct(*w) || isspace(*w)))||
-	(*w=='\0' && (ispunct(*s) || isspace(*s))) )
-	return 0;			/* 1st word equal */
-    else
-	return toupper(*s)-toupper(*w);	/* smaller/greater */
+	if ((*s=='\0' && (ispunct(*w) || isspace(*w)))||
+	    (*w=='\0' && (ispunct(*s) || isspace(*s))) )
+		return 0;			/* 1st word equal */
+	else
+		return toupper(*s)-toupper(*w);	/* smaller/greater */
 }
 
 
@@ -463,11 +467,13 @@ char *GetModuleResource(char *indata, char **resource, char *module_name)
  * ret_suffixnum (0 = no suffix, 1 = first suffix in suffixlist ...).
  *
  **************************************************************************/
-int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
-				int num, char *suffixlist, int ret_suffixnum[])
+int GetSuffixedIntegerArguments(
+  char *action, char **ret_action, int *retvals, int num, char *suffixlist,
+  int *ret_suffixnum)
 {
   int i;
   int j;
+  int n;
   char *token;
   int suffixes;
 
@@ -481,7 +487,7 @@ int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
     token = PeekToken(action, &action);
     if (token == NULL)
       break;
-    if (sscanf(token, "%d", &(retvals[i])) != 1)
+    if (sscanf(token, "%d%n", &(retvals[i]), &n) < 1)
       break;
     if (suffixes != 0 && ret_suffixnum != NULL)
     {
@@ -507,6 +513,11 @@ int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
       if (j == suffixes)
 	ret_suffixnum[i] = 0;
     }
+    else if (token[n] != 0 && !isspace(token[n]))
+    {
+      /* there is a suffix but no suffix list was specified */
+      break;
+    }
   }
   if (ret_action != NULL)
     *ret_action = action;
@@ -526,7 +537,7 @@ int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
  * it is big enough before calling this function.
  *
  **************************************************************************/
-int SuffixToPercentValue(int value, int suffix, int unit_table[])
+int SuffixToPercentValue(int value, int suffix, int *unit_table)
 {
   return (value * unit_table[suffix]) / 100;
 }
@@ -538,10 +549,10 @@ int SuffixToPercentValue(int value, int suffix, int unit_table[])
  * If ret_action is non-NULL, a pointer to the next token is returned there.
  *
  **************************************************************************/
-int GetIntegerArguments(char *action, char **ret_action, int retvals[],int num)
+int GetIntegerArguments(char *action, char **ret_action, int *retvals,int num)
 {
-  return GetSuffixedIntegerArguments(action, ret_action, retvals, num, NULL,
-				     NULL);
+  return GetSuffixedIntegerArguments(
+    action, ret_action, retvals, num, NULL, NULL);
 }
 
 
@@ -559,7 +570,7 @@ int GetIntegerArguments(char *action, char **ret_action, int retvals[],int num)
  * in token after the match.
  *
  **************************************************************************/
-int GetTokenIndex(char *token, char *list[], int len, char **next)
+int GetTokenIndex(char *token, char **list, int len, char **next)
 {
   int i;
   int l;
@@ -598,7 +609,7 @@ int GetTokenIndex(char *token, char *list[], int len, char **next)
  * token (just like the return value of GetNextToken).
  *
  **************************************************************************/
-char *GetNextTokenIndex(char *action, char *list[], int len, int *index)
+char *GetNextTokenIndex(char *action, char **list, int len, int *index)
 {
   char *token;
   char *next;
@@ -706,8 +717,8 @@ int GetTwoPercentArguments(char *action, int *val1, int *val2, int *val1_unit,
  * token matches none of these strings the default_ret value is returned and
  * the action itself is passed back in ret_action. If the no_toggle flag is
  * non-zero, the "toggle" string is handled as no match. */
-int ParseToggleArgument(char *action, char **ret_action, int default_ret,
-			char no_toggle)
+int ParseToggleArgument(
+  char *action, char **ret_action, int default_ret, char no_toggle)
 {
   int index;
   int rc;
