@@ -26,21 +26,6 @@
 
 #include "fvwmrect.h"
 
-/* Allow GCC extensions to work, if you have GCC */
-
-#ifndef __attribute__
-/* This feature is available in gcc versions 2.5 and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
-#  define __attribute__(x)
-# endif
-/* The __-protected variants of `format' and `printf' attributes
-   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __format__ format
-#  define __printf__ printf
-# endif
-#endif
-
 /***********************************************************************
  * Generic debugging
  ***********************************************************************/
@@ -217,6 +202,12 @@ Pixel GetHilite(Pixel);
 int pixel_to_color_string(
   Display *dpy, Colormap cmap, Pixel pixel, char *output, Bool use_hash);
 
+Pixel GetSimpleColor(char *name);
+/* handles colorset color names too */
+Pixel GetColor(char *name);
+
+/* duplicate an already allocated color */
+Pixel fvwmlib_clone_color(Pixel p);
 
 /***********************************************************************
  * Wrappers around various X11 routines
@@ -225,7 +216,7 @@ int pixel_to_color_string(
 typedef struct FvwmFont
 {
   XFontStruct *font;		/* font structure */
-#ifdef I18N_MB
+#ifdef MULTIBYTE
   XFontSet fontset;		/* font set */
 #endif
   int height;			/* height of the font */
@@ -233,7 +224,7 @@ typedef struct FvwmFont
 } FvwmFont;
 
 XFontStruct *GetFontOrFixed(Display *disp, char *fontname);
-#ifdef I18N_MB
+#ifdef MULTIBYTE
 XFontSet GetFontSetOrFixed(Display *disp, char *fontname);
 #endif
 Bool LoadFvwmFont(Display *dpy, char *fontname, FvwmFont *ret_font);
@@ -276,10 +267,16 @@ Bool GetResourceString(
  * Stuff for Graphics.c and ModGraph.c
  ***********************************************************************/
 
-void RelieveRectangle(Display *dpy, Drawable d, int x,int y,int w,int h,
-		      GC ReliefGC, GC ShadowGC, int line_width);
-void RelieveRectangle2(Display *dpy, Drawable d, int x,int y,int w,int h,
-		      GC ReliefGC, GC ShadowGC, int line_width);
+void do_relieve_rectangle(
+	Display *dpy, Drawable d, int x, int y, int w, int h,
+	GC ReliefGC, GC ShadowGC, int line_width, Bool use_alternate_shading);
+
+#define RelieveRectangle(dpy, d, x, y, w, h, ReliefGC, ShadowGC, line_width) \
+        do_relieve_rectangle( \
+                dpy, d, x, y, w, h, ReliefGC, ShadowGC, line_width, False)
+#define RelieveRectangle2(dpy, d, x, y, w, h, ReliefGC, ShadowGC, line_width) \
+        do_relieve_rectangle( \
+                dpy, d, x, y, w, h, ReliefGC, ShadowGC, line_width, True)
 
 Pixmap CreateStretchXPixmap(Display *dpy, Pixmap src, int src_width,
 			    int src_height, int src_depth,
@@ -373,28 +370,39 @@ int GetEqualSizeChildren(
  ***********************************************************************/
 
 /* contexts for button presses */
-#define C_NO_CONTEXT	0x00
-#define C_WINDOW	0x01
-#define C_TITLE		0x02
-#define C_ICON		0x04
-#define C_ROOT		0x08
-#define C_FRAME		0x10
-#define C_SIDEBAR       0x20
-#define C_L1            0x40
-#define C_R1            0x80
-#define C_L2           0x100
-#define C_R2           0x200
-#define C_L3           0x400
-#define C_R3           0x800
-#define C_L4          0x1000
-#define C_R4          0x2000
-#define C_L5          0x4000
-#define C_R5          0x8000
-#define C_UNMANAGED  0x10000
-#define C_RALL       (C_R1|C_R2|C_R3|C_R4|C_R5)
-#define C_LALL       (C_L1|C_L2|C_L3|C_L4|C_L5)
-#define C_ALL   (C_WINDOW|C_TITLE|C_ICON|C_ROOT|C_FRAME|C_SIDEBAR|\
-                 C_LALL|C_RALL)
+#define C_NO_CONTEXT	    0x00
+#define C_WINDOW	    0x01
+#define C_TITLE		    0x02
+#define C_ICON		    0x04
+#define C_ROOT		    0x08
+/*                          0x10*/
+/*                          0x20*/
+#define C_L1                0x40
+#define C_R1                0x80
+#define C_L2               0x100
+#define C_R2               0x200
+#define C_L3               0x400
+#define C_R3               0x800
+#define C_L4              0x1000
+#define C_R4              0x2000
+#define C_L5              0x4000
+#define C_R5              0x8000
+#define C_UNMANAGED      0x10000
+#define C_EWMH_DESKTOP   0x20000
+#define C_F_TOPLEFT     0x100000
+#define C_F_TOPRIGHT    0x200000
+#define C_F_BOTTOMLEFT  0x400000
+#define C_F_BOTTOMRIGHT 0x800000
+#define C_SB_LEFT      0x1000000
+#define C_SB_RIGHT     0x2000000
+#define C_SB_TOP       0x4000000
+#define C_SB_BOTTOM    0x8000000
+#define C_FRAME       (C_F_TOPLEFT|C_F_TOPRIGHT|C_F_BOTTOMLEFT|C_F_BOTTOMRIGHT)
+#define C_SIDEBAR     (C_SB_LEFT|C_SB_RIGHT|C_SB_TOP|C_SB_BOTTOM)
+#define C_RALL        (C_R1|C_R2|C_R3|C_R4|C_R5)
+#define C_LALL        (C_L1|C_L2|C_L3|C_L4|C_L5)
+#define C_ALL         (C_WINDOW|C_TITLE|C_ICON|C_ROOT|C_FRAME|C_SIDEBAR|\
+                       C_LALL|C_RALL|C_EWMH_DESKTOP)
 
 #define ALL_MODIFIERS (ShiftMask|LockMask|ControlMask|Mod1Mask|Mod2Mask|\
                        Mod3Mask|Mod4Mask|Mod5Mask)
