@@ -18,21 +18,21 @@
 #include <fvwm/window_flags.h>
 
 /*
-    =====================================================================
+    *********************************************************************
     All new-style module packets (i.e., those that are not simply arrays
     of long ints, as used by the older modules) should have a structure
     definition in this file.
-    =====================================================================
+    *********************************************************************
 */
 
 
 
 
-/*  ============================================================
+/*  ************************************************************
     The M_CONFIGURE_WINDOW packet.
     This is the same packet as the M_ADD_WINDOW packet, the
     only difference being the type.
-    ============================================================  */
+    ************************************************************  */
 /*  RBW- typedef struct config_win_packet  */
 typedef struct ConfigWinPacket
 {
@@ -69,8 +69,25 @@ unsigned long int  TextPixel;
 unsigned long int  BackPixel;
 
 /*  Everything below this is post-GSFR  */
+unsigned long int  ewmh_hint_layer;
+unsigned long int  ewmh_hint_desktop;
+unsigned long int  ewmh_window_type;
 window_flags       flags;
 
 } ConfigWinPacket;
 
+typedef struct MiniIconPacket
+{
+	Window             w;
+	Window             frame;
+	FvwmWindow         *fvwmwin;
+	unsigned long int  width;
+	unsigned long int  height;
+	unsigned long int  depth;
+	unsigned long int  picture;
+	unsigned long int  mask;
+	unsigned long int  alpha;
+	char *name;
+} MiniIconPacket;
+
 #endif /* _VPACKET_ */
