@@ -58,6 +58,7 @@
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
 #include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
 #include "FvwmAnimate.h"
 
 #define AS_PI 3.14159265358979323846
@@ -813,7 +814,7 @@ int main(int argc, char **argv) {
   }
   /* FvwmAnimate must use the root visuals so trash the fvwm one */
   putenv("FVWM_VISUALID=");
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   Scr.root = DefaultRootWindow(dpy);
   Scr.screen = DefaultScreen(dpy);
 
@@ -829,7 +830,7 @@ int main(int argc, char **argv) {
   running = True;                       /* out of initialization phase */
   SendFinishedStartupNotification(Channel); /* tell fvwm we're running */
   SetSyncMask(Channel,M_ICONIFY|M_DEICONIFY|M_STRING); /* lock on send mask */
-  SetNoGrabMask(Channel,M_ICONIFY|M_DEICONIFY); /* ignore during recapture */
+  SetNoGrabMask(Channel,M_ICONIFY|M_DEICONIFY|M_STRING); /* ignore during recapture */
   Loop();                               /* start running */
   return 0;
 }
@@ -1051,7 +1052,7 @@ void ParseConfigLine(char *buf) {
   }
   /* capture the ImagePath setting, don't worry about ColorLimit */
   if (strncasecmp(buf, "ImagePath",9)==0) {
-    SetImagePath(&buf[9]);
+    PictureSetImagePath(&buf[9]);
   }
   /* Search for MyName (normally *FvwmAnimate) */
   else if (strncasecmp(buf, MyName, MyNameLen) == 0) {/* If its for me */
@@ -1125,6 +1126,10 @@ void ParseConfigLine(char *buf) {
         break;
       case Iterations_arg:              /* Iterations */
         Animate.iterations = atoi(q);
+        /* Silently fix up iterations less than 1. */
+        if (Animate.iterations <= 0) {
+          Animate.iterations = 1;
+        }
         break;
       case Effect_arg:                /* Effect */
       case Resize_arg:                /* -or - Resize */
@@ -1154,6 +1159,10 @@ void ParseConfigLine(char *buf) {
         break;
       case Width_arg:                 /* Width */
         Animate.width = atoi(q);
+        /* Silently fix up width less than 0. */
+        if (Animate.width < 0) {
+          Animate.width = 0;
+        }
         CreateDrawGC();                 /* update GC */
         break;
       default:
@@ -1182,20 +1191,20 @@ static void CreateDrawGC(void) {
   pixmap = None;
   gcv.function = GXxor;                 /* default is to xor the lines */
   if (Animate.pixmap) {                 /* if pixmap called for */
-    Picture *picture;
+    FvwmPicture *picture;
 
-    picture = GetPicture(dpy, RootWindow(dpy,Scr.screen), 0, Animate.pixmap, 0);
+    picture = PGetFvwmPicture(dpy, RootWindow(dpy,Scr.screen), 0,
+			      Animate.pixmap, 0);
     if (!picture)
       fprintf(stderr, "%s: Could not load pixmap '%s'\n",
 	      MyName + 1, Animate.pixmap);
     else {
-      if (picture->depth == DefaultDepth(dpy, Scr.screen)) {
-	pixmap = XCreatePixmap(dpy, RootWindow(dpy, Scr.screen), picture->width,
-			       picture->height, picture->depth);
-	XCopyArea(dpy, picture->picture, pixmap, DefaultGC(dpy, Scr.screen), 0,
-                  0, picture->width, picture->height, 0, 0);
-      }
-      DestroyPicture(dpy, picture);
+      pixmap = XCreatePixmap(dpy, RootWindow(dpy, Scr.screen), picture->width,
+			       picture->height, Pdepth);
+      PGraphicsCopyPixmaps(dpy, picture->picture, None, None, picture->depth,
+			   pixmap, None,
+			   0, 0, picture->width, picture->height, 0, 0);
+      PDestroyFvwmPicture(dpy, picture);
     }
   } else if (Animate.color) {           /* if color called for */
     if (XParseColor(dpy,DefaultColormap(dpy,Scr.screen),Animate.color, &xcol)) {
@@ -1349,16 +1358,16 @@ Save not done! Error\n",
   fprintf(config_file,"# This file created by %s\n\n",MyName+1);
   for (i=0; i < NUM_EFFECTS; i++) {
     if (effects[i].function == Animate.resize) {
-      fprintf(config_file,"*%sEffect %s\n",MyName+1,effects[i].name);
+      fprintf(config_file,"*%s: Effect %s\n",MyName+1,effects[i].name);
       break;
     } /* found match */
   } /* all possible effects */
-  fprintf(config_file,"*%sIterations %d\n",MyName+1,Animate.iterations);
-  fprintf(config_file,"*%sWidth %d\n",MyName+1,Animate.width);
-  fprintf(config_file,"*%sTwist %f\n",MyName+1,Animate.twist);
-  fprintf(config_file,"*%sDelay %d\n",MyName+1,Animate.delay);
+  fprintf(config_file,"*%s: Iterations %d\n",MyName+1,Animate.iterations);
+  fprintf(config_file,"*%s: Width %d\n",MyName+1,Animate.width);
+  fprintf(config_file,"*%s: Twist %f\n",MyName+1,Animate.twist);
+  fprintf(config_file,"*%s: Delay %d\n",MyName+1,Animate.delay);
   if (Animate.color) {
-    fprintf(config_file,"*%sColor %s\n",MyName+1,Animate.color);
+    fprintf(config_file,"*%s: Color %s\n",MyName+1,Animate.color);
   }
   /* Note, add "time" if that ever works. dje. 10/14/98 */
   fclose(config_file);
