@@ -41,24 +41,19 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 #include <X11/keysym.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
-#ifdef XPM
-#include <X11/xpm.h>
-#endif
 
 #include "libs/defaults.h"
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/Grab.h"
+#include <libs/gravity.h>
 #include "fvwm/fvwm.h"
 #include "libs/Module.h"
 #include "libs/fvwmsignal.h"
@@ -107,11 +102,11 @@ void Loop(void);
 void RedrawWindow(button_info*);
 void RecursiveLoadData(button_info*,int*,int*);
 void CreateUberButtonWindow(button_info*,int,int);
-void nocolor(const char *a, const char *b) __attribute__((__noreturn__));
 int My_XNextEvent(Display *dpy, XEvent *event);
 void process_message(unsigned long type,unsigned long *body);
 extern void send_clientmessage (Display *disp, Window w, Atom a,
 				Time timestamp);
+void parse_message_line(char *line);
 void CheckForHangon(unsigned long*);
 static Window GetRealGeometry(
   Display*,Window,int*,int*,unsigned int*,unsigned int*, unsigned int*,
@@ -122,7 +117,8 @@ static void GetPanelGeometry(
 void swallow(unsigned long*);
 void AddButtonAction(button_info*,int,char*);
 char *GetButtonAction(button_info*,int);
-static void change_colorset(int colorset);
+static void update_root_transparency(XEvent *Event);
+static void change_colorset(int colorset, XEvent *Event);
 
 void DebugEvents(XEvent*);
 
@@ -134,7 +130,7 @@ Display *Dpy;
 Window Root;
 Window MyWindow;
 char *MyName;
-static int screen;
+int screen;
 
 static int x_fd;
 static fd_set_size_t fd_width;
@@ -149,8 +145,8 @@ static Pixel hilite_pix, back_pix, shadow_pix, fore_pix;
 GC  NormalGC;
 /* needed for relief drawing only */
 GC  ShadowGC;
-static int Width,Height;
 
+static int Width,Height;
 static int x= -30000,y= -30000;
 int w = -1;
 int h = -1;
@@ -182,6 +178,7 @@ int save_color_limit;                   /* Color limit, if any */
 
 Bool do_allow_bad_access = False;
 Bool was_bad_access = False;
+Bool swallowed = False;
 
 /* ------------------------------ Misc functions ----------------------------*/
 
@@ -196,72 +193,6 @@ char *mymalloc(int length)
 }
 #endif
 
-#ifdef I18N_MB
-char NoName[] = "Untitled";
-
-/*
- * fake XFetchName() function
- */
-static char **XFetchName_islist = NULL; /* XXX */
-static Status MyXFetchName(Display *dpy, Window win, char **winname)
-{
-  XTextProperty text;
-  char **list;
-  int nitems;
-
-  if (XGetWMName(dpy, win, &text))
-  {
-    if (text.value)
-    {
-      text.nitems = strlen(text.value);
-      if (text.encoding == XA_STRING)
-      {
-        /* STRING encoding, use this as it is */
-        *winname = (char *)text.value;
-      }
-      else
-      {
-        /* not STRING encoding, try to convert */
-        if (XmbTextPropertyToTextList(dpy, &text, &list, &nitems) >= Success
-            && nitems > 0 && *list)
-	{
-          XFree(text.value);
-          *winname = *list;
-          XFetchName_islist = list; /* XXX */
-        }
-	else
-	{
-          if (list)
-	    XFreeStringList(list);
-          XFree(text.value); /* return of XGetWMName() */
-          if (XGetWMName(dpy, win, &text)) /* XXX: read again ? */
-	  {
-	    *winname = (char *)text.value;
-	  }
-	  else
-	  {
-	    *winname = NoName;
-	  }
-        }
-      }
-      return 1;
-    }
-  }
-  return 0;
-}
-static int MyXFree(void *data)
-{
-  if (XFetchName_islist != NULL) {
-    XFreeStringList(XFetchName_islist);
-    XFetchName_islist = NULL;
-    return 0;
-  }
-  return XFree(data);
-}
-#define XFetchName(x,y,z) MyXFetchName(x,y,z)
-#define XFree(x) MyXFree(x)
-#endif
-
 /**
 *** Some fancy routines straight out of the manual :-) Used in DeadPipe.
 **/
@@ -353,6 +284,14 @@ static void DeadPipeCleanup(void)
 #ifdef DEBUG_HANGON
 	  fprintf(stderr,"now unswallowed\n");
 #endif
+	  if (b->swallow & b_FvwmModule)
+	  {
+	    char cmd[256];
+
+	    sprintf(cmd,"PropertyChange %u %u %lu",
+		    MX_PROPERTY_CHANGE_SWALLOW, 0, swin);
+	    SendText(fd,cmd,0);
+	  }
 	  XReparentWindow(Dpy,swin,Root,b->x,b->y);
 	  XMoveWindow(Dpy,swin,b->x,b->y);
 	  XResizeWindow(Dpy,swin,b->w,b->h);
@@ -375,12 +314,12 @@ static void DeadPipeCleanup(void)
   {
     /* The picture pointer is NULL if the pixmap came from a colorset. */
     if(b->flags&b_Icon && b->icon != NULL)
-      DestroyPicture(Dpy, b->icon);
+      PDestroyFvwmPicture(Dpy, b->icon);
     if(b->flags&b_IconBack && b->backicon != NULL)
-      DestroyPicture(Dpy, b->icon);
+      PDestroyFvwmPicture(Dpy, b->icon);
     if(b->flags&b_Container && b->c->flags&b_IconBack &&
        !(b->c->flags&b_TransBack) && b->c->backicon != NULL)
-      DestroyPicture(Dpy, b->c->backicon);
+      PDestroyFvwmPicture(Dpy, b->c->backicon);
   }
 }
 
@@ -493,15 +432,26 @@ void AddButtonAction(button_info *b,int n,char *action)
 /**
 *** GetButtonAction()
 **/
-char *GetButtonAction(button_info *b,int n)
+char *GetButtonAction(button_info *b, int n)
 {
-  if(!b || !(b->flags&b_Action) || !(b->action) || n < 0 ||
-     n > NUMBER_OF_MOUSE_BUTTONS)
-    return NULL;
-  if (!b->action[n])
-    return NULL;
+	rectangle r;
+	char *act;
 
-  return expand_action(b->action[n], b);
+	if(!b || !(b->flags&b_Action) || !(b->action) || n < 0 ||
+	   n > NUMBER_OF_MOUSE_BUTTONS)
+	{
+		return NULL;
+	}
+	if (!b->action[n])
+	{
+		return NULL;
+	}
+	get_button_root_geometry(&r, b);
+	act = module_expand_action(
+		Dpy, screen, b->action[n], &r, UberButton->c->fore,
+		UberButton->c->back);
+
+	return act;
 }
 
 /**
@@ -521,7 +471,7 @@ void SetTransparentBackground(button_info *ub,int w,int h)
     unsigned int border_width_return;
     unsigned int depth_return;
     int number, i;
-    XFontStruct *font;
+    FlocaleFont *Ffont;
     Region shape_r;
     static GC trans_gc = NULL;
 
@@ -628,10 +578,11 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       }
 
       /* handle button's title */
-      font=buttonFont(b);
-      if(b->flags&b_Title && font)
+      Ffont=buttonFont(b);
+      if(b->flags&b_Title && Ffont)
       {
-	XSetFont(Dpy,trans_gc,font->fid);
+	if (Ffont->font)
+	  XSetFont(Dpy,trans_gc,Ffont->font->fid);
 	DrawTitle(b,pmap_mask,trans_gc);
       }
     }
@@ -662,7 +613,7 @@ int myErrorHandler(Display *dpy, XErrorEvent *event)
   }
   if((event->error_code == BadWindow) || (event->error_code == BadDrawable)
      || (event->error_code == BadMatch) || (event->request_code==X_GrabButton)
-     || (event->request_code == X_GetGeometry) 
+     || (event->request_code == X_GetGeometry)
      || (event->error_code == BadPixmap))
     return 0;
 
@@ -686,9 +637,8 @@ int main(int argc, char **argv)
   button_info *b,*ub;
   int geom_option_argc = 0;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmButtons");
+
   MyName = GetFileNameFromPath(argv[0]);
 
 #ifdef HAVE_SIGACTION
@@ -793,7 +743,7 @@ int main(int argc, char **argv)
 	    XDisplayName(NULL));
     exit (1);
   }
-  InitPictureCMap(Dpy);
+  PictureInitCMap(Dpy);
   FScreenInit(Dpy);
   /* Initialise default colorset */
   AllocColorset(0);
@@ -929,7 +879,8 @@ int main(int argc, char **argv)
 
   SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST | M_MAP | M_WINDOW_NAME
 		 | M_RES_CLASS | M_CONFIG_INFO | M_END_CONFIG_INFO | M_RES_NAME
-		 | M_SENDCONFIG | M_ADD_WINDOW | M_CONFIGURE_WINDOW);
+		 | M_SENDCONFIG | M_ADD_WINDOW | M_CONFIGURE_WINDOW | M_STRING);
+  SetMessageMask(fd, MX_PROPERTY_CHANGE);
 
   /* request a window list, since this triggers a response which
    * will tell us the current desktop and paging status, needed to
@@ -963,13 +914,19 @@ void Loop(void)
 {
   XEvent Event;
   KeySym keysym;
-  char buffer[10],*tmp,*act;
+  char buffer[10],*act;
   int i,i2,button;
+  int x;
+  int y;
   button_info *ub,*b;
 #ifndef OLD_EXPOSE
-  int ex=10000,ey=10000,ex2=0,ey2=0;
+  static int ex=10000,ey=10000,ex2=0,ey2=0;
+  button_info *tmpb;
 #endif
+  FlocaleNameString tmp;
 
+  tmp.name = NULL;
+  tmp.name_list = NULL;
   while( !isTerminated )
   {
     if(My_XNextEvent(Dpy,&Event))
@@ -995,6 +952,8 @@ void Loop(void)
 #ifdef OLD_EXPOSE
 	if(Event.xexpose.count == 0)
 	{
+	  if(Event.xexpose.window != MyWindow)
+	    break;
 	  button=-1;ub=UberButton;
 	  while(NextButton(&ub,&b,&button,1))
 	  {
@@ -1006,6 +965,8 @@ void Loop(void)
 	    ready++;
 	}
 #else
+	if(Event.xexpose.window != MyWindow)
+	  break;
 	ex=min(ex,Event.xexpose.x);
 	ey=min(ey,Event.xexpose.y);
 	ex2=max(ex2,Event.xexpose.x+Event.xexpose.width);
@@ -1016,18 +977,29 @@ void Loop(void)
 	  button=-1;ub=UberButton;
 	  while(NextButton(&ub,&b,&button,1))
 	  {
-	    if(b->flags&b_Container)
+	    Bool r = False;
+
+	    tmpb = b;
+	    while(tmpb->parent != NULL)
 	    {
-	      x=buttonXPos(b,buttonNum(b));
-	      y=buttonYPos(b,buttonNum(b));
+	      if(tmpb->flags&b_Container)
+	      {
+		x=buttonXPos(tmpb,buttonNum(tmpb));
+		y=buttonYPos(tmpb,buttonNum(tmpb));
+	      }
+	      else
+	      {
+		x=buttonXPos(tmpb,button);
+		y=buttonYPos(tmpb,button);
+	      }
+	      if (!(ex > x + buttonWidth(tmpb) || ex2 < x ||
+		 ey > y + buttonHeight(tmpb) || ey2 < y))
+	      {
+		r = True;
+	      }
+	      tmpb = tmpb->parent;
 	    }
-	    else
-	    {
-	      x=buttonXPos(b,button);
-	      y=buttonYPos(b,button);
-	    }
-	    if(!(ex > x + buttonWidth(b) || ex2 < x ||
-		 ey > y + buttonHeight(b) || ey2 < y))
+	    if(r)
 	    {
 	      if(ready<1 && !(b->flags&b_Container))
 		MakeButton(b);
@@ -1044,18 +1016,28 @@ void Loop(void)
 
       case ConfigureNotify:
       {
+	XEvent event;
 	unsigned int depth,tw,th,border_width;
 	Window root;
 
-	if (!XGetGeometry(Dpy, MyWindow, &root, &x, &y, &tw, &th, &border_width,
-			  &depth))
+	while (XCheckTypedWindowEvent(Dpy, MyWindow, ConfigureNotify, &event))
+	{
+	  if (!event.xconfigure.send_event &&
+	      Event.xconfigure.window != MyWindow)
+	    continue;
+	  Event.xconfigure.x = event.xconfigure.x;
+	  Event.xconfigure.y = event.xconfigure.y;
+	  Event.xconfigure.send_event = True;
+	}
+	if (!XGetGeometry(Dpy, MyWindow, &root, &x, &y, &tw, &th,
+			  &border_width, &depth))
 	{
 	  break;
 	}
-	if(tw!=Width || th!=Height)
+	if(tw != Width || th != Height)
 	{
-	  Width=tw;
-	  Height=th;
+	  Width= tw;
+	  Height= th;
 	  SetButtonSize(UberButton,Width,Height);
 	  button=-1;
 	  ub=UberButton;
@@ -1064,11 +1046,19 @@ void Loop(void)
 	    SetTransparentBackground(UberButton, Width, Height);
 	  }
 	  for (i = 0; i < nColorsets; i++)
-	    change_colorset(i);
+	    change_colorset(i, &Event);
 	  while(NextButton(&ub,&b,&button,0))
 	    MakeButton(b);
+	  XClearWindow(Dpy, MyWindow);
 	  RedrawWindow(NULL);
 	}
+	else if (Event.xconfigure.window == MyWindow &&
+		 Event.xconfigure.send_event &&
+		 (UberButton->c->flags & b_Colorset) &&
+		 Colorset[UberButton->c->colorset].pixmap == ParentRelative)
+	{
+	  update_root_transparency(&Event);
+	}
       }
       break;
 
@@ -1103,8 +1093,21 @@ void Loop(void)
 	{
 	  break;
 	}
+        if (Event.xbutton.window == MyWindow)
+        {
+          x = Event.xbutton.x;
+          y = Event.xbutton.y;
+        }
+        else
+        {
+          Window dummy;
+
+          XTranslateCoordinates(
+            Dpy, Event.xbutton.window, MyWindow, Event.xbutton.x,
+            Event.xbutton.y, &x, &y, &dummy);
+        }
 	CurrentButton = b =
-	  select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+	  select_button(UberButton, x, y);
 	is_pointer_in_current_button = True;
 
 	act = NULL;
@@ -1123,19 +1126,19 @@ void Loop(void)
 	}
 	if (act && strncasecmp(act,"popup",5) != 0)
 	{
-	  if (act)
-	  {
-	    free(act);
-	    act = NULL;
-	  }
+	  free(act);
+	  act = NULL;
 	  break;
 	}
 	else /* i.e. action is Popup */
 	{
 	  XUngrabPointer(Dpy,CurrentTime); /* And fall through */
 	}
-	free(act);
-	act = NULL;
+	if (act)
+	{
+	  free(act);
+	  act = NULL;
+	}
 	/* fall through */
 
       case KeyRelease:
@@ -1145,7 +1148,20 @@ void Loop(void)
 	  CurrentButton = NULL;
 	  break;
 	}
-	b = select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+        if (Event.xbutton.window == MyWindow)
+        {
+          x = Event.xbutton.x;
+          y = Event.xbutton.y;
+        }
+        else
+        {
+          Window dummy;
+
+          XTranslateCoordinates(
+            Dpy, Event.xbutton.window, MyWindow, Event.xbutton.x,
+            Event.xbutton.y, &x, &y, &dummy);
+        }
+	b = select_button(UberButton, x, y);
 	act = GetButtonAction(b,Event.xbutton.button);
 	if (b && !act && (b->flags & b_Panel))
 	{
@@ -1196,18 +1212,18 @@ void Loop(void)
 	      else
 		i2=i;
 
-	      tmp=mymalloc(strlen(act)+1);
-	      strcpy(tmp,"Exec ");
+	      tmp.name = mymalloc(strlen(act)+1);
+	      strcpy(tmp.name, "Exec ");
 	      while(act[i2]!=0 && isspace((unsigned char)act[i2]))
 		i2++;
-	      strcat(tmp,&act[i2]);
+	      strcat(tmp.name ,&act[i2]);
 	      if (is_transient)
 	      {
 		/* delete the window before continuing */
 		XDestroyWindow(Dpy, MyWindow);
 		XSync(Dpy, 0);
 	      }
-	      MySendText(fd,tmp,0);
+	      MySendText(fd,tmp.name,0);
 	      if (is_transient)
 	      {
 		/* and exit */
@@ -1217,7 +1233,7 @@ void Loop(void)
 	      {
 		XWithdrawWindow(Dpy, MyWindow, screen);
 	      }
-	      free(tmp);
+	      free(tmp.name);
 	    } /* exec */
 	    else if(strncasecmp(act,"DumpButtons",11)==0)
 	      DumpButtons(UberButton);
@@ -1280,10 +1296,11 @@ void Loop(void)
 	      if(b->flags&b_Title)
 		free(b->title);
 	      b->flags|=b_Title;
-	      if (XFetchName(Dpy,swin,&tmp))
+	      FlocaleGetNameProperty(XGetWMName, Dpy, swin, &tmp);
+	      if (tmp.name != NULL)
 	      {
-		CopyString(&b->title,tmp);
-		XFree(tmp);
+		CopyString(&b->title,tmp.name);
+		FlocaleFreeNameProperty(&tmp);
 	      }
 	      else
 	      {
@@ -1365,7 +1382,9 @@ void Loop(void)
 		b->flags |= (b_Panel | b_Hangon);
 		b->newflags.panel_mapped = 0;
 	      }
-	      p = expand_action(b->spawn, NULL);
+	      p = module_expand_action(
+		      Dpy, screen, b->spawn, NULL, UberButton->c->fore,
+		      UberButton->c->back);
 	      if (p)
 	      {
 		MySendText(fd, p, 0);
@@ -1447,9 +1466,9 @@ void RedrawWindow(button_info *b)
 /**
 *** LoadIconFile()
 **/
-int LoadIconFile(char *s,Picture **p)
+int LoadIconFile(char *s, FvwmPicture **p)
 {
-  *p=CachePicture(Dpy,Root,imagePath,s, save_color_limit);
+  *p=PCacheFvwmPicture(Dpy,Root,imagePath,s, save_color_limit);
   if(*p)
     return 1;
   return 0;
@@ -1462,14 +1481,7 @@ int LoadIconFile(char *s,Picture **p)
 void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
 {
   int i,j,x=0,y=0;
-  XFontStruct *font;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-  XFontSet fontset = None;
-#endif
+  FlocaleFont *Ffont;
 
   if (!b)
     return;
@@ -1579,29 +1591,13 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
 #endif
 
     if(strncasecmp(b->font_string,"none",4)==0)
-      b->font=NULL;
-#ifdef I18N_MB
-    else if(!(b->fontset=XCreateFontSet(Dpy,b->font_string,&ml,&mc,&ds)))
     {
-      b->font = NULL;
-      b->flags&=~b_Font;
-      fprintf(stderr,"%s: Couldn't load fontset %s\n",MyName,
-	      b->font_string);
+      b->Ffont=NULL;
     }
-    else
-    {
-      /* fontset found */
-      XFontsOfFontSet(fontset, &fs_list, &ml);
-      b->font = fs_list[0];
-    }
-#else
-    else if(!(b->font=XLoadQueryFont(Dpy,b->font_string)))
+    else if (!(b->Ffont = FlocaleLoadFont(Dpy, b->font_string, MyName)))
     {
       b->flags&=~b_Font;
-      fprintf(stderr,"%s: Couldn't load font %s\n",MyName,
-	      b->font_string);
     }
-#endif
   }
 
   if(b->flags&b_Container && b->c->flags&b_Font)
@@ -1610,52 +1606,13 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
     fprintf(stderr,", font2 \"%s\"",b->c->font_string);
 #endif
     if(strncasecmp(b->c->font_string,"none",4)==0)
-      b->c->font=NULL;
-#ifdef I18N_MB
-    else if(!(b->c->fontset=XCreateFontSet(Dpy,b->c->font_string,&ml,&mc,&ds)))
     {
-      fprintf(stderr,"%s: Couldn't load fontset %s\n",MyName,
-	      b->c->font_string);
-      if(b==UberButton)
-      {
-#ifdef STRICTLY_FIXED
-	if(!(b->c->fontset=XCreateFontSet(Dpy,"fixed",&ml,&mc,&ds)))
-	{
-#else
-	if(!(b->c->fontset=XCreateFontSet(
-	  Dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)))
-	{
-#endif
-	  fprintf(stderr,"%s: Couldn't load fontset fixed\n",MyName);
-	  b->c->font = NULL;
-	}
-      }
-      else
-      {
-	b->c->font = NULL;
-	b->c->flags&=~b_Font;
-      }
+      b->c->Ffont = NULL;
     }
-    else
+    else if (!(b->c->Ffont = FlocaleLoadFont(Dpy, b->c->font_string, MyName)))
     {
-      /* fontset found */
-      XFontsOfFontSet(b->c->fontset, &fs_list, &ml);
-      b->c->font = fs_list[0];
+      b->c->flags&=~b_Font;
     }
-#else
-    else if(!(b->c->font=XLoadQueryFont(Dpy,b->c->font_string)))
-    {
-      fprintf(stderr,"%s: Couldn't load font %s\n",MyName,
-	      b->c->font_string);
-      if(b==UberButton)
-      {
-	if(!(b->c->font=XLoadQueryFont(Dpy,"fixed")))
-	  fprintf(stderr,"%s: Couldn't load font fixed\n",MyName);
-      }
-      else
-	b->c->flags&=~b_Font;
-    }
-#endif
   }
 
   /* Calculate subbutton sizes */
@@ -1699,24 +1656,20 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
   else
     b->flags&=~b_Icon;
 
-#ifdef I18N_MB
-  if(b->flags&b_Title && (fontset = buttonFontSet(b)) && (font=buttonFont(b)))
-#else
-  if(b->flags&b_Title && (font=buttonFont(b)))
-#endif
+  if(b->flags&b_Title && (Ffont = buttonFont(b)))
   {
 #ifdef DEBUG_LOADDATA
     fprintf(stderr,", title \"%s\"",b->title);
 #endif
     if(buttonJustify(b)&b_Horizontal)
     {
-      i+=buttonXPad(b)+XTextWidth(font,b->title,strlen(b->title));
-      j=max(j,font->ascent+font->descent);
+      i+=buttonXPad(b)+FlocaleTextWidth(Ffont,b->title,strlen(b->title));
+      j=max(j,Ffont->height);
     }
     else
     {
-      i=max(i,XTextWidth(font,b->title,strlen(b->title)));
-      j+=font->ascent+font->descent;
+      i=max(i,FlocaleTextWidth(Ffont,b->title,strlen(b->title)));
+      j+=Ffont->height;
     }
   }
 
@@ -1777,6 +1730,8 @@ static void HandlePanelPress(button_info *b)
   }
   else
   {
+    XWMHints wmhints;
+
     /* Make sure the icon is unmapped first. Needed to work properly with
      * shaded and iconified windows. */
     XWithdrawWindow(Dpy, b->PanelWin, screen);
@@ -1788,6 +1743,10 @@ static void HandlePanelPress(button_info *b)
     mysizehints.x = 32767;
     mysizehints.y = 32767;
     XSetWMNormalHints(Dpy, b->PanelWin, &mysizehints);
+    /* make sure its not mapped as an icon */
+    wmhints.flags = StateHint;
+    wmhints.initial_state = NormalState;
+    XSetWMHints(Dpy, b->PanelWin, &wmhints);
 
     /* map the window in the void */
     XMoveWindow(Dpy, b->PanelWin, 32767, 32767);
@@ -2054,9 +2013,9 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
   gcm = GCForeground|GCBackground;
   gcv.foreground = fore_pix;
   gcv.background = back_pix;
-  if(ub && ub->c && ub->c->font && ub->font)
+  if(ub && ub->c && ub->c->Ffont &&  ub->c->Ffont->font && ub->Ffont)
   {
-    gcv.font = ub->c->font->fid;
+    gcv.font = ub->c->Ffont->font->fid;
     gcm |= GCFont;
   }
   NormalGC = fvwmlib_XCreateGC(Dpy, MyWindow, gcm, &gcv);
@@ -2083,58 +2042,6 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
   free(myclasshints.res_name);
 }
 
-/* ----------------------------- color functions --------------------------- */
-
-#ifdef XPM
-#  define MyAllocColor(a,b,c) PleaseAllocColor(c)
-#else
-#  define MyAllocColor(a,b,c) XAllocColor(a,b,c)
-#endif
-
-/**
-*** PleaseAllocColor()
-*** Allocs a color through XPM, which does it's closeness thing if out of
-*** space.
-**/
-#ifdef XPM
-int PleaseAllocColor(XColor *color)
-{
-  char *xpm[] = {"1 1 1 1",NULL,"x"};
-  XpmAttributes attr;
-  XImage *dummy1=None,*dummy2=None;
-  static char buf[20];
-
-  sprintf(buf,"x c #%04x%04x%04x",color->red,color->green,color->blue);
-  xpm[1]=buf;
-  attr.valuemask=XpmCloseness|XpmVisual|XpmColormap|XpmDepth;
-  attr.closeness=40000; /* value used by fvwm and fvwmlib */
-  attr.visual = Pvisual;
-  attr.colormap = Pcmap;
-  attr.depth = Pdepth;
-
-  if(XpmCreateImageFromData(Dpy,xpm,&dummy1,&dummy2,&attr)!=XpmSuccess)
-  {
-    fprintf(stderr,"%s: Unable to get similar color\n",MyName);
-    exit(1);
-  }
-  color->pixel=XGetPixel(dummy1,0,0);
-  if(dummy1!=None)XDestroyImage(dummy1);
-  if(dummy2!=None)XDestroyImage(dummy2);
-  return 1;
-}
-#endif
-
-/**
-*** nocolor()
-*** Complain
-**/
-void nocolor(const char *a, const char *b)
-{
-  fprintf(stderr,"%s: Can't %s %s, quitting, sorry...\n", MyName, a,b);
-  exit(1);
-}
-
-
 /* --------------------------------------------------------------------------*/
 
 #ifdef DEBUG_EVENTS
@@ -2328,9 +2235,69 @@ void change_swallowed_window_colorset(button_info *b, Bool do_clear)
   return;
 }
 
-static void recursive_change_colorset(container_info *c, int colorset)
+static void send_bg_change_to_module(button_info *b, XEvent *Event)
+{
+  int bx,by,bpx,bpy,bf;
+
+  if (Event != NULL)
+  {
+    buttonInfo(b,&bx,&by,&bpx,&bpy,&bf);
+    Event->xconfigure.x = UberButton->x + bx;
+    Event->xconfigure.y = UberButton->y + by;
+    Event->xconfigure.width = b->icon_w;
+    Event->xconfigure.height = b->icon_h;
+    Event->xconfigure.window = SwallowedWindow(b);
+    XSendEvent(Dpy, SwallowedWindow(b), False, NoEventMask, Event);
+  }
+  else
+    {
+      char cmd[256];
+      sprintf(cmd,"PropertyChange %u %u %lu",
+	      MX_PROPERTY_CHANGE_BACKGROUND, 0, SwallowedWindow(b));
+      SendText(fd,cmd,0);
+    }
+}
+
+static void update_root_transparency(XEvent *Event)
+{
+  button_info *ub,*b;
+  int button;
+
+  if (!(UberButton->c->flags & b_Colorset) ||
+      Colorset[UberButton->c->colorset].pixmap != ParentRelative)
+  {
+    return;
+  }
+
+  XClearWindow(Dpy, MyWindow);
+  RedrawWindow(NULL);
+  /* Handle the swallowed window */
+  ub = UberButton;
+  button=-1;
+  while(NextButton(&ub,&b,&button,0))
+  {
+    if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow))
+    {
+      /* should handle 2 cases: module or not */
+      if (b->swallow & b_FvwmModule)
+      {
+	send_bg_change_to_module(b, Event);
+      }
+      else if ((b->flags & b_Colorset) &&
+	       Colorset[buttonColorset(b)].pixmap == ParentRelative)
+      {
+	/* it is an X application */
+	change_swallowed_window_colorset(b, True);
+      }
+    }
+  }
+}
+
+static void recursive_change_colorset(container_info *c, int colorset,
+				      XEvent *Event)
 {
   int i;
+  Bool bg_change = False;
 
   for (i = c->num_buttons; --i >= 0; )
   {
@@ -2338,31 +2305,58 @@ static void recursive_change_colorset(container_info *c, int colorset)
 
     if (!b)
       continue;
+
+    bg_change = False;
+    if (buttonColorset(b) == colorset)
+    {
+      bg_change = True;
+    }
+    else  if (buttonColorset(b) > -1 &&
+	      Colorset[buttonColorset(b)].pixmap == ParentRelative &&
+	      (UberButton->c->flags & b_Colorset &&
+	       colorset == UberButton->c->colorset))
+    {
+      bg_change = True;
+    }
+    else  if ((b->flags & b_Swallow) && (b->swallow & b_FvwmModule) &&
+	      (UberButton->c->flags & b_Colorset &&
+	       colorset == UberButton->c->colorset))
+    {
+      /* swallowed module */
+      bg_change = True;
+    }
+
     if (b->flags & b_Container)
     {
-      if (buttonColorset(b) == colorset)
+      if (bg_change)
       {
 	/* re-apply colorset to button */
 	RedrawButton(b, True);
       }
       /* recursively update containers */
-      recursive_change_colorset(b->c, colorset);
+      recursive_change_colorset(b->c, colorset, Event);
     }
     else if (b->flags & b_Swallow)
     {
       /* swallowed window */
-      if ((buttonSwallowCount(b) == 3) && (b->flags & b_Swallow) &&
-	  (b->flags & b_Colorset) && (b->IconWin != None) &&
-          (buttonColorset(b) == colorset))
+      if (buttonSwallowCount(b) == 3 && b->IconWin != None && bg_change)
       {
-	/* re-apply colorset to window background */
-	change_swallowed_window_colorset(b, True);
+	RedrawButton(b, True);
+	if (Event == NULL && (b->swallow & b_FvwmModule))
+	{
+	  XSync(Dpy, 0);
+	  send_bg_change_to_module(b, Event);
+	}
+	else if (b->flags & b_Colorset)
+	{
+	  change_swallowed_window_colorset(b, True);
+	}
       }
     }
     else
     {
       /* simple button */
-      if (buttonColorset(b) == colorset)
+      if (bg_change)
       {
 	/* re-apply colorset to button */
 	RedrawButton(b, True);
@@ -2373,7 +2367,7 @@ static void recursive_change_colorset(container_info *c, int colorset)
   return;
 }
 
-static void change_colorset(int colorset)
+static void change_colorset(int colorset, XEvent *Event)
 {
   if (UberButton->c->flags & b_Colorset && colorset == UberButton->c->colorset)
   {
@@ -2382,7 +2376,7 @@ static void change_colorset(int colorset)
       &Colorset[colorset], Pdepth, NormalGC, True);
   }
 
-  recursive_change_colorset(UberButton->c, colorset);
+  recursive_change_colorset(UberButton->c, colorset, Event);
 
   return;
 }
@@ -2397,13 +2391,12 @@ static void handle_config_info_packet(unsigned long *body)
   if (StrEquals(token, "Colorset"))
   {
     colorset = LoadColorset(tline);
-    change_colorset(colorset);
+    change_colorset(colorset, NULL);
   }
   else if (StrEquals(token, XINERAMA_CONFIG_STRING))
   {
     FScreenConfigureModule(tline);
   }
-
   return;
 }
 
@@ -2446,12 +2439,37 @@ void process_message(unsigned long type,unsigned long *body)
     {
       button_lborder = button_rborder = button_tborder = button_bborder
 	= cfgpacket->border_width;
-      if (HAS_BOTTOM_TITLE(cfgpacket))
-	button_bborder += cfgpacket->title_height;
-      else
+      switch (GET_TITLE_DIR(cfgpacket))
+      {
+      case DIR_N:
 	button_tborder += cfgpacket->title_height;
+	break;
+      case DIR_S:
+	button_bborder += cfgpacket->title_height;
+	      break;
+      case DIR_W:
+	button_lborder += cfgpacket->title_height;
+	      break;
+      case DIR_E:
+	button_rborder += cfgpacket->title_height;
+	      break;
+      }
     }
     break;
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+	((!swallowed && body[2] == 0) || (swallowed && body[2] == MyWindow)))
+    {
+      update_root_transparency(NULL);
+    }
+    else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == MyWindow)
+    {
+      swallowed = body[1];
+    }
+    break;
+  case M_STRING:
+    parse_message_line((char *)&body[3]);
+    break;
   default:
     break;
   }
@@ -2751,11 +2769,14 @@ static void GetPanelGeometry(
 **/
 void swallow(unsigned long *body)
 {
-  char *temp;
   button_info *ub=UberButton,*b;
   int button=-1;
+  int i;
   unsigned int d;
   Window p;
+  FlocaleNameString temp;
+  temp.name = NULL;
+  temp.name_list = NULL;
 
   while(NextButton(&ub,&b,&button,0))
   {
@@ -2823,6 +2844,21 @@ void swallow(unsigned long *body)
 	  return;
 	}
 	/*error checking*/
+        for (i = 0; !(b->flags & b_ActionIgnoresClientWindow) &&
+		     i <= NUMBER_OF_MOUSE_BUTTONS; i++)
+        {
+          if (b->action != NULL && b->action[i] != NULL)
+          {
+            XGrabButton(
+              Dpy, i, 0, b->IconWin, False,
+              ButtonPressMask | ButtonReleaseMask, GrabModeAsync,
+              GrabModeAsync, None, None);
+            if (i == 0)
+            {
+              break;
+            }
+          }
+        }
 	XReparentWindow(Dpy,swin,MyWindow,-32768,-32768);
 	XSync(Dpy, 0);
 	MyXUngrabServer(Dpy);
@@ -2835,10 +2871,11 @@ void swallow(unsigned long *body)
 	  if (b->flags & b_Title)
 	    free(b->title);
 	  b->flags |= b_Title;
-	  if (XFetchName(Dpy, swin, &temp))
+	  FlocaleGetNameProperty(XGetWMName, Dpy, swin, &temp);
+	  if (temp.name != NULL)
 	  {
-	    CopyString(&b->title, temp);
-	    XFree(temp);
+	    CopyString(&b->title, temp.name);
+	    FlocaleFreeNameProperty(&temp);
 	  }
 	  else
 	  {
@@ -2849,8 +2886,16 @@ void swallow(unsigned long *body)
 	MakeButton(b);
 	XMapWindow(Dpy, swin);
 	XSync(Dpy, 0);
+	if (b->swallow & b_FvwmModule)
+	{
+	  /* if we swallow a module we send an avertisment */
+	  char cmd[256];
 
-	if (b->flags & b_Colorset)
+	  sprintf(cmd,"PropertyChange %u %u %lu",
+		  MX_PROPERTY_CHANGE_SWALLOW, 1, SwallowedWindow(b));
+	  SendText(fd,cmd,0);
+	}
+	if ((b->flags & b_Colorset) &&  !(b->swallow & b_FvwmModule))
 	{
 	  /* A short delay to give the application the chance to set the
 	   * background itself, so we can override it. If we don't do this, the
@@ -2888,3 +2933,137 @@ void swallow(unsigned long *body)
     }
   }
 }
+
+
+/* SendToModule options */
+static void change_title(button_info *b, char *line)
+{
+  if (!(b->flags & b_Title)) {
+    fprintf(stderr, "%s: cannot create a title, only change one\n", MyName);
+    return;
+  }
+  free(b->title);
+  CopyString(&b->title, line);
+  RedrawButton(b, 2);
+  return;
+}
+
+static void change_icon(button_info *b, char *line)
+{
+  FvwmPicture *new_icon;
+  int l;
+
+  if (line == NULL) {
+    return;
+  }
+  if (!(b->flags & b_Icon)) {
+    fprintf(stderr, "%s: cannot create an icon, only change one\n", MyName);
+    return;
+  }
+  l = strlen(line);
+  if (l > 0 && line[l - 1] == '\n') {
+    line[l - 1] = 0;
+  }
+  if (LoadIconFile(line, &new_icon) == 0) {
+    fprintf(stderr, "%s: cannot load icon %s\n", MyName, line);
+    return;
+  }
+  free(b->icon_file);
+  CopyString(&b->icon_file, line);
+  PDestroyFvwmPicture(Dpy, b->icon);
+  XDestroyWindow(Dpy, b->IconWin);
+  b->IconWin = None;
+  b->icon = new_icon;
+  CreateIconWindow(b);
+  ConfigureIconWindow(b);
+  XMapWindow(Dpy, b->IconWin);
+  RedrawButton(b, 2);
+  return;
+}
+
+static char *message_options[] = {"Title", "Icon", NULL};
+
+void parse_message_line(char *line)
+{
+  char *rest;
+  char *next;
+  char *s;
+  int type;
+  int count, i;
+  int n;
+  int mask;
+  int x;
+  int y;
+  button_info *b, *ub = UberButton;
+
+  type = GetTokenIndex(line, message_options, -1, &rest);
+  if (type == -1) {
+    fprintf(stderr, "%s: Message not understood: %s\n", MyName, line);
+    return;
+  }
+
+  /* find out which button */
+  s = PeekToken(rest, &next);
+  if (s != NULL && *s == '+')
+  {
+    unsigned int JunkWidth;
+    unsigned int JunkHeight;
+
+    mask = XParseGeometry(s, &x, &y, &JunkWidth, &JunkHeight);
+    if (!(mask & XValue) || (mask & XNegative) ||
+	!(mask & YValue) || (mask & YNegative))
+    {
+      fprintf(stderr, "%s: illegal button position '%s'\n", MyName, s);
+      return;
+    }
+    rest = next;
+    if (x < 0 || y < 0) {
+      fprintf(stderr, "%s: button column/row must not be negative\n", MyName);
+      return;
+    }
+    b = get_xy_button(ub, y, x);
+    if (b == NULL) {
+      fprintf(stderr, "%s: button at column %d row %d not found\n", MyName,
+	      x, y);
+      return;
+    }
+  }
+  else
+  {
+    n = GetIntegerArguments(rest, &rest, &x, 1);
+    if (n < 1) {
+      fprintf(stderr, "%s: missing button number\n", MyName);
+      return;
+    }
+    if (x < 0) {
+      fprintf(stderr, "%s: button number must not be negative\n", MyName);
+      return;
+    }
+    i = count = -1;
+    /* find the button */
+    while (NextButton(&ub, &b, &i, 0)) {
+      if (++count == x)
+	break;
+    }
+    if (count != x || b == NULL) {
+      fprintf(stderr, "%s: button number %d not found\n", MyName, x);
+      return;
+    }
+  }
+
+  switch(type) {
+  case 0:
+    change_title(b, rest);
+    break;
+  case 1:
+    change_icon(b, rest);
+    break;
+  }
+  if (FShapesSupported)
+  {
+    if (UberButton->c->flags&b_TransBack)
+      SetTransparentBackground(UberButton,Width,Height);
+  }
+
+  return;
+}
