@@ -50,6 +50,7 @@ extern char *config_file;
 extern int button_width;
 extern int button_height;
 extern int has_button_geometry;
+extern int screen;
 
 /* contains the character that terminated the last string from seekright */
 static char terminator = '\0';
@@ -246,7 +247,7 @@ static void ParseTitle(char **ss, byte *flags, byte *mask)
 *** ParseSwallow()
 *** Parses the options possible to Swallow
 **/
-static void ParseSwallow(char **ss,byte *flags,byte *mask)
+static void ParseSwallow(char **ss,unsigned int *flags,unsigned int *mask)
 {
   char *swallowopts[] =
   {
@@ -256,6 +257,7 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
     "respawn","norespawn",
     "useold", "noold",
     "usetitle", "notitle",
+    "fvwmmodule", "nofvwmmodule",
     NULL
   };
   char *t,*s=*ss;
@@ -315,6 +317,14 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
       *flags&=~b_UseTitle;
       *mask|=b_UseTitle;
       break;
+    case 12: /* FvwmModule */
+      *flags|=b_FvwmModule;
+      *mask|=b_FvwmModule;
+      break;
+    case 13: /* NoFvwmModule */
+      *flags&=~b_FvwmModule;
+      *mask|=b_FvwmModule;
+      break;
     default:
       t=seekright(&s);
       fprintf(stderr,"%s: Illegal Swallow option \"%s\"\n",MyName,
@@ -331,8 +341,9 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
 *** ParsePanel()
 *** Parses the options possible to Panel
 **/
-static void ParsePanel(char **ss, byte *flags, byte *mask, char *direction,
-		       int *steps, int *delay, panel_flags_type *panel_flags,
+static void ParsePanel(char **ss, unsigned int *flags, unsigned int *mask,
+		       char *direction, int *steps, int *delay,
+		       panel_flags_type *panel_flags,
 		       int *indicator_size, int *rela_x, int *rela_y,
 		       char *position, char *context)
 {
@@ -780,7 +791,7 @@ static void ParseButton(button_info **uberb,char *s)
       "padding",
       "swallow",
       "panel",
-      "action",
+      "actionignoresclientwindow",
       "container",
       "end",
       "nosize",
@@ -789,6 +800,7 @@ static void ParseButton(button_info **uberb,char *s)
       "right",
       "center",
       "colorset",
+      "action",
       NULL
     };
     s = trimleft(s);
@@ -1031,7 +1043,9 @@ static void ParseButton(button_info **uberb,char *s)
 	  {
 	    char *p;
 
-	    p = expand_action(o, NULL);
+	    p = module_expand_action(
+		    Dpy, screen, o, NULL, UberButton->c->fore,
+		    UberButton->c->back);
 	    if (p)
 	    {
 	      if(!(buttonSwallow(b)&b_UseOld))
@@ -1051,54 +1065,21 @@ static void ParseButton(button_info **uberb,char *s)
 	  if(o)
 	    free(o);
 	}
+	/* check if it is a module by command line inspection if
+	 * this hints has not been given in the swallow option */
+	if (is_swallow && !(b->swallow_mask & b_FvwmModule))
+	{
+	  if (b->spawn != NULL &&
+	      (strncasecmp(b->spawn,"module",6) == 0 ||
+	       strncasecmp(b->spawn,"fvwm",4) == 0))
+	  {
+	    b->swallow |= b_FvwmModule;
+	  }
+	}
 	break;
 
-	/* --------------------------- action ------------------------ */
-
-      case 9: /* Action */
-	s = trimleft(s);
-	i=0;
-	if(*s=='(')
-	{
-	  s++;
-	  if(strncasecmp(s,"mouse",5)!=0)
-	  {
-	    fprintf(stderr,"%s: Couldn't parse action\n",MyName);
-	  }
-	  s+=5;
-	  i=strtol(s,&t,10);
-	  s=t;
-	  while(*s && *s!=')')
-	    s++;
-	  if(*s==')')
-            s++;
-	}
-        {
-          char *r;
-          char *u = s;
-
-          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
-          r = s;
-          if (t && r > u + 1)
-          {
-            /* remove unquoted trailing spaces */
-            r -= 2;
-            while (r >= u && isspace(*r))
-              r--;
-            r++;
-            if (isspace(*r))
-            {
-              t[strlen(t) - (s - r - 1)] = 0;
-            }
-          }
-        }
-	if(t)
-	{
-	  AddButtonAction(b,i,t);
-	  free(t);
-	}
-	else
-	  fprintf(stderr,"%s: Missing action argument\n",MyName);
+      case 9: /* ActionIgnoresClientWindow */
+	b->flags |= b_ActionIgnoresClientWindow;
 	break;
 
 	/* -------------------------- container ---------------------- */
@@ -1171,6 +1152,54 @@ static void ParseButton(button_info **uberb,char *s)
 	}
 	break;
 
+	/* --------------------------- action ------------------------ */
+
+      case 18: /* Action */
+	s = trimleft(s);
+	i=0;
+	if(*s=='(')
+	{
+	  s++;
+	  if(strncasecmp(s,"mouse",5)!=0)
+	  {
+	    fprintf(stderr,"%s: Couldn't parse action\n",MyName);
+	  }
+	  s+=5;
+	  i=strtol(s,&t,10);
+	  s=t;
+	  while(*s && *s!=')')
+	    s++;
+	  if(*s==')')
+            s++;
+	}
+        {
+          char *r;
+          char *u = s;
+
+          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
+          r = s;
+          if (t && r > u + 1)
+          {
+            /* remove unquoted trailing spaces */
+            r -= 2;
+            while (r >= u && isspace(*r))
+              r--;
+            r++;
+            if (isspace(*r))
+            {
+              t[strlen(t) - (s - r - 1)] = 0;
+            }
+          }
+        }
+	if(t)
+	{
+	  AddButtonAction(b,i,t);
+	  free(t);
+	}
+	else
+	  fprintf(stderr,"%s: Missing action argument\n",MyName);
+	break;
+
       default:
 	t=seekright(&s);
 	fprintf(stderr,"%s: Illegal button option \"%s\"\n",MyName,
@@ -1325,6 +1354,8 @@ static void ParseConfigLine(button_info **ubb,char *s)
 	break;
       UberButton->w = 0;
       UberButton->h = 0;
+      UberButton->x = 0;
+      UberButton->y = 0;
       if (flags&WidthValue)
 	button_width = width;
       if (flags&HeightValue)
@@ -1460,171 +1491,6 @@ static void ParseConfigFile(button_info *ub)
   fclose(f);
 }
 
-char *expand_action(char *in_action, button_info *b)
-{
-  char *variables[] =
-  {
-    "$",
-    "fg",
-    "bg",
-    "left",
-    "-left",
-    "right",
-    "-right",
-    "top",
-    "-top",
-    "bottom",
-    "-bottom",
-    "width",
-    "height",
-    NULL
-  };
-  char *action = NULL;
-  char *src;
-  char *dest;
-  char *string = NULL;
-  char *rest;
-  int px;
-  int py;
-  int val = 0;
-  int offset;
-  int x;
-  int y;
-  int f;
-  int i;
-  unsigned int w = 0;
-  unsigned int h = 0;
-  Window win;
-  extern int dpw;
-  extern int dph;
-
-  /* create a temporary storage for expanding */
-  action = (char *)malloc(MAX_MODULE_INPUT_TEXT_LEN);
-  if (!action)
-  {
-    /* could not alloc memory */
-    return NULL;
-  }
-
-  /* calculate geometry */
-  if (b)
-  {
-    w = buttonWidth(b);
-    h = buttonHeight(b);
-    buttonInfo(b, &x, &y, &px, &py, &f);
-    XTranslateCoordinates(Dpy, MyWindow, Root, x, y, &x, &y, &win);
-  }
-
-  for (src = in_action, dest = action; *src != 0; src++)
-  {
-    if (*src != '$')
-    {
-      *(dest++) = *src;
-    }
-    else
-    {
-      char *dest_org = dest;
-      Bool is_string = False;
-      Bool is_value = False;
-
-      *(dest++) = *(src++);
-      i = GetTokenIndex(src, variables, -1, &rest);
-      if (i == -1)
-      {
-	src--;
-	continue;
-      }
-      switch (i)
-      {
-      case 0: /* $ */
-	continue;
-      case 1: /* fg */
-	string = UberButton->c->fore;
-	is_string = True;
-	break;
-      case 2: /* bg */
-	string = UberButton->c->back;
-	is_string = True;
-	break;
-      case 3: /* left */
-	val = x;
-	is_value = True;
-	break;
-      case 4: /* -left */
-	val = dpw - x - 1;
-	is_value = True;
-	break;
-      case 5: /* right */
-	val = x + w;
-	is_value = True;
-	break;
-      case 6: /* -right */
-	val = dpw - x - w - 1;
-	is_value = True;
-	break;
-      case 7: /* top */
-	val = y;
-	is_value = True;
-	break;
-      case 8: /* -top */
-	val = dph - y - 1;
-	is_value = True;
-	break;
-      case 9: /* bottom */
-	val = y + h;
-	is_value = True;
-	break;
-      case 10: /* -bottom */
-	val = dph - y - h - 1;
-	is_value = True;
-	break;
-      case 11: /* width */
-	val = w;
-	is_value = True;
-	break;
-      case 12: /* height */
-	val = h;
-	is_value = True;
-	break;
-      default: /* unknown */
-	src--;
-	continue;
-      } /* switch */
-      dest = dest_org;
-      src = --rest;
-      if (is_value)
-      {
-	if (MAX_MODULE_INPUT_TEXT_LEN - (dest - action) <= 16)
-	{
-	  /* out of space */
-	  free(action);
-	  return NULL;
-	}
-	/* print the number into the string */
-	sprintf(dest, "%d%n", val, &offset);
-	dest += offset;
-      }
-      else if (is_string)
-      {
-	if (MAX_MODULE_INPUT_TEXT_LEN - (dest - action) <= strlen(string))
-	{
-	  /* out of space */
-	  free(action);
-	  return NULL;
-	}
-	/* print the colour name into the string */
-	if (string)
-	{
-	  sprintf(dest, "%s%n", string, &offset);
-	  dest += offset;
-	}
-      }
-    } /* if */
-  } /* for */
-  *dest = 0;
-  return action;
-}
-
 void parse_window_geometry(char *geom)
 {
   int flags;
@@ -1636,6 +1502,8 @@ void parse_window_geometry(char *geom)
   flags = FScreenParseGeometry(geom,&g_x,&g_y,&width,&height);
   UberButton->w = 0;
   UberButton->h = 0;
+  UberButton->x = 0;
+  UberButton->y = 0;
   if (flags&WidthValue)
     w = width;
   if (flags&HeightValue)
