@@ -179,7 +179,7 @@ int main ( int argc, char *argv[])
       Opt_info = atoi( optarg );
       break;
     case 'v':
-      printf("%s %s\n", MYNAME, MYVERSION );
+      printf("%s %s\n", MYNAME, VERSION );
       exit(0);
     case 'w':
       Tv.tv_usec = atoi( optarg ) % 1000000;
@@ -735,14 +735,10 @@ void list_configure(unsigned long *body)
 	    DO_SHRINK_WINDOWSHADE( cfgpacket ) );
     printf( "do_stack_transient_parent: %d\n",
 	    DO_STACK_TRANSIENT_PARENT( cfgpacket ) );
-    printf( "do_start_iconic: %d\n",
-	    DO_START_ICONIC( cfgpacket ) );
     printf( "do_window_list_skip: %d\n",
 	    DO_SKIP_WINDOW_LIST( cfgpacket ) );
     printf( "focus_mode: %d\n",
 	    GET_FOCUS_MODE( cfgpacket ) );
-    printf( "has_bottom_title: %d\n",
-	    HAS_BOTTOM_TITLE( cfgpacket ) );
     printf( "has_depressable_border: %d\n",
 	    HAS_DEPRESSABLE_BORDER( cfgpacket ) );
     printf( "has_mwm_border: %d\n",
@@ -825,6 +821,8 @@ void list_configure(unsigned long *body)
 	    IS_WINDOW_FONT_LOADED( cfgpacket ) );
     printf( "is_window_shaded %d\n",
 	    IS_SHADED( cfgpacket ) );
+    printf( "title_dir: %d\n",
+	    GET_TITLE_DIR( cfgpacket ) );
   }
 
   printf( "0x%08lx %-20s %ld\n",
