@@ -1,6 +1,4 @@
-/* $Id: FvwmCommandS.c,v 1.20 2000/05/30 00:51:15 domivogt Exp $
- * $Source: /home/cvs/fvwm/fvwm/modules/FvwmCommand/FvwmCommandS.c,v $
- *
+/*
  * Fvwm command input interface.
  *
  * Copyright 1997, Toshi Isogai. No guarantees or warantees or anything
@@ -32,15 +30,16 @@
 #define MYNAME   "FvwmCommandS"
 #define MAXHOSTNAME 32
 
-int Fd[2]; /* pipes to fvwm */
-int FfdC; /* command fifo file discriptors */
-int FfdM; /* message fifo file discriptors */
+static int Fd[2]; /* pipes to fvwm */
+static int FfdC; /* command fifo file discriptors */
+static int FfdM; /* message fifo file discriptors */
+static struct stat stat_buf;
 
-char *FfdC_name, *FfdM_name; /* fifo names */
+static char *FfdC_name, *FfdM_name; /* fifo names */
 
-ino_t FfdC_ino, FfdM_ino; /* fifo inode numbers */
+static ino_t FfdC_ino, FfdM_ino; /* fifo inode numbers */
 
-char hostname[MAXHOSTNAME];
+static char hostname[MAXHOSTNAME];
 
 int open_fifos(const char *f_stem);
 void close_fifos(void);
@@ -51,7 +50,21 @@ void process_message(unsigned long type,unsigned long *body);
 void relay_packet(unsigned long, unsigned long, unsigned long *);
 void server(char *);
 static RETSIGTYPE sig_handler(int);
-int write_f(int fd, char *p, int len);
+static char *bugger_off = "killme\n";
+
+/* a queue of messages for FvwmCommand to receive */
+typedef struct Q
+{
+  unsigned long length;
+  unsigned long sent;
+  char *body;
+  struct Q *next;
+} Q;
+/* head and tail of the queue */
+static Q *Qstart = NULL;
+static Q *Qlast = NULL;
+/* flag to indicate new queue items available */
+static Bool queueing = False;
 
 int main(int argc, char *argv[])
 {
@@ -59,8 +72,8 @@ int main(int argc, char *argv[])
 
   if (argc < FARGS)
   {
-    fprintf(stderr,"%s Version %s should only be executed by fvwm!\n",
-	    MYNAME, MYVERSION);
+    fprintf(stderr, "%s version %s should only be executed by fvwm!\n",
+	    MYNAME, VERSION);
     exit(1);
   }
 
@@ -84,8 +97,8 @@ int main(int argc, char *argv[])
     sigaddset(&sigact.sa_mask, SIGTERM);
     sigaddset(&sigact.sa_mask, SIGPIPE);
 
-#ifdef SA_INTERRUPT
-    sigact.sa_flags = SA_INTERRUPT;
+#ifdef SA_RESTART
+    sigact.sa_flags = SA_RESTART;
 #else
     sigact.sa_flags = 0;
 #endif
@@ -107,11 +120,11 @@ int main(int argc, char *argv[])
   signal(SIGHUP, sig_handler);
   signal(SIGTERM, sig_handler);
 #ifdef HAVE_SIGINTERRUPT
-  siginterrupt(SIGINT, 1);
-  siginterrupt(SIGHUP, 1);
-  siginterrupt(SIGQUIT, 1);
-  siginterrupt(SIGTERM, 1);
-  siginterrupt(SIGPIPE, 1);
+  siginterrupt(SIGINT, 0);
+  siginterrupt(SIGHUP, 0);
+  siginterrupt(SIGQUIT, 0);
+  siginterrupt(SIGTERM, 0);
+  siginterrupt(SIGPIPE, 0);
 #endif
 #endif
 
@@ -119,6 +132,7 @@ int main(int argc, char *argv[])
   Fd[1] = atoi(argv[2]);
 
   server(fifoname);
+  close_pipes();
   return 1;
 }
 
@@ -128,7 +142,6 @@ int main(int argc, char *argv[])
 static RETSIGTYPE
 sig_handler(int signo)
 {
-  close_pipes();
   fvwmSetTerminate(signo);
 }
 
@@ -137,38 +150,35 @@ sig_handler(int signo)
  */
 void server (char *name)
 {
-  char *home;
   char *f_stem;
   int  len;
-  fd_set fdset;
+  fd_set fdrset, fdwset;
   char buf[MAX_MODULE_INPUT_TEXT_LEN + 1];  /* command receiving buffer */
   char cmd[MAX_MODULE_INPUT_TEXT_LEN + 1];
   int  ix,cix;
+  struct timeval tv;
+
+  tv.tv_sec = 10;
+  tv.tv_usec = 0;
 
   if (name == NULL)
   {
     char *dpy_name;
 
     /* default name */
-    home = getenv("HOME");
-    if  (home == NULL)
-      home = "";
     dpy_name = getenv("DISPLAY");
     if (!dpy_name)
       dpy_name = ":0";
     if (strncmp(dpy_name, "unix:", 5) == 0)
       dpy_name += 4;
-    f_stem = safemalloc(strlen(home) + strlen(F_NAME) + MAXHOSTNAME +
-			strlen(dpy_name) + 4);
-    strcpy(f_stem, home);
-    if (f_stem[strlen(f_stem)-1] != '/')
-    {
-      strcat(f_stem, "/");
-    }
+    f_stem = safemalloc(11 + strlen(F_NAME) + MAXHOSTNAME + strlen(dpy_name));
+    if ((stat("/var/tmp", &stat_buf) == 0) && (stat_buf.st_mode & S_IFDIR))
+      strcpy (f_stem, "/var/tmp/");
+    else
+      strcpy (f_stem, "/tmp/");
     strcat(f_stem, F_NAME);
 
     /* Make it unique */
-    strcat(f_stem, "-");
     if (!dpy_name[0] || ':' == dpy_name[0])
     {
       gethostname(hostname, MAXHOSTNAME);
@@ -193,36 +203,93 @@ void server (char *name)
 
   while (!isTerminated)
   {
-    FD_ZERO(&fdset);
-    FD_SET(FfdC, &fdset);
-    FD_SET(Fd[1], &fdset);
+    int ret;
 
-    if (fvwmSelect(FD_SETSIZE, &fdset, 0, 0, NULL) < 0)
-      if (errno == EINTR)
+    FD_ZERO(&fdrset);
+    FD_ZERO(&fdwset);
+    FD_SET(FfdC, &fdrset);
+    FD_SET(Fd[1], &fdrset);
+    if (queueing)
+      FD_SET(FfdM, &fdwset);
+
+    ret = fvwmSelect(FD_SETSIZE, &fdrset, &fdwset, 0, queueing ? &tv : NULL);
+
+    if (ret < 0)
+      continue;
+
+    if (queueing && ret == 0) {
+      /* a timeout has occurred, this means the pipe to FvwmCommand is full
+       * dump any messages in the queue that have not been partially sent */
+      Q *q1, *q2;
+
+#ifdef PARANOMIA
+      /* do nothing if there are no messages (should never happen) */
+      if (!Qstart) {
+        queueing = False;
         continue;
+      }
+#endif
+      q1 = Qstart->next;
 
-    if (FD_ISSET(Fd[1], &fdset))
+      /* if the first message has been partially sent don't remove it */
+      if (!Qstart->sent) {
+        free(Qstart->body);
+        free(Qstart);
+        Qstart = NULL;
+      } else
+        fprintf(stderr, "FvwmCommandS: leaving a partially sent message in the queue\n");
+
+      /* now remove the rest of the message queue */
+      while ((q2 = q1) != NULL) {
+        q1 = q1->next;
+        free(q2->body);
+        free(q2);
+      }
+
+      /* there is either one message left (partially complete) or none */
+      Qlast = Qstart;
+      queueing = False;
+      continue;
+    }
+
+    if (FD_ISSET(Fd[1], &fdrset))
     {
       FvwmPacket* packet = ReadFvwmPacket(Fd[1]);
 
       if (packet == NULL)
       {
-        close_pipes();
-        exit(0);
+        break;
       }
       process_message(packet->type, packet->body);
     }
 
-    if (FD_ISSET(FfdC, &fdset))
+    if (FD_ISSET(FfdC, &fdrset))
     {
+      /*
+       * This descriptor is non-blocking, hence we should never
+       * block here! Also, the select() call should guarantee that
+       * there is something here to read, and the signal handling
+       * should be restarting interrupted reads. Together, these
+       * should severely restrict the types of errors that we can see.
+       */
       len = read(FfdC, buf, MAX_MODULE_INPUT_TEXT_LEN);
-      if (len == 0)
-        continue;
-      else if (len < 0)
+      if (len <= 0)
       {
-        if (errno != EAGAIN && errno != EINTR)
-          err_quit("reading fifo");
-        continue;
+        if ((len < 0) && (errno == EAGAIN))
+        {
+          /*
+           * This probably won't happen - the select() has told us that
+           * there's something to read. The only possible thing that could
+           * cause this is if somebody else read the data first ... (who?)
+           */
+          continue;
+        }
+
+        /*
+         * Any other error, such as an invalid descriptor (?) or someone
+         * closing the remote end of our pipe, and we give up!
+         */
+        err_quit("reading fifo");
       }
 
       /* in case of multiple long lines */
@@ -230,14 +297,14 @@ void server (char *name)
       {
         cmd[cix] = buf[ix];
         if (cmd[cix] == '\n')
-	{
+        {
           cmd[cix] = '\0';
           cix = 0;
-          if (StrEquals(cmd, "killme"))
+          if (StrHasPrefix(bugger_off, cmd))
             /* fvwm will close our pipes when it has processed this */
             SendQuitNotification(Fd);
           else
-	    SendText(Fd,cmd,0);
+            SendText(Fd, cmd, 0);
         }
 	else if (cix >= MAX_MODULE_INPUT_TEXT_LEN)
 	{
@@ -250,6 +317,27 @@ void server (char *name)
 	}
       } /* for */
     } /* FD_ISSET */
+
+    if (queueing && FD_ISSET(FfdM, &fdwset))
+    {
+      int sent;
+      Q *q = Qstart;
+
+      /* send one packet to FvwmCommand, try to send it all but cope
+       * with partial success */
+      sent = write(FfdM, q->body + q->sent, q->length - q->sent);
+      if (sent == q->length - q->sent) {
+        Qstart = q->next;
+        free(q->body);
+        free(q);
+        if (Qstart == NULL) {
+          Qlast = NULL;
+          queueing = False;
+        }
+      } else if (sent >= 0)
+        q->sent += sent;
+    }
+
   } /* while */
 }
 
@@ -296,8 +384,6 @@ void close_fifos(void)
  */
 int open_fifos(const char *f_stem)
 {
-  struct stat stat_buf;
-
   /* create 2 fifos */
   FfdC_name = malloc(strlen(f_stem) + 2);
   if (FfdC_name == NULL)
@@ -326,7 +412,7 @@ int open_fifos(const char *f_stem)
    * there is no need to wait before creating our own */
   if (FfdC > 0)
   {
-    write_f(FfdC, "killme\n", 9);
+    write(FfdC, bugger_off, strlen(bugger_off));
     close(FfdC);
   }
 
@@ -361,14 +447,38 @@ int open_fifos(const char *f_stem)
     return -1;
   }
   else
+  {
     FfdC_ino = stat_buf.st_ino;
+    if (!(S_IFIFO & stat_buf.st_mode))
+    {
+      err_msg("command fifo is not a fifo");
+      return -1;
+    }
+    if (stat_buf.st_mode & (S_IRWXG | S_IRWXO))
+    {
+      err_msg("command fifo is too permissive");
+      return -1;
+    }
+  }
   if (fstat(FfdM, &stat_buf) != 0)
   {
     err_msg("stat()ing message fifo");
     return -1;
   }
   else
+  {
     FfdM_ino = stat_buf.st_ino;
+    if (!(S_IFIFO & stat_buf.st_mode))
+    {
+      err_msg("message fifo is not a fifo");
+      return -1;
+    }
+    if (stat_buf.st_mode & (S_IRWXG | S_IRWXO))
+    {
+      err_msg("message fifo is too permissive");
+      return -1;
+    }
+  }
 
   return 0;
 }
@@ -449,49 +559,33 @@ void err_quit(const char *msg)
 
 /*
  * relay packet to front-end
+ * this is now implemented by simply adding the packet to a queue
+ * and letting the main select loop handle sending from the queue to
+ * the front end. In this way FvwmCommandS is always responsive to commands
+ * from the input pipes. (it will also die a lot faster when fvwm quits)
  */
 void relay_packet(unsigned long type, unsigned long length,
                   unsigned long *body)
 {
-  write_f(FfdM, (char*)&type, SOL);
-  write_f(FfdM, (char*)&length, SOL);
-  write_f(FfdM, (char*)body, length);
-}
+  Q *new;
 
-/*
- * write to fifo
- */
-int write_f(int fd, char *p, int len)
-{
-  int i, n;
-  struct timeval tv;
-  int again;
+  if (!length || !body)
+    return;
 
-  again = 0;
-  tv.tv_sec = 0;
-  tv.tv_usec = 50000;
+  new = (Q *)safemalloc(sizeof(Q));
 
-  for (i = 0; i < len; )
-  {
-    n = write(fd, &p[i], len - i);
-    if (n < 0)
-    {
-      if(errno == EINTR)
-      {
-	continue;
-      }
-      else if (errno == EAGAIN)
-      {
-	if (again++ < 5)
-	{
-	  select(0, 0, 0, 0, &tv);
-	  continue;
-	}
-	return -1; /* give up this message */
-      }
-      err_quit("writing fifo");
-    }
-    i += n;
-  }
-  return 0;
+  new->length = length + 2 * SOL;
+  new->sent = 0L;
+  new->body = safemalloc(new->length);
+  memcpy(new->body, &type, SOL);
+  memcpy(new->body + SOL, &length, SOL);
+  memcpy(new->body + 2 * SOL, body, length);
+  new->next = NULL;
+
+  if (Qlast)
+    Qlast->next = new;
+  Qlast = new;
+  if (!Qstart)
+    Qstart = Qlast;
+  queueing = True;
 }
