@@ -51,7 +51,7 @@
 #include "FvwmCpp.h"
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #define Resolution(pixels, mm) ((((pixels) * 2000 / (mm)) + 1) / 2)
@@ -190,7 +190,7 @@ int main(int argc, char **argv)
     }
 
   /* set up G */
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
 
   /* tell fvwm we're running if -lock is not used */
   if (!lock)
@@ -269,8 +269,9 @@ static char *cpp_defs(Display *display, const char *host, char *cpp_options, cha
   */
   /* first try to unlink it */
   unlink(tmp_name);
-  if ((fd = open(tmp_name, O_WRONLY|O_EXCL|O_CREAT, 0600)) < 0)
+  if ((fd = open(tmp_name, O_WRONLY|O_EXCL|O_CREAT, 0644)) < 0)
   {
+    fprintf(stderr, "error opening file %s\n", tmp_name);
     perror("exclusive open for output file failed in cpp_defs");
     exit(0377);
   }
@@ -412,9 +413,12 @@ static char *cpp_defs(Display *display, const char *host, char *cpp_options, cha
 #ifdef	XPM
   strcat(options, "XPM ");
 #endif
-#ifdef	I18N_MB
+#ifdef	MULTIBYTE
     strcat(options, "I18N_MB ");
 #endif
+#ifdef	MULTIBYTE
+    strcat(options, "MULTIBYTE ");
+#endif
 
   strcat(options, "Cpp ");
 
