@@ -179,6 +179,9 @@ int main(int argc, char **argv)
 
   fd_width = GetFdWidth();
 
+  /* select all extended messages */
+  SetMessageMask(fd, M_EXTENDED_MSG | MAX_XMSG_MASK);
+
   /* Create a list of all windows */
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
@@ -262,10 +265,18 @@ void process_message(unsigned long type, const unsigned long *body)
       fprintf(output,"window name\n");
       list_window_name(body);
       break;
+    case M_VISIBLE_NAME:
+      fprintf(output,"visible window name\n");
+      list_window_name(body);
+      break;
     case M_ICON_NAME:
       fprintf(output,"icon name\n");
       list_window_name(body);
       break;
+    case MX_VISIBLE_ICON_NAME:
+      fprintf(output,"visible icon name\n");
+      list_window_name(body);
+      break;
     case M_RES_CLASS:
       fprintf(output,"window class\n");
       list_window_name(body);
@@ -281,6 +292,10 @@ void process_message(unsigned long type, const unsigned long *body)
       fprintf(output,"icon location\n");
       list_icon(body);
       break;
+    case M_ICON_FILE:
+      fprintf(output,"icon file\n");
+      list_window_name(body);
+      break;
     case M_MAP:
       fprintf(output,"map\n");
       list_winid(body);
@@ -531,7 +546,7 @@ void list_configure(const unsigned long *body)
   fprintf(output,"\t desk %ld\n",p->desk);
   fprintf(output,"\t layer %ld\n",p->layer);
   fprintf(output,"\t title height %ld\n",p->title_height);
-  fprintf(output,"\t border width %ld\n",p->border_width);
+  fprintf(output,"\t border width %hd\n",p->border_width);
   fprintf(output,"\t window base width %ld\n",p->hints_base_width);
   fprintf(output,"\t window base height %ld\n",p->hints_base_height);
   fprintf(output,"\t window resize width increment %ld\n",p->hints_width_inc);
@@ -573,10 +588,8 @@ void list_configure(const unsigned long *body)
     fprintf(output,"\t\tdo_shrink_windowshade: %d\n", DO_SHRINK_WINDOWSHADE(p));
     fprintf(output,"\t\tdo_stack_transient_parent: %d\n",
 	    DO_STACK_TRANSIENT_PARENT(p));
-    fprintf(output,"\t\tdo_start_iconic: %d\n", DO_START_ICONIC(p));
     fprintf(output,"\t\tdo_window_list_skip: %d\n", DO_SKIP_WINDOW_LIST(p));
     fprintf(output,"\t\tfocus_mode: %d\n", GET_FOCUS_MODE(p));
-    fprintf(output,"\t\thas_bottom_title: %d\n", HAS_BOTTOM_TITLE(p));
     fprintf(output,"\t\thas_depressable_border: %d\n",
 	    HAS_DEPRESSABLE_BORDER(p));
     fprintf(output,"\t\thas_mwm_border: %d\n", HAS_MWM_BORDER(p));
@@ -613,6 +626,7 @@ void list_configure(const unsigned long *body)
     fprintf(output,"\t\tis_partially_visible: %d\n", IS_PARTIALLY_VISIBLE(p));
     fprintf(output,"\t\tis_pixmap_ours: %d\n", IS_PIXMAP_OURS(p));
     fprintf(output,"\t\tis_placed_wb3: %d\n", IS_PLACED_WB3(p));
+    fprintf(output,"\t\tis_placed_by_fvwm: %d\n", IS_PLACED_BY_FVWM(p));
     fprintf(output,"\t\tis_size_inc_set: %d\n", IS_SIZE_INC_SET(p));
     fprintf(output,"\t\tis_transient: %d\n", IS_TRANSIENT(p));
     fprintf(output,"\t\tis_window_drawn_once: %d\n", IS_WINDOW_DRAWN_ONCE(p));
@@ -621,6 +635,7 @@ void list_configure(const unsigned long *body)
 	    IS_WINDOW_BEING_MOVED_OPAQUE(p));
     fprintf(output,"\t\tis_window_font_loaded: %d\n", IS_WINDOW_FONT_LOADED(p));
     fprintf(output,"\t\tis_window_shaded %d\n", IS_SHADED(p));
+    fprintf(output,"\t\ttitle_dir: %d\n", GET_TITLE_DIR(p));
     fflush(output);
   }
 }
