@@ -46,14 +46,12 @@
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
-#ifdef XPM
-#include <X11/xpm.h>
-#endif /* XPM */
 
 #include "libs/fvwmlib.h"
+#include "libs/FShape.h"
 #include "libs/Module.h"
 #include "libs/Colorset.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 #include "libs/fvwmsignal.h"
 #include "libs/FScreen.h"
 
@@ -96,6 +94,7 @@ XGlobals xg;    /*X related globals*/
 DragWellMenuGlobals mg; /*menu related globals */
 DragWellButton dragBut; /*not really used as a button but rather a drag well*/
 char dropData[MAX_PATH_LEN];    /*data used for the drop*/
+Bool Swallowed = False;
 
 /* Don't know what this does yet...*/
 void deadPipe(int nonsense)
@@ -274,9 +273,13 @@ void veryLongLoop()
 	case Expose:
 	  drawDragWellButton(&dragBut); /*draws the drag icon*/
 	  break;
+	case ConfigureNotify:
+	  if (xg.colorset >= 0 && Colorset[xg.colorset].pixmap == ParentRelative)
+	    XClearArea(xg.dpy, xg.win, 0,0,0,0, True);
+	  break;
 	}
-    } else if (eventType==FOUND_FVWM_MESSAGE) { /*we have a fvwm2 message */
-      /* two cases, one where item is a file, other where we have a subdir*/
+    } else if (eventType==FOUND_FVWM_MESSAGE) { /* we have an fvwm message */
+      /* two cases, one where item is a file, other where we have a subdir */
       parseFvwmMessage(fvwmMessage);
       dragwellAnimate();
     } else {
@@ -438,9 +441,10 @@ void XStartup(char *appName)
 
   xg.appName = (char *) malloc(sizeof(char) * (strlen(appNameStart)+1));
   strcpy(xg.appName,appNameStart); /*save the name of the application*/
-  InitPictureCMap(xg.dpy);
+  PictureInitCMap(xg.dpy);
   FScreenInit(xg.dpy);
   AllocColorset(0);
+  FShapeInit(xg.dpy);
 
   /*get X stuff*/
   xg.xfd = XConnectionNumber(xg.dpy);
@@ -608,6 +612,13 @@ void parseOptions(void)
     } else if (StrEquals(resource, "Geometry")) {
       int g_x,g_y,flags;
       unsigned width,height;
+      xg.w = 48;
+      xg.h = 48;
+      xg.x = 0;
+      xg.y = 0;
+      xg.xneg = 0;
+      xg.yneg = 0;
+      xg.usposition = 0;
       flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue) {
 	xg.w = width;
@@ -632,6 +643,10 @@ void parseOptions(void)
     } else if (StrEquals(resource, "DragWellGeometry")) {
       int g_x,g_y,flags;
       unsigned width,height;
+      xg.dbw = 30;
+      xg.dbh = 30;
+      xg.dbx = 9;
+      xg.dby = 9;
       flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue) {
 	xg.dbw = width;
@@ -720,7 +735,7 @@ static void change_colorset(int colorset)
 /***************************************************************************
  *
  * myXNextEvent - waits for the next event, which is either an XEvent,
- *                  or a fvwm2 event.
+ *                  or an fvwm event.
  * Arguements:
  *   event - the XEvent that is possibly found.
  *   fvwmMessage - the FvwmMessage that is possibly found.
@@ -746,7 +761,7 @@ int myXNextEvent(XEvent *event, char *fvwmMessage)
 
   FD_ZERO(&in_fdset);
   FD_SET(xg.xfd,&in_fdset); /*look for x events*/
-  FD_SET(fd[1],&in_fdset); /*look for fvwm2 events*/
+  FD_SET(fd[1],&in_fdset); /*look for fvwm events*/
   if (select(xg.fdWidth,SELECT_FD_SET_CAST &in_fdset, 0, 0, NULL) > 0) /*wait */
   {
     if(FD_ISSET(xg.xfd, &in_fdset))
@@ -813,6 +828,24 @@ int myXNextEvent(XEvent *event, char *fvwmMessage)
 	      return FOUND_FVWM_NON_MESSAGE;
 	    }
 	  }
+	  else if (packet->type == MX_PROPERTY_CHANGE)
+	  {
+	    if (packet->body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+		xg.colorset >= 0 &&
+		Colorset[xg.colorset].pixmap == ParentRelative &&
+		((!Swallowed && packet->body[2] == 0) || 
+		 (Swallowed && packet->body[2] == xg.win)))
+	    {
+	      XClearArea(xg.dpy, xg.win, 0,0,0,0, True);
+	    }
+	    else if  (packet->body[0] == MX_PROPERTY_CHANGE_SWALLOW &&
+		      packet->body[2] == xg.win)
+	    {
+	      Swallowed = packet->body[1];
+	    }
+	    return FOUND_FVWM_NON_MESSAGE;
+	  }
+	  
           else
           {
 	    return FOUND_FVWM_NON_MESSAGE;
