@@ -113,9 +113,9 @@
 #define INFO(x)
 #endif
 
-#define BUILTIN_STARTUP		MAX_MESSAGES
-#define BUILTIN_SHUTDOWN	MAX_MESSAGES+1
-#define BUILTIN_UNKNOWN		MAX_MESSAGES+2
+#define BUILTIN_STARTUP		(MAX_TOTAL_MESSAGES)
+#define BUILTIN_SHUTDOWN	(MAX_TOTAL_MESSAGES + 1)
+#define BUILTIN_UNKNOWN		(MAX_TOTAL_MESSAGES + 2)
 #define MAX_BUILTIN		3
 
 /* globals */
@@ -148,7 +148,7 @@ typedef struct
   int action_arg;
 } event_entry;
 
-event_entry event_table[MAX_MESSAGES+MAX_BUILTIN] =
+event_entry event_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN] =
 {
   { "new_page",	-1 },
   { "new_desk", 0 },
@@ -176,11 +176,17 @@ event_entry event_table[MAX_MESSAGES+MAX_BUILTIN] =
   { "mini_icon", -1 },
   { "windowshade", 0 },
   { "dewindowshade", 0 },
-  { "lockonsend", -1 },
+  { "visible_name", 0 },
   { "sendconfig", -1 },
   { "restack", -1 },
   { "add_window", 0 },
   { "configure_window", 0 },
+  /* begin of extended message area */
+  { "visible_icon_name", 0 },
+  { "enter_window", 0 },
+  { "leave_window", 0 },
+  { "property_change", 0},
+  /* built in events */
 #ifdef M_BELL
   { "beep", -1 },
 #endif
@@ -194,11 +200,11 @@ event_entry event_table[MAX_MESSAGES+MAX_BUILTIN] =
 };
 
 /* define the action table  */
-char	*action_table[MAX_MESSAGES+MAX_BUILTIN];
+char	*action_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN];
 
 #ifdef HAVE_RPLAY
 /* define the rplay table */
-RPLAY	*rplay_table[MAX_MESSAGES+MAX_BUILTIN];
+RPLAY	*rplay_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN];
 #endif
 
 static volatile sig_atomic_t isTerminated = False;
@@ -207,7 +213,8 @@ int main(int argc, char **argv)
 {
   char *s;
   unsigned long	header[FvwmPacketHeaderSize], body[FvwmPacketBodyMaxSize];
-  int			total, remaining, count, event;
+  int total, remaining, count, event;
+  int is_extended_msg;
 
   INFO("--- started ----\n");
 
@@ -274,8 +281,9 @@ int main(int argc, char **argv)
     if (start_audio_delay) last_time = time(0);
     /* tell fvwm we're running */
 
+    /* SetMessageMask(fd, MAX_MSG_MASK); */
+    SetMessageMask(fd, M_EXTENDED_MSG | MAX_XMSG_MASK);
     /* migo (19-Aug-2000): synchronize on M_DESTROY_WINDOW */
-    /* SetMessageMask(fd, MAX_MASK); */
     SetSyncMask(fd, M_DESTROY_WINDOW);
 
     SendFinishedStartupNotification(fd);
@@ -325,14 +333,21 @@ int main(int argc, char **argv)
        */
       event = -1;
       msg_bit = header[1];
+      is_extended_msg = (msg_bit & M_EXTENDED_MSG);
+      msg_bit &= ~M_EXTENDED_MSG;
       while (msg_bit)
       {
 	event++;
 	msg_bit >>= 1;
       }
-      if (event < 0 || event >= MAX_MESSAGES)
-	event=BUILTIN_UNKNOWN;
-
+      if (event == -1)
+	event = BUILTIN_UNKNOWN;
+      else if (event >= MAX_MESSAGES && !is_extended_msg)
+	event = BUILTIN_UNKNOWN;
+      else if (event >= MAX_EXTENDED_MESSAGES && is_extended_msg)
+	event = BUILTIN_UNKNOWN;
+      else if (is_extended_msg)
+	event += MAX_MESSAGES;
       execute_event(event, body);		/* execute action */
 
 CONTINUE:
@@ -396,7 +411,7 @@ void execute_event(short event, unsigned long *body)
 	  {
 	    if(PassID && (event_table[event].action_arg != -1))
 	    {
-	      if (M_NEW_DESK == (1 << event))
+	      if ((M_NEW_DESK == (1 << event)) && event < MAX_EXTENDED_MESSAGES)
 		sprintf(buf, "%s %s %ld", cmd_line, action_table[event],
 			body[event_table[event].action_arg]);
 	      else
@@ -451,7 +466,7 @@ void config(void)
 
   /* Intialize all the actions */
 
-  for (i = 0; i < MAX_MESSAGES+MAX_BUILTIN; i++)
+  for (i = 0; i < MAX_TOTAL_MESSAGES+MAX_BUILTIN; i++)
   {
     action_table[i] = NULL;
 #ifdef HAVE_RPLAY
@@ -494,17 +509,8 @@ void config(void)
 	    free(cmd_line);
 	  if (token)
 	  {
-	    if (e - (char**)table == 4)
-	    {
-	      cmd_line = (char *)safemalloc(strlen(token)+11);
-	      strcpy(cmd_line, "Exec exec ");
-	      strcat(cmd_line, token);
-	    }
-	    else
-	    {
-	      cmd_line = (char *)safemalloc(strlen(token)+1);
-	      strcpy(cmd_line, token);
-	    }
+	    cmd_line = (char *)safemalloc(strlen(token)+1);
+	    strcpy(cmd_line, token);
 	  }
 	  else
 	  {
@@ -597,7 +603,7 @@ void config(void)
 	  continue;
 	}
 
-	for (found = 0,i = 0; !found && i < MAX_MESSAGES+MAX_BUILTIN;
+	for (found = 0,i = 0; !found && i < MAX_TOTAL_MESSAGES+MAX_BUILTIN;
 	     i++)
 	{
 	  INFO(event_table[i].name);
