@@ -33,9 +33,6 @@
 #include <X11/Xlib.h>
 #include <X11/X.h>
 #include <X11/Xutil.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 #include <X11/cursorfont.h>
 #define XK_MISCELLANY
 #include <X11/keysymdef.h>
@@ -44,8 +41,8 @@
 #include "libs/fvwmlib.h"
 #include "libs/fvwmsignal.h"
 
-#include <libs/Picture.h>               /* for InitPictureCMap */
-#include "libs/Colorset.h"               /* for InitPictureCMap */
+#include <libs/PictureBase.h>            /* for PictureInitCMap */
+#include "libs/Colorset.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 
@@ -65,6 +62,7 @@ Line *cur_line = &root_line;            /* curr line in parse rtns */
 char preload_yorn='n';           /* init to non-preload */
 Item *item;                             /* current during parse */
 Item *cur_sel, *cur_button;             /* current during parse */
+Item *timer = NULL;			/* timeout tracking */
 Display *dpy;
 int fd_x;                  /* fd for X connection */
 Window root, ref;
@@ -75,11 +73,12 @@ char bg_state = 'd';                    /* in default state */
   /* s = set by command (must be in "d" state for first "back" cmd to set it) */
   /* u = used (color allocated, too late to accept "back") */
 char endDefaultsRead = 'n';
-char *font_names[3];
+char *font_names[4];
 char *screen_background_color;
 char *MyName;
 int MyNameLen;
 int Channel[2];
+Bool Swallowed = False;
 
 /* Font/color stuff
    The colors are:
@@ -87,8 +86,8 @@ int Channel[2];
    . copied into items during parse
    . displayed in the customization dialog
    */
-static int colorset = -1;
-static int itemcolorset = 0;
+int colorset = -1;
+int itemcolorset = 0;
 
 /* prototypes */
 static void AssignDrawTable(Item *);
@@ -96,6 +95,37 @@ static void AddItem();
 static void PutDataInForm(char *);
 static void ReadFormData();
 static void FormVarsCheck(char **);
+static RETSIGTYPE TimerHandler(int);
+
+static void SetupTimer()
+{
+#ifdef HAVE_SIGACTION
+  {
+    struct sigaction  sigact;
+
+#ifdef SA_INTERRUPT
+    sigact.sa_flags = SA_INTERRUPT;
+#else
+    sigact.sa_flags = 0;
+#endif
+    sigemptyset(&sigact.sa_mask);
+    sigaddset(&sigact.sa_mask, SIGALRM);
+    sigact.sa_handler = TimerHandler;
+
+    sigaction(SIGALRM, &sigact, NULL);
+  }
+#else
+#ifdef USE_BSD_SIGNALS
+  fvwmSetSignalMask( sigmask(SIGALRM) );
+#endif
+  signal(SIGALRM, TimerHandler);  /* Dead pipe == Fvwm died */
+#ifdef HAVE_SIGINTERRUPT
+  siginterrupt(SIGALRM, 1);
+#endif
+#endif
+
+  alarm(1);
+}
 
 /* copy a string until '"', or '\n', or '\0' */
 static char *CopyQuotedString (char *cp)
@@ -138,19 +168,6 @@ static char *CopySolidString (char *cp)
   }
 }
 
-/* get the font height */
-static int FontHeight (XFontStruct *xfs)
-{
-  return (xfs->ascent + xfs->descent);
-}
-
-/* get the font width, for fixed-width font only */
-int FontWidth (XFontStruct *xfs)
-{
-  return (xfs->per_char[0].width);
-}
-
-
 /* Command parsing section */
 
 /* FvwmAnimate++ type command table */
@@ -188,6 +205,8 @@ static void ct_Text(char *);
 static void ct_InputPointer(char *);
 static void ct_InputPointerBack(char *);
 static void ct_InputPointerFore(char *);
+static void ct_Timeout(char *);
+static void ct_TimeoutFont(char *);
 static void ct_Title(char *);
 static void ct_UseData(char *);
 static void ct_padVText(char *);
@@ -229,6 +248,8 @@ static struct CommandTable ct_table[] =
   {"Position",ct_Position},
   {"Selection",ct_Selection},
   {"Text",ct_Text},
+  {"Timeout",ct_Timeout},
+  {"TimeoutFont",ct_TimeoutFont},
   {"Title",ct_Title},
   {"UseData",ct_UseData},
   {"WarpPointer",ct_WarpPointer}
@@ -256,7 +277,8 @@ static struct CommandTable def_table[] =
   {"ItemBack",ct_ItemBack},
   {"ItemColorset",ct_ItemColorset},
   {"ItemFore",ct_ItemFore},
-  {"Read",ct_Read}
+  {"Read",ct_Read},
+  {"TimeoutFont",ct_TimeoutFont}
 };
 
 /* If there were vars on the command line, do env var sustitution on
@@ -433,6 +455,10 @@ static void ct_Geometry(char *cp)
   int flags;
   unsigned int dummy;
 
+  CF.gx = 0;
+  CF.gy = 0;
+  CF.xneg = 0;
+  CF.yneg = 0;
   while (isspace(*cp))
     cp++;
   flags = FScreenParseGeometry(cp, &x, &y, &dummy, &dummy);
@@ -508,6 +534,13 @@ static void ct_Font(char *cp)
   font_names[f_text] = safestrdup(cp);
   myfprintf((stderr, "Font: %s\n", font_names[f_text]));
 }
+static void ct_TimeoutFont(char *cp)
+{
+  if (font_names[f_timeout])
+    free(font_names[f_timeout]);
+  font_names[f_timeout] = safestrdup(cp);
+  myfprintf((stderr, "TimeoutFont: %s\n", font_names[f_timeout]));
+}
 static void ct_ButtonFont(char *cp)
 {
   if (font_names[f_button])
@@ -617,23 +650,23 @@ static void ct_Message(char *cp)
   item->text.value = safemalloc(80);    /* point at last error recvd */
   item->text.n = 80;
   strcpy(item->text.value,"A mix of chars. MM20"); /* 20 mixed width chars */
-  item->header.size_x = (XTextWidth(item->header.dt_ptr->dt_font_struct,
-                                   item->text.value,
-                                   item->text.n/4) * 4) + 2 * TEXT_SPC;
-  item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
+  item->header.size_x = (FlocaleTextWidth(item->header.dt_ptr->dt_Ffont,
+                                          item->text.value,
+                                          item->text.n/4) * 4) + 2 * TEXT_SPC;
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
     + CF.padVText;
-
   memset(item->text.value,' ',80);      /* Clear it out */
   myfprintf((stderr, "Message area [%d, %d]\n",
              item->header.size_x, item->header.size_y));
   AddToLine(item);
   CF.last_error = item;                 /* save location of message item */
 }
+
 /* allocate colors and fonts needed */
 static void CheckAlloc(Item *this_item,DrawTable *dt)
 {
-  static XGCValues xgcv;
-  static int xgcv_mask = GCBackground | GCForeground | GCFont;
+  XGCValues xgcv;
+  int xgcv_mask = GCForeground;
 
   if (dt->dt_used == 2) {               /* fonts colors shadows */
     return;
@@ -648,12 +681,18 @@ static void CheckAlloc(Item *this_item,DrawTable *dt)
 
     xgcv.foreground = dt->dt_colors[c_fg];
     xgcv.background = dt->dt_colors[c_bg];
-    xgcv.font = dt->dt_font;
+    if (dt->dt_Ffont->font != NULL)
+    {
+      xgcv_mask |= GCFont;
+      xgcv.font = dt->dt_Ffont->font->fid;
+    }
+    xgcv_mask |= GCBackground;
     dt->dt_GC = fvwmlib_XCreateGC(dpy, CF.frame, xgcv_mask, &xgcv);
 
     dt->dt_used = 1;                    /* fore/back font allocated */
   }
-  if (this_item->type == I_TEXT) {      /* If no shadows needed */
+  if (this_item->type == I_TEXT
+      || this_item->type == I_TIMEOUT) {      /* If no shadows needed */
     return;
   }
   dt->dt_colors[c_item_fg] = (itemcolorset < 0)
@@ -664,8 +703,13 @@ static void CheckAlloc(Item *this_item,DrawTable *dt)
     : Colorset[itemcolorset].bg;
   xgcv.foreground = dt->dt_colors[c_item_fg];
   xgcv.background = dt->dt_colors[c_item_bg];
-  xgcv.font = dt->dt_font;
-  dt->dt_item_GC = fvwmlib_XCreateGC(dpy, CF.frame, GCForeground | GCFont, &xgcv);
+  xgcv_mask = GCForeground;
+  if (dt->dt_Ffont->font != NULL)
+  {
+    xgcv_mask |= GCFont;
+    xgcv.font = dt->dt_Ffont->font->fid;
+  }
+  dt->dt_item_GC = fvwmlib_XCreateGC(dpy, CF.frame, xgcv_mask, &xgcv);
   if (Pdepth < 2) {
     dt->dt_colors[c_itemlo] = BlackPixel(dpy, screen);
     dt->dt_colors[c_itemhi] = WhitePixel(dpy, screen);
@@ -698,6 +742,8 @@ static void AssignDrawTable(Item *adt_item)
   match_item_back = color_names[c_item_bg];
   if (adt_item->type == I_TEXT) {
     match_font = font_names[f_text];
+  } else if (adt_item->type == I_TIMEOUT) {
+    match_font = font_names[f_timeout];
   } else if (adt_item->type == I_INPUT) {
     match_font = font_names[f_input];
   } else {
@@ -735,8 +781,9 @@ static void AssignDrawTable(Item *adt_item)
   new_dt->dt_color_names[c_item_fg] = safestrdup(match_item_fore);
   new_dt->dt_color_names[c_item_bg] = safestrdup(match_item_back);
   new_dt->dt_used = 0;                  /* show nothing allocated */
-  new_dt->dt_font_struct = GetFontOrFixed(dpy, new_dt->dt_font_name);
-  new_dt->dt_font = new_dt->dt_font_struct->fid;
+  new_dt->dt_Ffont = FlocaleLoadFont(dpy, new_dt->dt_font_name, MyName+1);
+  FlocaleAllocateWinString(&new_dt->dt_Fstr);
+
   myfprintf((stderr,"Created drawtable with %s %s %s %s %s\n",
              new_dt->dt_color_names[c_fg], new_dt->dt_color_names[c_bg],
              new_dt->dt_color_names[c_item_fg],
@@ -774,12 +821,11 @@ static void ct_Text(char *cp)
     item->text.value = "";
   item->text.n = strlen(item->text.value);
 
-  item->header.size_x = XTextWidth(item->header.dt_ptr->dt_font_struct,
+  item->header.size_x = FlocaleTextWidth(item->header.dt_ptr->dt_Ffont,
                                    item->text.value,
                                    item->text.n) + 2 * TEXT_SPC;
-  item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
-    + CF.padVText;
-
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
+     + CF.padVText;
   myfprintf((stderr, "Text \"%s\" [%d, %d]\n", item->text.value,
              item->header.size_x, item->header.size_y));
   AddToLine(item);
@@ -796,6 +842,82 @@ static void ct_Title(char *cp)
     CF.title = "";
   myfprintf((stderr, "Title \"%s\"\n", CF.title));
 }
+static void ct_Timeout(char *cp)
+{
+  /* syntax: *FFTimeout seconds <Command> "Text" */
+  char *tmpcp, *tmpbuf;
+
+  if (timer != NULL) {
+    fprintf(stderr,"Only one timeout per form allowed, skipped %s.\n",cp);
+    return;
+  }
+  AddItem();
+  bg_state = 'u';                       /* indicate b/g color now used. */
+  item->type = I_TIMEOUT;
+  /* Item now added to list of items, now it needs a pointer
+     to the correct DrawTable. */
+  AssignDrawTable(item);
+  item->header.name = "";
+
+  item->timeout.timeleft = atoi(cp);
+  if (item->timeout.timeleft < 0)
+  {
+    item->timeout.timeleft = 0;
+  }
+  else if (item->timeout.timeleft > 99999)
+  {
+    item->timeout.timeleft = 99999;
+  }
+  timer = item;
+
+  while (*cp && *cp != '\n' && !isspace((unsigned char)*cp)) cp++;
+							/* skip timeout */
+  while (*cp && *cp != '\n' && isspace((unsigned char)*cp)) cp++;
+							/* move up to command */
+
+  if (!*cp || *cp == '\n') {
+    fprintf(stderr,"Improper arguments specified for FvwmForm Timeout.\n");
+    return;
+  }
+
+  if (*cp == '\"') {
+    item->timeout.command = CopyQuotedString(++cp);
+    /* skip over the whole quoted string to continue parsing */
+    cp += strlen(item->timeout.command);
+  }
+  else {
+    tmpbuf = safestrdup(cp);
+    tmpcp = tmpbuf;
+    while (!isspace((unsigned char)*tmpcp)) tmpcp++;
+    *tmpcp = '\0';                        /* cutoff command at first word */
+    item->timeout.command = safestrdup(tmpbuf);
+    free(tmpbuf);
+    while (!isspace((unsigned char)*cp)) cp++; /* move past command again */
+  }
+
+  while (*cp && *cp != '\n' && isspace((unsigned char)*cp)) cp++;
+						/* move up to next arg */
+
+  if (!*cp || *cp == '\n') {
+    fprintf(stderr,"Improper arguments specified for FvwmForm Timeout.\n");
+    return;
+  }
+
+  if (*cp == '\"') {
+    item->timeout.text = CopyQuotedString(++cp);
+  } else
+    item->timeout.text = "";
+  item->timeout.len = strlen(item->timeout.text);
+
+  item->header.size_x = FlocaleTextWidth(item->header.dt_ptr->dt_Ffont,
+                                   item->timeout.text,
+                                   item->timeout.len) + 2 * TEXT_SPC;
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
+     + CF.padVText;
+  myfprintf((stderr, "Timeout %d \"%s\" [%d, %d]\n", item->timeout.timeleft,
+		item->timeout.text, item->header.size_x, item->header.size_y));
+  AddToLine(item);
+}
 static void ct_padVText(char *cp)
 {
   /* syntax: *FFText "<padVText pixels>" */
@@ -827,9 +949,9 @@ static void ct_Input(char *cp)
   item->input.value = (char *)safemalloc(item->input.buf);
   item->input.value[0] = 0;             /* avoid reading unitialized data */
 
-  item->header.size_x = FontWidth(item->header.dt_ptr->dt_font_struct)
+  item->header.size_x = item->header.dt_ptr->dt_Ffont->max_char_width
     * item->input.size + 2 * TEXT_SPC + 2 * BOX_SPC;
-  item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
     + 3 * TEXT_SPC + 2 * BOX_SPC;
   myfprintf((stderr,"Input size_y is %d\n",item->header.size_y));
 
@@ -929,9 +1051,9 @@ static void PutDataInForm(char *cp)
         strcpy(item->input.init_value,cp); /* new initial value in field */
 	if (item->input.value)
 	  free(item->input.value);
-        item->input.value = safemalloc(var_len+1);
+        item->input.buf = var_len+1;
+        item->input.value = safemalloc(item->input.buf);
         strcpy(item->input.value,cp);     /* new value in field */
-        /* New value, but don't change length */
         free(var_name);                 /* goto's have their uses */
         return;
       }
@@ -1040,13 +1162,13 @@ static void ct_Choice(char *cp)
   }
 
   cur_sel->selection.choices[cur_sel->selection.n++] = item;
-  item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
-    + 2 * TEXT_SPC;
-  item->header.size_x = FontHeight(item->header.dt_ptr->dt_font_struct)
-    + 4 * TEXT_SPC +
-    XTextWidth(item->header.dt_ptr->dt_font_struct,
-               item->choice.text, item->choice.n);
-
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
+     + 2 * TEXT_SPC;
+  /* this is weird, the x dimension is the sum of the height and width? */
+  item->header.size_x = item->header.dt_ptr->dt_Ffont->height
+     + 4 * TEXT_SPC +
+     FlocaleTextWidth(item->header.dt_ptr->dt_Ffont,
+       item->choice.text, item->choice.n);
   myfprintf((stderr, "Choice %s, \"%s\", [%d, %d]\n", item->header.name,
           item->choice.text, item->header.size_x, item->header.size_y));
   AddToLine(item);
@@ -1079,11 +1201,11 @@ static void ct_Button(char *cp)
   else
     item->button.keypress = -1;
   item->button.len = strlen(item->button.text);
-  item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
-    + 2 * TEXT_SPC + 2 * BOX_SPC;
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height
+     + 2 * TEXT_SPC + 2 * BOX_SPC;
   item->header.size_x = 2 * TEXT_SPC + 2 * BOX_SPC
-    + XTextWidth(item->header.dt_ptr->dt_font_struct, item->button.text,
-                 item->button.len);
+  + FlocaleTextWidth(item->header.dt_ptr->dt_Ffont, item->button.text,
+    item->button.len);
   AddToLine(item);
   cur_button = item;
   myfprintf((stderr,"Created button, fore %s, bg %s, text %s\n",
@@ -1116,6 +1238,7 @@ static void InitConstants () {
   font_names[0]=safestrdup("8x13bold");
   font_names[1]=safestrdup("8x13bold");
   font_names[2]=safestrdup("8x13bold");
+  font_names[3]=safestrdup("8x13bold");
   screen_background_color=safestrdup("Light Gray");
   CF.p_c[input_fore].pointer_color.pixel = WhitePixel(dpy, screen);
   CF.p_c[input_back].pointer_color.pixel = BlackPixel(dpy, screen);
@@ -1343,22 +1466,44 @@ static void Restart ()
 /* redraw the frame */
 void RedrawFrame ()
 {
-  int x, y;
   Item *item;
 
+  Bool clear = False;
+  if (FftSupport)
+  {
+    item = root_item_ptr;
+    while(item != 0 && !clear)
+    {
+      if ((item->type == I_TEXT || item->type == I_CHOICE) &&
+	  item->header.dt_ptr->dt_Ffont->fftf.fftfont != NULL)
+	clear = True;
+      item = item->header.next;
+    }
+    if (clear)
+      XClearWindow(dpy, CF.frame);
+  }
+
   for (item = root_item_ptr; item != 0;
        item = item->header.next) {      /* all items */
     switch (item->type) {
     case I_TEXT:
       RedrawText(item);
       break;
+    case I_TIMEOUT:
+      RedrawTimeout(item);
+      break;
     case I_CHOICE:
-      x = item->header.pos_x + TEXT_SPC + item->header.size_y;
-      y = item->header.pos_y + TEXT_SPC +
-        item->header.dt_ptr->dt_font_struct->ascent;
-      XDrawString(dpy, CF.frame, item->header.dt_ptr->dt_GC,
-                       x, y, item->choice.text,
-                       item->choice.n);
+      item->header.dt_ptr->dt_Fstr->win = CF.frame;
+      item->header.dt_ptr->dt_Fstr->gc  = item->header.dt_ptr->dt_GC;
+      item->header.dt_ptr->dt_Fstr->str = item->choice.text;
+      item->header.dt_ptr->dt_Fstr->x   = item->header.pos_x
+        + TEXT_SPC + item->header.size_y;
+      item->header.dt_ptr->dt_Fstr->y   = item->header.pos_y + TEXT_SPC
+        + item->header.dt_ptr->dt_Ffont->ascent;
+      item->header.dt_ptr->dt_Fstr->len = item->choice.n;
+      FlocaleDrawString(dpy,
+                        item->header.dt_ptr->dt_Ffont,
+                        item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
       break;
     }
   }
@@ -1366,20 +1511,76 @@ void RedrawFrame ()
 
 void RedrawText(Item *item)
 {
-  int x, y;
-  int len;
   char *p;
 
   CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
-  x = item->header.pos_x + TEXT_SPC;
-  y = item->header.pos_y + ( CF.padVText / 2 ) +
-    item->header.dt_ptr->dt_font_struct->ascent;
-  len = item->text.n;
-  if ((p = memchr(item->text.value, '\0', len)) != NULL)
-    len = p - item->text.value;
-  XDrawString(dpy, CF.frame, item->header.dt_ptr->dt_GC,
-	      x, y, item->text.value, len);
+  item->header.dt_ptr->dt_Fstr->len = item->text.n;
+  if ((p = memchr(item->text.value, '\0', item->header.dt_ptr->dt_Fstr->len))
+      != NULL)
+    item->header.dt_ptr->dt_Fstr->len = p - item->text.value;
+  item->header.dt_ptr->dt_Fstr->win = CF.frame;
+  item->header.dt_ptr->dt_Fstr->gc  = item->header.dt_ptr->dt_GC;
+  item->header.dt_ptr->dt_Fstr->str = item->text.value;
+  item->header.dt_ptr->dt_Fstr->x   = item->header.pos_x + TEXT_SPC;
+  item->header.dt_ptr->dt_Fstr->y   = item->header.pos_y + ( CF.padVText / 2 ) +
+    item->header.dt_ptr->dt_Ffont->ascent;
+  FlocaleDrawString(dpy,
+                    item->header.dt_ptr->dt_Ffont,
+                    item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
+  return;
+}
 
+void RedrawTimeout(Item *item)
+{
+  char *p;
+  char *tmpbuf, *tmpptr, *tmpbptr;
+  int reallen;
+
+  XClearArea(dpy, CF.frame,
+               item->header.pos_x, item->header.pos_y,
+               item->header.size_x, item->header.size_y,
+               False);
+
+  tmpbuf = safemalloc(item->timeout.len + 6);
+  tmpbptr = tmpbuf;
+  for (tmpptr = item->timeout.text; *tmpptr != '\0' &&
+		!(tmpptr[0] == '%' && tmpptr[1] == '%'); tmpptr++) {
+    *tmpbptr = *tmpptr;
+    tmpbptr++;
+  }
+  if (tmpptr[0] == '%') {
+    tmpptr++; tmpptr++;
+    sprintf(tmpbptr, "%d", item->timeout.timeleft);
+    tmpbptr += strlen(tmpbptr);
+  }
+  for (; *tmpptr != '\0'; tmpptr++) {
+    *tmpbptr = *tmpptr;
+    tmpbptr++;
+  }
+  *tmpbptr = '\0';
+  
+  reallen = strlen(tmpbuf);
+  item->header.size_x = FlocaleTextWidth(item->header.dt_ptr->dt_Ffont,
+                                   tmpbuf, reallen) + 2 * TEXT_SPC;
+  item->header.size_y = item->header.dt_ptr->dt_Ffont->height + CF.padVText;
+
+  CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
+  item->header.dt_ptr->dt_Fstr->len = reallen;
+  if ((p = memchr(item->timeout.text, '\0', item->header.dt_ptr->dt_Fstr->len))
+      != NULL)
+    item->header.dt_ptr->dt_Fstr->len = p - tmpbuf;
+  item->header.dt_ptr->dt_Fstr->win = CF.frame;
+  item->header.dt_ptr->dt_Fstr->gc  = item->header.dt_ptr->dt_GC;
+  if (item->header.dt_ptr->dt_Fstr->str != NULL)
+    free(item->header.dt_ptr->dt_Fstr->str);
+  item->header.dt_ptr->dt_Fstr->str = safestrdup(tmpbuf);
+  item->header.dt_ptr->dt_Fstr->x   = item->header.pos_x + TEXT_SPC;
+  item->header.dt_ptr->dt_Fstr->y   = item->header.pos_y + ( CF.padVText / 2 ) +
+    item->header.dt_ptr->dt_Ffont->ascent;
+  FlocaleDrawString(dpy,
+                    item->header.dt_ptr->dt_Ffont,
+                    item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
+  free(tmpbuf);
   return;
 }
 
@@ -1431,7 +1632,8 @@ void RedrawItem (Item *item, int click)
                   xsegs, 4);
 
     if (click) {
-      x = BOX_SPC + TEXT_SPC + FontWidth(item->header.dt_ptr->dt_font_struct)
+      x = BOX_SPC + TEXT_SPC +
+        item->header.dt_ptr->dt_Ffont->max_char_width
         * CF.abs_cursor - 1;
       XSetForeground(dpy, item->header.dt_ptr->dt_item_GC,
                      item->header.dt_ptr->dt_colors[c_item_bg]);
@@ -1443,23 +1645,35 @@ void RedrawItem (Item *item, int click)
     len = item->input.n - item->input.left;
     XSetForeground(dpy, item->header.dt_ptr->dt_item_GC,
                    item->header.dt_ptr->dt_colors[c_item_fg]);
+    item->header.dt_ptr->dt_Fstr->win = item->header.win;
+    item->header.dt_ptr->dt_Fstr->gc  = item->header.dt_ptr->dt_item_GC;
     if (len > item->input.size)
       len = item->input.size;
     else
-      XDrawString(dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
-		       BOX_SPC + TEXT_SPC
-                       + FontWidth(item->header.dt_ptr->dt_font_struct) * len,
-		       BOX_SPC + TEXT_SPC
-                       + item->header.dt_ptr->dt_font_struct->ascent,
-		       item->input.blanks, item->input.size - len);
-    XDrawString(dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
-		     BOX_SPC + TEXT_SPC,
-		     BOX_SPC + TEXT_SPC +
-                     item->header.dt_ptr->dt_font_struct->ascent,
-		     item->input.value + item->input.left, len);
+    {
+      item->header.dt_ptr->dt_Fstr->str = item->input.blanks;
+      item->header.dt_ptr->dt_Fstr->x   = BOX_SPC + TEXT_SPC
+                  + item->header.dt_ptr->dt_Ffont->max_char_width
+                  * len;
+      item->header.dt_ptr->dt_Fstr->y   = BOX_SPC + TEXT_SPC
+                  + item->header.dt_ptr->dt_Ffont->ascent;
+      item->header.dt_ptr->dt_Fstr->len = item->input.size - len;
+      FlocaleDrawString(dpy,
+                        item->header.dt_ptr->dt_Ffont,
+                        item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
+    }
+    item->header.dt_ptr->dt_Fstr->str = item->input.value;
+    item->header.dt_ptr->dt_Fstr->x   = BOX_SPC + TEXT_SPC;
+    item->header.dt_ptr->dt_Fstr->y   = BOX_SPC + TEXT_SPC
+      + item->header.dt_ptr->dt_Ffont->ascent;
+    item->header.dt_ptr->dt_Fstr->len = len;
+    FlocaleDrawString(dpy,
+                      item->header.dt_ptr->dt_Ffont,
+                      item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
     if (item == CF.cur_input && !click) {
-      x = BOX_SPC + TEXT_SPC
-        + FontWidth(item->header.dt_ptr->dt_font_struct) * CF.abs_cursor - 1;
+      x = BOX_SPC + TEXT_SPC +
+        item->header.dt_ptr->dt_Ffont->max_char_width
+        * CF.abs_cursor - 1;
       XDrawLine(dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
 		x, BOX_SPC, x, dy - BOX_SPC);
       myfprintf((stderr,"Line %d/%d - %d/%d\n",
@@ -1555,11 +1769,26 @@ void RedrawItem (Item *item, int click)
                   xsegs, 4);
     XSetForeground(dpy, item->header.dt_ptr->dt_item_GC,
                    item->header.dt_ptr->dt_colors[c_item_fg]);
-    XDrawString(
-      dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
-      BOX_SPC + TEXT_SPC,
-      BOX_SPC + TEXT_SPC + item->header.dt_ptr->dt_font_struct->ascent,
-      item->button.text, item->button.len);
+    item->header.dt_ptr->dt_Fstr->win = item->header.win;
+    item->header.dt_ptr->dt_Fstr->gc  = item->header.dt_ptr->dt_item_GC;
+    item->header.dt_ptr->dt_Fstr->str = item->button.text;
+    item->header.dt_ptr->dt_Fstr->x   = BOX_SPC + TEXT_SPC;
+    item->header.dt_ptr->dt_Fstr->y   = BOX_SPC + TEXT_SPC
+      + item->header.dt_ptr->dt_Ffont->ascent;
+    item->header.dt_ptr->dt_Fstr->len = item->button.len;
+    if (FftSupport)
+    {
+      if (item->header.dt_ptr->dt_Ffont->fftf.fftfont != NULL)
+	XClearArea(dpy, item->header.win,
+		   BOX_SPC + TEXT_SPC - 1, BOX_SPC,
+		   item->header.size_x
+		   - (2 * BOX_SPC) - 2 - TEXT_SPC,
+		   (item->header.size_y - 1)
+		   - 2 * BOX_SPC + 1, False);
+    }
+    FlocaleDrawString(dpy,
+                      item->header.dt_ptr->dt_Ffont,
+                      item->header.dt_ptr->dt_Fstr, FWS_HAVE_LENGTH);
     myfprintf((stderr,"Just put %s into a button\n",
                item->button.text));
     break;
@@ -1567,6 +1796,27 @@ void RedrawItem (Item *item, int click)
   XFlush(dpy);
 }
 
+/* update transparency if backgroude colorset is transparent */
+void UpdateRootTransapency(void)
+{
+  Item *item;
+
+  if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+  {
+    /* window has moved redraw the background if it is transparent */
+    XClearArea(dpy, CF.frame, 0,0,0,0, True);
+    if (itemcolorset > -1 &&
+	Colorset[itemcolorset].pixmap == ParentRelative)
+    {
+      for (item = root_item_ptr; item != 0; item = item->header.next)
+      {
+	if (item->header.win != None)
+	  XClearArea(dpy, item->header.win, 0,0,0,0, True);
+      }
+    }
+  }
+}
+
 /* execute a command */
 void DoCommand (Item *cmd)
 {
@@ -1755,6 +2005,7 @@ static void OpenWindows ()
        item = item->header.next) {      /* all items */
     switch (item->type) {
     case I_INPUT:
+      myfprintf((stderr,"Checking alloc during OpenWindow on input\n"));
       CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
       item->header.win =
 	XCreateSimpleWindow(dpy, CF.frame,
@@ -1774,6 +2025,7 @@ static void OpenWindows ()
       }
       break;
     case I_CHOICE:
+      myfprintf((stderr,"Checking alloc during Openwindow on choice\n"));
       CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
       item->header.win =
 	XCreateSimpleWindow(dpy, CF.frame,
@@ -1818,6 +2070,7 @@ static void OpenWindows ()
   Restart();
   if (colorset >= 0)
   {
+    CheckAlloc(root_item_ptr,root_item_ptr->header.dt_ptr);
     SetWindowBackground(dpy, CF.frame, CF.max_width, CF.total_height,
                         &Colorset[(colorset)], Pdepth,
                         root_item_ptr->header.dt_ptr->dt_GC, True);
@@ -1853,6 +2106,17 @@ static void process_message(unsigned long type, unsigned long *body)
     myfprintf((stderr,"process_message: Got command: %s\n", (char *)&body[3]));
     ParseActiveMessage((char *)&body[3]);
     break;
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == CF.frame)))
+    {
+      UpdateRootTransapency();
+    }
+    else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == CF.frame)
+    {
+	Swallowed = body[1];
+    }
+    break;
   case M_ERROR:
   case M_STRING:
     if (CF.last_error) {                /* if form has message area */
@@ -1992,6 +2256,7 @@ static void am_Stop(char *cp)
 static void MainLoop (void)
 {
   fd_set fds;
+  fd_set_size_t fd_width = GetFdWidth();
 
   while ( !isTerminated ) {
     FD_ZERO(&fds);
@@ -1999,7 +2264,7 @@ static void MainLoop (void)
     FD_SET(fd_x, &fds);
 
     XFlush(dpy);
-    if (fvwmSelect(32, &fds, NULL, NULL, NULL) > 0) {
+    if (fvwmSelect(fd_width, &fds, NULL, NULL, NULL) > 0) {
       if (FD_ISSET(Channel[1], &fds))
 	ReadFvwm();
       if (FD_ISSET(fd_x, &fds))
@@ -2016,23 +2281,75 @@ TerminateHandler(int sig)
   fvwmSetTerminate(sig);
 }
 
+/* signal-handler to make the timer work */
+static RETSIGTYPE
+TimerHandler(int sig)
+{
+  int dn;
+  char *sp;
+  char *parsed_command;
+
+  timer->timeout.timeleft--;
+  if (timer->timeout.timeleft <= 0) {
+    /* pre-command */
+    if (!XWithdrawWindow(dpy, CF.frame, screen))
+    {
+      /* hm, what can we do now? just ignore this situation. */
+    }
+
+    /* construct command */
+    parsed_command = ParseCommand(0, timer->timeout.command, '\0', &dn, &sp);
+    myfprintf((stderr, "Final command: %s\n", parsed_command));
+
+    /* send command */
+    if ( parsed_command[0] == '!') {    /* If command starts with ! */
+      system(parsed_command+1);         /* Need synchronous execution */
+    } else {
+      SendText(Channel,parsed_command, ref);
+    }
+
+    /* post-command */
+    if (CF.last_error) {                  /* if form has last_error field */
+      memset(CF.last_error->text.value, ' ', CF.last_error->text.n); /* clear */
+      /* To do this more elegantly, the window resize logic should recalculate
+         size_x for the Message as the window resizes.  Right now, just clear
+         a nice wide area. dje */
+      XClearArea(dpy,CF.frame,
+                 CF.last_error->header.pos_x,
+                 CF.last_error->header.pos_y,
+                 /* CF.last_error->header.size_x, */
+                 2000,
+                 CF.last_error->header.size_y, False);
+    } /* end form has last_error field */
+    if (CF.grab_server)
+      XUngrabServer(dpy);
+    /* This is a temporary bug workaround for the pipe drainage problem */
+    SendQuitNotification(Channel);    /* let commands complete */
+    /* Note how the window is withdrawn, but execution continues until
+       the quit notifcation catches up with this module...
+       Should not be a problem, there shouldn't be any more commands
+       coming into FvwmForm.  dje */
+  }
+  else {
+    RedrawTimeout(timer);
+    alarm(1);
+  }
+}
+
 
 /* main procedure */
 int main (int argc, char **argv)
 {
   int i;
   FILE *fdopen();
-  char *s;                              /* FvwmAnimate */
-  char mask_mesg[20];
+  char *s;
   char cmd[200];
 
 #ifdef DEBUGTOFILE
   freopen(".FvwmFormDebug","w",stderr);
 #endif
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, getenv("LC_CTYPE"), "", "FvwmForm");
 
   /* From FvwmAnimate start */
   /* Save our program  name - for error events */
@@ -2047,15 +2364,6 @@ int main (int argc, char **argv)
   *MyName='*';
   strcpy(MyName+1, s);		/* append name */
 
-  /* convert aliases FormFvwmMyFlavor. used in 2.3 to FvwmForm-MyFlavor */
-  if (matchWildcards("FormFvwm*.", s))
-  {
-    strcpy(MyName + 1, "FvwmForm-");
-    strncpy(MyName + 10, s + 8, MyNameLen - 10);
-    fprintf(stderr, "FvwmForm: Please use alias name %s instead of %s\n",
-      MyName + 1, s);
-  }
-
   myfprintf((stderr,"%s: Starting, argv[0] is %s, len %d\n",MyName+1,
              argv[0],MyNameLen));
 
@@ -2125,7 +2433,7 @@ int main (int argc, char **argv)
   if (ref == 0) ref = None;
   myfprintf((stderr, "ref == %d\n", (int)ref));
 
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   /* prevent core dumps if fvwm doesn't provide any colorsets */
   AllocColorset(0);
@@ -2148,12 +2456,14 @@ int main (int argc, char **argv)
 
   MassageConfig();                      /* add data, calc window x/y */
 
-  /* Now tell fvwm we want *Alias commands in real time */
-  sprintf(mask_mesg,"SET_MASK %lu\n",(unsigned long)
-          (M_SENDCONFIG|M_CONFIG_INFO|M_ERROR|M_STRING));
-  SendInfo(Channel, mask_mesg, 0);      /* tell fvwm about our mask */
+  /* tell fvwm about our mask */
+  SetMessageMask(Channel, M_SENDCONFIG|M_CONFIG_INFO|M_ERROR|M_STRING);
+  SetMessageMask(Channel, MX_PROPERTY_CHANGE);
   OpenWindows();                        /* create initial window */
   SendFinishedStartupNotification(Channel);/* tell fvwm we're running */
+  if (timer != NULL) {
+     SetupTimer();
+  }
   MainLoop();                           /* start */
 
   return 0;                             /* */
