@@ -24,6 +24,7 @@
 
  */
 
+#include "libs/Flocale.h"               /* for font definition stuff */
 /*
  * This next stuff should be more specific and customizable.
  * For example padVText (above) was one of the things TXT_SPC
@@ -43,12 +44,14 @@
 #define ITEMS_PER_EXPANSION 32
 #define CHOICES_PER_SEL_EXPANSION 8
 #define BUTTON_COMMAND_EXPANSION 8
+#define TIMEOUT_COMMAND_EXPANSION 8
 
 #define I_TEXT          1
 #define I_INPUT         2
 #define I_SELECT        3
 #define I_CHOICE        4
 #define I_BUTTON        5
+#define I_TIMEOUT       6
 
 #define IS_SINGLE       1
 #define IS_MULTIPLE     2
@@ -70,8 +73,8 @@ typedef struct _drawtable {
                                            use them all */
   GC dt_GC;                             /* graphic ctx used for text */
   GC dt_item_GC;                        /* graphic ctx used for graphics */
-  Font dt_font;
-  XFontStruct *dt_font_struct;
+  FlocaleFont *dt_Ffont;                /* Fvwm font structure */
+  FlocaleWinString *dt_Fstr;            /* Fvwm string (handles multibyte) */
 } DrawTable;
 
 /* An "item" is something in a form. Part of the structure is common
@@ -137,6 +140,13 @@ typedef union _item {
     int button_array_size;              /* current size of next array */
     char **commands;    /* Fvwm command to execute */
   } button;
+  struct {		/* I_TIMEOUT */
+    struct _head head;
+    int timeleft;       /* seconds left on timer */
+    int len;            /* text length */
+    char *text;         /* text string */
+    char *command;    /* Fvwm command(s) to execute */
+  } timeout;
 } Item;
 
 #define L_LEFT        1
@@ -157,6 +167,13 @@ typedef struct _line {
   Item **items;                         /* ptr to array of items */
 } Line;
 
+/* Need a struct for the pointer color.
+   Since colors are unsigned, we need a flag to indicate whether
+   the color was ever set. */
+typedef struct _ptrc {
+  XColor pointer_color;
+  char used;
+} Ptr_color;
 
 /* There is one "form". Start of attempt to change that. */
 typedef struct _form {
@@ -185,15 +202,20 @@ typedef struct _form {
   char *expand_buffer;                  /* buffer to expand commands in */
   int expand_buffer_size;               /* current size */
   char have_env_var;                    /* at least one env var on cmd line */
+  Cursor pointer[3];
+  Ptr_color p_c[6];                     /* fg/bg for pointers, used flag */
+  int activate_on_press;                /* activate button on press */
 } Form;
 
+enum {input_pointer,button_pointer,button_in_pointer};
+enum {input_fore,input_back,
+      button_fore,button_back,
+      button_in_fore,button_in_back};
+
 extern Form cur_form;                   /* current form */
 #define CF cur_form                     /* I may want to undo this... */
 
   /* Globals: */
-/* These aren't customizable as to shape or color, so for now, they aren't
-   in the "form" structure */
-extern Cursor xc_ibeam, xc_hand;
 
 /* Link list roots */
 extern Item *root_item_ptr;             /* pointer to root of item list */
@@ -211,10 +233,12 @@ enum { c_bg, c_fg, c_item_bg, c_item_fg, c_itemlo, c_itemhi };
 extern char *color_names[4];
 extern char bg_state;
 extern char endDefaultsRead;
-enum { f_text, f_input, f_button };
-extern char *font_names[3];
+enum { f_text, f_input, f_button, f_timeout };
+extern char *font_names[4];
 extern char *screen_background_color;
 
+extern int colorset;
+extern int itemcolorset;
 
 /* From FvwmAnimate start */
 extern char *MyName;
@@ -240,7 +264,9 @@ extern int Channel[2];
 /* prototypes */
 void ReadXServer();                     /* ReadXServer.c */
 void RedrawText(Item *item);            /* FvwmForm.c */
+void RedrawTimeout(Item *item);         /* FvwmForm.c */
 void RedrawItem (Item *item, int click); /* FvwmForm.c */
+void UpdateRootTransapency(void);        /* FvwmForm.c */
 void DoCommand (Item *cmd);             /* FvwmForm.c */
 int FontWidth (XFontStruct *xfs);       /* FvwmForm.c */
 void RedrawFrame ();                    /* FvwmForm.c */
