@@ -41,6 +41,7 @@
 
 #include "config.h"
 #include "libs/fvwmlib.h"
+#include "libs/Colorset.h"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -79,7 +80,16 @@ void ReadXServer ()
     if (event.xany.window == CF.frame) {
       switch (event.type) {
       case ConfigureNotify:             /* has window be reconfigured */
-        ResizeFrame();                  /* adjust yourself... */
+	if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative && 
+	    event.xconfigure.send_event)
+	{
+	  UpdateRootTransapency();
+	}
+	else
+	{
+	  /* adjust yourself... */
+	  ResizeFrame();
+	}
         break;
 #if 0
       case SelectionClear:
@@ -93,6 +103,9 @@ void ReadXServer ()
         break;
 #endif
       case Expose:
+	while (XCheckTypedWindowEvent(dpy, CF.frame, Expose, &event));
+	if (event.xexpose.count != 0)
+	  break;
 	RedrawFrame();
 	if (CF.grab_server && !CF.server_grabbed) {
 	  if (GrabSuccess ==
@@ -146,6 +159,9 @@ void ReadXServer ()
 	}
         switch (ks) {                   /* regular key, may need adjustment */
         case XK_Tab:
+#ifdef XK_XKB_KEYS
+        case XK_ISO_Left_Tab:
+#endif
           if (event.xkey.state & ShiftMask) { /* shifted key */
             buf[0] = '\020';          /* chg shift tab to ^P */
           }
@@ -175,7 +191,19 @@ void ReadXServer ()
 	    }
 	  }
 	  break;
-	}
+	} else if (CF.cur_input == CF.cur_input->input.next_input) {
+          /* 1 ip field */
+          switch (buf[0]) {
+          case '\020':                  /* ^P previous field */
+            process_history(-1);
+            goto redraw_newcursor;
+            break;
+          case '\016':                  /* ^N  next field */
+            process_history(1);
+            goto redraw_newcursor;
+            break;
+          } /* end switch */
+        } /* end one input field */
 	switch (buf[0]) {
 	case '\001':  /* ^A */
 	  old_cursor = CF.abs_cursor;
@@ -302,7 +330,7 @@ void ReadXServer ()
 	{
 	  XSetForeground(dpy, CF.cur_input->header.dt_ptr->dt_item_GC,
                          CF.cur_input->header.dt_ptr->dt_colors[c_item_bg]);
-          /* Since XDrawString is being used, I changed this to clear the
+          /* Since DrawString is being used, I changed this to clear the
              entire input field.  dje 10/24/99. */
 	  XClearArea(dpy, CF.cur_input->header.win,
                      BOX_SPC + TEXT_SPC - 1, BOX_SPC,
@@ -319,16 +347,21 @@ void ReadXServer ()
                          CF.cur_input->header.dt_ptr->dt_colors[c_item_fg]);
 	  if (len > CF.cur_input->input.size)
 	    len = CF.cur_input->input.size;
-	  XDrawString(dpy, CF.cur_input->header.win,
-                           CF.cur_input->header.dt_ptr->dt_item_GC,
-			   BOX_SPC + TEXT_SPC,
-			   BOX_SPC + TEXT_SPC +
-                           CF.cur_input->header.dt_ptr->dt_font_struct->ascent,
-			   CF.cur_input->input.value +
-                           CF.cur_input->input.left, len);
+          CF.cur_input->header.dt_ptr->dt_Fstr->win = CF.cur_input->header.win;
+          CF.cur_input->header.dt_ptr->dt_Fstr->gc  =
+            CF.cur_input->header.dt_ptr->dt_item_GC;
+          CF.cur_input->header.dt_ptr->dt_Fstr->str = CF.cur_input->input.value;
+          CF.cur_input->header.dt_ptr->dt_Fstr->x   = BOX_SPC + TEXT_SPC;
+          CF.cur_input->header.dt_ptr->dt_Fstr->y   = BOX_SPC + TEXT_SPC
+            + CF.cur_input->header.dt_ptr->dt_Ffont->ascent;
+          CF.cur_input->header.dt_ptr->dt_Fstr->len = len;
+          FlocaleDrawString(dpy,
+                            CF.cur_input->header.dt_ptr->dt_Ffont,
+                            CF.cur_input->header.dt_ptr->dt_Fstr,
+                            FWS_HAVE_LENGTH);
 	  x = BOX_SPC + TEXT_SPC +
-            FontWidth(CF.cur_input->header.dt_ptr->dt_font_struct) *
-            CF.abs_cursor - 1;
+            CF.cur_input->header.dt_ptr->dt_Ffont->max_char_width
+            * CF.abs_cursor - 1;
 	  dy = CF.cur_input->header.size_y - 1;
 	  XDrawLine(dpy, CF.cur_input->header.win,
                     CF.cur_input->header.dt_ptr->dt_item_GC,
@@ -346,7 +379,8 @@ void ReadXServer ()
       if (event.xany.window == item->header.win) {
 	switch (event.type) {
 	case Expose:
-	  RedrawItem(item, 0);
+	  if (event.xexpose.count == 0)
+	    RedrawItem(item, 0);
 	  break;
 	case ButtonPress:
 	  if (item->type == I_INPUT) {
@@ -355,9 +389,9 @@ void ReadXServer ()
 	    CF.cur_input = item;
 	    RedrawItem(old_item, 1);
 	    CF.abs_cursor = (event.xbutton.x - BOX_SPC -
-			  TEXT_SPC +
-                          FontWidth(item->header.dt_ptr->dt_font_struct) / 2)
-	      / FontWidth(item->header.dt_ptr->dt_font_struct);
+                             TEXT_SPC +
+                             item->header.dt_ptr->dt_Ffont->max_char_width / 2)
+	      / item->header.dt_ptr->dt_Ffont->max_char_width;
 	    if (CF.abs_cursor < 0)
 	      CF.abs_cursor = 0;
 	    if (CF.abs_cursor > item->input.size)
@@ -382,16 +416,49 @@ void ReadXServer ()
 	    ToggleChoice(item);
 	  if (item->type == I_BUTTON) {
 	    RedrawItem(item, 1);        /* push button in */
-            usleep(MICRO_S_FOR_10MS);   /* make sure its visible */
-            RedrawItem(item, 0);        /* pop button out */
-            DoCommand(item);            /* execute the button command */
+            if (CF.activate_on_press) {
+              usleep(MICRO_S_FOR_10MS);   /* make sure its visible */
+              RedrawItem(item, 0);        /* pop button out */
+              DoCommand(item);            /* execute the button command */
+            } else {
+              XGrabPointer(dpy, item->header.win,
+                           False,         /* owner of events */
+                           ButtonReleaseMask, /* events to report */
+                           GrabModeAsync, /* keyboard mode */
+                           GrabModeAsync, /* pointer mode */
+                           None,          /* confine to */
+                           /* I sort of like this, the hand points in
+                              the other direction and the color is
+                              reversed. I don't know what other GUIs do,
+                              Java doesn't do anything, neither does anything
+                              else I can find...dje */
+                           CF.pointer[button_in_pointer],   /* cursor */
+                           CurrentTime);
+            } /* end activate on press */
 	  }
 	  break;
 	case ButtonRelease:
-          /* There used to be logic in here to execute the button action
-             only on release.  It missed some release events, and I didn't
-             see the value to the feature.  So now buttons are handled with
-             only the press event.  dje 11/26/99. */
+          if (!CF.activate_on_press) {
+            RedrawItem(item, 0);
+            if (CF.grab_server && CF.server_grabbed) {
+              /* You have to regrab the pointer, or focus
+                 can go to another window.
+                 grab...
+              */
+              XGrabPointer(dpy, CF.frame, True, 0, GrabModeAsync, GrabModeAsync,
+                           None, None, CurrentTime);
+              XFlush(dpy);
+            } else {
+              XUngrabPointer(dpy, CurrentTime);
+              XFlush(dpy);
+            }
+            if (event.xbutton.x >= 0 &&
+                event.xbutton.x < item->header.size_x &&
+                event.xbutton.y >= 0 &&
+                event.xbutton.y < item->header.size_y) {
+              DoCommand(item);
+            }
+          }
 	  break;
 	}
       }
@@ -491,6 +558,10 @@ static int process_tabtypes(unsigned char * buf) {
 
 static void process_regular_char_input(unsigned char *buf) {
   char *sp, *dp, *ep;
+  /* n is the space actually used.
+     buf is the size of the buffer
+     size is the size of the field on the screen
+     size is used as the increment, arbitrarily. */
   if (++(CF.cur_input->input.n) >= CF.cur_input->input.buf) {
     CF.cur_input->input.buf += CF.cur_input->input.size;
     CF.cur_input->input.value =
