@@ -38,7 +38,6 @@
 
 #include <libs/Module.h>
 #include <libs/fvwmlib.h>
-#include <libs/Picture.h>
 #include <fvwm/fvwm.h>
 #include <libs/vpacket.h>
 
@@ -342,6 +341,7 @@ process_message (unsigned long type,
   int button = 0;
   GtkWidget *widget;
   window_list_options *opts;
+
   char name[128];
 
   switch (type)
@@ -405,7 +405,7 @@ process_message (unsigned long type,
     case M_DESTROY_WINDOW:
       g_hash_table_remove (window_list_entries, &(body[0]));
       break;
-    case M_WINDOW_NAME:
+    case M_VISIBLE_NAME:
       {
 	window_list_entry *wle = lookup_window_list_entry (body[0]);
 	if (wle->name)
@@ -413,7 +413,7 @@ process_message (unsigned long type,
 	wle->name = safestrdup ((char*) (&body[3]));
       }
       break;
-    case M_ICON_NAME:
+    case MX_VISIBLE_ICON_NAME:
       {
 	window_list_entry *wle = lookup_window_list_entry (body[0]);
 	if (wle->icon_name)
@@ -423,10 +423,12 @@ process_message (unsigned long type,
       break;
     case M_MINI_ICON:
       {
-	window_list_entry *wle = lookup_window_list_entry (body[0]);
+	MiniIconPacket *mip = (MiniIconPacket *) body;
+	window_list_entry *wle = lookup_window_list_entry (mip->w);
+
 	if (wle->mini_icon)
 	  free(wle->mini_icon);
-	wle->mini_icon = safestrdup ((char*) (&body[8]));
+	wle->mini_icon = safestrdup (mip->name);
       }
       break;
     }
@@ -498,6 +500,7 @@ main (int argc, char **argv)
 
   parse_options ();
 
+  /* normal messages */
   SetMessageMask (fvwm_fd,
 		  M_STRING |
 		  M_CONFIG_INFO |
@@ -506,9 +509,12 @@ main (int argc, char **argv)
 		  M_DESTROY_WINDOW |
 		  M_CONFIGURE_WINDOW |
 		  M_NEW_DESK |
-		  M_WINDOW_NAME |
-		  M_ICON_NAME |
+		  M_VISIBLE_NAME |
 		  M_MINI_ICON);
+  /* extended messages */
+  SetMessageMask (fvwm_fd,
+		  MX_VISIBLE_ICON_NAME);
+
 
   SendText (fvwm_fd, "Send_WindowList", 0);
 
