@@ -30,13 +30,13 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkprivate.h>
 #include "gtkpixmapmenuitem.h"
-#ifdef IMLIB
+#ifdef GDK_IMLIB
 #include <gdk_imlib.h>
 #endif
 
 #include "libs/fvwmlib.h"
 #include "libs/Module.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 
 
 extern int fvwm_fd[2];
@@ -162,12 +162,12 @@ menu_item_new_with_pixmap_and_label (char *file, char *l_label, char *r_label)
   char *path;
   gint accel_key = 0, underbar_pos = -1;
   gchar *pattern, *converted_label;
-#ifdef IMLIB
+#ifdef GDK_IMLIB
   GdkImlibImage *im;
   int w, h;
 #endif
 
-  path = findImageFile (file ? file : "", image_path, R_OK);
+  path = PictureFindImageFile (file ? file : "", image_path, R_OK);
 
   if (path == NULL)
     {
@@ -175,7 +175,7 @@ menu_item_new_with_pixmap_and_label (char *file, char *l_label, char *r_label)
     }
   else
     {
-#ifdef IMLIB
+#ifdef GDK_IMLIB
       im = gdk_imlib_load_image (path);
       if (im)
 	{
