@@ -27,7 +27,6 @@
 
 #define NONE 0
 #define VERTICAL 1
-
 #define HORIZONTAL 2
 
 #include "config.h"
@@ -54,29 +53,19 @@
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
-
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif /* SHAPE */
 
 #include "libs/defaults.h"
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Colorset.h"
 #include "fvwm/fvwm.h"
+#include "libs/PictureGraphics.h"
 #include "FvwmIconBox.h"
 
 
 char *MyName;
 
-XFontStruct *font;
-#ifdef I18N_MB
-XFontSet fontset;
-#endif
-
 Display *dpy;			/* which display are we talking to */
 int x_fd;
 fd_set_size_t fd_width;
@@ -95,6 +84,7 @@ int colorset = -1;
 int Iconcolorset = -1;
 int IconHicolorset = -1;
 Bool UseSkipList = False;
+Bool Swallowed = False;
 
 /* same strings as in misc.c */
 char NoClass[] = "NoClass";
@@ -106,6 +96,8 @@ Pixel act_icon_fore_pix, act_icon_back_pix,
   act_icon_hilite_pix, act_icon_shadow_pix;
 
 GC  NormalGC,ShadowGC,ReliefGC,IconShadowGC,IconReliefGC;
+FlocaleFont *Ffont;
+FlocaleWinString *FwinString;
 Window main_win;
 Window holder_win;
 Window icon_win;
@@ -128,9 +120,10 @@ int UWidth, UHeight;
 		       LeaveWindowMask | PointerMotionMask)
 
 unsigned long m_mask = M_CONFIGURE_WINDOW|M_ADD_WINDOW|M_DESTROY_WINDOW|
-		 M_END_WINDOWLIST| M_ICONIFY|M_DEICONIFY|M_ICON_NAME|
-		 M_RES_NAME|M_RES_CLASS|M_WINDOW_NAME|M_ICON_FILE|
-		 M_DEFAULTICON|M_CONFIG_INFO|M_END_CONFIG_INFO;
+  M_END_WINDOWLIST| M_ICONIFY|M_DEICONIFY|
+  M_RES_NAME|M_RES_CLASS|M_VISIBLE_NAME|M_ICON_FILE|
+  M_DEFAULTICON|M_CONFIG_INFO|M_END_CONFIG_INFO;
+unsigned long mx_mask = MX_VISIBLE_ICON_NAME|MX_PROPERTY_CHANGE;
 
 struct icon_info *Hilite;
 int main_width, main_height;
@@ -148,7 +141,7 @@ int interval = 8;
 int motion = NONE;
 int primary = LEFT, secondary = BOTTOM;
 int xneg = 0, yneg = 0;
-int ClickTime = 150;
+int ClickTime = DEFAULT_CLICKTIME;
 unsigned int bar_width = 9;
 int redraw_flag = 3;
 int icon_relief = 4;
@@ -184,6 +177,7 @@ int sortby = UNSORT;
 char* AnimCommand = NULL;
 
 int save_color_limit = 0;                   /* color limit from config */
+static Bool have_double_click = False;
 static Bool is_dead_pipe = False;
 
 static RETSIGTYPE TerminateHandler(int);
@@ -191,13 +185,8 @@ static int myErrorHandler(Display *dpy, XErrorEvent *event);
 static void CleanUp(void);
 static void change_colorset(int color);
 
-#ifdef I18N_MB
-#ifdef __STDC__
-#define XTextWidth(x,y,z)       XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z)       XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
+Bool do_allow_bad_access = False;
+Bool was_bad_access = False;
 
 /************************************************************************
   Main
@@ -210,9 +199,7 @@ int main(int argc, char **argv)
   char *temp, *s;
   XIconSize* size;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmIconBox");
 
   temp = argv[0];
   s=strrchr(argv[0], '/');
@@ -283,9 +270,10 @@ int main(int argc, char **argv)
 	      XDisplayName(display_name));
       exit (1);
     }
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
+  FShapeInit(dpy);
   x_fd = XConnectionNumber(dpy);
 
   fd_width = GetFdWidth();
@@ -300,11 +288,10 @@ int main(int argc, char **argv)
 
   XSetErrorHandler(myErrorHandler);
 
-  /* SetMessageMask(fd, m_mask); */
-
   ParseOptions();
-
-  SetMessageMask(fd, m_mask); /* it may have changed */
+  /* m_mask/mx_mask may have changed in the ParseOptions call*/
+  SetMessageMask(fd, m_mask);
+  SetMessageMask(fd, mx_mask);
 
   if ((local_flags & SETWMICONSIZE) && (size = XAllocIconSize()) != NULL){
     size->max_width  = size->min_width  = max_icon_width + icon_relief;
@@ -317,7 +304,7 @@ int main(int argc, char **argv)
 
   CreateWindow();
 
-  SendFvwmPipe(fd,"Send_WindowList",0);
+  SendText(fd,"Send_WindowList",0);
   atexit(CleanUp);
 
   /* tell fvwm we're running */
@@ -356,9 +343,12 @@ void Loop(void)
 	    {
 	    case Expose:
 	      if(Event.xexpose.count == 0){
-		if (Event.xany.window == main_win){
-		  RedrawWindow();
-		}else{
+		if (Event.xany.window == main_win)
+		{
+			RedrawWindow();
+		}
+		else
+		{
 		  tmp = Head;
 		  while(tmp != NULL){
 		    if (Event.xany.window == tmp->icon_pixmap_w){
@@ -375,6 +365,13 @@ void Loop(void)
 	      break;
 
 	    case ConfigureNotify:
+	      if (Event.xconfigure.window == icon_win &&
+		  (colorset < 0 ||
+		   Colorset[colorset].pixmap != ParentRelative))
+	      {
+		/* needs this event for transparency */
+		break;
+	      }
 	      if (!XGetGeometry(dpy,main_win,&root,&x,&y,
 				(unsigned int *)&tw,(unsigned int *)&th,
 				(unsigned int *)&border_width,
@@ -424,6 +421,17 @@ void Loop(void)
 		  RedrawWindow();
 		}
 	      }
+	      else if (ready &&
+		       (Event.xconfigure.send_event ||
+			Event.xconfigure.window == icon_win) && colorset >= 0 &&
+		       Colorset[colorset].pixmap == ParentRelative)
+	      {
+		/* moved */
+		XClearArea(dpy, icon_win, 0,0,0,0, False);
+		/* XClearArea(dpy, icon_win, 0,0,0,0, True); is not enough
+		 * when you unshade */
+		RedrawWindow();
+	      }
 	      break;
 	    case KeyPress:
 	      ExecuteKey(Event);
@@ -564,29 +572,31 @@ void Loop(void)
 		  XFree (tmp->wmhints);
 		tmp->wmhints = XGetWMHints(dpy, tmp->id);
 		if (tmp->wmhints && (tmp->wmhints->flags & IconPixmapHint)){
-#ifdef SHAPE
-		/* turn off "old" shape mask */
-		if (tmp->icon_maskPixmap != None)
-		  XShapeCombineMask(dpy, tmp->icon_pixmap_w,
-				    ShapeBounding, 0, 0, None, ShapeSet);
-#endif
+		  /* turn off "old" shape mask */
+		  if (FShapesSupported && tmp->icon_maskPixmap != None)
+		  {
+		    FShapeCombineMask(
+		      dpy, tmp->icon_pixmap_w, FShapeBounding, 0, 0, None,
+		      FShapeSet);
+		  }
 		  if (tmp->iconPixmap != None)
 		    XFreePixmap(dpy, tmp->iconPixmap);
 		  GetIconBitmap(tmp);
-#ifdef SHAPE
-		  if (tmp->icon_maskPixmap != None) {
-		    int hr = (Pdefault | (tmp->icon_depth == 1)
-			      | IS_PIXMAP_OURS(tmp)) ? icon_relief/2 : 0;
+		  if (FShapesSupported && tmp->icon_maskPixmap != None)
+		  {
+		    int hr;
+		    hr =
+		      (Pdefault || (tmp->icon_depth == 1) ||
+		       IS_PIXMAP_OURS(tmp)) ? icon_relief/2 : 0;
 		    /* This XSync is necessary, do not remove */
 		    /* without it the icon window will disappear */
 		    /* server: Exceed, client Sparc/Solaris 2.6 */
 		    /* don't know if this is a server or xlib bug */
 		    XSync(dpy, False);
-		    XShapeCombineMask(dpy, tmp->icon_pixmap_w,
-				      ShapeBounding, hr, hr,
-				      tmp->icon_maskPixmap, ShapeSet);
+		    FShapeCombineMask(
+		      dpy, tmp->icon_pixmap_w, FShapeBounding, hr, hr,
+		      tmp->icon_maskPixmap, FShapeSet);
 		  }
-#endif
 		  AdjustIconWindow(tmp, i);
 		  if (max_icon_height != 0)
 		  RedrawIcon(tmp, 1);
@@ -704,117 +714,171 @@ void RedrawIcons(void)
 
 void RedrawIcon(struct icon_info *item, int f)
 {
-  unsigned long plane = 1;
-  int hr, len;
-  int diff, lm ,w, h, tw;
-  char label[256];
+	unsigned long plane = 1;
+	int hr, len;
+	int diff, lm ,w, h, tw;
+	char label[256];
 
-  hr = icon_relief/2;
+	hr = icon_relief/2;
 
-  if (Hilite == item){
-    XSetForeground(dpy, NormalGC, act_icon_fore_pix);
-    XSetBackground(dpy, NormalGC, act_icon_back_pix);
-    XSetForeground(dpy, IconReliefGC, act_icon_hilite_pix);
-    XSetForeground(dpy, IconShadowGC, act_icon_shadow_pix);
+	if (Hilite == item)
+	{
+		XSetForeground(dpy, NormalGC, act_icon_fore_pix);
+		XSetBackground(dpy, NormalGC, act_icon_back_pix);
+		XSetForeground(dpy, IconReliefGC, act_icon_hilite_pix);
+		XSetForeground(dpy, IconShadowGC, act_icon_shadow_pix);
 
-    if (max_icon_height != 0 && (IS_ICON_OURS(item)))
-      XSetWindowBackground(dpy, item->icon_pixmap_w, act_icon_back_pix);
-    XSetWindowBackground(dpy, item->IconWin, act_icon_back_pix);
-  }
-
-  /* icon pixmap */
-  if ((f & 1) && (IS_ICON_OURS(item))){
-    if (item->iconPixmap != None && item->icon_pixmap_w != None){
-      if (item->icon_depth == 1) {
-	XCopyPlane(dpy, item->iconPixmap, item->icon_pixmap_w, NormalGC,
-		   0, 0, item->icon_w, item->icon_h,
-		   hr, hr, plane);
-      } else {
-        if (Pdefault || IS_PIXMAP_OURS(item)) {
-	  XCopyArea(dpy, item->iconPixmap, item->icon_pixmap_w, NormalGC,
-		    0, 0, item->icon_w, item->icon_h, hr, hr);
-        } else {
-	  XCopyArea(dpy, item->iconPixmap, item->icon_pixmap_w,
-		    DefaultGC(dpy, screen),
-		    0, 0, item->icon_w, item->icon_h, 0, 0);
+		if (max_icon_height != 0 && (IS_ICON_OURS(item)))
+		{
+			if (item->icon_alphaPixmap != None)
+				XSetWindowBackgroundPixmap(dpy,
+							   item->icon_pixmap_w,
+							   ParentRelative);
+			else
+				XSetWindowBackground(dpy, item->icon_pixmap_w,
+						     act_icon_back_pix);
+		}
+		XSetWindowBackground(dpy, item->IconWin, act_icon_back_pix);
 	}
-      }
-    }
 
-    if (!(IS_ICON_SHAPED(item))
-	&& (Pdefault || (item->icon_depth == 1) || IS_PIXMAP_OURS(item))) {
-      if (item->icon_w > 0 && item->icon_h > 0)
-	RelieveRectangle(dpy, item->icon_pixmap_w, 0, 0, item->icon_w
-		         +icon_relief - 1,
-		         item->icon_h + icon_relief - 1, IconReliefGC,
-		         IconShadowGC, 2);
-      else
-	RelieveRectangle(dpy, item->icon_pixmap_w, 0, 0, max_icon_width
-		         + icon_relief - 1,
-		         max_icon_height + icon_relief - 1, IconReliefGC,
-		         IconShadowGC, 2);
-    }
-  }
+	/* icon pixmap */
+	if ((f & 1) && (IS_ICON_OURS(item)))
+	{
+		if (item->iconPixmap != None && item->icon_pixmap_w != None)
+		{
+			if (item->icon_depth == 1)
+			{
+				XCopyPlane(dpy,
+					   item->iconPixmap,
+					   item->icon_pixmap_w,
+					   NormalGC,
+					   0, 0, item->icon_w, item->icon_h,
+					   hr, hr, plane);
+			}
+			else if (IS_PIXMAP_OURS(item))
+			{
+				if (item->icon_alphaPixmap != None)
+					XClearWindow(dpy, item->icon_pixmap_w);
+				PGraphicsCopyPixmaps(dpy,
+						     item->iconPixmap,
+						     item->icon_maskPixmap,
+						     item->icon_alphaPixmap,
+						     item->icon_depth,
+						     item->icon_pixmap_w,
+						     NormalGC,
+						     0, 0,
+						     item->icon_w, item->icon_h,
+						     hr, hr);  
+			}
+			else if (Pdefault)
+			{
+				XCopyArea(dpy, item->iconPixmap,
+					  item->icon_pixmap_w,
+					  NormalGC,
+					  0, 0,
+					  item->icon_w, item->icon_h, hr, hr);
+			}
+			else
+			{
+				XCopyArea(dpy, item->iconPixmap,
+					  item->icon_pixmap_w,
+					  DefaultGC(dpy, screen),
+					  0, 0,
+					  item->icon_w, item->icon_h, 0, 0);
+			}
+		}
 
-  /* label */
-  if (f & 2){
-    w = max_icon_width + icon_relief;
-    h = max_icon_height + icon_relief;
+		if (!(IS_ICON_SHAPED(item))
+		    && (Pdefault || (item->icon_depth == 1) ||
+			IS_PIXMAP_OURS(item))) 
+		{
+			if (item->icon_w > 0 && item->icon_h > 0)
+				RelieveRectangle(dpy, item->icon_pixmap_w,
+						 0, 0, item->icon_w
+						 +icon_relief - 1,
+						 item->icon_h + icon_relief - 1,
+						 IconReliefGC,
+						 IconShadowGC, 2);
+			else
+				RelieveRectangle(dpy, item->icon_pixmap_w,
+						 0, 0, max_icon_width
+						 + icon_relief - 1,
+						 max_icon_height + 
+						 icon_relief - 1,
+						 IconReliefGC,
+						 IconShadowGC, 2);
+		}
+	}
 
-    if (IS_ICONIFIED(item)){
-      sprintf(label, "(%s)", item->name);
-    }else
-      strcpy(label, item->name);
+	/* label */
+	if (f & 2)
+	{
+		w = max_icon_width + icon_relief;
+		h = max_icon_height + icon_relief;
 
-    len = strlen(label);
-    tw = XTextWidth(font, label, len);
-    diff = max_icon_width + icon_relief - tw;
-    lm = diff/2;
-    lm = lm > 4 ? lm : 4;
+		if (IS_ICONIFIED(item))
+		{
+			sprintf(label, "(%s)", item->name);
+		}
+		else
+		{
+			strcpy(label, item->name);
+		}
 
-    if (Hilite == item){
-      XRaiseWindow(dpy, item->IconWin);
-      XMoveResizeWindow(dpy, item->IconWin,
-			item->x + min(0, (diff - 8))/2,
-			item->y + h,
-			max(tw + 8, w), 6 + font->ascent +
-			font->descent);
-      XClearWindow(dpy, item->IconWin);
-#ifdef I18N_MB
-      XmbDrawString(dpy, item->IconWin, fontset, NormalGC, lm, 3 + font->ascent,
-#else
-      XDrawString(dpy, item->IconWin, NormalGC, lm, 3 + font->ascent,
-#endif
-		  label, len);
-      RelieveRectangle(dpy, item->IconWin, 0, 0,
-		       max(tw + 8, w) - 1, 6 + font->ascent +
-		       font->descent - 1, IconReliefGC, IconShadowGC, 2);
-    }else{
-      XMoveResizeWindow(dpy, item->IconWin, item->x, item->y + h,
-			w, 6 + font->ascent + font->descent);
-      XClearWindow(dpy, item->IconWin);
-#ifdef I18N_MB
-      XmbDrawString(dpy, item->IconWin, fontset, NormalGC, lm, 3 + font->ascent,
-#else
-      XDrawString(dpy, item->IconWin, NormalGC, lm, 3 + font->ascent,
-#endif
-		  label, len);
-      RelieveRectangle(dpy, item->IconWin, 0, 0,
-		       w - 1, 5 + font->ascent + font->descent,
-		       IconReliefGC, IconShadowGC, 2);
-    }
-  }
+		len = strlen(label);
+		tw = FlocaleTextWidth(Ffont, label, len);
+		diff = max_icon_width + icon_relief - tw;
+		lm = diff/2;
+		lm = lm > 4 ? lm : 4;
 
-  if (Hilite == item){
-    XSetForeground(dpy, NormalGC, icon_fore_pix);
-    XSetBackground(dpy, NormalGC, icon_back_pix);
-    XSetForeground(dpy, IconReliefGC, icon_hilite_pix);
-    XSetForeground(dpy, IconShadowGC, icon_shadow_pix);
+		FwinString->str = label;
+		FwinString->win = item->IconWin;
+		FwinString->gc =  NormalGC;
+		FwinString->x = lm;
+		FwinString->y = 3 + Ffont->ascent;
 
-    if (max_icon_height != 0 && (IS_ICON_OURS(item)))
-      XSetWindowBackground(dpy, item->icon_pixmap_w, icon_back_pix);
-    XSetWindowBackground(dpy, item->IconWin, icon_back_pix);
-  }
+		if (Hilite == item)
+		{
+			XRaiseWindow(dpy, item->IconWin);
+			XMoveResizeWindow(dpy, item->IconWin,
+					  item->x + min(0, (diff - 8))/2,
+					  item->y + h,
+					  max(tw + 8, w), 6 + Ffont->height);
+			XClearWindow(dpy, item->IconWin);
+			FlocaleDrawString(dpy, Ffont, FwinString, 0);
+			RelieveRectangle(dpy, item->IconWin, 0, 0,
+					 max(tw + 8, w) - 1,
+					 6 + Ffont->height - 1,
+					 IconReliefGC, IconShadowGC, 2);
+		}
+		else
+		{
+			XMoveResizeWindow(dpy, item->IconWin,
+					  item->x, item->y + h,
+					  w, 6 + Ffont->height);
+			XClearWindow(dpy, item->IconWin);
+			FlocaleDrawString(dpy, Ffont, FwinString, 0);
+			RelieveRectangle(dpy, item->IconWin, 0, 0,
+					 w - 1, 5 + Ffont->height,
+					 IconReliefGC, IconShadowGC, 2);
+		}
+	}
+
+	if (Hilite == item)
+	{
+		XSetForeground(dpy, NormalGC, icon_fore_pix);
+		XSetBackground(dpy, NormalGC, icon_back_pix);
+		XSetForeground(dpy, IconReliefGC, icon_hilite_pix);
+		XSetForeground(dpy, IconShadowGC, icon_shadow_pix);
+
+		if (max_icon_height != 0 && (IS_ICON_OURS(item)) && 
+		    item->icon_alphaPixmap == None)
+		{
+			XSetWindowBackground(dpy, item->icon_pixmap_w,
+					     icon_back_pix);
+		}
+		XSetWindowBackground(dpy, item->IconWin, icon_back_pix);
+	}
 }
 
 void animate(struct icon_info *item, unsigned long *body)
@@ -1006,12 +1070,6 @@ void CreateWindow(void)
   XSizeHints mysizehints;
   XTextProperty name;
   XClassHint class_hints;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
 
   h_margin = margin1*2 + bar_width + margin2 + 8;
   v_margin = margin1*2 + bar_width + margin2 + 8;
@@ -1020,31 +1078,12 @@ void CreateWindow(void)
   _XA_WM_PROTOCOLS = XInternAtom (dpy, "WM_PROTOCOLS", False);
 
   /* load the font */
-#ifdef I18N_MB
-  if ((fontset = XCreateFontSet(dpy, font_string, &ml, &mc, &ds)) == NULL)
+  Ffont = FlocaleLoadFont(dpy, font_string, MyName);
+  if (Ffont == NULL)
   {
-#ifdef STRICTLY_FIXED
-    if ((fontset = XCreateFontSet(dpy, "fixed", &ml, &mc, &ds)) == NULL)
-#else
-    if ((fontset = XCreateFontSet(dpy, "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*", &ml, &mc, &ds)) == NULL)
-#endif
-    {
-      fprintf(stderr,"%s: No fonts available\n",MyName);
-      exit(1);
-    }
+    fprintf(stderr,"%s: No fonts available, exiting\n",MyName);
+    exit(1);
   }
-  XFontsOfFontSet(fontset, &fs_list, &ml);
-  font = fs_list[0];
-#else
-  if ((font = XLoadQueryFont(dpy, font_string)) == NULL)
-  {
-    if ((font = XLoadQueryFont(dpy, "fixed")) == NULL)
-    {
-      fprintf(stderr,"%s: No fonts available\n",MyName);
-      exit(1);
-    }
-  }
-#endif
 
   if ((local_flags & HIDE_H))
     v_margin -= bar_width + margin2 + 4;
@@ -1052,8 +1091,7 @@ void CreateWindow(void)
     h_margin -= bar_width + margin2 + 4;
 
   UWidth = max_icon_width + icon_relief + interval;
-  UHeight = font->ascent + font->descent + max_icon_height +
-    icon_relief + 6 + interval;
+  UHeight = Ffont->height + max_icon_height + icon_relief + 6 + interval;
   Width = UWidth * num_columns + interval -1;
   Height = UHeight * num_rows + interval -1;
 
@@ -1168,6 +1206,8 @@ void CreateWindow(void)
 				 icon_win_width,
 				 icon_win_height,
 				 0,fore_pix,back_pix);
+  /* needed for transparency */
+  XSelectInput(dpy,icon_win,StructureNotifyMask);
 
   gcm = GCForeground|GCBackground;
   gcv.foreground = hilite_pix;
@@ -1184,17 +1224,26 @@ void CreateWindow(void)
   gcv.foreground = icon_shadow_pix;
   IconShadowGC = fvwmlib_XCreateGC(dpy, main_win, gcm, &gcv);
 
-  gcm = GCForeground|GCBackground|GCFont;
+  gcm = GCForeground|GCBackground;
+  if (Ffont->font != NULL)
+  {
+    gcm |= GCFont;
+    gcv.font = Ffont->font->fid;
+  }
   gcv.foreground = icon_fore_pix;
-  gcv.font =  font->fid;
   NormalGC = fvwmlib_XCreateGC(dpy, main_win, gcm, &gcv);
 
+  /* init our Flocale window string for text drawing */
+  FlocaleAllocateWinString(&FwinString);
 
   /* icon_win's background */
   if (colorset >= 0) {
     SetWindowBackground(dpy, icon_win, icon_win_width, icon_win_height,
                         &Colorset[(colorset)], Pdepth, NormalGC,
 			True);
+    SetWindowBackground(dpy, holder_win, mysizehints.width, mysizehints.height,
+                        &Colorset[(colorset)], Pdepth, NormalGC,
+			True);
     SetWindowBackground(dpy, main_win, mysizehints.width, mysizehints.height,
                         &Colorset[(colorset)], Pdepth, NormalGC,
 			True);
@@ -1299,7 +1348,8 @@ static void change_colorset(int color)
     SetWindowBackground(
       dpy, icon_win, icon_win_width, icon_win_height, &Colorset[(colorset)],
       Pdepth, NormalGC, True);
-    XSetWindowBackground(dpy, holder_win, back_pix);
+    SetWindowBackground(dpy, holder_win, main_width - h_margin,
+      main_height - v_margin, &Colorset[(colorset)], Pdepth, NormalGC, True);
     XSetWindowBackground(dpy, h_scroll_bar, back_pix);
     XSetWindowBackground(dpy, v_scroll_bar, back_pix);
     XSetWindowBackground(dpy, l_button, back_pix);
@@ -1567,25 +1617,38 @@ DeadPipe(int nonsense)
 static void
 CleanUp(void)
 {
+	struct icon_info *tmpi;
 #if 0
-  struct icon_info *tmpi, *tmpi2;
-  struct mousefunc *tmpm, *tmpm2;
-  struct keyfunc *tmpk, *tmpk2;
-  struct iconfile *tmpf, *tmpf2;
+	struct icon_info *tmpi, *tmpi2;
+	struct mousefunc *tmpm, *tmpm2;
+	struct keyfunc *tmpk, *tmpk2;
+	struct iconfile *tmpf, *tmpf2;
 #endif
 
-  if (!is_dead_pipe)
-  {
-    if ((local_flags & SETWMICONSIZE))
-    {
-      XDeleteProperty(dpy, Root, XA_WM_ICON_SIZE);
-      XFlush(dpy);
-    }
-  }
-
+	if (!is_dead_pipe)
+	{
+		if ((local_flags & SETWMICONSIZE))
+		{
+			XDeleteProperty(dpy, Root, XA_WM_ICON_SIZE);
+			XFlush(dpy);
+		}
+	}
+	/* DV: my, what is all this stuff good for? All memory gets freed
+	 * automatically! 
+	 * OC: Yes but the icon window must be reparented to the root window */
+	tmpi = Head;
+	while(tmpi != NULL)
+	{
+		if (!IS_ICON_OURS(tmpi) && tmpi->icon_pixmap_w &&
+		    tmpi->icon_w != 0)
+		{
+			XSelectInput(dpy, tmpi->icon_pixmap_w, NoEventMask);
+			XReparentWindow(dpy, tmpi->icon_pixmap_w, Root, 0, 0);
+			XUnmapWindow(dpy, tmpi->icon_pixmap_w);
+		}
+		tmpi = tmpi->next;
+	}
 #if 0
-  /* DV: my, what is all this stuff good for? All memory gets freed
-   * automatically! */
   if (FvwmDefaultIcon != NULL)
     free(FvwmDefaultIcon);
 
@@ -1651,6 +1714,12 @@ void ParseOptions(void)
     {
       if (strncasecmp(tline,CatString3("*", MyName, "Geometry"),Clength+9)==0)
       {
+	num_columns = 6;
+	num_rows = 1;
+	geom_x = -10000;
+	geom_y = -10000;
+	xneg = 0;
+	yneg = 0;
 	tmp = &tline[Clength+9];
 	while(((isspace((unsigned char)*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
 	  tmp++;
@@ -2001,7 +2070,10 @@ void parsemouse(char *tline)
   if (strncasecmp(start, "Click", 5) == 0)
     f->type = CLICK;
   else if (strncasecmp(start, "DoubleClick", 11) == 0)
+  {
     f->type = DOUBLE_CLICK;
+    have_double_click = True;
+  }
 
   /* actions */
   tline = end;
@@ -2311,7 +2383,7 @@ void process_message(unsigned long type, unsigned long *body)
   case M_RES_CLASS:
     UpdateItem(type, body[0], (char *)&body[3]);
     break;
-  case M_WINDOW_NAME:
+  case M_VISIBLE_NAME:
     tmp = UpdateItem(type, body[0], (char *)&body[3]);
     if (!ready || tmp == NULL)
       break;
@@ -2338,7 +2410,7 @@ void process_message(unsigned long type, unsigned long *body)
     }
     }
     break;
-  case M_ICON_NAME:
+  case MX_VISIBLE_ICON_NAME:
     tmp = UpdateItem(type, body[0], (char *)&body[3]);
     if (!ready || tmp == NULL)
     break;
@@ -2435,20 +2507,32 @@ void process_message(unsigned long type, unsigned long *body)
     ready = 1;
     break;
   case M_CONFIG_INFO:
-  {
-    char *tline, *token;
-    int color;
-    tline = (char*)&(body[3]);
-    token = PeekToken(tline, &tline);
-    if (StrEquals(token, "Colorset")) {
-      color = LoadColorset(tline);
-      change_colorset(color);
-    }
-    else if (StrEquals(token, XINERAMA_CONFIG_STRING))
     {
-      FScreenConfigureModule(tline);
-    }
+      char *tline, *token;
+      int color;
+      tline = (char*)&(body[3]);
+      token = PeekToken(tline, &tline);
+      if (StrEquals(token, "Colorset")) {
+	color = LoadColorset(tline);
+	change_colorset(color);
+      }
+      else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+      {
+	  FScreenConfigureModule(tline);
+      }
   }
+  break;
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+	(colorset > -1 && Colorset[colorset].pixmap == ParentRelative) &&
+	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == icon_win)))
+    {
+      XClearArea(dpy, icon_win, 0,0,0,0, False);
+    }
+    else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == icon_win)
+    {
+      Swallowed = body[1];
+    }
     break;
   default:
     break;
@@ -2629,7 +2713,7 @@ struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
       strcpy(str, item);
 
       switch (type){
-      case M_ICON_NAME:
+      case MX_VISIBLE_ICON_NAME:
 	if (tmp->name)
 	  free(tmp->name);
 	tmp->name = str;
@@ -2640,7 +2724,7 @@ struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
 	tmp->icon_file = str;
 	tmp->extra_flags &= ~DEFAULTICON;
 	return tmp;
-      case M_WINDOW_NAME:
+      case M_VISIBLE_NAME:
 	if (tmp->window_name)
 	  free(tmp->window_name);
 	tmp->window_name = str;
@@ -2877,35 +2961,73 @@ void freeitem(struct icon_info *item, int d)
 
 
 /************************************************************************
-  IsClick
-  	Based on functions.c from Fvwm:
-	Copyright 1988, Evans and Sutherland Computer Corporation,
-       	Copyright 1989, Massachusetts Institute of Technology,
-       	Copyright 1993, Robert Nation.
+ * CheckActionType
+ *   Based on functions.c from Fvwm:
  ***********************************************************************/
-Bool IsClick(int x,int y,unsigned EndMask, XEvent *d)
+static int CheckActionType(
+  int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed)
 {
   int xcurrent,ycurrent,total = 0;
+  int dist;
+  Bool do_sleep = False;
 
   xcurrent = x;
   ycurrent = y;
-  while((total < ClickTime)&&
-        (x - xcurrent < 5)&&(x - xcurrent > -5)&&
-        (y - ycurrent < 5)&&(y - ycurrent > -5))
+  dist = DEFAULT_MOVE_THRESHOLD;
+
+  while (total < ClickTime || !may_time_out)
+  {
+    if (!(x - xcurrent <= dist && xcurrent - x <= dist &&
+	  y - ycurrent <= dist && ycurrent - y <= dist))
     {
-      usleep(10000);
-      total+=10;
-      if(XCheckMaskEvent (dpy,EndMask, d))
-	return True;
-      if(XCheckMaskEvent (dpy,ButtonMotionMask|PointerMotionMask, d))
-        {
-          xcurrent = d->xmotion.x_root;
-          ycurrent = d->xmotion.y_root;
-        }
+      return TIMEOUT;
     }
-  return False;
+
+    if (do_sleep)
+    {
+      usleep(20000);
+    }
+    else
+    {
+      usleep(1);
+      do_sleep = 1;
+    }
+    total += 20;
+    if (XCheckMaskEvent(dpy, ButtonReleaseMask|ButtonMotionMask|
+			PointerMotionMask|ButtonPressMask|ExposureMask, d))
+    {
+      do_sleep = 0;
+      switch (d->xany.type)
+      {
+      case ButtonRelease:
+	return CLICK;
+      case MotionNotify:
+	if ((d->xmotion.state & DEFAULT_ALL_BUTTONS_MASK) ||
+            !is_button_pressed)
+	{
+	  xcurrent = d->xmotion.x_root;
+	  ycurrent = d->xmotion.y_root;
+	}
+        else
+	{
+	  return CLICK;
+	}
+	break;
+      case ButtonPress:
+	if (may_time_out)
+	  is_button_pressed = True;
+	break;
+      default:
+	/* can't happen */
+	break;
+      }
+    }
+  }
+
+  return TIMEOUT;
 }
 
+
 /************************************************************************
  * ExecuteAction
 * 	Based on part of ComplexFunction() of functions.c from fvwm:
@@ -2920,26 +3042,31 @@ void ExecuteAction(int x, int y, struct icon_info *item)
   XEvent d;
   struct mousefunc *tmp;
 
-   /* Wait and see if we have a click, or a move */
-  /* wait 100 msec, see if the used releases the button */
-  if(IsClick(x,y,ButtonReleaseMask,&d))
+  /* Wait and see if we have a click, or a move */
+  /* wait forever, see if the user releases the button */
+  type = CheckActionType(x, y, &d, False, True);
+  if (type == CLICK)
+  {
+    ev = &d;
+    /* If it was a click, wait to see if its a double click */
+    if (have_double_click)
     {
-      type = CLICK;
-      ev = &d;
-    }
-
-  /* If it was a click, wait to see if its a double click */
-  if((type == CLICK) && (IsClick(x,y,ButtonPressMask, &d)))
-    {
-      type = ONE_AND_A_HALF_CLICKS;
-      ev = &d;
-    }
-  if((type == ONE_AND_A_HALF_CLICKS) &&
-     (IsClick(x,y,ButtonReleaseMask, &d)))
-    {
-      type = DOUBLE_CLICK;
-      ev = &d;
+      type = CheckActionType(x, y, &d, True, False);
+      switch (type)
+      {
+      case CLICK:
+	type = DOUBLE_CLICK;
+	ev = &d;
+	break;
+      case TIMEOUT:
+	type = CLICK;
+	break;
+      default:
+	/* can't happen */
+	break;
+      }
     }
+  }
   tmp = MouseActions;
 
   while (tmp != NULL){
@@ -3089,14 +3216,20 @@ char *stripcpy2(char *source)
 static int
 myErrorHandler(Display *dpy, XErrorEvent *event)
 {
-  /* some errors are ignored, mostly due to colorsets changing too fast */
-  if (event->error_code == BadWindow)
-    return 0;
-  if (event->error_code == BadDrawable)
-    return 0;
-  if (event->error_code == BadPixmap)
-    return 0;
+	if((event->error_code == BadAccess) && do_allow_bad_access)
+	{
+		/* may happen when reparenting a icon window */
+		was_bad_access = 1;
+		return 0;
+	}
+	/* some errors are ignored, mostly due to colorsets changing too fast */
+	if (event->error_code == BadWindow)
+		return 0;
+	if (event->error_code == BadDrawable)
+		return 0;
+	if (event->error_code == BadPixmap)
+		return 0;
 
-  PrintXErrorAndCoredump(dpy, event, MyName);
-  return 0;
+	PrintXErrorAndCoredump(dpy, event, MyName);
+	return 0;
 }
