@@ -22,18 +22,10 @@
 
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#ifdef __STDC__
-#define XTextWidth(x,y,z)       XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z)       XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
 
+#include <libs/Flocale.h>
 #include <libs/Picture.h>
 #include <libs/Colorset.h>
-
 #ifndef FVWM_VERSION
 #define FVWM_VERSION 2
 #endif
@@ -200,10 +192,8 @@ typedef struct Function {
 typedef struct win_data {
   struct button *button;
   /* stuff shadowed in the Button structure */
-#ifdef MINI_ICONS
-  Picture pic;
-  Picture old_pic;
-#endif
+  FvwmPicture pic;
+  FvwmPicture old_pic;
   Uchar iconified, state;
 
   Ulong desknum;
@@ -223,6 +213,8 @@ typedef struct win_data {
   char *classname;
   char *titlename;
   char *iconname;
+  char *visible_name;
+  char *visible_icon_name;
 } WinData;
 
 typedef struct button {
@@ -230,9 +222,7 @@ typedef struct button {
   int x, y, w, h; /* current coords of button */
   struct {
     int dirty_flags;
-#ifdef MINI_ICONS
-    Picture pic;
-#endif
+    FvwmPicture pic;
     WinData *win;
     char *display_string;
     int x, y, w, h;
@@ -276,9 +266,18 @@ typedef enum {
   SortNone,          /* no sorting */
   SortId,            /* sort by window id */
   SortName,          /* case insensitive name sorting */
-  SortNameCase       /* case sensitive name sorting */
+  SortNameCase,      /* case sensitive name sorting */
+  SortWeighted       /* custom sort order */
 } SortType;
 
+typedef struct {
+  char *resname;
+  char *classname;
+  char *titlename;
+  char *iconname;
+  int weight;
+} WeightedSort;
+
 typedef struct win_manager {
   unsigned int magic;
   int index;
@@ -291,13 +290,8 @@ typedef struct win_manager {
   GC hiContext[NUM_CONTEXTS], backContext[NUM_CONTEXTS],
     reliefContext[NUM_CONTEXTS];
   GC shadowContext[NUM_CONTEXTS], flatContext[NUM_CONTEXTS];
-  XFontStruct *ButtonFont;
-#ifdef I18N_MB
-  XFontSet ButtonFontset;
-#endif
-#ifdef MINI_ICONS
+  FlocaleFont *FButtonFont;
   int draw_icons;
-#endif
   int shaped;
   StringList show;
   StringList dontshow;
@@ -315,6 +309,8 @@ typedef struct win_manager {
   Uchar followFocus;
   Uchar usewinlist;
   SortType sort;
+  WeightedSort *weighted_sorts;
+  int weighted_sorts_len, weighted_sorts_size;
   char *AnimCommand;
   Uchar showonlyiconic;
   rectangle managed_g;    /* dimensions of managed screen portion */
@@ -335,13 +331,12 @@ typedef struct win_manager {
   ManGeometry geometry, drawn_geometry;
   Button *select_button, *focus_button;
   Uchar window_mapped, drawn_mapping;
-#ifdef SHAPE
   ShapeState shape, drawn_shape;
-#endif
   ButtonArray buttons;
 
   /* Fvwm state */
   int we_are_drawing, configures_expected;
+  Bool swallowed;
 } WinManager;
 
 #define MANAGER_EMPTY(man) ((man)->buttons.num_windows == 0)
@@ -355,7 +350,6 @@ typedef struct {
   int transient;
   WinData *focus_win;
   WinData *select_win;
-  int shapes_supported;
   int got_window_list;
 } GlobalData;
 
@@ -376,6 +370,7 @@ extern Display *theDisplay;
 extern char *Module;
 extern int ModuleLen;
 extern ContextDefaults contextDefaults[];
+extern int mods_unused;
 
 extern void ReadFvwmPipe(void);
 extern void *Malloc (size_t size);
@@ -386,6 +381,7 @@ extern char *copy_string (char **target, const char *src);
 
 extern void init_globals (void);
 extern int allocate_managers (int num);
+extern int expand_weighted_sorts (void);
 
 extern WinData *new_windata (void);
 extern void free_windata (WinData *p);
