@@ -16,9 +16,7 @@
 #include "FvwmIconMan.h"
 #include "readconfig.h"
 #include "xmanager.h"
-
-static char const rcsid[] =
-  "$Id: functions.c,v 1.6 1999/06/27 14:47:13 domivogt Exp $";
+#include "libs/Module.h"
 
 static Button *get_select_button (void);
 
@@ -282,7 +280,7 @@ int builtin_sendcommand (int numargs, BuiltinArg *args)
     return 0;
   }
 
-  SendFvwmPipe (args[0].value.string_value, win->app_id);
+  SendFvwmPipe (Fvwm_fd, args[0].value.string_value, win->app_id);
 
   return 0;
 }
