@@ -24,9 +24,6 @@
 #include <libs/FShape.h>
 #include <libs/Module.h>
 
-static char const rcsid[] =
-  "$Id: readconfig.c,v 1.38.2.2 2002/06/11 01:01:12 migo Exp $";
-
 /************************************************************************
  *
  * Builtin functions:
@@ -726,7 +723,8 @@ Binding *ParseMouseEntry (char *tline)
 
   find_context(modifiers,&mods,key_modifiers,tline);
   if((mods & AnyModifier)&&(mods&(~AnyModifier))) {
-    ConsoleMessage ("Binding specified AnyModifier and other modifers too. Excess modifiers will be ignored.");
+    ConsoleMessage ("Binding specified AnyModifier and other modifers too. "
+                    "Excess modifiers will be ignored.");
   }
 
   new = (Binding *)safemalloc(sizeof(Binding));
@@ -897,8 +895,9 @@ static int GetConfigLineWrapper (int *fd, char **tline)
 
   GetConfigLine (fd, tline);
   if (*tline) {
-    if (strncasecmp(*tline, "Colorset", 8) == 0) {
-         LoadColorset(&(*tline)[8]);
+    if (strncasecmp(*tline, "Colorset", 8) == 0)
+    {
+      LoadColorset(&(*tline)[8]);
     }
     else if (strncasecmp(*tline, XINERAMA_CONFIG_STRING,
 			 sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
@@ -906,6 +905,10 @@ static int GetConfigLineWrapper (int *fd, char **tline)
       FScreenConfigureModule(
 	(*tline) + sizeof(XINERAMA_CONFIG_STRING) - 1);
     }
+    else if (strncasecmp(*tline, "IgnoreModifiers", 15) == 0)
+    {
+      sscanf((*tline) + 16, "%d", &mods_unused);
+    }
     temp = strchr (*tline, '\n');
     if (temp) {
       *temp = '\0';
@@ -1123,6 +1126,38 @@ static void handle_button_config (int manager, int context, char *option)
 	       copy_string (&globals.managers[id].backColorName[context], p));
 }
 
+static void add_weighted_sort(WinManager *man, WeightedSort *weighted_sort) {
+  WeightedSort *p;
+  int i;
+
+  if (man->weighted_sorts_len == man->weighted_sorts_size) {
+    i = man->weighted_sorts_size;
+    man->weighted_sorts_size += 16;
+    man->weighted_sorts =
+      (WeightedSort *)saferealloc ((char *)man->weighted_sorts,
+			 man->weighted_sorts_size * sizeof (WeightedSort));
+  }
+  p = &man->weighted_sorts[man->weighted_sorts_len];
+  p->resname = NULL;
+  p->classname = NULL;
+  p->titlename = NULL;
+  p->iconname = NULL;
+  if (weighted_sort->resname) {
+    copy_string(&p->resname, weighted_sort->resname);
+  }
+  if (weighted_sort->classname) {
+    copy_string(&p->classname, weighted_sort->classname);
+  }
+  if (weighted_sort->titlename) {
+    copy_string(&p->titlename, weighted_sort->titlename);
+  }
+  if (weighted_sort->iconname) {
+    copy_string(&p->iconname, weighted_sort->iconname);
+  }
+  p->weight = weighted_sort->weight;
+  ++man->weighted_sorts_len;
+}
+
 void read_in_resources (char *file)
 {
   char *p, *q;
@@ -1130,6 +1165,7 @@ void read_in_resources (char *file)
   char *option1;
   Binding *binding = NULL;
   Resolution r;
+  Reverse rv;
 
   if (!init_config_file (file))
     return;
@@ -1346,33 +1382,36 @@ void read_in_resources (char *file)
 	  Free(token);
       }
       else if (!strcasecmp (option1, "drawicons")) {
-#ifdef MINI_ICONS
-	p = read_next_cmd (READ_ARG);
-	if (!p) {
-	  ConsoleMessage ("Bad line: %s\n", current_line);
-	  ConsoleMessage ("Need argument to drawicons\n");
-	  continue;
-	}
-	if (!strcasecmp (p, "true")) {
-	  i = 1;
-	}
-    /* [NFM 3 Dec 97] added support for FvwmIconMan*drawicons "always" */
-	else if (!strcasecmp (p, "always")) {
-	  i = 2;
-	}
-	else if (!strcasecmp (p, "false")) {
-	  i = 0;
-	}
-	else {
-	  ConsoleMessage ("Bad line: %s\n", current_line);
-	  ConsoleMessage ("What is this: %s?\n", p);
-	  continue;
-	}
-	ConsoleDebug (CONFIG, "Setting drawicons to: %d\n", i);
-	SET_MANAGER (manager, draw_icons, i);
-#else
-	ConsoleMessage ("DrawIcons support not compiled in\n");
-#endif
+        if (!FMiniIconsSupported)
+        {
+          ConsoleMessage ("DrawIcons support not compiled in\n");
+        }
+        else
+        {
+	  p = read_next_cmd (READ_ARG);
+	  if (!p) {
+            ConsoleMessage ("Bad line: %s\n", current_line);
+            ConsoleMessage ("Need argument to drawicons\n");
+            continue;
+          }
+          if (!strcasecmp (p, "true")) {
+            i = 1;
+          }
+          /* [NFM 3 Dec 97] added support for drawicons "always" */
+          else if (!strcasecmp (p, "always")) {
+            i = 2;
+          }
+          else if (!strcasecmp (p, "false")) {
+	    i = 0;
+          }
+          else {
+            ConsoleMessage ("Bad line: %s\n", current_line);
+            ConsoleMessage ("What is this: %s?\n", p);
+            continue;
+          }
+          ConsoleDebug (CONFIG, "Setting drawicons to: %d\n", i);
+          SET_MANAGER (manager, draw_icons, i);
+        }
       }
       else if (!strcasecmp (option1, "followfocus")) {
 	p = read_next_cmd (READ_ARG);
@@ -1417,19 +1456,16 @@ void read_in_resources (char *file)
        SET_MANAGER (manager, showonlyiconic, i);
       }
       else if (!strcasecmp (option1, "font")) {
-	char *f;
 	p = read_next_cmd (READ_REST_OF_LINE);
 	trim(p);
 	if (!p) {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  continue;
 	}
-	CopyStringWithQuotes(&f, p);
-	ConsoleDebug (CONFIG, "font: %s\n", f);
+	ConsoleDebug (CONFIG, "font: %s\n", p);
 
 	SET_MANAGER (manager, fontname,
-		     copy_string (&globals.managers[id].fontname, f));
-	free(f);
+		     copy_string (&globals.managers[id].fontname, p));
       }
       else if (!strcasecmp (option1, "foreground")) {
 	p = read_next_cmd (READ_ARG);
@@ -1535,18 +1571,18 @@ void read_in_resources (char *file)
 	}
 	ConsoleDebug (CONFIG, "reverse: %s\n", p);
 	if (!strcasecmp (p, "none"))
-	  r = REVERSE_NONE;
+	  rv = REVERSE_NONE;
 	else if (!strcasecmp (p, "icon"))
-	  r = REVERSE_ICON;
+	  rv = REVERSE_ICON;
 	else if (!strcasecmp (p, "normal"))
-	  r = REVERSE_NORMAL;
+	  rv = REVERSE_NORMAL;
 	else {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  ConsoleMessage ("What kind of reverse is this?\n");
 	  continue;
 	}
 
-	SET_MANAGER (manager, rev, r);
+	SET_MANAGER (manager, rev, rv);
       }
       else if (!strcasecmp (option1, "shape")) {
 	p = read_next_cmd (READ_ARG);
@@ -1625,9 +1661,12 @@ void read_in_resources (char *file)
 	else if (!strcasecmp (p, "none")) {
 	  i = SortNone;
 	}
+	else if (!strcasecmp (p, "weighted")) {
+	  i = SortWeighted;
+	}
 	else if (!strcasecmp (p, "false") || !strcasecmp (p, "true")) {
 	  /* Old options */
-	  ConsoleMessage ("FvwmIconMan*sort option no longer takes "
+	  ConsoleMessage ("FvwmIconMan: sort option no longer takes "
 			  "true or false value\n"
 			  "Please read the latest manpage\n");
 	  continue;
@@ -1640,6 +1679,60 @@ void read_in_resources (char *file)
 	ConsoleDebug (CONFIG, "Setting sort to: %d\n", i);
 	SET_MANAGER (manager, sort, i);
       }
+      else if (!strcasecmp (option1, "sortweight")) {
+	WeightedSort weighted_sort;
+	p = read_next_cmd (READ_ARG);
+	if (!p) {
+	  ConsoleMessage ("Bad line: %s\n", current_line);
+	  continue;
+	}
+	if (extract_int (p, &n) == 0) {
+	  ConsoleMessage ("This is not a number: %s\n", p);
+	  ConsoleMessage ("Bad line: %s\n", current_line);
+	  continue;
+	}
+	weighted_sort.resname = NULL;
+	weighted_sort.classname = NULL;
+	weighted_sort.titlename = NULL;
+	weighted_sort.iconname = NULL;
+	weighted_sort.weight = n;
+	p = read_next_cmd (READ_ARG);
+	while (p) {
+	  if (!strncasecmp(p, "resource=", 9)) {
+	    copy_string(&weighted_sort.resname, p + 9);
+	  } else if (!strncasecmp(p, "class=", 6)) {
+	    copy_string(&weighted_sort.classname, p + 6);
+	  } else if (!strncasecmp(p, "title=", 6)) {
+	    copy_string(&weighted_sort.titlename, p + 6);
+	  } else if (!strncasecmp(p, "icon=", 5)) {
+	    copy_string(&weighted_sort.iconname, p + 5);
+	  } else {
+	    ConsoleMessage ("Unknown sortweight field: %s\n", p);
+	    ConsoleMessage ("Bad line: %s\n", current_line);
+	  }
+	  p = read_next_cmd (READ_ARG);
+	}
+	if (manager == -1) {
+	  for (i = 0; i < globals.num_managers; i++) {
+	    add_weighted_sort (&globals.managers[i], &weighted_sort);
+	  }
+	}
+	else {
+	  add_weighted_sort (&globals.managers[manager], &weighted_sort);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.resname);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.classname);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.titlename);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.iconname);
+	}
+      }
       else if (!strcasecmp (option1, "NoIconAction")) {
 	char *token;
 	p = read_next_cmd (READ_REST_OF_LINE);
