@@ -22,9 +22,6 @@
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 
-static char const rcsid[] =
-  "$Id: x.c,v 1.53 2001/09/16 15:51:18 domivogt Exp $";
-
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
 
 Display *theDisplay;
@@ -119,7 +116,7 @@ static void handle_buttonevent (XEvent *theEvent, WinManager *man)
 	if(((MouseEntry->Button_Key == theEvent->xbutton.button)||
 	    (MouseEntry->Button_Key == 0))&&
 	   ((MouseEntry->Modifier == AnyModifier)||
-	    (MouseEntry->Modifier == (modifier& (~LockMask)))))
+	    (MouseEntry->Modifier == (modifier& (~mods_unused)))))
 	{
 	  Function *ftype = (Function *)(MouseEntry->Action2);
 	  ConsoleDebug (X11, "\tgot a mouse binding\n");
@@ -207,6 +204,11 @@ void xevent_loop (void)
     }
 
     man = find_windows_manager (theEvent.xany.window);
+    if (!man && theEvent.type == ConfigureNotify)
+    {
+      /* swallowed icon man */
+      man = find_windows_manager (theEvent.xconfigure.window);
+    }
     if (!man) {
       ConsoleDebug (X11, "Event doesn't belong to a manager\n");
       continue;
@@ -232,7 +234,7 @@ void xevent_loop (void)
       for (key = man->bindings[KEYPRESS]; key != NULL; key = key->NextBinding)
       {
 	if ((key->Button_Key == theEvent.xkey.keycode) &&
-	    ((key->Modifier == (modifier&(~LockMask)))||
+	    ((key->Modifier == (modifier&(~mods_unused)))||
 	     (key->Modifier == AnyModifier)))
 	{
 	  Function *ftype = (Function *)(key->Action2);
@@ -309,7 +311,7 @@ void xevent_loop (void)
 	  moved = True;
       }
       theEvent = saveEvent;
-
+	      
       if (man->geometry.dir & GROW_FIXED)
       {
 	man->geometry.rows =
@@ -350,6 +352,8 @@ void xevent_loop (void)
       /* must refresh transparent windows when moved */
       if (moved && man->pixmap[DEFAULT] == ParentRelative)
       {
+	man->geometry.x = theEvent.xconfigure.x;
+	man->geometry.y = theEvent.xconfigure.y;
         XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
       }
 
@@ -499,12 +503,7 @@ void X_init_manager (int man_id)
   WinManager *man;
   unsigned int width, height;
   int i, x, y, geometry_mask;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
+
   ConsoleDebug (X11, "In X_init_manager\n");
 
   man = &globals.managers[man_id];
@@ -519,41 +518,14 @@ void X_init_manager (int man_id)
 
   ConsoleDebug (X11, "boxwidth = %d\n", man->geometry.boxwidth);
 
-  if (man->fontname) {
-#ifdef I18N_MB
-    man->ButtonFontset = XCreateFontSet (theDisplay, man->fontname, &ml, &mc, &ds);
-    if (!man->ButtonFontset) {
-      if (!(man->ButtonFontset = XCreateFontSet (theDisplay, FONT_STRING, &ml, &mc, &ds))) {
-        ConsoleMessage ("Can't get fontset\n");
-        ShutMeDown (1);
-      }
-    }
-    XFontsOfFontSet(man->ButtonFontset,&fs_list,&ml);
-    man->ButtonFont = fs_list[0];
-#else
-    man->ButtonFont = XLoadQueryFont (theDisplay, man->fontname);
-    if (!man->ButtonFont) {
-      if (!(man->ButtonFont = XLoadQueryFont (theDisplay, FONT_STRING))) {
-	ConsoleMessage ("Can't get font\n");
-	ShutMeDown (1);
-      }
-    }
-#endif
-  }
-  else {
-#ifdef I18N_MB
-    if (!(man->ButtonFontset = XCreateFontSet (theDisplay, FONT_STRING, &ml, &mc, &ds))) {
-      ConsoleMessage ("Can't get fontset\n");
-      ShutMeDown (1);
-    }
-    XFontsOfFontSet(man->ButtonFontset,&fs_list,&ml);
-    man->ButtonFont = fs_list[0];
-#else
-    if (!(man->ButtonFont = XLoadQueryFont (theDisplay, FONT_STRING))) {
-      ConsoleMessage ("Can't get font\n");
-      ShutMeDown (1);
-    }
-#endif
+  if (man->fontname == NULL)
+    man->FButtonFont = FlocaleLoadFont(theDisplay, FONT_STRING, MyName);
+  else
+    man->FButtonFont = FlocaleLoadFont(theDisplay, man->fontname, MyName)  ;
+  if (man->FButtonFont == NULL) 
+  {
+    ConsoleMessage ("Can't get font, exiting\n");
+    ShutMeDown (1);
   }
 
   for ( i = 0; i < NUM_CONTEXTS; i++ ) {
@@ -603,13 +575,12 @@ void X_init_manager (int man_id)
     }
   }
 
-  man->fontheight = man->ButtonFont->ascent +
-    man->ButtonFont->descent;
+  man->fontheight = man->FButtonFont->height;
 
   /* silly hack to guess the minimum char width of the font
      doesn't have to be perfect. */
 
-  man->fontwidth = XTextWidth (man->ButtonFont, ".", 1);
+  man->fontwidth = FlocaleTextWidth (man->FButtonFont, ".", 1);
 
   /* First:  get button geometry
      Second: get geometry from geometry string
@@ -650,6 +621,10 @@ void X_init_manager (int man_id)
       NULL, scr, &man->managed_g.x, &man->managed_g.y,
       &man->managed_g.width, &man->managed_g.height);
 
+    if (geometry_mask & XValue)
+      man->geometry.x += man->managed_g.x;
+    if (geometry_mask & YValue)
+	man->geometry.y += man->managed_g.y;
     if ((geometry_mask & XValue) || (geometry_mask & YValue)) {
       man->sizehints_flags |= USPosition;
       if (geometry_mask & XNegative)
@@ -685,22 +660,24 @@ void X_init_manager (int man_id)
 
   man->geometry.width  = man->geometry.cols * man->geometry.boxwidth;
   man->geometry.height = man->geometry.rows * man->geometry.boxheight;
-
   if ((geometry_mask & XValue) && (geometry_mask & XNegative))
-    man->geometry.x +=
-      globals.screen_g.x + globals.screen_g.width - man->geometry.width;
+    man->geometry.x += man->managed_g.width - man->geometry.width;
   if ((geometry_mask & YValue) && (geometry_mask & YNegative))
-    man->geometry.y +=
-      globals.screen_g.y + globals.screen_g.height - man->geometry.height;
-
+    man->geometry.y += man->managed_g.height - man->geometry.height;
   if (globals.transient) {
     Window dummyroot, dummychild;
     int junk;
     unsigned int ujunk;
     fscreen_scr_arg fscr;
 
-    XQueryPointer(theDisplay, theRoot, &dummyroot, &dummychild,
-		  &man->geometry.x, &man->geometry.y, &junk, &junk, &ujunk);
+    if (XQueryPointer(
+	  theDisplay, theRoot, &dummyroot, &dummychild, &man->geometry.x,
+	  &man->geometry.y, &junk, &junk, &ujunk) == False)
+    {
+      /* pointer is on a different screen - that's okay here */
+      man->geometry.y = 0;
+      man->geometry.x = 0;
+    }
     fscr.xypos.x = man->geometry.x;
     fscr.xypos.y = man->geometry.y;
     FScreenGetScrRect(
@@ -790,7 +767,6 @@ void create_manager_window (int man_id)
   sizehints.x = man->geometry.x;
   sizehints.y = man->geometry.y;
 
-
   ConsoleDebug (X11, "hints: x, y, w, h = %d %d %d %d)\n",
 		sizehints.x, sizehints.y,
 		sizehints.base_width, sizehints.base_height);
@@ -866,7 +842,8 @@ void create_manager_window (int man_id)
 
     man->hiContext[i] =
       fvwmlib_XCreateGC (theDisplay, man->theWindow, gcmask, &gcval);
-    XSetFont (theDisplay, man->hiContext[i], man->ButtonFont->fid);
+    if (man->FButtonFont->font != NULL)
+      XSetFont (theDisplay, man->hiContext[i], man->FButtonFont->font->fid);
     XSetForeground (theDisplay, man->hiContext[i], man->forecolor[i]);
 
     gcmask = GCForeground | GCBackground;
@@ -940,7 +917,7 @@ void init_display (void)
   }
   XSetErrorHandler (handle_error);
   _XA_WM_DEL_WIN = XInternAtom(theDisplay, "WM_DELETE_WINDOW", 0);
-  InitPictureCMap (theDisplay);
+  PictureInitCMap (theDisplay);
   FScreenInit(theDisplay);
   AllocColorset(0);
   FShapeInit(theDisplay);
