@@ -22,12 +22,10 @@
 #include "FvwmIconMan.h"
 #include "x.h"
 #include "xmanager.h"
-
-static char const rcsid[] =
-  "$Id: xmanager.c,v 1.50.2.2 2002/04/22 22:01:41 migo Exp $";
+#include "libs/PictureGraphics.h"
 
 extern char *MyName;
-
+extern FlocaleWinString *FwinString;
 
 /* button dirty bits: */
 #define ICON_STATE_CHANGED  1
@@ -361,11 +359,11 @@ static char *make_display_string (WinData *win, char *format, int len)
     if (*in_p == '%') {
       switch (*(++in_p)) {
       case 'i':
-	COPY (iconname);
+	COPY (visible_icon_name);
 	break;
 
       case 't':
-	COPY (titlename);
+	COPY (visible_name);
 	break;
 
       case 'r':
@@ -517,11 +515,10 @@ static void set_window_button (WinData *win, int index)
   b = win->manager->buttons.buttons[index];
 
   /* Can optimize here */
-
-#ifdef MINI_ICONS
-  b->drawn_state.pic = win->pic;
-#endif
-
+  if (FMiniIconsSupported)
+  {
+    b->drawn_state.pic = win->pic;
+  }
   b->drawn_state.win = win;
   copy_string(&b->drawn_state.display_string, win->display_string);
   b->drawn_state.iconified = win->iconified;
@@ -633,24 +630,25 @@ void force_manager_redraw (WinManager *man)
   draw_manager (man);
 }
 
-#ifdef MINI_ICONS
-
-void set_win_picture (WinData *win, Pixmap picture, Pixmap mask,
+void set_win_picture (WinData *win, Pixmap picture, Pixmap mask, Pixmap alpha,
 		      unsigned int depth, unsigned int width,
 		      unsigned int height)
 {
+  if (!FMiniIconsSupported)
+  {
+    return;
+  }
   if (win->button)
     win->button->drawn_state.dirty_flags |= PICTURE_CHANGED;
   win->old_pic = win->pic;
   win->pic.picture = picture;
   win->pic.mask = mask;
+  win->pic.alpha = alpha;
   win->pic.width = width;
   win->pic.height = height;
   win->pic.depth = depth;
 }
 
-#endif
-
 void set_win_iconified (WinData *win, int iconified)
 {
   /* This change has become necessary because with colorsets we don't know
@@ -723,8 +721,8 @@ void set_win_displaystring (WinData *win)
   int maxlen;
 
   if (!man || ((man->format_depend & CLASS_NAME) && !win->classname)
-      || ((man->format_depend & ICON_NAME) && !win->iconname)
-      || ((man->format_depend & TITLE_NAME) && !win->titlename)
+      || ((man->format_depend & ICON_NAME) && !win->visible_icon_name)
+      || ((man->format_depend & TITLE_NAME) && !win->visible_name)
       || ((man->format_depend & RESOURCE_NAME) && !win->resname)) {
     return;
   }
@@ -1054,7 +1052,7 @@ static void get_title_geometry (WinManager *man, ButtonGeometry *g)
   text_pad = center_padding (man->fontheight, g->button_h);
 
   g->text_y = g->button_y + text_pad;
-  g->text_base = g->text_y + man->ButtonFont->ascent;
+  g->text_base = g->text_y + man->FButtonFont->ascent;
 }
 
 static void get_button_geometry (WinManager *man, Button *button,
@@ -1074,8 +1072,7 @@ static void get_button_geometry (WinManager *man, Button *button,
   g->button_h = button->h;
 
 /* [BV 16-Apr-97] Mini Icons work on black-and-white too */
-#ifdef MINI_ICONS
-  if (man->draw_icons && win && win->pic.picture) {
+  if (FMiniIconsSupported && man->draw_icons && win && win->pic.picture) {
     /* If no window, then icon_* aren't used, so doesn't matter what
        they are */
     g->icon_w = min (win->pic.width, g->button_h);
@@ -1085,16 +1082,13 @@ static void get_button_geometry (WinManager *man, Button *button,
     g->icon_y = g->button_y + icon_pad;
   }
   else {
-#endif
     g->icon_h = man->geometry.boxheight - 8;
     g->icon_w = g->icon_h;
 
     icon_pad = center_padding (g->icon_h, g->button_h);
     g->icon_x = g->button_x + icon_pad;
     g->icon_y = g->button_y + icon_pad;
-#ifdef MINI_ICONS
   }
-#endif
 
   g->text_x = g->icon_x + g->icon_w + 2;
   g->text_w = g->button_w - 4 - (g->text_x - g->button_x);
@@ -1105,7 +1099,7 @@ static void get_button_geometry (WinManager *man, Button *button,
   text_pad = center_padding (man->fontheight, g->button_h);
 
   g->text_y = g->button_y + text_pad;
-  g->text_base = g->text_y + man->ButtonFont->ascent;
+  g->text_base = g->text_y + man->FButtonFont->ascent;
 }
 
 static void draw_3d_icon (WinManager *man, int box, ButtonGeometry *g,
@@ -1126,17 +1120,12 @@ static void iconify_box (WinManager *man, WinData *win, int box,
 			 ButtonGeometry *g, int iconified, Contexts contextId,
 			 int button_already_cleared)
 {
-#ifdef MINI_ICONS
-  XGCValues gcv;
-  unsigned long gcm;
-#endif
 
   if (!man->window_up)
     return;
 
 /* [BV 16-Apr-97] Mini Icons work on black-and-white too */
-#ifdef MINI_ICONS
-  if (man->draw_icons && win->pic.picture) {
+  if (FMiniIconsSupported && man->draw_icons && win->pic.picture) {
     if (iconified == 0 && man->draw_icons != 2) {
       if (!button_already_cleared) {
 	XFillRectangle (theDisplay, man->theWindow,
@@ -1145,22 +1134,13 @@ static void iconify_box (WinManager *man, WinData *win, int box,
       }
     }
     else {
-      gcm = GCClipMask|GCClipXOrigin|GCClipYOrigin;
-      gcv.clip_mask = win->pic.mask;
-      gcv.clip_x_origin = g->icon_x;
-      gcv.clip_y_origin = g->icon_y;
-      XChangeGC (theDisplay, man->hiContext[contextId], gcm, &gcv);
-
-      XCopyArea(theDisplay, win->pic.picture, man->theWindow,
-		man->hiContext[contextId], 0, 0, g->icon_w, g->icon_h,
-		g->icon_x, g->icon_y);
-      gcm = GCClipMask;
-      gcv.clip_mask = None;
-      XChangeGC(theDisplay, man->hiContext[contextId], gcm, &gcv);
+      PGraphicsCopyFvwmPicture(theDisplay, &win->pic, man->theWindow,
+			       man->hiContext[contextId],
+			       0, 0, g->icon_w, g->icon_h,
+			       g->icon_x, g->icon_y);
     }
   }
   else {
-#endif
     if (Pdepth > 2) {
       draw_3d_icon (man, box, g, iconified, contextId);
     }
@@ -1174,10 +1154,7 @@ static void iconify_box (WinManager *man, WinData *win, int box,
 		  g->icon_x, g->icon_y, g->icon_w, g->icon_h, 0, 360 * 64);
       }
     }
-#ifdef MINI_ICONS
   }
-#endif
-
 }
 
 int change_windows_manager (WinData *win)
@@ -1344,9 +1321,8 @@ static void draw_button (WinManager *man, int button, int force)
 	draw_icon = 1;
 	draw_string = 1;
       }
-#ifdef MINI_ICONS
-      if (PICTURE_CHANGED) {
-	Picture tpic;
+      if (FMiniIconsSupported && PICTURE_CHANGED) {
+	FvwmPicture tpic;
 
 	ConsoleDebug (X11, "\tPicture changed\n");
 	tpic = win->pic;
@@ -1358,7 +1334,6 @@ static void draw_button (WinManager *man, int button, int force)
 	draw_string = 1;
 	clear_old_pic = 1;
       }
-#endif
       if (ICON_STATE_CHANGED && b->drawn_state.iconified != win->iconified) {
 	ConsoleDebug (X11, "\tIcon changed\n");
 	b->drawn_state.iconified = win->iconified;
@@ -1436,14 +1411,28 @@ static void draw_button (WinManager *man, int button, int force)
 			man->backContext[button_state],
 			g.text_x, g.text_y, g.text_w, g.text_h);
       }
-#ifdef I18N_MB
-      XmbDrawString (theDisplay, man->theWindow, man->ButtonFontset,
-#else
-      XDrawString (theDisplay, man->theWindow,
-#endif
-		   man->hiContext[button_state],
-		   g.text_x, g.text_base, b->drawn_state.display_string,
-		   strlen (b->drawn_state.display_string));
+      FwinString->str =  b->drawn_state.display_string;
+      FwinString->win = man->theWindow;
+      FwinString->gc = man->hiContext[button_state];
+      FwinString->x = g.text_x;
+      FwinString->y = g.text_base;
+      FwinString->len = strlen(b->drawn_state.display_string);
+      if (FftSupport)
+      {
+	if (man->FButtonFont->fftf.fftfont != NULL)
+	{
+	  while(
+	    FwinString->len >= 0 &&
+	    FlocaleTextWidth(man->FButtonFont, FwinString->str,
+			     FwinString->len)
+	    > g.text_w)
+	  {
+	    FwinString->len--;
+	  }
+	}
+      }
+      FlocaleDrawString(
+	theDisplay, man->FButtonFont, FwinString, FWS_HAVE_LENGTH);
       XSetClipMask (theDisplay, man->hiContext[button_state], None);
     }
   }
@@ -1474,22 +1463,27 @@ static void draw_empty_manager (WinManager *man)
   clear_empty_region (man);
   get_title_geometry (man, &g);
 
-  if (len > 0)
-    XFillRectangle (theDisplay, man->theWindow, man->backContext[state],
-                    g.button_x, g.button_y, g.button_w, g.button_h);
+  if (len > 0) {
+    if (man->colorsets[state] >= 0
+        && Colorset[man->colorsets[state]].pixmap == ParentRelative) {
+      XClearArea (theDisplay, man->theWindow, g.button_x, g.button_y,
+                  g.button_w, g.button_h, False);
+    } else {
+      XFillRectangle (theDisplay, man->theWindow, man->backContext[state],
+                      g.button_x, g.button_y, g.button_w, g.button_h);
+    }
+  }
   if (Pdepth > 2) {
     get_gcs (man, state, 0, &context1, &context2);
     draw_relief (man, state, &g, context1, context2);
   }
-  else {
-  }
   ClipRectangle (man, state, g.text_x, g.text_y, g.text_w, g.text_h);
-#ifdef I18N_MB
-  XmbDrawString (theDisplay, man->theWindow, man->ButtonFontset, man->hiContext[state],
-#else
-  XDrawString (theDisplay, man->theWindow, man->hiContext[state],
-#endif
-	       g.text_x, g.text_base, man->titlename, len);
+  FwinString->str = man->titlename;
+  FwinString->win = man->theWindow;
+  FwinString->gc = man->hiContext[state];
+  FwinString->x = g.text_x;
+  FwinString->y = g.text_base;
+  FlocaleDrawString (theDisplay, man->FButtonFont, FwinString, 0);
   XSetClipMask (theDisplay, man->hiContext[state], None);
 }
 
@@ -1512,13 +1506,12 @@ void draw_manager (WinManager *man)
     update_geometry = 1;
     force_draw = 1;
   }
-  if (force_draw || man->buttons.num_windows != 0)
-    clear_empty_region (man);
 
   if (redraw_all || (man->dirty_flags & MAPPING_CHANGED)) {
     force_draw = 1;
     ConsoleDebug (X11, "manager %s: mapping changed\n", man->titlename);
   }
+
   if (FShapesSupported && man->shaped)
   {
     if (redraw_all || (man->dirty_flags & SHAPE_CHANGED)) {
@@ -1532,6 +1525,7 @@ void draw_manager (WinManager *man)
       update_geometry = 1;
     }
   }
+
   if (redraw_all || (man->dirty_flags & GEOMETRY_CHANGED)) {
     ConsoleDebug (X11, "\tredrawing all buttons\n");
     update_geometry = 1;
@@ -1542,6 +1536,15 @@ void draw_manager (WinManager *man)
       set_button_geometry (man, man->buttons.buttons[i]);
   }
 
+  if (force_draw || update_geometry)
+  {
+    clear_empty_region (man);
+    /* maybe not usefull but safe */
+    if (man->colorsets[DEFAULT] >= 0 &&
+	 Colorset[man->colorsets[DEFAULT]].pixmap == ParentRelative)
+      force_draw = True;
+  }
+
   man->dirty_flags = 0;
   man->buttons.dirty_flags = 0;
   man->buttons.drawn_num_buttons = man->buttons.num_buttons;
@@ -1568,8 +1571,36 @@ void draw_manager (WinManager *man)
 }
 
 
+static int compute_weight(WinData *win)
+{
+  WinManager *man;
+  WeightedSort *sort;
+  int i;
+
+  man = win->manager;
+  for (i = 0; i < man->weighted_sorts_len; i++) {
+    sort = &man->weighted_sorts[i];
+    if (sort->resname && !matchWildcards(sort->resname, win->resname)) {
+      continue;
+    }
+    if (sort->classname && !matchWildcards(sort->classname, win->classname)) {
+      continue;
+    }
+    if (sort->titlename && !matchWildcards(sort->titlename, win->titlename)) {
+      continue;
+    }
+    if (sort->iconname && !matchWildcards(sort->iconname, win->iconname)) {
+      continue;
+    }
+    return sort->weight;
+  }
+  return 0;
+}
+
 static int compare_windows(SortType type, WinData *a, WinData *b)
 {
+  int wa, wb;
+
   if (type == SortId) {
     return a->app_id - b->app_id;
   }
@@ -1579,6 +1610,14 @@ static int compare_windows(SortType type, WinData *a, WinData *b)
   else if (type == SortNameCase) {
     return strcmp (a->display_string, b->display_string);
   }
+  else if (type == SortWeighted) {
+    wa = compute_weight(a);
+    wb = compute_weight(b);
+    if (wa != wb) {
+      return wa - wb;
+    }
+    return strcmp(a->display_string, b->display_string);
+  }
   else {
     ConsoleMessage ("Internal error in compare_windows\n");
     return 0;
