@@ -46,17 +46,15 @@
 #include <X11/Intrinsic.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/Module.h"
-#include "libs/Picture.h"
 #include "libs/Colorset.h"
 #include "libs/fvwmsignal.h"
+#include "libs/Flocale.h"
+#include "libs/Parse.h"
 #include "FvwmIdent.h"
 
 static RETSIGTYPE TerminateHandler(int);
@@ -68,10 +66,9 @@ static int fd[2];
 static Display *dpy;			/* which display are we talking to */
 static Window Root;
 static GC gc;
-static XFontStruct *font;
-#ifdef I18N_MB
-static XFontSet fontset;
-#endif
+
+static FlocaleFont *Ffont;
+static FlocaleWinString *FwinString;
 
 static int screen;
 static int x_fd;
@@ -84,15 +81,20 @@ static int colorset = 0;
 
 static char *BackColor = "white";
 static char *ForeColor = "black";
-static char *font_string = "fixed";
+static char *font_string = NULL;
 
 static Pixel fore_pix;
+static Pixel back_pix;
 static Window main_win;
 static Window app_win;
+static Bool UsePixmapDrawing = False; /* if True draw everything in a pixamp
+				       * and set the window background. Use
+				       * this with Xft */
+static int main_width;
+static int main_height;
 
-static EventMask mw_events =
-  ExposureMask | ButtonPressMask | KeyPressMask |
-  ButtonReleaseMask | KeyReleaseMask | StructureNotifyMask;
+static EventMask mw_events = ButtonPressMask | KeyPressMask |
+  ButtonReleaseMask | KeyReleaseMask;
 
 static Atom wm_del_win;
 
@@ -106,6 +108,11 @@ static int max_col1, max_col2;
 static char id[15], desktop[10], swidth[10], sheight[10], borderw[10];
 static char geometry[30], mymin_aspect[11], max_aspect[11], layer[10];
 
+/* FIXME: default layer should be received from fvwm */
+#define default_layer 4
+static int minimal_layer = default_layer;
+static int my_layer = default_layer;
+
 /***********************************************************************
  *
  *  Procedure:
@@ -119,9 +126,7 @@ int main(int argc, char **argv)
   int Clength;
   char *tline;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmIdent");
 
   /* Save the program name for error messages and config parsing */
   temp = argv[0];
@@ -204,48 +209,82 @@ int main(int argc, char **argv)
   screen= DefaultScreen(dpy);
   Root = RootWindow(dpy, screen);
 
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   /* prevent core dumps if fvwm doesn't provide any colorsets */
   AllocColorset(0);
+  FlocaleAllocateWinString(&FwinString);
   FShapeInit(dpy);
 
   SetMessageMask(fd, M_CONFIGURE_WINDOW | M_WINDOW_NAME | M_ICON_NAME
                  | M_RES_CLASS | M_RES_NAME | M_END_WINDOWLIST | M_CONFIG_INFO
                  | M_END_CONFIG_INFO | M_SENDCONFIG);
+  SetMessageMask(fd, MX_PROPERTY_CHANGE);
   /* scan config file for set-up parameters */
   /* Colors and fonts */
 
   InitGetConfigLine(fd,MyName);
   GetConfigLine(fd,&tline);
 
-  while(tline != (char *)0) {
-    if(strlen(tline)>1) {
-      if(strncasecmp(tline, CatString3(MyName,"Font",""),Clength+4)==0) {
-        CopyStringWithQuotes(&font_string,&tline[Clength+4]);
+  while (tline != (char *)0)
+  {
+    if (strlen(tline) <= 1)
+    {
+      continue;
+    }
+    if (strncasecmp(tline, MyName, Clength) == 0)
+    {
+      tline += Clength;
+      if (strncasecmp(tline, "Font", 4) == 0)
+      {
+        CopyString(&font_string, &tline[4]);
       }
-      else if(strncasecmp(tline,CatString3(MyName,"Fore",""), Clength+4)==0) {
-        CopyString(&ForeColor,&tline[Clength+4]);
+      else if (strncasecmp(tline, "Fore", 4) == 0)
+      {
+        CopyString(&ForeColor, &tline[4]);
         colorset = -1;
       }
-      else if(strncasecmp(tline,CatString3(MyName, "Back",""), Clength+4)==0) {
-        CopyString(&BackColor,&tline[Clength+4]);
+      else if (strncasecmp(tline, "Back", 4) == 0)
+      {
+        CopyString(&BackColor, &tline[4]);
         colorset = -1;
       }
-      else if(strncasecmp(tline,CatString3(MyName,"Colorset",""),Clength+8)==0){
-        sscanf(&tline[Clength+8], "%d", &colorset);
+      else if (strncasecmp(tline, "Colorset", 8) == 0)
+      {
+        sscanf(&tline[8], "%d", &colorset);
         AllocColorset(colorset);
       }
-      else if(strncasecmp(tline, "Colorset", 8) == 0){
-        LoadColorset(&tline[8]);
-      }
-      else if(strncasecmp(tline, XINERAMA_CONFIG_STRING,
-			  sizeof(XINERAMA_CONFIG_STRING) - 1) == 0){
-	FScreenConfigureModule(
-	  tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
+      else if (strncasecmp(tline, "MinimalLayer", 12) == 0)
+      {
+        char *layer_str = PeekToken(&tline[12], NULL);
+        if (layer_str == NULL)
+        {
+          minimal_layer = default_layer;
+        }
+        else if (sscanf(layer_str, "%d", &minimal_layer) != 1)
+        {
+          if (strncasecmp(layer_str, "none", 4) == 0)
+          {
+            minimal_layer = -1;
+          }
+          else
+          {
+            minimal_layer = default_layer;
+          }
+        }
       }
     }
-    GetConfigLine(fd,&tline);
+    else if (strncasecmp(tline, "Colorset", 8) == 0)
+    {
+      LoadColorset(&tline[8]);
+    }
+    else if (strncasecmp(
+      tline, XINERAMA_CONFIG_STRING, sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
+    {
+      FScreenConfigureModule(
+        tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
+    }
+    GetConfigLine(fd, &tline);
   }
 
   if(app_win == 0)
@@ -256,7 +295,7 @@ int main(int argc, char **argv)
   /* Create a list of all windows */
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
-  SendInfo(fd,"Send_WindowList",0);
+  SendText(fd, "Send_WindowList", 0);
 
   /* tell fvwm we're running */
   SendFinishedStartupNotification(fd);
@@ -365,7 +404,14 @@ void list_configure(unsigned long *body)
       target.width_inc = cfgpacket->hints_width_inc;
       target.height_inc = cfgpacket->hints_height_inc;
       target.gravity = cfgpacket->hints_win_gravity;
+      target.ewmh_hint_layer = cfgpacket->ewmh_hint_layer;
+      target.ewmh_hint_desktop = cfgpacket->ewmh_hint_desktop;
+      target.ewmh_window_type = cfgpacket->ewmh_window_type;
       found = 1;
+
+      my_layer = (int)target.layer;
+      if (my_layer < minimal_layer)
+        my_layer = minimal_layer;
     }
 }
 
@@ -423,14 +469,23 @@ void list_res_name(unsigned long *body)
     }
 }
 
+void list_property_change(unsigned long *body)
+{
+  if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND && body[2] == 0 &&
+      (colorset >= 0 && Colorset[colorset].pixmap == ParentRelative))
+    {
+      XClearArea(dpy, main_win, 0,0,0,0, True);
+    }
+}
+
 /*************************************************************************
  *
  * End of window list, open an x window and display data in it
  *
  ************************************************************************/
-XSizeHints mysizehints;
 void list_end(void)
 {
+  XSizeHints mysizehints;
   XGCValues gcv;
   unsigned long gcm;
   int lmax,height;
@@ -439,12 +494,6 @@ void list_end(void)
   XSetWindowAttributes attributes;
   int is_key_pressed = 0;
   int is_button_pressed = 0;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
 
   if(!found)
     {
@@ -455,23 +504,15 @@ void list_end(void)
   /* tell fvwm to only send config messages */
   SetMessageMask(fd, M_CONFIG_INFO | M_SENDCONFIG);
 
-#ifdef I18N_MB
-  if ((fontset = XCreateFontSet(dpy, font_string, &ml, &mc, &ds)) == NULL) {
-#ifdef STRICTLY_FIXED
-      if ((fontset = XCreateFontSet(dpy, "fixed", &ml, &mc, &ds)) == NULL)
-#else
-      if ((fontset = XCreateFontSet(dpy, "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*", &ml, &mc, &ds)) == NULL)
-#endif
-	  exit(1);
+  if ((Ffont = FlocaleLoadFont(dpy, font_string, MyName)) == NULL)
+  {
+    fprintf(stderr,"%s: cannot load font, exiting\n", MyName);
+    exit(1);
   }
-  XFontsOfFontSet(fontset, &fs_list, &ml);
-  font = fs_list[0];
-#else
-  /* load the font */
-  if ((font = XLoadQueryFont(dpy, font_string)) == NULL)
-    if ((font = XLoadQueryFont(dpy, "fixed")) == NULL)
-      exit(1);
-#endif
+
+  /* chose the rendering methode */
+  if (FftSupport && Ffont->fftf.fftfont != NULL)
+	  UsePixmapDrawing = True;
 
   /* make window infomation list */
   MakeList();
@@ -479,12 +520,12 @@ void list_end(void)
   /* size and create the window */
   lmax = max_col1 + max_col2 + 15;
 
-  height = ListSize * (font->ascent + font->descent);
+  height = ListSize * (Ffont->height);
 
   mysizehints.flags=
     USSize|USPosition|PWinGravity|PResizeInc|PBaseSize|PMinSize|PMaxSize;
-  mysizehints.width = lmax + 10;
-  mysizehints.height= height + 10;
+  main_width = mysizehints.width = lmax + 10;
+  main_height = mysizehints.height = height + 10;
   mysizehints.width_inc = 1;
   mysizehints.height_inc = 1;
   mysizehints.base_height = mysizehints.height;
@@ -507,6 +548,7 @@ void list_end(void)
     if (!XQueryPointer(
 	  dpy, Root, &JunkW, &JunkW, &x, &y, &JunkC, &JunkC, &JunkM))
     {
+      /* pointer is on a different screen */
       x = 0;
       y = 0;
     }
@@ -533,20 +575,18 @@ void list_end(void)
 
   if (Pdepth < 2)
   {
-    attributes.background_pixel = GetColor("white");
+    back_pix = GetColor("white");
     fore_pix = GetColor("black");
   }
   else
   {
-    attributes.background_pixel = (colorset < 0)
-				  ? GetColor(BackColor)
-    				  : Colorset[colorset].bg;
-    fore_pix = (colorset < 0) ? GetColor(ForeColor)
-			      : Colorset[colorset].fg;
+    back_pix = (colorset < 0)? GetColor(BackColor) : Colorset[colorset].bg;
+    fore_pix = (colorset < 0)? GetColor(ForeColor) : Colorset[colorset].fg;
   }
 
   attributes.colormap = Pcmap;
   attributes.border_pixel = 0;
+  attributes.background_pixel = back_pix;
   main_win = XCreateWindow(dpy, Root, x, y,
 			   mysizehints.width, mysizehints.height, 0, Pdepth,
 			   InputOutput, Pvisual,
@@ -556,21 +596,29 @@ void list_end(void)
   XSetWMProtocols(dpy,main_win,&wm_del_win,1);
 
   XSetWMNormalHints(dpy,main_win,&mysizehints);
+  if (!UsePixmapDrawing)
+    mw_events |= ExposureMask;
   /* have to ask for configure events when transparent */
   if (colorset >= 0 && Colorset[colorset].pixmap == ParentRelative)
-    mw_events |= StructureNotifyMask;
+    mw_events |= StructureNotifyMask|ExposureMask;
+
+
   XSelectInput(dpy, main_win, mw_events);
   change_window_name(&MyName[1]);
 
-  gcm = GCForeground|GCFont;
+  gcm = GCForeground;
   gcv.foreground = fore_pix;
-  gcv.font = font->fid;
+  if (Ffont->font != NULL)
+  {
+    gcm |= GCFont;
+    gcv.font = Ffont->font->fid;
+  }
   gc = fvwmlib_XCreateGC(dpy, main_win, gcm, &gcv);
-
-  if (colorset >= 0)
-    SetWindowBackground(dpy, main_win, mysizehints.width, mysizehints.height,
+  if (UsePixmapDrawing) 
+    PixmapDrawWindow(main_width, main_height);
+  else if (colorset >= 0)
+    SetWindowBackground(dpy, main_win, main_width, main_height,
 			&Colorset[(colorset)], Pdepth, gc, True);
-
   XMapWindow(dpy,main_win);
 
   /* Window is created. Display it until the user clicks or deletes it. */
@@ -591,8 +639,10 @@ void list_end(void)
       XNextEvent(dpy,&Event);
       switch(Event.type) {
         case Expose:
-          while (XCheckTypedEvent(dpy, Expose, &Event)) ;
-	  RedrawWindow();
+          while (XCheckTypedEvent(dpy, Expose, &Event));
+	  if (FftSupport && Ffont->fftf.fftfont != NULL)
+		  XClearWindow(dpy, main_win);
+	  DrawItems(main_win);
 	  break;
         case KeyPress:
 	  is_key_pressed = 1;
@@ -615,7 +665,7 @@ void list_end(void)
 		fd, M_CONFIGURE_WINDOW | M_WINDOW_NAME | M_ICON_NAME
 		| M_RES_CLASS | M_RES_NAME | M_END_WINDOWLIST | M_CONFIG_INFO
 		| M_END_CONFIG_INFO | M_SENDCONFIG);
-	      SendInfo(fd,"Send_WindowList",0);
+	      SendText(fd, "Send_WindowList", 0);
 	      XDestroyWindow(dpy, main_win);
 	      DestroyList();
 	      fvwmlib_get_target_window(dpy, screen, MyName, &app_win, True);
@@ -633,9 +683,12 @@ void list_end(void)
 	    exit(0);
 	  break;
 	case ReparentNotify:
-	  sprintf(buf, "Layer 0 %d", (int)target.layer);
-	  SendInfo(fd, buf, main_win);
-	  SendInfo(fd, "Raise", main_win);
+	  if (minimal_layer >= 0)
+	  {
+	    sprintf(buf, "Layer 0 %d", my_layer);
+	    SendText(fd, buf, main_win);
+	  }
+	  SendText(fd, "Raise", main_win);
 	  break;
 	case ConfigureNotify:
 	{
@@ -656,9 +709,9 @@ void list_end(void)
 	    x = Event.xconfigure.x;
 	    y = Event.xconfigure.y;
 	    /* flush any expose events */
-	    while (XCheckTypedEvent(dpy, Expose, &Event)) ;
+	    while (XCheckTypedEvent(dpy, Expose, &Event));
 	    XClearWindow(dpy, main_win);
-	    RedrawWindow();
+	    DrawItems(main_win);
 	  }
 	  break;
 	}
@@ -675,73 +728,122 @@ void list_end(void)
       packet = ReadFvwmPacket(fd[1]);
       if (packet == NULL)
 	exit(0);
-      if (colorset >= 0 && packet && packet->type == M_CONFIG_INFO) {
+      if (packet &&  packet->type == MX_PROPERTY_CHANGE)
+      {
+	list_property_change(packet->body);
+      }
+      if (packet && packet->type == M_CONFIG_INFO) {
 	tline = (char*)&(packet->body[3]);
 	tline = GetNextToken(tline, &token);
-	if (StrEquals(token, "Colorset")) {
+	if (StrEquals(token, "Colorset") && colorset >= 0) {
 	  /* track all colorset changes and update display if necessary */
 	  if (LoadColorset(tline) == colorset) {
-	    XSetForeground(dpy, gc, Colorset[colorset].fg);
-	    SetWindowBackground(dpy, main_win, mysizehints.width,
-				mysizehints.height,
-				&Colorset[colorset], Pdepth, gc,
-				True);
 	    /* ask for movement events iff transparent */
 	    if (Colorset[colorset].pixmap == ParentRelative)
-	      mw_events |= StructureNotifyMask;
+	      mw_events |= StructureNotifyMask|ExposureMask;
 	    else
-	      mw_events &= ~StructureNotifyMask;
+	    {
+	      mw_events &= ~(StructureNotifyMask);
+	      if (UsePixmapDrawing)
+		mw_events &= ~(ExposureMask);
+	    }
 	    XSelectInput(dpy, main_win, mw_events);
+	    XSetForeground(dpy, gc, Colorset[colorset].fg);
+	    if (UsePixmapDrawing)
+	      PixmapDrawWindow(main_width, main_height);
+	    else
+	      SetWindowBackground(dpy, main_win, main_width, main_height,
+				&Colorset[colorset], Pdepth, gc, True);
 	  }
 	}
 	else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
 	  FScreenConfigureModule(tline);
 	}
-	free(token);
+	if (token)
+	  free(token);
       }
     }
   }
 }
 
-
-
 /************************************************************************
  *
- * Draw the window
+ * Draw the items
  *
  ***********************************************************************/
-void RedrawWindow(void)
+void DrawItems(Drawable d)
 {
-  int fontheight,i=0;
-  struct Item *cur = itemlistRoot;
+	int fontheight,i=0;
+	struct Item *cur = itemlistRoot;
 
-  fontheight = font->ascent + font->descent;
+	fontheight = Ffont->height;
+	FwinString->win = d;
+	FwinString->gc = gc;
 
-  while(cur != NULL)
-    {
-      /* first column */
-#ifdef I18N_MB
-      XmbDrawString(dpy, main_win, fontset, gc, 5,
-		  5 + font->ascent + i * fontheight, cur->col1,
-		  strlen(cur->col1));
-#else
-      XDrawString(dpy, main_win, gc, 5,
-		  5 + font->ascent + i * fontheight, cur->col1,
-		  strlen(cur->col1));
-#endif
-      /* second column */
-#ifdef I18N_MB
-      XmbDrawString(dpy, main_win, fontset, gc, 10 + max_col1,
-		  5 + font->ascent + i * fontheight, cur->col2,
-		  strlen(cur->col2));
-#else
-      XDrawString(dpy, main_win, gc, 10 + max_col1,
-		  5 + font->ascent + i * fontheight, cur->col2,
-		  strlen(cur->col2));
-#endif
-      ++i;
-      cur = cur->next;
-    }
+	while(cur != NULL)
+	{
+		/* first column */
+		FwinString->str = cur->col1;
+		FwinString->x = 5;
+		FwinString->y = 5 + Ffont->ascent + i * fontheight;
+		FlocaleDrawString(dpy, Ffont, FwinString, 0);
+
+		/* second column */
+		FwinString->str = cur->col2;
+		FwinString->x = 10 + max_col1;
+		FlocaleDrawString(dpy, Ffont, FwinString, 0);
+
+		++i;
+		cur = cur->next;
+	}
+}
+
+void PixmapDrawWindow(int w, int h)
+{
+	Pixmap pix;
+	XGCValues gcv;
+	unsigned long gcm;
+
+	if (colorset >= 0)
+	{
+		Pixmap cs_pix;
+		cs_pix = CreateBackgroundPixmap(dpy, main_win, w, h,
+						&Colorset[(colorset)],
+						Pdepth, gc, False);
+		if (cs_pix == ParentRelative)
+		{
+			pix = cs_pix;
+		}
+		else
+		{
+			pix = CreateTiledPixmap(dpy, cs_pix, 0,0,w,h,Pdepth, gc);
+			XFreePixmap(dpy, cs_pix);
+		}
+	}
+	else
+	{
+		gcm = GCForeground;
+		gcv.foreground = back_pix;
+		XChangeGC(dpy, gc, gcm, &gcv);
+		pix = XCreatePixmap(dpy, main_win, w, h, Pdepth);
+		XFillRectangle(dpy, pix, gc, 0, 0, w, h);
+		gcv.foreground = fore_pix;
+		XChangeGC(dpy, gc, gcm, &gcv);
+	}
+
+	if (pix != ParentRelative)
+	{
+		DrawItems(pix);
+		XSetWindowBackgroundPixmap(dpy, main_win, pix);
+		XClearWindow(dpy, main_win);
+		XFreePixmap(dpy, pix);
+	}
+	else
+	{
+		XSetWindowBackgroundPixmap(dpy, main_win, pix);
+		XClearWindow(dpy, main_win);
+		DrawItems(main_win);
+	}
 }
 
 /**************************************************************************
@@ -789,8 +891,8 @@ void AddToList(char *s1, char* s2)
   int tw1, tw2;
   struct Item* item, *cur = itemlistRoot;
 
-  tw1 = XTextWidth(font, s1, strlen(s1));
-  tw2 = XTextWidth(font, s2, strlen(s2));
+  tw1 = FlocaleTextWidth(Ffont, s1, strlen(s1));
+  tw2 = FlocaleTextWidth(Ffont, s2, strlen(s2));
   max_col1 = max_col1 > tw1 ? max_col1 : tw1;
   max_col2 = max_col2 > tw2 ? max_col2 : tw2;
 
@@ -1000,4 +1102,89 @@ void MakeList(void)
       } /* end getsizehints worked */
     }
   }
+
+  /* EWMH window type */
+  if (target.ewmh_window_type == EWMH_WINDOW_TYPE_DESKTOP_ID)
+    AddToList("EWMH Window Type:","Desktop");
+  else if (target.ewmh_window_type == EWMH_WINDOW_TYPE_DIALOG_ID)
+    AddToList("EWMH Window Type:","Dialog");
+  else if (target.ewmh_window_type == EWMH_WINDOW_TYPE_DOCK_ID)
+    AddToList("EWMH Window Type:","Dock");
+  else if (target.ewmh_window_type == EWMH_WINDOW_TYPE_MENU_ID)
+    AddToList("EWMH Window Type:","Menu");
+  else if (target.ewmh_window_type == EWMH_WINDOW_TYPE_NORMAL_ID)
+    AddToList("EWMH Window Type:","Normal");
+  else if (target.ewmh_window_type == EWMH_WINDOW_TYPE_TOOLBAR_ID)
+    AddToList("EWMH Window Type:","ToolBar");
+
+  /* EWMH wm state */
+  {
+    char ewmh_init_state[512] = "";
+    Bool add_to_list = False;
+
+
+    if (HAS_EWMH_INIT_FULLSCREEN_STATE(targ) == EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "FullScreen ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_HIDDEN_STATE(targ) == EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "Iconic ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_MAXHORIZ_STATE(targ) == EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "MaxHoriz ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_MAXVERT_STATE(targ) == EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "MaxVert ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_MODAL_STATE(targ) == EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "Modal ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_SHADED_STATE(targ)== EWMH_STATE_HAS_HINT)
+    {
+      strcat(ewmh_init_state, "Shaded ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_SKIP_PAGER_STATE(targ) == EWMH_STATE_HAS_HINT ||
+	HAS_EWMH_INIT_SKIP_TASKBAR_STATE(targ) == EWMH_STATE_HAS_HINT )
+    {
+      strcat(ewmh_init_state, "SkipList ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_STICKY_STATE(targ) == EWMH_STATE_HAS_HINT ||
+	(HAS_EWMH_INIT_WM_DESKTOP(targ) ==  EWMH_STATE_HAS_HINT &&
+	(target.ewmh_hint_desktop == 0xFFFFFFFE ||
+	 target.ewmh_hint_desktop == 0xFFFFFFFF)))
+    {
+      strcat(ewmh_init_state, "Sticky ");
+      add_to_list = True;
+    }
+    if (target.ewmh_hint_layer > 0)
+    {
+      strcat(ewmh_init_state, "StaysOnTop ");
+      add_to_list = True;
+    }
+    if (HAS_EWMH_INIT_WM_DESKTOP(targ) == EWMH_STATE_HAS_HINT &&
+	target.ewmh_hint_desktop < 256)
+    {
+      strcat(ewmh_init_state, "StartOnDesk");
+      sprintf(ewmh_init_state, "%s %lu",
+	      ewmh_init_state, target.ewmh_hint_desktop);
+      add_to_list = True;
+    }
+    if (add_to_list)
+    {
+      /* should remove ending space */
+      AddToList("EWMH Init State:",ewmh_init_state);
+    }
+  }
+
 }
