@@ -49,8 +49,9 @@
 #include "libs/Module.h"
 
 #include "FvwmM4.h"
-#include "libs/Picture.h"
 #include "libs/fvwmlib.h"
+#include "libs/FShape.h"
+#include "libs/PictureBase.h"
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #define Resolution(pixels, mm) ((((pixels) * 2000 / (mm)) + 1) / 2)
@@ -230,7 +231,7 @@ int main(int argc, char **argv)
     }
 
   /* set up G */
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
 
   /* tell fvwm we're running if -lock is not used */
   if (!lock)
@@ -312,6 +313,7 @@ static char *m4_defs(
   unlink(tmp_name);
   if ((fd = open(tmp_name, O_WRONLY|O_EXCL|O_CREAT, 0644)) < 0)
   {
+    fprintf(stderr, "error opening file %s\n", tmp_name);
     perror("exclusive open for output file failed in m4_defs");
     exit(0377);
   }
@@ -463,15 +465,17 @@ static char *m4_defs(
 
   /* Add options together */
   *options = '\0';
-#ifdef	SHAPE
-  strcat(options, "SHAPE ");
-#endif
+  if (FHaveShapeExtension)
+    strcat(options, "SHAPE ");
 #ifdef	XPM
   strcat(options, "XPM ");
 #endif
-#ifdef  I18N_MB
+#ifdef  MULTIBYTE
   strcat(options, "I18N_MB ");
 #endif
+#ifdef  MULTIBYTE
+  strcat(options, "MULTIBYTE ");
+#endif
 
   strcat(options, "M4 ");
 
