@@ -41,15 +41,13 @@
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
 #include <X11/keysym.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
 #ifdef DEBUG
 #  define FVWM_DEBUG_MSGS   /* Do we need this? */
 #endif
@@ -64,7 +62,8 @@ int fd[2];
 
 PagerStringList *FindDeskStrings(int desk);
 PagerStringList *NewPagerStringItem(PagerStringList *last, int desk);
-extern XFontStruct *windowFont;
+extern FlocaleFont *FwindowFont;
+extern Pixmap default_pixmap;
 
 /*************************************************************************
  *
@@ -85,7 +84,7 @@ char *font_string = NULL;
 char *smallFont = NULL;
 char *HilightC = NULL;
 
-Picture *HilightPixmap = NULL;
+FvwmPicture *HilightPixmap = NULL;
 int HilightDesks = 1;
 
 char *WindowBack = NULL;
@@ -98,7 +97,9 @@ unsigned int WindowBorderWidth = DEFAULT_PAGER_WINDOW_BORDER_WIDTH;
 unsigned int MinSize = 2 * DEFAULT_PAGER_WINDOW_BORDER_WIDTH + 1;
 Bool WindowBorders3d = False;
 
-Picture *PixmapBack = NULL;
+Bool UseSkipList = False;
+
+FvwmPicture *PixmapBack = NULL;
 
 char *ImagePath = NULL;
 
@@ -148,8 +149,9 @@ PagerStringList string_list = { NULL, 0, -1, -1, -1, NULL, NULL, NULL };
 Bool is_transient = False;
 Bool do_ignore_next_button_release = False;
 Bool error_occured = False;
+Bool Swallowed = False;
 
-
+static void SetDeskLabel(int desk, const char *label);
 static RETSIGTYPE TerminateHandler(int);
 void ExitPager(void);
 
@@ -169,9 +171,8 @@ int main(int argc, char **argv)
   int JunkX, JunkY;
   unsigned JunkMask;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmPager");
+
   /* Save our program  name - for error messages */
   MyName = GetFileNameFromPath(argv[0]);
 
@@ -309,7 +310,7 @@ int main(int argc, char **argv)
       exit (1);
     }
   x_fd = XConnectionNumber(dpy);
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
@@ -324,6 +325,7 @@ int main(int argc, char **argv)
 #endif
 
   SetMessageMask(fd,
+		 M_VISIBLE_NAME |
                  M_ADD_WINDOW|
                  M_CONFIGURE_WINDOW|
                  M_DESTROY_WINDOW|
@@ -337,25 +339,32 @@ int main(int argc, char **argv)
 		 M_DEICONIFY|
 		 M_RES_NAME|
 		 M_RES_CLASS|
-		 M_WINDOW_NAME|
-		 M_ICON_NAME|
 		 M_CONFIG_INFO|
 		 M_END_CONFIG_INFO|
 		 M_MINI_ICON|
 		 M_END_WINDOWLIST|
 		 M_RESTACK);
+  SetMessageMask(fd,
+		 MX_VISIBLE_ICON_NAME|
+		 MX_PROPERTY_CHANGE);
 #ifdef DEBUG
   fprintf(stderr,"[main]: calling ParseOptions\n");
 #endif
   ParseOptions();
   if (is_transient)
+  {
+    if (XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &window_x, &window_y, &JunkX,
+	  &JunkY, &JunkMask) == False)
     {
-      XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		    &window_x, &window_y, &JunkX, &JunkY, &JunkMask);
-      usposition = 1;
-      xneg = 0;
-      yneg = 0;
+      /* pointer is on a different screen */
+      window_x = 0;
+      window_y = 0;
     }
+    usposition = 1;
+    xneg = 0;
+    yneg = 0;
+  }
 #ifdef DEBUG
   fprintf(stderr,
 	  "[main]: back from calling ParseOptions, calling init pager\n");
@@ -373,15 +382,9 @@ int main(int argc, char **argv)
   if (WindowLabelFormat == NULL)
     WindowLabelFormat = safestrdup("%i");
 
-  if (font_string == NULL)
-    font_string = safestrdup("fixed");
-
   if ((HilightC == NULL) && (HilightPixmap == NULL))
     HilightDesks = 0;
 
-  if (BalloonFont == NULL)
-    BalloonFont = safestrdup("fixed");
-
   if (BalloonBorderColor == NULL)
     BalloonBorderColor = safestrdup("black");
 
@@ -538,10 +541,10 @@ void process_message( FvwmPacket* packet )
       break;
     case M_RES_CLASS:
     case M_RES_NAME:
-    case M_WINDOW_NAME:
+    case M_VISIBLE_NAME:
       list_window_name(body,type);
       break;
-    case M_ICON_NAME:
+    case MX_VISIBLE_ICON_NAME:
       list_icon_name(body);
       break;
     case M_MINI_ICON:
@@ -556,6 +559,8 @@ void process_message( FvwmPacket* packet )
     case M_CONFIG_INFO:
       list_config_info(body);
       break;
+    case MX_PROPERTY_CHANGE:
+      list_property_change(body);
     default:
       /* ignore unknown packet */
       break;
@@ -681,7 +686,7 @@ void list_configure(unsigned long *body)
     t->text = body[22];
     t->back = body[23];
   }
-  if(IS_ICONIFIED(t))
+  if (IS_ICONIFIED(t))
   {
     t->x = t->icon_x;
     t->y = t->icon_y;
@@ -700,12 +705,14 @@ void list_configure(unsigned long *body)
     t->width = t->frame_width;
     t->height = t->frame_height;
   }
-  if(t->desk != cfgpacket->desk)
+  if (t->desk != cfgpacket->desk)
   {
     ChangeDeskForWindow(t, cfgpacket->desk);
   }
   else
+  {
     MoveResizePagerView(t, False);
+  }
 }
 
 /***********************************************************************
@@ -876,7 +883,7 @@ void list_new_desk(unsigned long *body)
 
     if (Desks[0].bgPixmap != NULL)
     {
-      DestroyPicture(dpy, Desks[0].bgPixmap);
+      PDestroyFvwmPicture(dpy, Desks[0].bgPixmap);
       Desks[0].bgPixmap = NULL;
     }
 
@@ -962,25 +969,6 @@ void list_new_desk(unsigned long *body)
     XClearWindow(dpy, Desks[0].w);
     XClearWindow(dpy, Desks[0].title_w);
   } /* if (fAlwaysCurrentDesk && oldDesk != Scr.CurrentDesk) */
-  else if (!fAlwaysCurrentDesk)
-  {
-    int i;
-    char *name;
-    char line[100];
-
-    i = Scr.CurrentDesk - desk1;
-    if (i >= 0 && i < ndesks && Desks[i].label != NULL)
-    {
-      name = Desks[i].label;
-    }
-    else
-    {
-      sprintf(line, "Desk %d", Scr.CurrentDesk);
-      name = &(line[0]);
-    }
-      XStoreName(dpy, Scr.Pager_w, name);
-      XSetIconName(dpy, Scr.Pager_w, name);
-  }
 
   MovePage(True);
   DrawGrid(oldDesk - desk1,1);
@@ -1170,14 +1158,14 @@ void list_window_name(unsigned long *body,unsigned long type)
           free(t->res_name);
         CopyString(&t->res_name,(char *)(&body[3]));
         break;
-      case M_WINDOW_NAME:
+      case M_VISIBLE_NAME:
         if(t->window_name != NULL)
           free(t->window_name);
         CopyString(&t->window_name,(char *)(&body[3]));
         break;
       }
       /* repaint by clearing window */
-      if ((windowFont != NULL) && (t->icon_name != NULL)
+      if ((FwindowFont != NULL) && (t->icon_name != NULL)
 	   && !(MiniIcons && t->mini_icon.picture)) {
 	if (t->PagerView)
 	  XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
@@ -1224,7 +1212,7 @@ void list_icon_name(unsigned long *body)
 	free(t->icon_name);
       CopyString(&t->icon_name,(char *)(&body[3]));
       /* repaint by clearing window */
-      if ((windowFont != NULL) && (t->icon_name != NULL)
+      if ((FwindowFont != NULL) && (t->icon_name != NULL)
 	   && !(MiniIcons && t->mini_icon.picture)) {
 	if (t->PagerView)
 	  XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
@@ -1239,17 +1227,20 @@ void list_mini_icon(unsigned long *body)
 {
   PagerWindow	*t;
   Window target_w;
+  MiniIconPacket *mip = (MiniIconPacket *) body;
+
   target_w = body[0];
   t = Start;
-  while (t && (t->w != target_w))
+  while (t && (t->w != mip->w))
     t = t->next;
   if (t)
   {
-    t->mini_icon.width   = body[3];
-    t->mini_icon.height  = body[4];
-    t->mini_icon.depth   = body[5];
-    t->mini_icon.picture = body[6];
-    t->mini_icon.mask    = body[7];
+    t->mini_icon.width   = mip->width;
+    t->mini_icon.height  = mip->height;
+    t->mini_icon.depth   = mip->depth;
+    t->mini_icon.picture = mip->picture;
+    t->mini_icon.mask    = mip->mask;
+    t->mini_icon.alpha   = mip->alpha;
     /* repaint by clearing window */
     if (MiniIcons && t->mini_icon.picture) {
       if (t->PagerView)
@@ -1365,8 +1356,59 @@ void list_config_info(unsigned long *body)
   {
     FScreenConfigureModule(tline);
   }
+  else if (StrEquals(token, "DesktopName"))
+  {
+    int val;
+    if (GetIntegerArguments(tline, &tline, &val, 1) > 0)
+    {
+      SetDeskLabel(val, (const char *)tline);
+    }
+    else
+    {
+      return;
+    }
+    if (fAlwaysCurrentDesk)
+    {
+      if (Scr.CurrentDesk == val)
+	val = 0;
+    }
+    else if ((val >= desk1) && (val <=desk2))
+    {
+      val = val - desk1;
+    }
+    DrawGrid(val, True);
+  }
 }
 
+void list_property_change(unsigned long *body)
+{
+  int i;
+
+  if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND)
+  {
+
+    if (default_pixmap == ParentRelative &&
+	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
+    {
+      ReConfigure();
+    }
+    if (BalloonView != None)
+    {
+      for(i=0;i<ndesks;i++)
+      {
+	if (Desks[i].ballooncolorset > -1 &&
+	    Colorset[Desks[i].ballooncolorset].pixmap == ParentRelative)
+	{
+	  XClearArea(dpy, Desks[i].balloon.w, 0, 0, 0, 0, True);
+	}
+      }
+    }
+  }
+  else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == Scr.Pager_w)
+  {
+    Swallowed = body[1];
+  }
+}
 
 /***************************************************************************
  *
@@ -1485,6 +1527,42 @@ static void ParseColorset(char *arg1, char *arg2, void *offset_deskinfo,
   return;
 }
 
+static void SetDeskLabel(int desk, const char *label)
+{
+  PagerStringList *item;
+
+  if (fAlwaysCurrentDesk)
+  {
+    item = FindDeskStrings(desk);
+    if (item->next != NULL)
+    {
+      /* replace label */
+      if (item->next->label != NULL)
+      {
+	free(item->next->label);
+	item->next->label = NULL;
+      }
+      CopyString(&(item->next->label), label);
+    }
+    else
+    {
+      /* new Dcolor and desktop */
+      item = NewPagerStringItem(item, desk);
+      CopyString(&(item->label), label);
+    }
+    if (desk == Scr.CurrentDesk)
+    {
+      free(Desks[0].label);
+      CopyString(&Desks[0].label, label);
+    }
+  }
+  else if((desk >= desk1)&&(desk <=desk2))
+  {
+    free(Desks[desk - desk1].label);
+    CopyString(&Desks[desk - desk1].label, label);
+  }
+}
+
 /*****************************************************************************
  *
  * This routine is responsible for reading and parsing the config file
@@ -1571,6 +1649,15 @@ void ParseOptions(void)
       }
       continue;
     }
+    else if (StrEquals(token, "DesktopName"))
+    {
+      int val;
+      if (GetIntegerArguments(next, &next, &val, 1) > 0)
+      {
+	SetDeskLabel(val, (const char *)next);
+      }
+      continue;
+    }
 
     tline2 = GetModuleResource(tline, &resource, MyName);
     if (!resource)
@@ -1687,38 +1774,7 @@ void ParseOptions(void)
 	desk = desk1;
 	sscanf(arg1,"%d",&desk);
       }
-      if (fAlwaysCurrentDesk)
-      {
-	PagerStringList *item;
-
-	item = FindDeskStrings(desk);
-	if (item->next != NULL)
-	{
-	  /* replace label */
-	  if (item->next->label != NULL)
-	  {
-	    free(item->next->label);
-	    item->next->label = NULL;
-	  }
-	  CopyString(&(item->next->label), arg2);
-	}
-	else
-	{
-	  /* new Dcolor and desktop */
-	  item = NewPagerStringItem(item, desk);
-	  CopyString(&(item->label), arg2);
-	}
-	if (desk == Scr.CurrentDesk)
-	{
-	  free(Desks[0].label);
-	  CopyString(&Desks[0].label, arg2);
-	}
-      }
-      else if((desk >= desk1)&&(desk <=desk2))
-      {
-	free(Desks[desk - desk1].label);
-	CopyString(&Desks[desk - desk1].label, arg2);
-      }
+      SetDeskLabel(desk, (const char *)arg2);
     }
     else if (StrEquals(resource, "Font"))
     {
@@ -1741,7 +1797,14 @@ void ParseOptions(void)
 	CopyString(&font_string,next);
       }
       if(strncasecmp(font_string,"none",4) == 0)
+      {
 	uselabel = 0;
+	if (font_string)
+	{
+	  free(font_string);
+	  font_string = NULL;
+	}
+      }
     }
     else if (StrEquals(resource, "Fore"))
     {
@@ -1826,30 +1889,30 @@ void ParseOptions(void)
 	{
 	  if (item->next->bgPixmap != NULL)
 	  {
-	    DestroyPicture(dpy, item->next->bgPixmap);
+	    PDestroyFvwmPicture(dpy, item->next->bgPixmap);
 	    item->next->bgPixmap = NULL;
 	  }
-	  item->next->bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					       ImagePath,
-					       arg2, 0);
+	  item->next->bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
+						    ImagePath,
+						    arg2, 0);
 	}
 	else
 	{
 	  /* new Dcolor and desktop */
 	  item = NewPagerStringItem(item, desk);
-	  item->bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					 ImagePath,
-					 arg2, 0);
+	  item->bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
+					      ImagePath,
+					      arg2, 0);
 	}
 	if (desk == Scr.CurrentDesk)
 	{
 	  if (Desks[0].bgPixmap != NULL)
 	  {
-	    DestroyPicture(dpy, Desks[0].bgPixmap);
+	    PDestroyFvwmPicture(dpy, Desks[0].bgPixmap);
 	    Desks[0].bgPixmap = NULL;
 	  }
 
-	  Desks[0].bgPixmap = CachePicture (dpy, Scr.Pager_w,
+	  Desks[0].bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
 					    ImagePath,
 					    arg2, 0);
 	}
@@ -1860,12 +1923,12 @@ void ParseOptions(void)
 
 	if (Desks[dNr].bgPixmap != NULL)
 	{
-	  DestroyPicture(dpy, Desks[dNr].bgPixmap);
+	  PDestroyFvwmPicture(dpy, Desks[dNr].bgPixmap);
 	  Desks[dNr].bgPixmap = NULL;
 	}
-	Desks[dNr].bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					    ImagePath,
-					    arg2, 0);
+	Desks[dNr].bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
+						 ImagePath,
+						 arg2, 0);
       }
 
 #ifdef DEBUG
@@ -1879,13 +1942,13 @@ void ParseOptions(void)
       if(Pdepth > 1)
       {
 	if (PixmapBack) {
-	  DestroyPicture (dpy, PixmapBack);
+	  PDestroyFvwmPicture (dpy, PixmapBack);
 	  PixmapBack = NULL;
 	}
 
-	PixmapBack = CachePicture (dpy, Scr.Pager_w,
-				   ImagePath,
-				   arg1, 0);
+	PixmapBack = PCacheFvwmPicture (dpy, Scr.Pager_w,
+					ImagePath,
+					arg1, 0);
 #ifdef DEBUG
 	fprintf(stderr,
 		"[ParseOptions]: Global: bgPixmap = %s\n", arg1);
@@ -1898,13 +1961,13 @@ void ParseOptions(void)
       if(Pdepth > 1)
       {
 	if (HilightPixmap) {
-	  DestroyPicture (dpy, HilightPixmap);
+	  PDestroyFvwmPicture (dpy, HilightPixmap);
 	  HilightPixmap = NULL;
 	}
 
-	HilightPixmap = CachePicture (dpy, Scr.Pager_w,
-				      ImagePath,
-				      arg1, 0);
+	HilightPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
+					ImagePath,
+					arg1, 0);
 
 #ifdef DEBUG
 	fprintf(stderr,
@@ -2036,6 +2099,10 @@ void ParseOptions(void)
 	free(WindowLabelFormat);
       CopyString(&WindowLabelFormat,arg1);
     }
+    else if (StrEquals(resource,"UseSkipList"))
+    {
+      UseSkipList = True;
+    }
     else if (StrEquals(resource, "MoveThreshold"))
     {
       int val;
@@ -2109,7 +2176,7 @@ void ParseOptions(void)
     {
       if (BalloonFont)
 	free(BalloonFont);
-      CopyStringWithQuotes(&BalloonFont, next);
+      CopyString(&BalloonFont, next);
     }
 
     else if (StrEquals(resource, "BalloonBorderColor"))
