@@ -15,6 +15,7 @@
 
 #include <libs/Picture.h>
 #include <libs/vpacket.h>
+#include <libs/Flocale.h>
 
 typedef struct ScreenInfo
 {
@@ -75,6 +76,10 @@ typedef struct pager_window
   Pixel text;
   Pixel back;
   window_flags flags;
+  struct
+  {
+    unsigned is_mapped : 1;
+  } myflags;
   Window icon_w;
   Window icon_pixmap_w;
   char *icon_name;
@@ -82,7 +87,7 @@ typedef struct pager_window
   char *res_name;
   char *res_class;
   char *window_label; /* This is displayed inside the mini window */
-  Picture mini_icon;
+  FvwmPicture mini_icon;
   int pager_view_x;
   int pager_view_y;
   int pager_view_width;
@@ -101,10 +106,7 @@ typedef struct balloon_window
 {
   Window w;              /* ID of balloon window */
   PagerWindow *pw;       /* pager window it's associated with */
-  XFontStruct *font;
-#ifdef I18N_MB
-  XFontSet fontset;
-#endif
+  FlocaleFont *Ffont;
   char *label;           /* the label displayed inside the balloon */
   int height;            /* height of balloon window based on font */
   int border;            /* border width */
@@ -118,7 +120,7 @@ typedef struct desk_info
   Window title_w;
   Window CPagerWin;
   BalloonWindow balloon;
-  Picture *bgPixmap;		/* Pixmap used as background. */
+  FvwmPicture *bgPixmap;		/* Pixmap used as background. */
   int colorset;
   int highcolorset;
   int ballooncolorset;
@@ -140,7 +142,7 @@ typedef struct pager_string_list
   int ballooncolorset;
   char *Dcolor;
   char *label;
-  Picture *bgPixmap;		/* Pixmap used as background. */
+  FvwmPicture *bgPixmap;		/* Pixmap used as background. */
 } PagerStringList;
 
 /*************************************************************************
@@ -156,7 +158,7 @@ void ParseOptions(void);
 
 void list_add(unsigned long *body);
 void list_configure(unsigned long *body);
-void list_colorset(unsigned long *body);
+void list_config_info(unsigned long *body);
 void list_destroy(unsigned long *body);
 void list_focus(unsigned long *body);
 void list_toggle(unsigned long *body);
@@ -173,6 +175,7 @@ void list_class(unsigned long *body);
 void list_res_name(unsigned long *body);
 void list_mini_icon(unsigned long *body);
 void list_restack(unsigned long *body, unsigned long length);
+void list_property_change(unsigned long *body);
 void list_end(void);
 int My_XNextEvent(Display *dpy, XEvent *event);
 
@@ -208,7 +211,7 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only);
 void IconSwitchPage(XEvent *Event);
 void IconMoveWindow(XEvent *Event,PagerWindow *t);
 void HandleEnterNotify(XEvent *Event);
-void HandleExpose(XEvent *Event, Bool redraw_subwindows);
+void HandleExpose(XEvent *Event);
 void MoveStickyWindows(void);
 void MapBalloonWindow(PagerWindow *t, Bool is_icon_view);
 void UnmapBalloonWindow();
