@@ -25,6 +25,7 @@
 #include "libs/FShape.h"
 #include "libs/fvwmsignal.h"
 #include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
 
 #ifdef MEMDEBUG			/* For debugging */
 #include <unchecked.h>
@@ -59,6 +60,7 @@ int fd[2]; 			/* pipe pair */
 int fd_err;
 int x_fd;			/* fd for X */
 Window ref;
+FlocaleWinString *FwinString;
 
 extern int yyparse(void);
 extern void (*TabCom[25]) (int NbArg,long *TabArg);
@@ -103,7 +105,7 @@ ShutdownX(void)
   /* execute the QuitFunc */
   if (x11base->quitfunc != NULL)
     ExecBloc(x11base->quitfunc);
-  
+
   if (is_dead_pipe)
     return;
 
@@ -311,10 +313,11 @@ void Xinit(int IsFather)
   }
   screen = DefaultScreen(dpy);
   Root = RootWindow(dpy,screen);
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
+  FlocaleAllocateWinString(&FwinString);
   XSetErrorHandler(myErrorHandler);
 
 #ifdef MEMDEBUG
@@ -348,24 +351,65 @@ void Xinit(int IsFather)
 /***********************/
 void LoadIcon(struct XObj *xobj)
 {
-  Picture *pic;
+	char *path;
+	int depth;
+	FvwmPictureFlags fpf;
 
-  if ((xobj->icon) != NULL)
-  {
-    pic = CachePicture(dpy,x11base->win,imagePath,xobj->icon,save_color_limit);
-    if (!pic)
-    {
-      fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to load pixmap %s\n",
-	      ScriptName,xobj->icon);
-      xobj->iconPixmap = None;
-      xobj->icon_maskPixmap = None;
-      return;
-    }
-    xobj->iconPixmap = pic->picture;
-    xobj->icon_maskPixmap = pic->mask;
-    xobj->icon_w = pic->width;
-    xobj->icon_h = pic->height;
-  }
+	fpf.alloc_pixels = 0;
+	fpf.alpha = 1;
+	if (xobj->iconPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->iconPixmap);
+		xobj->iconPixmap = None;
+	}
+	if (xobj->icon_maskPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->icon_maskPixmap);
+		xobj->icon_maskPixmap = None;
+	}
+	if (xobj->icon_alphaPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->icon_alphaPixmap);
+		xobj->icon_alphaPixmap = None;
+	}
+	
+	if ((xobj->icon) == NULL)
+		return;
+	if ((path = PictureFindImageFile(xobj->icon,imagePath,R_OK)) == NULL)
+	{
+		return;
+	}
+	if (!PImageLoadPixmapFromFile(dpy, x11base->root, path, save_color_limit,
+				      &xobj->iconPixmap, &xobj->icon_maskPixmap,
+				      &xobj->icon_alphaPixmap,
+				      &xobj->icon_w, &xobj->icon_h, &depth,
+				      0, NULL, fpf))
+	{
+		fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to "
+			"load pixmap %s\n",
+			ScriptName,xobj->icon);
+		free(path);
+		return;
+	}
+	if (depth <= 1)
+	{
+		XGCValues gcv;
+		GC m_gc;
+		Pixmap temp;
+
+		gcv.background= WhitePixel(dpy, screen);
+		gcv.foreground= BlackPixel(dpy, screen);
+		m_gc = fvwmlib_XCreateGC(dpy, x11base->win,
+				       GCForeground | GCBackground, &gcv);
+		temp = XCreatePixmap(dpy, x11base->win,
+				     xobj->icon_w, xobj->icon_h, Pdepth);
+		XCopyPlane(dpy, xobj->iconPixmap, temp, m_gc,
+			   0,0,xobj->icon_w, xobj->icon_h, 0,0,1);
+		XFreePixmap(dpy, xobj->iconPixmap);
+		XFreeGC(dpy,m_gc);
+		xobj->iconPixmap = temp;
+	}
+	free(path);
 }
 
 
@@ -410,7 +454,7 @@ void OpenWindow (void)
   /* Choix des evts recus par la fenetre */
   XSelectInput(dpy,x11base->win, KeyPressMask|ButtonPressMask|
 	       ExposureMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|
-	       ButtonMotionMask);
+	       ButtonMotionMask|StructureNotifyMask);
   XSelectInput(dpy, x11base->root, PropertyChangeMask);
 
   /* Specification des parametres utilises par le gestionnaire de fenetre */
@@ -552,7 +596,7 @@ void BuildGUI(int IsFather)
     else
       tabxobj[i]->title = (*tabobj)[i].title;
 
-    if ((*tabobj)[i].font == NULL)
+    if ((*tabobj)[i].font == NULL && x11base->font != NULL)
       tabxobj[i]->font = (char*)safestrdup(x11base->font);
     else
       tabxobj[i]->font = (*tabobj)[i].font;
@@ -600,7 +644,7 @@ void BuildGUI(int IsFather)
   x11base->periodictasks = scriptprop->periodictasks;
   /* the QuitFunc */
   x11base->quitfunc = scriptprop->quitfunc;
-   
+
   /*Si un bloc d'initialisation du script existe, on l'execute ici */
   if (scriptprop->initbloc != NULL)
   {
@@ -733,6 +777,28 @@ void SendMsgToScript(XEvent event)
   }
 }
 
+static
+void UpdateRootTransparency(void)
+{
+  int i;
+
+  if (x11base->colorset > 0 &&
+      Colorset[x11base->colorset].pixmap == ParentRelative)
+  {
+    XClearArea(dpy, x11base->win, 0, 0, 0, 0, True);
+    for (i=0; i<nbobj; i++)
+    {
+      if (Rectangle != tabxobj[i]->TypeWidget &&
+	  SwallowExec != tabxobj[i]->TypeWidget &&
+	  ((tabxobj[i]->colorset > 0 &&
+	   Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
+      {
+	XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
+      }
+    }
+  }
+}
+
 /* read an X event */
 void ReadXServer (void)
 {
@@ -773,6 +839,23 @@ void ReadXServer (void)
 	}
       }
       break;
+    case ConfigureNotify:
+    {
+      Bool moved = False;
+
+      moved = event.xconfigure.send_event;
+      while (XCheckTypedEvent(dpy, ConfigureNotify, &event))
+      {
+	/* check for movement */
+	if (event.xconfigure.send_event)
+	  moved = True;
+      }
+      if (moved)
+      {
+	UpdateRootTransparency();
+      }
+    }
+    break;
     case KeyPress:
       /* Touche presse dans un objet / Key press in an object */
       isTab = 0;
@@ -784,7 +867,7 @@ void ReadXServer (void)
       /* check for bindings defined by the Key instruction */
       if ((action = CheckBinding(BindingsList, STROKE_ARG(0)
 				 event.xkey.keycode,
-				 event.xkey.state, LockMask, 
+				 event.xkey.state, LockMask,
 				 C_WINDOW, KEY_BINDING)) != NULL)
       {
 	SendMsgAndString(action, "CheckBinding");
@@ -996,6 +1079,16 @@ void MainLoop (void)
 	      SetWindowBackground(dpy, x11base->win, x11base->size.width,
 				  x11base->size.height, &Colorset[n], Pdepth,
 				  x11base->gc, True);
+	      for (i=0; i<nbobj; i++)
+	      {
+		if (Rectangle != tabxobj[i]->TypeWidget &&
+		    SwallowExec != tabxobj[i]->TypeWidget &&
+		    ((tabxobj[i]->colorset > 0 &&
+		      Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
+		{
+		  XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
+		}
+	      }
 	    }
 	  }
 	  else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
@@ -1004,6 +1097,20 @@ void MainLoop (void)
 	  if (token)
 	    free(token);
 	}
+	else if (packet->type == MX_PROPERTY_CHANGE)
+	{
+	  if (packet->body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+	      ((!x11base->swallowed && packet->body[2] == 0) ||
+	       (x11base->swallowed && packet->body[2] == x11base->win)))
+	  {
+	    UpdateRootTransparency();
+	  }
+	  else if  (packet->body[0] == MX_PROPERTY_CHANGE_SWALLOW &&
+		    packet->body[2] == x11base->win)
+	  {
+	    x11base->swallowed = packet->body[1];
+	  }
+	}
 	else if (packet->type == M_STRING) {
 	  char *action, *token;
 	  action = (char*)&(packet->body[3]);
@@ -1072,9 +1179,7 @@ int main (int argc, char **argv)
   int IsFather;
   int i;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmScript");
 
   ModuleName = GetFileNameFromPath(argv[0]);
 
@@ -1105,19 +1210,20 @@ int main (int argc, char **argv)
   SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST| M_STRING |
 		 M_MAP|  M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
 		 M_END_CONFIG_INFO| M_WINDOW_NAME | M_SENDCONFIG);
-
+  SetMessageMask(fd, MX_PROPERTY_CHANGE);
   /* Enregistrement des arguments du script */
   x11base = (X11base*) safecalloc(1,sizeof(X11base));
   x11base->TabArg[0] = ModuleName;
   for (i=8-IsFather; i<argc; i++)
     x11base->TabArg[i-7+IsFather] = argv[i];
   /* Couleurs et fontes par defaut */
-  x11base->font = safestrdup("fixed");
+  x11base->font = NULL;
   x11base->forecolor = safestrdup("black");
   x11base->backcolor = safestrdup("grey85");
   x11base->shadcolor = safestrdup("grey55");
   x11base->hilicolor = safestrdup("grey100");
   x11base->colorset = -1;
+  x11base->swallowed = False;
 
   /* Initialisation du serveur X et de la fenetre */
   Xinit(IsFather);
