@@ -273,7 +273,7 @@ static char *FuncGetShad(int *NbArg, long *TabArg)
   char *tmp;
   long Id;
   XColor color;
-  
+
   (*NbArg)++;	    /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
@@ -847,7 +847,7 @@ static char *FuncSendMsgAndGet(int *NbArg,long *TabArg)
     free(com_name);
     str=(char*)safecalloc(2,sizeof(char));
     sprintf(str,(err) ? "0" : "1");
-    return str; 
+    return str;
   }
 
   /* get the answer from the receiver.                              *
@@ -945,7 +945,7 @@ static char *FuncParse(int *NbArg,long *TabArg)
     step++;
     i += toklen + 4;
   }
-  
+
   if (end > l || end <= start)
   {
     str=(char*)safecalloc(1,sizeof(char));
@@ -1150,14 +1150,7 @@ static void ChangeFont (int NbArg,long *TabArg)
   int i=0;
   char *arg[2];
   int IdItem;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#else
-  XFontStruct *xfont;
-#endif
+  FlocaleFont *Ffont;
 
   arg[0] = CalcArg(TabArg,&i);
   i++;
@@ -1166,32 +1159,20 @@ static void ChangeFont (int NbArg,long *TabArg)
   if (tabxobj[IdItem]->font)
     free(tabxobj[IdItem]->font);
   tabxobj[IdItem]->font = safestrdup(arg[1]);
-#ifdef I18N_MB
-  /* Hmm.. Fontset is not freed. However, original alogrithm does not consider
-   * the situation of font-loading-falure.
-  */
-  if ((tabxobj[IdItem]->xfontset = GetFontSetOrFixed(dpy,tabxobj[IdItem]->font))
-      == NULL) {
-    fprintf(stderr, "[%s][ChangeFont]: Couldn't load font %s. Exiting!\n",
-	    ScriptName,tabxobj[IdItem]->font);
-    exit(1);
-  }
-  XFontsOfFontSet(tabxobj[IdItem]->xfontset, &fs_list, &ml);
-  tabxobj[IdItem]->xfont = fs_list[0];
-  XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->xfont->fid);
-#else
-  if ((xfont = XLoadQueryFont(dpy,tabxobj[IdItem]->font)) == NULL)
-  {
-    fprintf(stderr,"[%s][ChangeFont]: Can't load font %s\n",
+
+  if ((Ffont =
+       FlocaleLoadFont(dpy, tabxobj[IdItem]->font, ScriptName)) == NULL) {
+    fprintf(stderr, "[%s][ChangeFont]: Couldn't load font %s\n",
 	    ScriptName,tabxobj[IdItem]->font);
   }
   else
   {
-    XFreeFont(dpy,tabxobj[IdItem]->xfont);
-    tabxobj[IdItem]->xfont = xfont;
-    XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->xfont->fid);
+    FlocaleUnloadFont(dpy, tabxobj[IdItem]->Ffont);
+    tabxobj[IdItem]->Ffont = Ffont;
+    if (Ffont->font != NULL)
+      XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->Ffont->font->fid);
   }
-#endif
+
   if (tabxobj[IdItem]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[IdItem]->win);
   tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
@@ -1446,8 +1427,8 @@ static void SendSign (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* WarpPointer *******/
-static void WarpPointer(int NbArg,long *TabArg)
+/******* MyWarpPointer *******/
+static void MyWarpPointer(int NbArg,long *TabArg)
 {
   int i=0;
   char *arg;
@@ -1774,7 +1755,7 @@ static void Key (int NbArg,long *TabArg)
     str = strcat(str, tmp);
     free(tmp);
   }
-  
+
   tmp = safestrdup(CatString3(widget, " ", sig));
   action = safestrdup(CatString3(tmp, " ", str));
   free(sig);
@@ -1801,9 +1782,9 @@ static void Key (int NbArg,long *TabArg)
     free(action);
     return;
   }
-  
+
   AddBinding(dpy, &BindingsList, KEY_BINDING, STROKE_ARG(0)
-	     0, keysym, key_string, modifier, C_WINDOW, 
+	     0, keysym, key_string, modifier, C_WINDOW,
 	     (void *)action, NULL);
   free(key_string);
   free(in_modifier);
@@ -1831,7 +1812,7 @@ void InitCom()
   TabCom[14]=IfThen;
   TabCom[15]=Loop;
   TabCom[16]=While;
-  TabCom[17]=WarpPointer;
+  TabCom[17]=MyWarpPointer;
   TabCom[18]=WriteToFile;
   TabCom[19]=ChangeBackColor;
   TabCom[21]=ChangeValueMax;
