@@ -18,7 +18,9 @@
 #include "libs/fvwmlib.h"
 #include "libs/Module.h"
 #include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
 
 #include <stdio.h>
 #include <signal.h>
@@ -36,17 +38,19 @@
 #include <sys/bsdtypes.h> /* Saul */
 #endif
 
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#ifdef __STDC__
-#define XTextWidth(x,y,z)	XmbTextEscapement(x ## set,y,z)
-#define XFreeFont(x,y)		XFreeFontSet(x,y ## set)
-#else
-#define XTextWidth(x,y,z)	XmbTextEscapement(x/**/set,y,z)
-#define XFreeFont(x,y)		XFreeFontSet(x,y/**/set)
-#endif
-#define DrawString(a,b,c,d,e,f,g,h,i,j,k) FakeDrawString(xobj->xfontset,a,b,c,d,e,f,g,h,i,j,k)
-#endif /* I18N_MB */
+/* flags */
+#define IS_HIDDEN(x) (x->flags[0])
+#define HAS_NO_RELIEF_STRING(x) (x->flags[1])
+#define HAS_NO_FOCUS(x) (x->flags[2])
+#define TEXT_POS_DEFAULT 0
+#define TEXT_POS_CENTER  1
+#define TEXT_POS_LEFT    2
+#define TEXT_POS_RIGHT   3
+#define IS_TEXT_POS_DEFAULT(x) (x->flags[3] == TEXT_POS_DEFAULT)
+#define GET_TEXT_POS(x) (x->flags[3])
+#define IS_TEXT_CENTER(x) (x->flags[3] == TEXT_POS_CENTER)
+#define IS_TEXT_LEFT(x)   (x->flags[3] == TEXT_POS_LEFT)
+#define IS_TEXT_RIGHT(x)  (x->flags[3] == TEXT_POS_RIGHT)
 
 #include <X11/keysymdef.h>
 #include <X11/keysym.h>
@@ -56,10 +60,6 @@
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
 
-#ifdef XPM
-#include <X11/xpm.h>
-#endif
-
 /* Prototype for yylex().  There must be a way to get this automatically from
    yacc/bison.  */
 extern int yylex(void);
@@ -120,6 +120,7 @@ typedef struct			/* Type pour la gestion X */
   char *icon;			/* Icone pour l'application iconisee */
   Bloc *periodictasks;		/* Tableau de taches periodiques */
   Bloc *initbloc;		/* Bloc d'initalisation */
+  Bloc *quitfunc;               /* Bloc executed at exit */
 } ScriptProp;
 
 
@@ -151,7 +152,7 @@ typedef struct			/* Type pour les boutons */
   int value3;
 } MyObject;
 
-typedef MyObject TabObj[100];
+typedef MyObject TabObj[1000];
 
 
 typedef struct
@@ -170,11 +171,13 @@ typedef struct
   char *icon;
   char *title;
   Bloc *periodictasks;
+  Bloc *quitfunc;
   int HaveXSelection;
-  char* TabScriptId[99];
+  char* TabScriptId[999];
   int NbChild;
   time_t BeginTime;
   char *TabArg[20];
+  Bool swallowed;
 } X11base ;
 
 typedef struct
@@ -210,11 +213,9 @@ struct XObj
   char *font;			/* Police utilisee */
   Pixmap iconPixmap;		/* Icone charge */
   Pixmap icon_maskPixmap;	/* Icone masque */
+  Pixmap icon_alphaPixmap;	/* alpha channel */
   int icon_w,icon_h;		/* Largeur et hauteur de l'icone */
-  XFontStruct *xfont;
-#ifdef I18N_MB
-  XFontSet xfontset;
-#endif
+  FlocaleFont *Ffont;
   int value;			/* Valeur courante */
   int value2;			/* Valeur minimale */
   int value3;			/* Valeur maximale */
