@@ -33,8 +33,9 @@
 
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
-#include "libs/Picture.h"
+#include "libs/FShape.h"
 #include "libs/Colorset.h"
+#include "libs/PictureBase.h"
 #include "FvwmScroll.h"
 
 char *MyName;
@@ -142,9 +143,10 @@ int main(int argc, char **argv)
   ScreenWidth = DisplayWidth(dpy,screen);
 
   SetMessageMask(fd, M_CONFIG_INFO | M_END_CONFIG_INFO | M_SENDCONFIG);
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   /* prevent core dumps if fvwm doesn't provide any colorsets */
   AllocColorset(0);
+  FShapeInit(dpy);
 
   /* scan config file for set-up parameters */
   /* Colors and fonts */
