@@ -38,28 +38,21 @@
 #include "libs/fvwmlib.h"
 #include "libs/Colorset.h"
 #include "libs/safemalloc.h"
+#include "libs/Flocale.h"
+#include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
 
 #include "ButtonArray.h"
 #include "Mallocs.h"
 
-#ifdef I18N_MB
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
-
-extern XFontStruct *ButtonFont, *SelButtonFont;
-#ifdef I18N_MB
-extern XFontSet ButtonFontset, SelButtonFontset;
-#endif
+extern FlocaleFont *FButtonFont, *FSelButtonFont;
 extern Display *dpy;
 extern Window win;
 extern GC shadow, hilite, graph, whitegc, blackgc, checkered, icongraph,
           iconhilite, iconshadow, focusgraph, focushilite, focusshadow;
 extern GC iconbackgraph;
 extern GC focusbackgraph;
+extern FlocaleWinString *FwinString;
 extern int button_width;
 extern int iconcolorset;
 extern int focuscolorset;
@@ -84,7 +77,7 @@ extern int NRows, RowHeight;
 void Draw3dRect(Window wn, int x, int y, int w, int h, int state,
 		Bool iconified)
 {
-  colorset_struct *cset;
+  colorset_struct *cset = &Colorset[0];  /* initialize to avoid warning */
   int d = 1;
 
   if (ThreeDfvwm)
@@ -140,16 +133,16 @@ void Draw3dRect(Window wn, int x, int y, int w, int h, int state,
     if (focuscolorset >= 0 && (cset->pixmap || cset->shape_mask))
     {
       /* we have a colorset background */
-      SetRectangleBackground(dpy, win, x + 2, y + 2, w - 4 + (2 - d), 
+      SetRectangleBackground(dpy, win, x + 2, y + 2, w - 4 + (2 - d),
 			     h - 4 + (2 - d), cset, Pdepth, focusgraph);
     }
     else
     {
-      XFillRectangle (dpy, wn, focusbackgraph, x + 2, y + 2, 
+      XFillRectangle (dpy, wn, focusbackgraph, x + 2, y + 2,
 		      w - 4 + (2 - d), h - 4 + (2 - d));
     }
     if (!NoBrightFocus)
-      XFillRectangle (dpy, wn, checkered, x + 2, y + 2, 
+      XFillRectangle (dpy, wn, checkered, x + 2, y + 2,
 		      w - 4 + (2 - d), h - 4 + (2 - d));
     XDrawLine (dpy, wn, blackgc, x, y, x+w-2, y);
     XDrawLine (dpy, wn, blackgc, x, y, x, y+h-2);
@@ -185,11 +178,11 @@ void Draw3dRect(Window wn, int x, int y, int w, int h, int state,
 /* -------------------------------------------------------------------------
    ButtonNew - Allocates and fills a new button structure
    ------------------------------------------------------------------------- */
-Button *ButtonNew(const char *title, Picture *p, int state, int count)
+Button *ButtonNew(const char *title, FvwmPicture *p, int state, int count)
 {
   Button *new;
 
-  if (title == NULL) 
+  if (title == NULL)
     return NULL;
   new = (Button *)safemalloc(sizeof(Button));
   new->title = safemalloc(strlen(title)+1);
@@ -197,6 +190,7 @@ Button *ButtonNew(const char *title, Picture *p, int state, int count)
   if (p != NULL) {
     new->p.picture = p->picture;
     new->p.mask    = p->mask;
+    new->p.alpha   = p->alpha;
     new->p.width   = p->width;
     new->p.height  = p->height;
     new->p.depth   = p->depth;
@@ -221,10 +215,7 @@ void ButtonDraw(Button *button, int x, int y, int w, int h)
   char *t3p = "...";
   int state, x3p, newx;
   int search_len;
-  XFontStruct *font;
-#ifdef I18N_MB
-  XFontSet fontset;
-#endif
+  FlocaleFont *Ffont;
   XGCValues gcv;
   unsigned long gcm;
   GC *drawgc;
@@ -244,31 +235,23 @@ void ButtonDraw(Button *button, int x, int y, int w, int h)
   if (state != BUTTON_UP) { x++; y++; }
 
   if (state == BUTTON_BRIGHT || button == StartButton)
-#ifdef I18N_MB
   {
-    font = SelButtonFont;
-    fontset = SelButtonFontset;
+    Ffont = FSelButtonFont;
   }
-#else
-    font = SelButtonFont;
-#endif
   else
-#ifdef I18N_MB
   {
-    font = ButtonFont;
-    fontset = ButtonFontset;
+    Ffont = FButtonFont;
   }
-#else
-    font = ButtonFont;
-#endif
 
-  gcm = GCFont;
-  gcv.font = font->fid;
-  XChangeGC(dpy, *drawgc, gcm, &gcv);
+  if (Ffont->font != NULL)
+  {
+    gcm = GCFont;
+    gcv.font = Ffont->font->fid;
+    XChangeGC(dpy, *drawgc, gcm, &gcv);
+  }
 
   newx = 4;
-
-  w3p = XTextWidth(font, t3p, 3);
+  w3p = FlocaleTextWidth(Ffont, t3p, 3);
 
   if (button->p.picture != 0)
   {
@@ -276,18 +259,14 @@ void ButtonDraw(Button *button, int x, int y, int w, int h)
     int pheight = min(button->p.height, h-2);
     int offset = (button->p.height > h) ? 0 : ((h - button->p.height) >> 1);
     int pwidth = min(button->p.width, w-5);
-    gcm = GCClipMask | GCClipXOrigin | GCClipYOrigin;
-    gcv.clip_mask = button->p.mask;
-    gcv.clip_x_origin = x + 3;
-    gcv.clip_y_origin = y + offset;
-    XChangeGC(dpy, hilite, gcm, &gcv);
-    XCopyArea(dpy, button->p.picture, win, hilite, 0, 0,
-                   pwidth, pheight,
-                   gcv.clip_x_origin, gcv.clip_y_origin);
-    gcm = GCClipMask;
-    gcv.clip_mask = None;
-    XChangeGC(dpy, hilite, gcm, &gcv);
-
+ 
+    if (button->p.alpha != None)
+    {
+      XClearArea(dpy, win, x+3, y+offset, pwidth, pheight, False);
+    }
+    PGraphicsCopyFvwmPicture(dpy, &(button->p), win, hilite,
+			     0, 0, pwidth, pheight,
+			     x+3, y+offset);
     newx += pwidth+2;
   }
 
@@ -299,47 +278,44 @@ void ButtonDraw(Button *button, int x, int y, int w, int h)
   if ((newx + w3p + 2) >= w)
   {
     t3p = "..\0";
-    w3p = XTextWidth(font, t3p, 2);
+    w3p = FlocaleTextWidth(Ffont, t3p, 2);
     if ((newx + w3p + 2) >= w)
     {
       t3p = ".\0";
-      w3p = XTextWidth(font, t3p, 1);
+      w3p = FlocaleTextWidth(Ffont, t3p, 1);
       if ((newx + w3p + 2) >= w)
 	return;
     }
   }
 
   search_len = strlen(button->title);
-
+  FwinString->win = win;
+  FwinString->y = y + FButtonFont->ascent + 4;
+  FwinString->gc = *drawgc;
   button->truncate = False;
-  if (XTextWidth(font, button->title, search_len) > w-newx-3) {
+
+  if (FlocaleTextWidth(Ffont, button->title, search_len) > w-newx-3) {
 
     x3p = 0;
     while (search_len >= 0
-	   && ((x3p = newx + XTextWidth(font, button->title, search_len))
+	   && ((x3p = newx + FlocaleTextWidth(Ffont, button->title, search_len))
 	       > w-w3p-3))
       search_len--;
 
-    /* It seems a little bogus that we don't see if the "..." _itself_
-       will fit on the button; what if it won't?  Oh well.  */
-#ifdef I18N_MB
-    XmbDrawString(dpy, win, fontset,
-#else
-    XDrawString(dpy, win,
-#endif
-		*drawgc, x + x3p, y+ButtonFont->ascent+4, t3p, 3);
+    FwinString->str = t3p;
+    FwinString->x = x + x3p;
+    FlocaleDrawString(dpy, Ffont, FwinString, 0);
     button->truncate = True;
   }
 
   /* Only print as much of the title as will fit.  */
   if (search_len)
-#ifdef I18N_MB
-    XmbDrawString(dpy, win, fontset,
-#else
-    XDrawString(dpy, win,
-#endif
-		*drawgc, x+newx, y+font->ascent+4,
-		button->title, search_len);
+  {
+    FwinString->str = button->title;
+    FwinString->x = x + newx;
+    FwinString->len = search_len;
+    FlocaleDrawString(dpy, Ffont, FwinString, FWS_HAVE_LENGTH);
+  }
 }
 
 
@@ -431,7 +407,7 @@ void UpdateArray(ButtonArray *array,int x,int y,int w, int h, int tw)
 /* -------------------------------------------------------------------------
    AddButton - Allocate space for and add the button to the list
    ------------------------------------------------------------------------- */
-void AddButton(ButtonArray *array, const char *title, Picture *p, int state,
+void AddButton(ButtonArray *array, const char *title, FvwmPicture *p, int state,
                int count, int iconified)
 {
   Button *new, *temp;
@@ -501,7 +477,7 @@ int UpdateButton(ButtonArray *array, int butnum, const char *title, int state)
 /* -------------------------------------------------------------------------
    UpdateButtonPicture - Change the picture of a button
    ------------------------------------------------------------------------- */
-int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
+int UpdateButtonPicture(ButtonArray *array, int butnum, FvwmPicture *p)
 {
   Button *temp;
 
@@ -513,6 +489,7 @@ int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
   if (temp->p.picture != p->picture || temp->p.mask != p->mask) {
     temp->p.picture = p->picture;
     temp->p.mask    = p->mask;
+    temp->p.alpha   = p->alpha;
     temp->p.width   = p->width;
     temp->p.height  = p->height;
     temp->p.depth   = p->depth;
@@ -679,19 +656,31 @@ int LocateButton(ButtonArray *array, int xp,  int yp, int *xb, int *yb,
 extern int StartButtonWidth;
 void ButtonCoordinates(ButtonArray *array, int numbut, int *xc, int *yc)
 {
-  Button *temp;
-  int x = 0;
-  int y = 0;
-  int r = 0;
+	Button *temp;
+	int x = 0;
+	int y = 0;
+	int r = 0;
 
-  for(temp=array->head; temp->count != numbut; temp=temp->next) {
-    if((x + 2*array->tw > array->w) && (r < NRows))
-      { x = 0; y += RowHeight+2; ++r; }
-    else
-     x += array->tw;
-  }
+	for(temp=array->head; temp->count != numbut; temp=temp->next)
+	{
+		if((x + 2*array->tw > array->w) && (r < NRows))
+		{
+			x = 0;
+			y += RowHeight+2;
+			++r;
+		}
+		else
+		{
+			x += array->tw;
+		}
+	}
 
-  *xc = x+StartButtonWidth+3;
-  *yc = y;
+	*xc = x+StartButtonWidth+3;
+	*yc = y;
 }
 
+void ButtonDimensions(ButtonArray *array, int *width, int *height)
+{
+	*width = array->tw;
+	*height = RowHeight+2;
+}
