@@ -54,6 +54,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <stdarg.h>
+#include <locale.h>
 
 #include <unistd.h>
 #include <ctype.h>
@@ -64,10 +65,6 @@
 
 #include <stdlib.h>
 
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
-
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"  /* for pixmaps routines */
 #include "libs/FScreen.h"
@@ -75,6 +72,8 @@
 #include "libs/safemalloc.h"
 #include "libs/fvwmsignal.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
+#include "libs/gravity.h"
 
 #include "FvwmTaskBar.h"
 #include "ButtonArray.h"
@@ -132,10 +131,8 @@ GC      checkered = None;
 int     colorset = -1;
 int     iconcolorset = -1;
 int     focuscolorset = -1;
-XFontStruct *ButtonFont, *SelButtonFont;
-#ifdef I18N_MB
-XFontSet ButtonFontset, SelButtonFontset;
-#endif
+FlocaleFont *FButtonFont, *FSelButtonFont;
+FlocaleWinString *FwinString;
 int fontheight;
 static Atom wm_del_win;
 Atom MwmAtom = None;
@@ -150,7 +147,7 @@ int  win_width    = 5,
      win_y,
      win_border = 4,
      win_has_title = 0,
-     win_has_bottom_title = 0,
+     win_title_dir = 0,
      win_title_height = 0,
      win_is_shaded = 0,
      button_width = DEFAULT_BTN_WIDTH,
@@ -162,12 +159,9 @@ int  win_width    = 5,
 			 *  0 hidden -> not hidden (for the events loop) */
      FocusInWin = 0;    /* 1 if the Taskbar has the focus */
 
-static int auto_stick_y = 0;
-
 static volatile sig_atomic_t AlarmSet = NOT_SET;
 static volatile sig_atomic_t tip_window_alarm = False;
 static volatile sig_atomic_t hide_taskbar_alarm = False;
-static volatile sig_atomic_t stick_taskbar_alarm = False;
 
 
 int UpdateInterval = 30;
@@ -175,7 +169,7 @@ int UpdateInterval = 30;
 ButtonArray buttons;
 List windows;
 
-char *ClickAction[NUMBER_OF_MOUSE_BUTTONS] =
+char *ClickAction[max(NUMBER_OF_MOUSE_BUTTONS, 3)] =
 {
   DEFAULT_CLICK1,
   DEFAULT_CLICK2,
@@ -189,7 +183,7 @@ char *ClickAction[NUMBER_OF_MOUSE_BUTTONS] =
      *FocusBackColor = NULL,
      *FocusForeColor = NULL,
      *geometry       = NULL,
-     *font_string    = "fixed",
+     *font_string    = NULL,
      *selfont_string = NULL;
 
 static char *AnimCommand = NULL;
@@ -230,22 +224,23 @@ extern TipStruct Tip;
 /* Imported from Start */
 extern int StartButtonWidth, StartButtonHeight;
 extern char *StartPopup;
+extern char *StartCommand;
 
 char *ImagePath = NULL;
 char *XineramaConfig = NULL;
 static int fscreen = 0;
+int ColorLimit = 0;
 
 static void ParseConfig( void );
 static void ParseConfigLine(char *tline);
 static void ShutMeDown(void);
 static RETSIGTYPE TerminateHandler(int sig);
 static RETSIGTYPE Alarm(int sig);
-static void SetAlarm(tb_alarm_t event);
-static void ClearAlarm(tb_alarm_t event);
+static void SetAlarm(int event);
+static void ClearAlarm(void);
 static void DoAlarmAction(void);
 static int ErrorHandler(Display*, XErrorEvent*);
 static Bool change_colorset(int cset, Bool force);
-static void WarpTaskBar(int y);
 int IsItemIndexIconSuppressed(List *list, int i);
 
 /******************************************************************************
@@ -259,6 +254,9 @@ int main(int argc, char **argv)
   char *s;
   int i;
 
+  FlocaleInit(LC_CTYPE, "", "", "FvwmTaskBar");
+  setlocale(LC_TIME, "");
+
   for (i = 3; i < NUMBER_OF_MOUSE_BUTTONS; i++)
   {
     ClickAction[i] = DEFAULT_CLICK_N;
@@ -270,9 +268,6 @@ int main(int argc, char **argv)
   if (s != NULL)
     temp = s + 1;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
 
   if((argc != 6)&&(argc != 7)) {
     fprintf(stderr,"%s Version %s should only be executed by fvwm!\n",temp,
@@ -342,13 +337,13 @@ int main(int argc, char **argv)
 #endif
 
   SetMessageMask(Fvwm_fd,M_ADD_WINDOW | M_CONFIGURE_WINDOW | M_DESTROY_WINDOW |
-		 M_WINDOW_NAME | M_ICON_NAME | M_RES_NAME | M_DEICONIFY |
-		 M_ICONIFY | M_END_WINDOWLIST | M_FOCUS_CHANGE |
+		 M_VISIBLE_NAME | M_RES_NAME |
+		 M_DEICONIFY | M_ICONIFY | M_END_WINDOWLIST | M_FOCUS_CHANGE |
 		 M_CONFIG_INFO | M_END_CONFIG_INFO | M_NEW_DESK | M_SENDCONFIG
-#ifdef MINI_ICONS
 		 | M_MINI_ICON
-#endif
 		);
+  /* extended messages */
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME | MX_PROPERTY_CHANGE);
 
   /* Parse the config file */
   ParseConfig();
@@ -367,7 +362,7 @@ int main(int argc, char **argv)
 
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
-  SendFvwmPipe(Fvwm_fd, "Send_WindowList",0);
+  SendText(Fvwm_fd, "Send_WindowList",0);
 
   /* tell fvwm we're running */
   SendFinishedStartupNotification(Fvwm_fd);
@@ -417,11 +412,13 @@ void EndLessLoop(void)
     {
 
       if (FD_ISSET(x_fd, &readset) || XPending(dpy))
+      {
         LoopOnEvents();
+      }
 
       if (FD_ISSET(Fvwm_fd[1], &readset))
       {
-	ReadFvwmPipe();
+        ReadFvwmPipe();
       }
     }
     DoAlarmAction();
@@ -456,13 +453,11 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   int i = -1;
   char *string;
   long Desk;
-  Picture p;
+  FvwmPicture p;
   struct ConfigWinPacket  *cfgpacket;
   int iconified;
   rectangle new_g;
 
-/*    memset(&p, 0, sizeof(Picture)); */
-
   switch(type)
   {
   case M_FOCUS_CHANGE:
@@ -500,7 +495,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       if (win_border != (short)cfgpacket->border_width ||
 	  win_title_height != (int)cfgpacket->title_height ||
 	  win_has_title != HAS_TITLE(cfgpacket) ||
-	  win_has_bottom_title != HAS_BOTTOM_TITLE(cfgpacket))
+	  win_title_dir != GET_TITLE_DIR(cfgpacket))
       {
 	XSizeHints hints;
 	long dumy;
@@ -509,7 +504,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	win_width = screen_g.width-(win_border<<1);
 	win_title_height = (int)cfgpacket->title_height;
 	win_has_title = HAS_TITLE(cfgpacket);
-	win_has_bottom_title = HAS_BOTTOM_TITLE(cfgpacket);
+	win_title_dir = GET_TITLE_DIR(cfgpacket);
 
 	if (AutoStick)
 	{
@@ -520,14 +515,16 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	    {
 	      win_y = screen_g.height -
 		(AutoHide ?
-		 VISIBLE_PIXELS() - win_title_height*win_has_bottom_title :
+		 VISIBLE_PIXELS() -
+		 win_title_height * HAS_TITLE_DIR(cfgpacket, DIR_S) :
 		 win_height + win_border);
 	    }
 	    else
 	    {
 	      win_y = AutoHide ?
-		VISIBLE_PIXELS() + win_title_height*win_has_bottom_title
-		- win_height : win_border + win_title_height;
+		VISIBLE_PIXELS() +
+		      win_title_height * HAS_TITLE_DIR(cfgpacket, DIR_S) -
+		      win_height : win_border + win_title_height;
 	    }
 	  }
 	  else
@@ -535,12 +532,12 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	    if (win_y > Midline)
 	    {
 	      win_y = screen_g.height - win_border -
-		(win_has_bottom_title)*win_height;
+		HAS_TITLE_DIR(cfgpacket, DIR_S) * win_height;
 	    }
 	    else
 	    {
 	      win_y = win_title_height + win_border -
-		(win_has_bottom_title)*win_height;
+		HAS_TITLE_DIR(cfgpacket, DIR_S) * win_height;
 	    }
 	  }
 	  win_y += screen_g.y;
@@ -553,7 +550,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 
 	XSetWMNormalHints(dpy,win,&hints);
 
-	if (AutoStick)
+ 	if (AutoStick)
 	  XMoveResizeWindow(dpy, win, win_x, win_y, win_width, win_height);
 	else
 	  XResizeWindow(dpy, win, win_width, win_height);
@@ -570,12 +567,12 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  if (win_y > Midline)
 	  {
 	    win_y = screen_g.height - win_border -
-	      (win_has_bottom_title)*win_height;
+	      (win_title_dir == DIR_S) * win_height;
 	  }
 	  else
 	  {
 	    win_y = win_title_height + win_border -
-	      (win_has_bottom_title)*win_height;
+	      (win_title_dir == DIR_S) * win_height;
 	  }
 	}
 	else
@@ -626,7 +623,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  is_on_desk = True;
 	  is_changing_desk = False;
 	}
-	else if (!IsItemIndexSticky(&windows,i))
+	else
 	{
 	  is_on_desk = (DeskNumber == cfgpacket->desk);
 	  is_changing_desk = (Desk != cfgpacket->desk);
@@ -692,7 +689,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	redraw = 1;
       }
     }
-    else if (!UseSkipList || !DO_SKIP_WINDOW_LIST(cfgpacket))
+    else
     {
       AddItem(&windows, cfgpacket->w,
 	      cfgpacket, cfgpacket->desk, Count++);
@@ -717,26 +714,29 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       redraw = 1;
     break;
 
-#ifdef MINI_ICONS
   case M_MINI_ICON:
-    if ((i = FindItem(&windows, body[0])) == -1) break;
-    p.picture = body[6];
-    p.mask    = body[7];
-    p.width   = body[3];
-    p.height  = body[4];
-    p.depth   = body[5];
+    if (!FMiniIconsSupported || (i = FindItem(&windows, body[0])) == -1)
+      break;
+    {
+	    MiniIconPacket *mip = (MiniIconPacket *) body;
+	    p.picture = mip->picture;
+	    p.mask    = mip->mask;
+	    p.alpha   = mip->alpha;
+	    p.width   = mip->width;
+	    p.height  = mip->height;
+	    p.depth   = mip->depth;
+    }
     UpdateItemPicture(&windows, i, &p);
     if (UpdateButton(&buttons, i, NULL, DONT_CARE) != -1) {
       UpdateButtonPicture(&buttons, i, &p);
       redraw = 0;
     }
     break;
-#endif
 
-  case M_WINDOW_NAME:
-  case M_ICON_NAME:
-    if ((type == M_ICON_NAME   && !UseIconNames) ||
-	(type == M_WINDOW_NAME &&  UseIconNames)) break;
+  case M_VISIBLE_NAME:
+  case MX_VISIBLE_ICON_NAME:
+    if ((type == MX_VISIBLE_ICON_NAME   && !UseIconNames) ||
+	(type == M_VISIBLE_NAME &&  UseIconNames)) break;
     string = (char *) &body[3];
     if ((i = UpdateItemName(&windows, body[0], (char *)&body[3])) == -1)
       break;
@@ -809,7 +809,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  SendText(Fvwm_fd, buff, 0);
 	}
 	temp->needsupdate = 1;
-	temp->iconified = iconified;
+  	temp->iconified = iconified;
 	DrawButtonArray(&buttons, 0);
       }
       if (AnimCommand && AnimCommand[0] != 0)
@@ -842,6 +842,14 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   case M_NEW_PAGE:
     break;
 
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND && body[2] == 0 &&
+	colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+    {
+      redraw = 1;
+    }
+    break;
+
   case M_CONFIG_INFO:
     {
       char *tline;
@@ -878,23 +886,12 @@ void redraw_buttons()
 
   for (item=windows.head; item; item=item->next)
   {
-    if ((DeskNumber != item->Desk && !IS_STICKY(item) &&
-	 (!IS_ICONIFIED(item) || !IS_ICON_STICKY(item))))
+    if ((DeskNumber == item->Desk || IS_STICKY(item)) &&
+	(!DO_SKIP_WINDOW_LIST(item) || !UseSkipList))
     {
-      continue;
+      AddButton(&buttons, item->name, &(item->p), BUTTON_UP, item->count,
+		IS_ICONIFIED(item));
     }
-    if (DO_SKIP_WINDOW_LIST(item) && UseSkipList)
-    {
-      continue;
-    }
-    if (PageOnly &&
-	!fvwmrect_do_rectangles_intersect(&item->win_g, &global_scr_g))
-    {
-      continue;
-    }
-    AddButton(
-	    &buttons, item->name, &(item->p), BUTTON_UP, item->count,
-	    IS_ICONIFIED(item));
   }
 
   RedrawWindow(1);
@@ -972,6 +969,7 @@ static char *configopts[] =
   "imagepath",
   "colorset",
   XINERAMA_CONFIG_STRING,
+  "ColorLimit",
   NULL
 };
 
@@ -1053,6 +1051,9 @@ static void ParseConfigLine(char *tline)
       }
       CopyString(&XineramaConfig, rest);
       break;
+    case 3: /* ColorLimit */
+      sscanf(rest, "%d", &ColorLimit);
+      break;
     default:
       /* unknown option */
       break;
@@ -1068,10 +1069,10 @@ static void ParseConfigLine(char *tline)
     switch(index)
     {
     case 0: /* Font */
-      CopyStringWithQuotes(&font_string, rest);
+      CopyString(&font_string, rest);
       break;
-    case 1: /* SelFont */
-      CopyStringWithQuotes(&selfont_string, rest);
+    case 1: /* Selfont */
+      CopyString(&selfont_string, rest);
       break;
     case 2: /* Geometry */
       while (isspace((unsigned char)*rest) && *rest != '\n' && *rest != 0)
@@ -1193,17 +1194,16 @@ static void ParseConfigLine(char *tline)
 static RETSIGTYPE
 Alarm(int nonsense)
 {
-  if (AlarmSet & SHOW_TIP)
+
+  switch(AlarmSet)
   {
+  case SHOW_TIP:
     tip_window_alarm = True;
-  }
-  if (AlarmSet & HIDE_TASK_BAR)
-  {
-    hide_taskbar_alarm = True;
-  }
-  if (AlarmSet & STICK_TASK_BAR)
-  {
-    stick_taskbar_alarm = True;
+    break;
+
+  case HIDE_TASK_BAR:
+      hide_taskbar_alarm = True;
+    break;
   }
 
   AlarmSet = NOT_SET;
@@ -1224,12 +1224,7 @@ DoAlarmAction(void)
     hide_taskbar_alarm = False;
     HideTaskBar();
   }
-  if (stick_taskbar_alarm)
-  {
-    stick_taskbar_alarm = False;
-    WarpTaskBar(auto_stick_y);
-  }
-  if (tip_window_alarm)
+  else if (tip_window_alarm)
   {
     tip_window_alarm = False;
     if (AutoHide && WindowState == 0)
@@ -1242,8 +1237,13 @@ DoAlarmAction(void)
       /* We are now "sure" that the TaskBar is not hidden for the
 	 Event loop. We send a motion notify for activating tips */
       WindowState = 1;
-      XQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
-		    &pos_x, &pos_y, &dummy);
+      if (XQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
+			&pos_x, &pos_y, &dummy) == False)
+      {
+	/* pointer is on a different screen */
+	abs_x = 0;
+	abs_y = 0;
+      }
       sevent.xmotion.x = pos_x;
       sevent.xmotion.y = pos_y;
       sevent.xany.type = MotionNotify;
@@ -1280,8 +1280,8 @@ void CheckForTip(int x, int y)
     num = LocateButton(&buttons, x, y, &bx, &by, &name, &trunc);
     if (num != -1) {
       if ((Tip.type != num)  ||
-	  (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
-	PopupTipWindow(bx+3, by, name);
+          (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
+        PopupTipWindow(bx+3, by, name);
       Tip.type = num;
     } else {
       Tip.type = NO_TIP;
@@ -1293,19 +1293,18 @@ void CheckForTip(int x, int y)
       SetAlarm(SHOW_TIP);
   }
   else {
-    ClearAlarm(SHOW_TIP);
+    ClearAlarm();
     if (Tip.open) ShowTipWindow(0);
   }
 }
 
-
 /******************************************************************************
   LoopOnEvents - Process all the X events we get
 ******************************************************************************/
 void LoopOnEvents(void)
 {
   int  num = 0;
-  char tmp[100];
+  char *tmp;
   XEvent Event;
   XEvent Event2;
   int x, y, redraw;
@@ -1337,8 +1336,26 @@ void LoopOnEvents(void)
 	if (Event.xbutton.button >= 1 &&
 	    Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS)
 	{
-	  SendFvwmPipe(Fvwm_fd, ClickAction[Event.xbutton.button-1],
-		       ItemID(&windows, num));
+	  rectangle r;
+	  Window tmpw;
+
+	  ButtonCoordinates(&buttons, num, &r.x, &r.y);
+	  ButtonDimensions(&buttons, &r.width, &r.height);
+	  XTranslateCoordinates(dpy, win, Root, r.x, r.y, &r.x, &r.y, &tmpw);
+	  tmp = module_expand_action(
+		  dpy, screen, ClickAction[Event.xbutton.button-1], &r, NULL,
+		  NULL);
+	  if (tmp)
+	  {
+	    SendFvwmPipe(Fvwm_fd, tmp, ItemID(&windows, num));
+	    free(tmp);
+	  }
+	  else
+	  {
+	    SendFvwmPipe(
+		    Fvwm_fd, ClickAction[Event.xbutton.button-1],
+		    ItemID(&windows, num));
+	  }
 	}
       }
 
@@ -1351,7 +1368,7 @@ void LoopOnEvents(void)
 	if (num == ButPressed)
 	  RadioButton(&buttons, num, BUTTON_DOWN);
 	if (num != -1)
-	  SendFvwmPipe(Fvwm_fd, "Focus 0", ItemID(&windows, num));
+	  SendText(Fvwm_fd, "Focus 0", ItemID(&windows, num));
       }
       ButPressed = -1;
       redraw = 0;
@@ -1370,8 +1387,38 @@ void LoopOnEvents(void)
 	  /* bar in bottom of the screen */
 	  y = win_y - screen_g.height;
 	}
-	sprintf(tmp,"Popup %s %d %d", StartPopup, x, y);
-	SendFvwmPipe(Fvwm_fd, tmp, 0);
+	if (StartCommand != NULL)
+	{
+	  rectangle r;
+	  Window tmpw;
+
+	  r.x = 0;
+	  r.y = 0;
+	  r.width = StartButtonWidth;
+	  r.height = StartButtonHeight;
+	  XTranslateCoordinates(dpy, win, Root, r.x, r.y, &r.x, &r.y, &tmpw);
+	  tmp = module_expand_action(dpy, screen, StartCommand, &r, NULL, NULL);
+	  if (tmp)
+	  {
+	    SendText(Fvwm_fd, tmp, win);
+	    free(tmp);
+	  }
+	  else
+	  {
+	    SendText(Fvwm_fd, StartCommand, win);
+	  }
+	}
+	if (StartPopup != NULL)
+	{
+	  tmp = (char *)safemalloc(strlen(StartPopup) + 7);
+	  sprintf(tmp,"Popup %s", StartPopup);
+	  SendText(Fvwm_fd, tmp, win);
+	  free(tmp);
+	}
+	if (StartPopup == NULL && StartCommand == NULL)
+	{
+	  SendText(Fvwm_fd, "Popup StartMenu", win);
+	}
       } else {
 	StartButtonUpdate(NULL, BUTTON_UP);
 	if (MouseInMail(Event.xbutton.x, Event.xbutton.y)) {
@@ -1428,7 +1475,7 @@ void LoopOnEvents(void)
 	}
       } else {
 	if (num != -1 && num != ButPressed)
-	  SendFvwmPipe(Fvwm_fd, "Focus 0", ItemID(&windows, num));
+	  SendText(Fvwm_fd, "Focus 0", ItemID(&windows, num));
       }
 
       CheckForTip(Event.xmotion.x, Event.xmotion.y);
@@ -1436,7 +1483,7 @@ void LoopOnEvents(void)
 
     case LeaveNotify:
       NewTimestamp = Event.xcrossing.time;
-      ClearAlarm(SHOW_TIP);
+      ClearAlarm();
       if (Tip.open)
 	ShowTipWindow(0);
 
@@ -1485,7 +1532,7 @@ void LoopOnEvents(void)
 	  redraw = 0;
 	}
       } else if (num != -1 && num != ButPressed)
-	SendFvwmPipe(Fvwm_fd, "Focus 0", ItemID(&windows, num));
+	SendText(Fvwm_fd, "Focus 0", ItemID(&windows, num));
 
       CheckForTip(Event.xmotion.x, Event.xmotion.y);
       break;
@@ -1508,10 +1555,11 @@ void LoopOnEvents(void)
       }
       else if (Event.xconfigure.x != win_x || Event.xconfigure.y != win_y)
       {
+	if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+	  redraw = 1;
 	if (AutoStick)
 	{
-	  SetAlarm(STICK_TASK_BAR);
-	  auto_stick_y = Event.xconfigure.y;
+	  WarpTaskBar(Event.xconfigure.y, 0);
 	}
 	else
 	{
@@ -1658,10 +1706,15 @@ static void CreateOrUpdateGCs(void)
 
    /* only the foreground changes for all GCs */
    gcval.background = pback;
-   gcval.font = SelButtonFont->fid;
    gcval.graphics_exposures = False;
 
-   gcmask = GCForeground | GCBackground | GCFont | GCGraphicsExposures;
+   gcmask = GCForeground | GCBackground | GCGraphicsExposures;
+
+   if (FSelButtonFont->font != NULL)
+   {
+     gcmask |= GCFont;
+     gcval.font = FSelButtonFont->font->fid;
+   }
    /* Normal */
    gcval.foreground = pfore;
    if (graph)
@@ -1747,7 +1800,7 @@ static void CreateOrUpdateGCs(void)
 
    /* brighting */
    gcmask = GCForeground | GCBackground | GCTile |
-	    GCFillStyle  | GCGraphicsExposures;
+            GCFillStyle  | GCGraphicsExposures;
    if (focuscolorset >= 0 || FocusBackColor != NULL)
      gcval.foreground = pfocusback;
    else
@@ -1777,7 +1830,7 @@ static Bool change_colorset(int cset, Bool force)
     {
       SetWindowBackground(
 	dpy, win, win_width, win_height, &Colorset[colorset],
-	Pdepth, graph, True);
+	Pdepth,	graph, True);
     }
     do_redraw = True;
   }
@@ -1797,17 +1850,13 @@ void StartMeUp(void)
    int ret;
    int i;
    XSetWindowAttributes attr;
-#ifdef I18N_MB
-  char **ml;
-  XFontStruct **fs_list;
-#endif
 
    if (!(dpy = XOpenDisplay(""))) {
      fprintf(stderr,"%s: can't open display %s", Module,
 	     XDisplayName(""));
      exit (1);
    }
-   InitPictureCMap(dpy);
+   PictureInitCMap(dpy);
    FScreenInit(dpy);
    if (XineramaConfig)
    {
@@ -1816,6 +1865,7 @@ void StartMeUp(void)
    }
    AllocColorset(0);
    FShapeInit(dpy);
+   FlocaleAllocateWinString(&FwinString);
    x_fd = XConnectionNumber(dpy);
    screen= DefaultScreen(dpy);
    Root = RootWindow(dpy, screen);
@@ -1838,39 +1888,19 @@ void StartMeUp(void)
    if (selfont_string == NULL)
      selfont_string = font_string;
 
-#ifdef I18N_MB
-   if ((ButtonFontset=GetFontSetOrFixed(dpy,font_string)) == NULL)
+   if ((FButtonFont = FlocaleLoadFont(dpy, font_string, Module)) == NULL)
    {
-	   fprintf(stderr,
-		   "%s: Couldn't load font. Exiting!\n",Module);
-	   exit(1);
+     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+     exit(1);
    }
-   XFontsOfFontSet(ButtonFontset,&fs_list,&ml);
-   ButtonFont = fs_list[0];
-   if ((SelButtonFontset=GetFontSetOrFixed(dpy,selfont_string)) == NULL)
+   if ((FSelButtonFont = FlocaleLoadFont(dpy, selfont_string, Module)) == NULL)
    {
-	   fprintf(stderr,
-		   "%s: Couldn't load selfont. Exiting!\n",Module);
-	   exit(1);
+     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+     exit(1);
    }
-   XFontsOfFontSet(SelButtonFontset,&fs_list,&ml);
-   SelButtonFont = fs_list[0];
-#else
-   if ((ButtonFont = XLoadQueryFont(dpy, font_string)) == NULL) {
-     if ((ButtonFont = XLoadQueryFont(dpy, "fixed")) == NULL) {
-       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n", Module);
-       exit(1);
-     }
-   }
-   if ((SelButtonFont = XLoadQueryFont(dpy, selfont_string)) == NULL) {
-     if ((SelButtonFont = XLoadQueryFont(dpy, "fixed")) == NULL) {
-       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n", Module);
-       exit(1);
-     }
-   }
-#endif
 
-   fontheight = SelButtonFont->ascent + SelButtonFont->descent;
+   fontheight = (FButtonFont->height > FSelButtonFont->height)?
+     FButtonFont->height : FSelButtonFont->height;
 
    NRows = 1;
    RowHeight = fontheight + 8;
@@ -2007,8 +2037,8 @@ void StartMeUp(void)
    }
 
    XSelectInput(dpy,win,(ExposureMask | KeyPressMask | PointerMotionMask |
-			 EnterWindowMask | LeaveWindowMask |
-			 StructureNotifyMask));
+                         EnterWindowMask | LeaveWindowMask |
+                         StructureNotifyMask));
 
    InitGoodies();
    atexit(ShutMeDown);
@@ -2090,12 +2120,12 @@ PropMwmHints prop;
 /***********************************************************************
  WarpTaskBar -- Enforce AutoStick feature
  ***********************************************************************/
-static void WarpTaskBar(int y)
+void WarpTaskBar(int y, Bool force)
 {
   /* The tests on y are really useful ! */
   if (!AutoHide &&
       ((y != screen_g.y + screen_g.height - win_height - win_border &&
-	y !=  screen_g.y + win_border))) {
+	y !=  screen_g.y + win_border) || force)) {
     if (y > Midline)
     {
       win_y = screen_g.height - win_height - win_border;
@@ -2112,7 +2142,12 @@ static void WarpTaskBar(int y)
 
   if (AutoHide)
      SetAlarm(HIDE_TASK_BAR);
-  ClearAlarm(STICK_TASK_BAR);
+
+  /* Prevent oscillations caused by race with
+     time delayed TaskBarHide().  Is there any way
+     to prevent these Xevents from being sent
+     to the server in the first place? */
+  PurgeConfigEvents();
 }
 
 /***********************************************************************
@@ -2140,7 +2175,7 @@ void RevealTaskBar()
   int new_win_y;
   int inc_y = 2;
 
-  ClearAlarm(HIDE_TASK_BAR);
+  ClearAlarm();
    /* do not reveal if the taskbar is already revealed */
   if (WindowState >= 0)
     return;
@@ -2182,18 +2217,21 @@ void HideTaskBar()
   int d_x, d_y, wx, wy;
   unsigned int mask;
 
-  ClearAlarm(HIDE_TASK_BAR);
+  ClearAlarm();
   /* do not hide if the taskbar is already hiden */
   if (WindowState == -1 || win_is_shaded)
     return;
 
   if (FocusInWin)
   {
-    XQueryPointer(dpy, win, &d_rt,&d_ch, &d_x, &d_y,
-		  &wx, &wy, &mask);
-    if (wy >= -(win_border + win_title_height*(1-win_has_bottom_title)) &&
+    if (XQueryPointer(
+		dpy, win, &d_rt,&d_ch, &d_x, &d_y, &wx, &wy, &mask) == False)
+    {
+      /* pointer is on a different screen - that's okay here */
+    }
+    if (wy >= - (win_border + win_title_height * (win_title_dir != DIR_S)) &&
 	wy < win_height + win_border +
-	win_title_height*win_has_bottom_title)
+	win_title_height * (win_title_dir == DIR_S))
     {
       if (wy < 0 || wy >= win_height || wx < 0 || wx >= win_width)
 	SetAlarm(HIDE_TASK_BAR);
@@ -2207,7 +2245,7 @@ void HideTaskBar()
   if (win_y < Midline)
   {
     new_win_y = screen_g.y + VISIBLE_PIXELS() +
-      win_title_height*win_has_bottom_title - win_height;
+      win_title_height * (win_title_dir == DIR_S) - win_height;
     for (; win_y>=new_win_y; win_y -=inc_y)
     {
       XMoveWindow(dpy, win, win_x, win_y);
@@ -2217,7 +2255,7 @@ void HideTaskBar()
   else
   {
     new_win_y = screen_g.y + screen_g.height - VISIBLE_PIXELS() +
-      win_title_height*win_has_bottom_title;
+      win_title_height * (win_title_dir == DIR_S);
     for (; win_y<=new_win_y; win_y +=inc_y)
     {
       XMoveWindow(dpy, win, win_x, win_y);
@@ -2233,10 +2271,10 @@ void HideTaskBar()
  SetAlarm -- Schedule a timeout event
  ************************************************************************/
 static void
-SetAlarm(tb_alarm_t event)
+SetAlarm(int event)
 {
   alarm(0);  /* remove a race-condition */
-  AlarmSet |= event;
+  AlarmSet = event;
   alarm(1);
 }
 
@@ -2244,13 +2282,22 @@ SetAlarm(tb_alarm_t event)
  ClearAlarm -- Disable timeout events
  ************************************************************************/
 static void
-ClearAlarm(tb_alarm_t event)
+ClearAlarm(void)
 {
-  AlarmSet &= ~event;
-  if (AlarmSet == 0)
-  {
-    alarm(0);
-  }
+  AlarmSet = NOT_SET;
+  alarm(0);
+}
+
+/***********************************************************************
+ PurgeConfigEvents -- Wait for and purge ConfigureNotify events.
+ ************************************************************************/
+void PurgeConfigEvents(void)
+{
+  XEvent Event;
+
+  XPeekEvent(dpy, &Event);
+  while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event))
+    ;
 }
 
 /************************************************************************
