@@ -61,7 +61,7 @@ void AddItem(List *list, long id, ConfigWinPacket *cfgpacket,
   memcpy(&new->flags, &cfgpacket->flags, sizeof(new->flags));
   new->Desk=Desk;
   new->count=count;
-  memset((void *)&(new->p), 0, sizeof(Picture));
+  memset((void *)&(new->p), 0, sizeof(FvwmPicture));
   new->next=NULL;
   if (list->tail==NULL) list->head=list->tail=new;
   else {
@@ -301,7 +301,7 @@ int IsItemIndexSticky(List *list, int i)
   Item *temp;
   for(temp=list->head;temp!=NULL && temp->count!=i;temp=temp->next);
   if (temp==NULL) return -1;
-  return IS_STICKY(temp);
+  return (IS_STICKY(temp) || (IS_ICONIFIED(temp) && IS_ICON_STICKY(temp)));
 }
 
 /******************************************************************************
@@ -364,7 +364,7 @@ void CopyItem(List *dest, List *source, int n)
 /******************************************************************************
   UpdateItemPicture - Adds the picture information in the list
 ******************************************************************************/
-void UpdateItemPicture(List *list, int n, Picture *p)
+void UpdateItemPicture(List *list, int n, FvwmPicture *p)
 {
   Item *temp;
 
@@ -396,7 +396,7 @@ int GetDeskNumber(List *list, int n, long *Desk)
 /******************************************************************************
   GetItemPicture - Returns the picture
 ******************************************************************************/
-Picture *GetItemPicture(List *list, int n)
+FvwmPicture *GetItemPicture(List *list, int n)
 {
   Item *temp;
 
