@@ -18,61 +18,84 @@
 #include <X11/Xlib.h>
 
 #include "libs/fvwmlib.h"
+#include "libs/Flocale.h"
+#include "libs/Picture.h"
 #include "ButtonArray.h"
 #include "Mallocs.h"
 
 extern Display *dpy;
 extern Window Root, win;
-extern XFontStruct *ButtonFont;
-#ifdef I18N_MB
-extern XFontSet ButtonFontset;
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
+extern FlocaleFont *FButtonFont;
 extern int Clength;
 extern char *ImagePath;
+extern int ColorLimit;
+
 Button *StartButton;
 int StartButtonWidth, StartButtonHeight;
 char *StartName     = NULL,
+     *StartCommand  = NULL,
      *StartPopup    = NULL,
      *StartIconName = NULL;
 
 
-void StartButtonParseConfig(const char *tline, char *Module)
+static char *startopts[] =
 {
-  if(strncasecmp(tline,CatString3(Module,"StartName",""), Clength+9)==0)
-    CopyString(&StartName,&tline[Clength+9]);
-  else if(strncasecmp(tline,CatString3(Module,"StartMenu",""), Clength+9)==0)
-    CopyString(&StartPopup,&tline[Clength+9]);
-  else if(strncasecmp(tline,CatString3(Module,"StartIcon",""), Clength+9)==0)
-    CopyString(&StartIconName,&tline[Clength+9]);
+  "StartName",
+  "StartMenu",
+  "StartIcon",
+  "StartCommand",
+  NULL
+};
+
+Bool StartButtonParseConfig(char *tline)
+{
+  char *rest;
+  char *option;
+  int i;
+
+  option = tline + Clength;
+  i = GetTokenIndex(option, startopts, -1, &rest);
+  while (*rest && *rest != '\n' && isspace(*rest))
+    rest++;
+  switch(i)
+  {
+  case 0: /* StartName */
+    CopyString(&StartName, rest);
+    break;
+  case 1: /* StartMenu */
+    CopyString(&StartPopup, rest);
+    break;
+  case 2: /* StartIcon */
+    CopyString(&StartIconName, rest);
+    break;
+  case 3: /* StartCommand */
+    CopyString(&StartCommand, rest);
+    break;
+  default:
+    /* unknown option */
+    return False;
+  } /* switch */
+
+  return True;
 }
 
 void StartButtonInit(int height)
 {
-  Picture *p = NULL;
+  FvwmPicture *p = NULL;
   int pw;
 
   /* some defaults */
   if (StartName  == NULL)
     UpdateString(&StartName, "Start");
-  if (StartPopup == NULL)
-    UpdateString(&StartPopup, "StartMenu");
   if (StartIconName == NULL)
     UpdateString(&StartIconName, "mini-start.xpm");
 
-  /** FIXME: what should the colour limit be?
-      I put in -1, which apparently imposes NO limit.
-  **/
-  p = GetPicture(dpy, win, ImagePath, StartIconName, -1);
+  p = PGetFvwmPicture(dpy, win, ImagePath, StartIconName, ColorLimit);
 
   StartButton = (Button *)ButtonNew(StartName, p, BUTTON_UP,0);
   if (p != NULL) pw = p->width+3; else pw = 0;
-  StartButtonWidth = XTextWidth(ButtonFont, StartName, strlen(StartName)) +
-    pw + 14;
+  StartButtonWidth = FlocaleTextWidth(FButtonFont, StartName, strlen(StartName))
+    + pw + 14;
   StartButtonHeight = height;
 }
 
