@@ -72,8 +72,9 @@
 #include <stdlib.h>
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Module.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 #include "libs/Colorset.h"
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -244,6 +245,7 @@ int main(int argc, char **argv)
       Buttons[i].icons[j].h = 0;
       Buttons[i].icons[j].mask = None;	/* pixmap for the icon mask */
       Buttons[i].icons[j].icon = None;
+      Buttons[i].icons[j].alpha = None;
       Buttons[i].icons[j].depth = 0;
     }
     Buttons[i].IconWin = None;
@@ -268,10 +270,11 @@ int main(int argc, char **argv)
 	    XDisplayName(display_name));
     exit (1);
   }
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   /* Initialise default colorset */
   AllocColorset(0);
+  FShapeInit(dpy);
 
   x_fd = XConnectionNumber(dpy);
 
@@ -375,7 +378,7 @@ int main(int argc, char **argv)
       goto Builtin;
     }
     Buttons[BACK_BUTTON].icons[0].file=BgPixmapFile;
-    if (GetXPMFile(BACK_BUTTON,0))
+    if (LoadIconFile(BACK_BUTTON,0))
       break;
     else
       goto Solid;
@@ -398,10 +401,7 @@ int main(int argc, char **argv)
   case TEXTURE_BUILTIN:
     Builtin:
     TextureType=TEXTURE_BUILTIN;
-    /* if (GetXPMData( BACK_BUTTON, button_xpm))
-       break;
-       pdg */
-  default:
+   default:
     Solid:
     TextureType=TEXTURE_SOLID;
     if (GetSolidXPM(BACK_BUTTON, BgColor))
@@ -711,11 +711,12 @@ void Loop(void)
 			    )
 			    break; */
 
-	  XQueryPointer(dpy,main_win,
-			&dummy_rt,&dummy_c,
-			&dummy_x,&dummy_y,
-			&pos_x,&pos_y,
-			&dummy);
+	  if (XQueryPointer(
+		dpy, main_win, &dummy_rt, &dummy_c, &dummy_x, &dummy_y,
+		&pos_x, &pos_y, &dummy) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	  base = 0;
 	  dummy_y = (pos_y/BUTTONHEIGHT);
 	  dummy_x= (pos_x/BUTTONWIDTH);
@@ -1584,6 +1585,10 @@ void ParseOptions(char *filename)
 
     if((strncasecmp(tline,CatString3("*", MyName, "Geometry"),Clength+9)==0))
     {
+      w= -1;
+      h= -1;
+      x= -100000;
+      y= -100000;
       tmp = &tline[Clength+9];
       while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
       {
