@@ -50,23 +50,15 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
 
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
 #include "Mallocs.h"
+#include "libs/PictureGraphics.h"
 
-#ifdef I18gN_MB
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
-
-extern XFontStruct *ButtonFont;
-#ifdef I18N_MB
-extern XFontSet ButtonFontset;
-#endif
+extern FlocaleFont *FButtonFont;
+extern FlocaleWinString *FwinString;
 extern Display *dpy;
 extern Window win;
 extern GC shadow[MAX_COLOUR_SETS],hilite[MAX_COLOUR_SETS];
@@ -88,7 +80,7 @@ extern int UseSkipList;
 /* -------------------------------------------------------------------------
    ButtonNew - Allocates and fills a new button structure
    ------------------------------------------------------------------------- */
-Button *ButtonNew(char *title, Picture *p, int up)
+Button *ButtonNew(char *title, FvwmPicture *p, int up)
 {
   Button *new;
 
@@ -99,6 +91,7 @@ Button *ButtonNew(char *title, Picture *p, int up)
   {
     new->p.picture = p->picture;
     new->p.mask = p->mask;
+    new->p.alpha = p->alpha;
     new->p.width = p->width;
     new->p.height = p->height;
     new->p.depth = p->depth;
@@ -190,7 +183,7 @@ void ReorderButtons(ButtonArray *array, int ButNum, int FlipFocus)
 /******************************************************************************
   AddButton - Allocate space for and add the button to the bottom
 ******************************************************************************/
-int AddButton(ButtonArray *array, char *title, Picture *p, int up)
+int AddButton(ButtonArray *array, char *title, FvwmPicture *p, int up)
 {
   Button *new;
 
@@ -208,7 +201,7 @@ int AddButton(ButtonArray *array, char *title, Picture *p, int up)
 
 /* in Taskbar this replaces below  ArrangeButtonArray (array);*/
 
-  new->tw=XTextWidth(ButtonFont,title,strlen(title));
+  new->tw=FlocaleTextWidth(FButtonFont,title,strlen(title));
   new->truncatewidth=0;
   new->next=NULL;
   new->needsupdate=1;
@@ -232,7 +225,7 @@ int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
     {
       temp->title=(char *)saferealloc(temp->title,strlen(title)+1);
       strcpy(temp->title,title);
-      temp->tw=XTextWidth(ButtonFont,title,strlen(title));
+      temp->tw=FlocaleTextWidth(FButtonFont,title,strlen(title));
       temp->truncatewidth = 0;
     }
     if (up!=-1)
@@ -247,7 +240,7 @@ int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
 /* -------------------------------------------------------------------------
    UpdateButtonPicture - Change the picture of a button
    ------------------------------------------------------------------------- */
-int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
+int UpdateButtonPicture(ButtonArray *array, int butnum, FvwmPicture *p)
 {
   Button *temp;
   temp=find_n(array,butnum);
@@ -256,6 +249,7 @@ int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
   {
     temp->p.picture = p->picture;
     temp->p.mask = p->mask;
+    temp->p.alpha = p->alpha;
     temp->p.width = p->width;
     temp->p.height = p->height;
     temp->p.depth = p->depth;
@@ -396,7 +390,7 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
   char *string;
   XGCValues gcv;
   unsigned long gcm;
-  XFontStruct *font;
+  FlocaleFont *Ffont;
 
   /* The margin we want between the relief/text/pixmaps */
 #define INNER_MARGIN 2
@@ -405,13 +399,22 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
   set=button->set;
   topgc = up ? hilite[set] : shadow[set];
   bottomgc = up ? shadow[set] : hilite[set];
-  font = ButtonFont;
+  Ffont = FButtonFont;
 
-  gcm = GCFont;
-  gcv.font = font->fid;
-  XChangeGC(dpy, graph[set], gcm, &gcv);
+  if (Ffont->font != NULL)
+  {
+    gcm = GCFont;
+    gcv.font = Ffont->font->fid;
+    XChangeGC(dpy, graph[set], gcm, &gcv);
+  }
 
-  Fontheight=ButtonFont->ascent+ButtonFont->descent;
+  Fontheight=FButtonFont->height;
+
+  if ((FftSupport && Ffont->fftf.fftfont != NULL) ||
+      (button->p.picture != 0 && button->p.alpha != 0))
+  {
+    clear_bg = True;
+  }
 
   /* handle transparency by clearing button, otherwise paint with background */
   if (clear_bg) {
@@ -421,23 +424,13 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
       XFillRectangle(dpy,win,background[set],x,y,w,h+1);
   }
 
-  if ((button->p.picture != 0)/* &&
-      (w + button->p.width + w3p + 3 > MIN_BUTTON_SIZE)*/) {
+  if (button->p.picture != 0) {
     /* clip pixmap to fit inside button */
     int height = min(button->p.height, h);
     int offset = (button->p.height > h) ? 0 : ((h - button->p.height) >> 1);
-    gcm = GCClipMask|GCClipXOrigin|GCClipYOrigin;
-    gcv.clip_mask = button->p.mask;
-    gcv.clip_x_origin = x + 2 + button->reliefwidth;
-    gcv.clip_y_origin = y + offset;
-    XChangeGC(dpy, hilite[set], gcm, &gcv);
-    XCopyArea(dpy, button->p.picture, win, hilite[set], 0, 0,
-                   button->p.width, height,
-                   gcv.clip_x_origin, gcv.clip_y_origin);
-    gcm = GCClipMask;
-    gcv.clip_mask = None;
-    XChangeGC(dpy, hilite[set], gcm, &gcv);
-
+    PGraphicsCopyFvwmPicture(dpy, &(button->p), win, hilite[set],
+			     0, 0, button->p.width, height,
+			     x + 2 + button->reliefwidth, y+offset);
     newx = button->p.width+2*INNER_MARGIN;
   }
   else
@@ -450,7 +443,7 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
         static int icon_offset = -1;
 
         if (icon_offset == -1)
-          icon_offset = XTextWidth(ButtonFont, "(", 1);
+          icon_offset = FlocaleTextWidth(FButtonFont, "(", 1);
         newx += icon_offset;
       }
     }
@@ -470,7 +463,8 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
     } else {
       if (TruncateLeft) {
 	/* move the pointer up until the rest fits */
-	while (*string && (newx + XTextWidth(ButtonFont, string, strlen(string))
+	while (*string && (newx +
+			   FlocaleTextWidth(FButtonFont, string, strlen(string))
 			   + 2 * button->reliefwidth + INNER_MARGIN) > w) {
 	  string++;
 	  len--;
@@ -478,21 +472,19 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
 	button->truncatewidth = w;
 	button->truncate_title = string;
       } else {
-	while ((len > 1) && (newx + XTextWidth(ButtonFont, string, len)
-			   + 2 * button->reliefwidth + INNER_MARGIN) > w)
+	while ((len > 1) && (newx +
+			     FlocaleTextWidth(FButtonFont, string, len)
+			     + 2 * button->reliefwidth + INNER_MARGIN) > w)
 	  len--;
       }
     }
   }
-#ifdef I18N_MB
-  XmbDrawString(dpy,win,ButtonFontset,graph[set],x+newx+button->reliefwidth,
-              y+1+button->reliefwidth+ButtonFont->ascent,
-              string,len);
-#else
-  XDrawString(dpy,win,graph[set],x+newx+button->reliefwidth,
-              y+1+button->reliefwidth+ButtonFont->ascent,
-              string,len);
-#endif
+  FwinString->str = string;
+  FwinString->len = len;
+  FwinString->x = x+newx+button->reliefwidth;
+  FwinString->y = y+1+button->reliefwidth+FButtonFont->ascent;
+  FwinString->gc = graph[set];
+  FlocaleDrawString(dpy, FButtonFont, FwinString, FWS_HAVE_LENGTH);
 
   /* Draw relief last */
   RelieveRectangle(dpy,win,x,y,w-1,h,topgc,bottomgc,button->reliefwidth);
@@ -661,18 +653,20 @@ void PrintButtons(ButtonArray *array)
     ConsoleMessage("   %s is %s\n",temp->title,(temp->up) ? "Up":"Down");
 }
 
-#ifdef MINI_ICONS
 /******************************************************************************
   ButtonPicture - Return the mini icon associated with the button
 ******************************************************************************/
-Picture *ButtonPicture(ButtonArray *array, int butnum)
+FvwmPicture *ButtonPicture(ButtonArray *array, int butnum)
 {
   Button *temp;
 
+  if (!FMiniIconsSupported)
+  {
+    return NULL;
+  }
   temp=find_n(array,butnum);
   return &(temp->p);
 }
-#endif
 
 /******************************************************************************
   IsButtonVisible - Says if the button should be in winlist
