@@ -62,6 +62,8 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/Flocale.h"
+#include "libs/gravity.h"
 
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
@@ -69,16 +71,6 @@
 #include "Mallocs.h"
 #include "libs/Colorset.h"
 
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#define XDrawString(t,u,v,w,x,y,z) XmbDrawString(t,u,ButtonFontset,v,w,x,y,z)
-#endif
-
 #define MAX_NO_ICON_ACTION_LENGTH (MAX_MODULE_INPUT_TEXT_LEN - 100)
 
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
@@ -109,10 +101,8 @@ GC shadow[MAX_COLOUR_SETS];
 GC hilite[MAX_COLOUR_SETS];
 GC background[MAX_COLOUR_SETS];
 Pixmap win_bg;
-XFontStruct *ButtonFont;
-#ifdef I18N_MB
-XFontSet ButtonFontset;
-#endif
+FlocaleFont *FButtonFont;
+FlocaleWinString *FwinString;
 int fontheight;
 int buttonheight;
 static Atom wm_del_win;
@@ -152,7 +142,7 @@ char *ClickAction[NUMBER_OF_BUTTONS] =
   *geometry="";
 int colorset[MAX_COLOUR_SETS];
 Pixmap pixmap[MAX_COLOUR_SETS];
-char *font_string = "fixed";
+char *font_string = NULL;
 Bool UseSkipList = False;
 Bool Anchor = True;
 Bool UseIconNames = False;
@@ -242,9 +232,7 @@ int main(int argc, char **argv)
     Clength = strlen(Module);
   }
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmWinList");
 
   Fvwm_fd[0] = atoi(argv[1]);
   Fvwm_fd[1] = atoi(argv[2]);
@@ -283,6 +271,7 @@ int main(int argc, char **argv)
   colorset[0] = colorset[1] = 0;
   colorset[2] = colorset[3] = 1;
   AllocColorset(1);
+  FlocaleAllocateWinString(&FwinString);
 
   /* Setup the XConnection */
   StartMeUp_I();
@@ -300,15 +289,15 @@ int main(int argc, char **argv)
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
   SetMessageMask(Fvwm_fd,
-#ifdef MINI_ICONS
 		 M_MINI_ICON |
-#endif
 		 M_CONFIGURE_WINDOW | M_ADD_WINDOW | M_DESTROY_WINDOW |
-		 M_WINDOW_NAME | M_ICON_NAME | M_DEICONIFY | M_ICONIFY |
-		 M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
+		 M_VISIBLE_NAME | M_DEICONIFY |
+		 M_ICONIFY | M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
 		 M_CONFIG_INFO | M_SENDCONFIG);
+  /* extended messages */
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME | MX_PROPERTY_CHANGE);
 
-  SendFvwmPipe(Fvwm_fd, "Send_WindowList",0);
+  SendText(Fvwm_fd, "Send_WindowList",0);
 
   /* Recieve all messages from Fvwm */
   atexit(ShutMeDown);
@@ -378,10 +367,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   char *name,*string;
   static int current_focus=-1;
   struct ConfigWinPacket  *cfgpacket;
-
-#ifdef MINI_ICONS
-  Picture p;
-#endif
+  FvwmPicture p;
 
   switch(type)
   {
@@ -395,10 +381,19 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       {
 	win_border_x = win_border_y = cfgpacket->border_width;
 	if (((win_grav == NorthWestGravity || win_grav == NorthEastGravity)
-	     && !HAS_BOTTOM_TITLE(cfgpacket)) ||
+	     && HAS_TITLE_DIR(cfgpacket, DIR_N)) ||
 	    ((win_grav == SouthWestGravity || win_grav == SouthEastGravity)
-	    && HAS_BOTTOM_TITLE(cfgpacket)))
+	    && HAS_TITLE_DIR(cfgpacket, DIR_S)))
+	{
 	  win_border_y +=  cfgpacket->title_height;
+	}
+	if (((win_grav == NorthWestGravity || win_grav == SouthWestGravity)
+	     && HAS_TITLE_DIR(cfgpacket, DIR_W)) ||
+	    ((win_grav == NorthEastGravity || win_grav == SouthEastGravity)
+	     && HAS_TITLE_DIR(cfgpacket, DIR_E)))
+	{
+	  win_border_x +=  cfgpacket->title_height;
+	}
       }
 #endif
       if ((i = FindItem(&windows,cfgpacket->w))!=-1)
@@ -431,17 +426,18 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       redraw=1;
       break;
 
-#ifdef MINI_ICONS
   case M_MINI_ICON:
-    if ((i=FindItem(&windows,body[0]))==-1) break;
-
+    if (!FMiniIconsSupported || (i=FindItem(&windows,body[0]))==-1)
+      break;
     if (UpdateButton(&buttons,i,NULL,-1)!=-1)
     {
-      p.width = body[3];
-      p.height = body[4];
-      p.depth = body[5];
-      p.picture = body[6];
-      p.mask = body[7];
+      MiniIconPacket *mip = (MiniIconPacket *) body;
+      p.picture = mip->picture;
+      p.mask    = mip->mask;
+      p.alpha   = mip->alpha;
+      p.width   = mip->width;
+      p.height  = mip->height;
+      p.depth   = mip->depth;
 
       UpdateButtonPicture(&buttons, i, &p);
 
@@ -449,11 +445,10 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       redraw = 1;
     }
     break;
-#endif
 
-    case M_WINDOW_NAME:
-    case M_ICON_NAME:
-      if ((type == M_ICON_NAME) ^ UseIconNames)
+    case M_VISIBLE_NAME:
+    case MX_VISIBLE_ICON_NAME:
+      if ((type == MX_VISIBLE_ICON_NAME) ^ UseIconNames)
 	break;
       string=(char *)&body[3];
       if ((i=UpdateItemName(&windows,body[0],string))==-1) break;
@@ -543,6 +538,13 @@ void ProcessMessage(unsigned long type,unsigned long *body)
         RedrawWindow(True, True);
       }
       break;
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND && body[2] == 0 &&
+	WindowState &&  win_bg == ParentRelative)
+    {
+      RedrawWindow(True, True);
+    }
+    break;
     case M_CONFIG_INFO:
       ParseConfigLine((char *)&body[3]);
       break;
@@ -622,8 +624,8 @@ ParseConfigLine(char *tline)
     return;
 
   if (strlen(tline) > 1) {
-    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4) == 0)
-      CopyStringWithQuotes(&font_string, &tline[Clength + 4]);
+    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4)	== 0)
+      CopyString(&font_string, &tline[Clength + 4]);
     else if (strncasecmp(tline, CatString3(Module, "Fore", ""), Clength + 4)
 	     == 0) {
       CopyString(&ForeColor[0], &tline[Clength + 4]);
@@ -775,14 +777,19 @@ void LoopOnEvents(void)
   XEvent Event;
 #if 1
   Window dummyroot,dummychild;
-  int x,x1,y,y1;
+  int x,xdummy,y,ydummy;
   unsigned int dummy1;
 
   if (Transient && !Checked)
   {
     if (Pressed)
     {
-      XQueryPointer(dpy,win,&dummyroot,&dummychild,&x1,&y1,&x,&y,&dummy1);
+      if (XQueryPointer(
+	    dpy, win, &dummyroot, &dummychild, &xdummy, &ydummy, &x, &y,
+	    &dummy1) == False)
+      {
+	/* pointer is on a different screen - that's okay here */
+      }
       num=WhichButton(&buttons,x,y);
       if (num!=-1)
       {
@@ -864,7 +871,7 @@ void LoopOnEvents(void)
 	  }
 	}
         if (Event.xconfigure.send_event && (win_x != Event.xconfigure.x
-            || win_y != Event.xconfigure.y)) {
+            || win_y != Event.xconfigure.y || win_bg == ParentRelative)) {
 	  win_x = Event.xconfigure.x;
 	  win_y = Event.xconfigure.y;
 	  if (win_bg == ParentRelative)
@@ -923,9 +930,7 @@ void AdjustWindow(Bool force)
 {
   int new_width=0,new_height=0,tw,i,total,all;
   char *temp;
-#ifdef MINI_ICONS
   int base_miw = 0;
-#endif
 
   total = ItemCountVisible(&windows);
   all = ItemCount(&windows);
@@ -939,29 +944,31 @@ void AdjustWindow(Bool force)
     return;
   }
 
-#ifdef MINI_ICONS
-  for(i=0; i<all; i++)
+  if (FMiniIconsSupported)
   {
-    if(IsButtonIndexVisible(&buttons,i) &&
-       ButtonPicture(&buttons,i)->picture != None)
+    for(i=0; i<all; i++)
     {
-      if (ButtonPicture(&buttons,i)->width > base_miw)
-	base_miw = ButtonPicture(&buttons,i)->width;
-      break;
+      if (IsButtonIndexVisible(&buttons,i) &&
+          ButtonPicture(&buttons,i)->picture != None)
+      {
+        if (ButtonPicture(&buttons,i)->width > base_miw)
+          base_miw = ButtonPicture(&buttons,i)->width;
+        break;
+      }
     }
+    base_miw += 2;
   }
-  base_miw += 2;
-#endif
   for(i=0;i<all;i++)
   {
     temp=ItemName(&windows,i);
     if(temp != NULL && IsItemIndexVisible(&windows,i))
     {
-	tw = 8 + XTextWidth(ButtonFont,temp,strlen(temp));
-	tw += XTextWidth(ButtonFont,"()",2);
-#ifdef MINI_ICONS
-	tw += base_miw;
-#endif
+	tw = 8 + FlocaleTextWidth(FButtonFont,temp,strlen(temp));
+	tw += FlocaleTextWidth(FButtonFont,"()",2);
+        if (FMiniIconsSupported)
+        {
+          tw += base_miw;
+        }
 	new_width=max(new_width,tw);
     }
   }
@@ -1125,8 +1132,14 @@ void MakeMeWindow(void)
   {
     fscreen_scr_arg fscr;
 
-    XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,
-		  &dummy1);
+    if (XQueryPointer(
+	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
+	  &dummy1) == False)
+    {
+      /* pointer is on a different screen */
+      hints.x = 0;
+      hints.y = 0;
+    }
     fscr.xypos.x = hints.x;
     fscr.xypos.y = hints.y;
     FScreenGetScrRect(
@@ -1240,6 +1253,8 @@ void MakeMeWindow(void)
   wm_del_win=XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
 
+  FwinString->win = win;
+
   {
     XTextProperty nametext;
     char *list[]={NULL,NULL};
@@ -1280,8 +1295,12 @@ void MakeMeWindow(void)
   {
     gcval.foreground=fore[i];
     gcval.background=back[i];
-    gcval.font=ButtonFont->fid;
-    gcmask=GCForeground|GCBackground|GCFont;
+    gcmask=GCForeground|GCBackground;
+    if (FButtonFont->font != NULL)
+    {
+      gcval.font=FButtonFont->font->fid;
+      gcmask |= GCFont;
+    }
     graph[i]=fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
     if(Pdepth < 2)
@@ -1348,8 +1367,14 @@ void MakeMeWindow(void)
       fprintf(stderr,"failed to grab pointer\n");
       exit(1);
     }
-    XQueryPointer(
-      dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
+    if (XQueryPointer(
+	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
+	  &dummy1) == False)
+    {
+      /* pointer is on a different screen */
+      hints.x = 0;
+      hints.y = 0;
+    }
     Pressed = !!SomeButtonDown(dummy1);
   }
 }
@@ -1365,7 +1390,7 @@ void StartMeUp_I(void)
       XDisplayName(""));
     exit (1);
   }
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
@@ -1378,12 +1403,6 @@ void StartMeUp_I(void)
 
 void StartMeUp_II(void)
 {
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
 
   if (geometry == NULL)
     UpdateString(&geometry, "");
@@ -1394,36 +1413,16 @@ void StartMeUp_II(void)
   FScreenGetScrRect(
     NULL, fscreen, &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height);
 
-#ifdef I18N_MB
-  if ((ButtonFontset=XCreateFontSet(dpy,font_string,&ml,&mc,&ds)) == NULL) {
-#ifdef STRICTLY_FIXED
-    if ((ButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL)
-    {
-      exit(1);
-    }
-#else
-    if ((ButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL)
-    {
-      exit(1);
-    }
-#endif
-  }
-  XFontsOfFontSet(ButtonFontset,&fs_list,&ml);
-  ButtonFont = fs_list[0];
-#else
-  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL)
+  if ((FButtonFont = FlocaleLoadFont(dpy,font_string,Module)) == NULL)
   {
-    if ((ButtonFont=XLoadQueryFont(dpy,"fixed"))==NULL)
-    {
-      exit(1);
-    }
+    fprintf(stderr,"%s: Cannot load font, Exitting\n", Module);
+    exit(1);
   }
-#endif
 
-  fontheight = ButtonFont->ascent+ButtonFont->descent;
+  fontheight = FButtonFont->height;
   buttonheight = fontheight + 3 + 2 * ReliefWidth;
 
-  win_width=XTextWidth(ButtonFont,"XXXXXXXXXXXXXXX",10);
+  win_width=FlocaleTextWidth(FButtonFont,"XXXXXXXXXXXXXXX",10);
 
 }
 
